BEGIN;
--
-- PostgreSQL database dump
--

SET client_encoding = 'SQL_ASCII';
SET standard_conforming_strings = off;
SET check_function_bodies = false;
SET client_min_messages = warning;
SET escape_string_warning = off;

--
-- Name: cabar; Type: SCHEMA; Schema: -; Owner: db_global_owner
--

CREATE SCHEMA cabar;


ALTER SCHEMA cabar OWNER TO db_global_owner;

--
-- Name: code; Type: SCHEMA; Schema: -; Owner: db_global_owner
--

CREATE SCHEMA code;


ALTER SCHEMA code OWNER TO db_global_owner;

--
-- Name: SCHEMA code; Type: COMMENT; Schema: -; Owner: db_global_owner
--

COMMENT ON SCHEMA code IS 'Schema to hold functions that will write code.';


--
-- Name: code_internal; Type: SCHEMA; Schema: -; Owner: db_global_owner
--

CREATE SCHEMA code_internal;


ALTER SCHEMA code_internal OWNER TO db_global_owner;

--
-- Name: SCHEMA code_internal; Type: COMMENT; Schema: -; Owner: db_global_owner
--

COMMENT ON SCHEMA code_internal IS 'Schema to hold the interals of code that writes code.';


--
-- Name: inheritance; Type: SCHEMA; Schema: -; Owner: db_global_owner
--

CREATE SCHEMA inheritance;


ALTER SCHEMA inheritance OWNER TO db_global_owner;

--
-- Name: object_monitor; Type: SCHEMA; Schema: -; Owner: db_global_owner
--

CREATE SCHEMA object_monitor;


ALTER SCHEMA object_monitor OWNER TO db_global_owner;

--
-- Name: pgq; Type: SCHEMA; Schema: -; Owner: skytools
--

CREATE SCHEMA pgq;


ALTER SCHEMA pgq OWNER TO skytools;

--
-- Name: replication; Type: SCHEMA; Schema: -; Owner: db_global_owner
--

CREATE SCHEMA replication;


ALTER SCHEMA replication OWNER TO db_global_owner;

--
-- Name: setting; Type: SCHEMA; Schema: -; Owner: db_global_owner
--

CREATE SCHEMA setting;


ALTER SCHEMA setting OWNER TO db_global_owner;

--
-- Name: test; Type: SCHEMA; Schema: -; Owner: db_global_owner
--

CREATE SCHEMA test;


ALTER SCHEMA test OWNER TO db_global_owner;

--
-- Name: tools; Type: SCHEMA; Schema: -; Owner: db_global_owner
--

CREATE SCHEMA tools;


ALTER SCHEMA tools OWNER TO db_global_owner;

SET search_path = pgq, pg_catalog;

--
-- Name: ret_batch_event; Type: TYPE; Schema: pgq; Owner: skytools
--

CREATE TYPE ret_batch_event AS (
	ev_id bigint,
	ev_time timestamp with time zone,
	ev_txid bigint,
	ev_retry integer,
	ev_type text,
	ev_data text,
	ev_extra1 text,
	ev_extra2 text,
	ev_extra3 text,
	ev_extra4 text
);


ALTER TYPE pgq.ret_batch_event OWNER TO skytools;

--
-- Name: ret_batch_info; Type: TYPE; Schema: pgq; Owner: skytools
--

CREATE TYPE ret_batch_info AS (
	queue_name text,
	consumer_name text,
	batch_start timestamp with time zone,
	batch_end timestamp with time zone,
	prev_tick_id bigint,
	tick_id bigint,
	lag interval
);


ALTER TYPE pgq.ret_batch_info OWNER TO skytools;

--
-- Name: ret_consumer_info; Type: TYPE; Schema: pgq; Owner: skytools
--

CREATE TYPE ret_consumer_info AS (
	queue_name text,
	consumer_name text,
	lag interval,
	last_seen interval,
	last_tick bigint,
	current_batch bigint,
	next_tick bigint
);


ALTER TYPE pgq.ret_consumer_info OWNER TO skytools;

--
-- Name: ret_queue_info; Type: TYPE; Schema: pgq; Owner: skytools
--

CREATE TYPE ret_queue_info AS (
	queue_name text,
	queue_ntables integer,
	queue_cur_table integer,
	queue_rotation_period interval,
	queue_switch_time timestamp with time zone,
	queue_external_ticker boolean,
	queue_ticker_max_count integer,
	queue_ticker_max_lag interval,
	queue_ticker_idle_period interval,
	ticker_lag interval
);


ALTER TYPE pgq.ret_queue_info OWNER TO skytools;

SET search_path = cabar, pg_catalog;

--
-- Name: component_schema__add(text, text, boolean); Type: FUNCTION; Schema: cabar; Owner: db_global_owner
--

CREATE FUNCTION component_schema__add(p_component text, p_schema_name text, p_create boolean) RETURNS void
    AS $$

DECLARE
fname CONSTANT text := 'cabar.component_schema__add';
v_component_id cabar.component_schemas.component_id%TYPE := cabar.components__get_id( p_component );
BEGIN
  -- The NOT NULL will blow up if this fails anyway, so no sense in creating an assert for it
  IF v_component_id IS NULL THEN
    RAISE EXCEPTION '%: component "%" does not exist (cabar.components__get_id returned null)', fname, p_component;
  END IF;

  IF p_create THEN
    PERFORM code.schema( p_schema_name, p_component || '_owner' );
  END IF;
  PERFORM tools.assert(
    'cabar__component_schema__add__schema_exists'
    , EXISTS( SELECT * FROM pg_namespace WHERE nspname = p_schema_name )
    , 'schema ' || p_schema_name || ' does not exist'
  );

  INSERT INTO cabar.component_schemas( component_id, schema_name ) VALUES( v_component_id, p_schema_name );
EXCEPTION WHEN unique_violation THEN
  RAISE DEBUG '%: component/schema combination %/% already exists', fname, p_component, p_schema_name;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION cabar.component_schema__add(p_component text, p_schema_name text, p_create boolean) OWNER TO db_global_owner;

--
-- Name: FUNCTION component_schema__add(p_component text, p_schema_name text, p_create boolean); Type: COMMENT; Schema: cabar; Owner: db_global_owner
--

COMMENT ON FUNCTION component_schema__add(p_component text, p_schema_name text, p_create boolean) IS 'Marks (or creates) a schema belonging to a particular cabar component.';


--
-- Name: component_schema__add(text, text); Type: FUNCTION; Schema: cabar; Owner: db_global_owner
--

CREATE FUNCTION component_schema__add(p_component text, p_schema_name text) RETURNS void
    AS $_$
SELECT cabar.component_schema__add( $1, $2, true )
$_$
    LANGUAGE sql;


ALTER FUNCTION cabar.component_schema__add(p_component text, p_schema_name text) OWNER TO db_global_owner;

--
-- Name: FUNCTION component_schema__add(p_component text, p_schema_name text); Type: COMMENT; Schema: cabar; Owner: db_global_owner
--

COMMENT ON FUNCTION component_schema__add(p_component text, p_schema_name text) IS 'Creates a schema belonging to a particular cabar component';


SET default_tablespace = '';

SET default_with_oids = false;

--
-- Name: components; Type: TABLE; Schema: cabar; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE components (
    id smallint NOT NULL,
    component_search_path text,
    component text NOT NULL,
    description text
);


ALTER TABLE cabar.components OWNER TO db_global_owner;

--
-- Name: TABLE components; Type: COMMENT; Schema: cabar; Owner: db_global_owner
--

COMMENT ON TABLE components IS 'Lookup table for component. The data in this table is considered to be static, meaning that it should only be changed via a patch. This is generated code. This is a seed table.';


--
-- Name: COLUMN components.component_search_path; Type: COMMENT; Schema: cabar; Owner: db_global_owner
--

COMMENT ON COLUMN components.component_search_path IS 'Custom search_path to be used in patches for this component. If NULL the default path defined in tools.patch will be used.';


--
-- Name: components__get(smallint); Type: FUNCTION; Schema: cabar; Owner: db_global_owner
--

CREATE FUNCTION components__get(p_id smallint) RETURNS components
    AS $_$
SELECT * FROM cabar.components WHERE id = $1
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION cabar.components__get(p_id smallint) OWNER TO db_global_owner;

--
-- Name: FUNCTION components__get(p_id smallint); Type: COMMENT; Schema: cabar; Owner: db_global_owner
--

COMMENT ON FUNCTION components__get(p_id smallint) IS 'Returns row from cabar.components based on id. This is generated code.';


--
-- Name: components__get_id(text); Type: FUNCTION; Schema: cabar; Owner: db_global_owner
--

CREATE FUNCTION components__get_id(p_component text) RETURNS smallint
    AS $_$
SELECT id FROM cabar.components WHERE lower(component) = lower($1)
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION cabar.components__get_id(p_component text) OWNER TO db_global_owner;

--
-- Name: FUNCTION components__get_id(p_component text); Type: COMMENT; Schema: cabar; Owner: db_global_owner
--

COMMENT ON FUNCTION components__get_id(p_component text) IS 'Returns ID value for cabar.components based on component. This is generated code.';


--
-- Name: components__get_text(smallint); Type: FUNCTION; Schema: cabar; Owner: db_global_owner
--

CREATE FUNCTION components__get_text(p_id smallint) RETURNS text
    AS $_$
SELECT component FROM cabar.components WHERE id = $1
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION cabar.components__get_text(p_id smallint) OWNER TO db_global_owner;

--
-- Name: FUNCTION components__get_text(p_id smallint); Type: COMMENT; Schema: cabar; Owner: db_global_owner
--

COMMENT ON FUNCTION components__get_text(p_id smallint) IS 'Returns component value for cabar.components based on id. This is generated code.';


--
-- Name: env__reset(text); Type: FUNCTION; Schema: cabar; Owner: db_global_owner
--

CREATE FUNCTION env__reset(p_setting text) RETURNS void
    AS $$
DECLARE
  c_value CONSTANT text := tools.session__get( 'cabar patch', lower( p_setting ) );
BEGIN
  IF cabar.env_settings__get_id( p_setting ) IS NULL THEN
    RAISE EXCEPTION 'Setting "%" is not valid', p_setting;
  END IF;

  IF c_value IS NULL THEN
    RAISE EXCEPTION 'Setting "%" is not set in your session', p_setting;
  END IF;

  PERFORM pg_catalog.set_config( p_setting, c_value, true );
END
$$
    LANGUAGE plpgsql;


ALTER FUNCTION cabar.env__reset(p_setting text) OWNER TO db_global_owner;

--
-- Name: env__reset(); Type: FUNCTION; Schema: cabar; Owner: db_global_owner
--

CREATE FUNCTION env__reset() RETURNS void
    AS $$
BEGIN
  PERFORM cabar.env__reset( env_setting ) FROM cabar.env_settings;
END
$$
    LANGUAGE plpgsql;


ALTER FUNCTION cabar.env__reset() OWNER TO db_global_owner;

--
-- Name: env_settings; Type: TABLE; Schema: cabar; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE env_settings (
    id smallint NOT NULL,
    env_setting text NOT NULL,
    description text
);


ALTER TABLE cabar.env_settings OWNER TO db_global_owner;

--
-- Name: TABLE env_settings; Type: COMMENT; Schema: cabar; Owner: db_global_owner
--

COMMENT ON TABLE env_settings IS 'Lookup table for env_setting. The data in this table is considered to be static, meaning that it should only be changed via a patch. This is generated code. This is a seed table.';


--
-- Name: env_settings__get(smallint); Type: FUNCTION; Schema: cabar; Owner: db_global_owner
--

CREATE FUNCTION env_settings__get(p_id smallint) RETURNS env_settings
    AS $_$
SELECT * FROM cabar.env_settings WHERE id = $1
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION cabar.env_settings__get(p_id smallint) OWNER TO db_global_owner;

--
-- Name: FUNCTION env_settings__get(p_id smallint); Type: COMMENT; Schema: cabar; Owner: db_global_owner
--

COMMENT ON FUNCTION env_settings__get(p_id smallint) IS 'Returns row from cabar.env_settings based on id. This is generated code.';


--
-- Name: env_settings__get_id(text); Type: FUNCTION; Schema: cabar; Owner: db_global_owner
--

CREATE FUNCTION env_settings__get_id(p_env_setting text) RETURNS smallint
    AS $_$
SELECT id FROM cabar.env_settings WHERE lower(env_setting) = lower($1)
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION cabar.env_settings__get_id(p_env_setting text) OWNER TO db_global_owner;

--
-- Name: FUNCTION env_settings__get_id(p_env_setting text); Type: COMMENT; Schema: cabar; Owner: db_global_owner
--

COMMENT ON FUNCTION env_settings__get_id(p_env_setting text) IS 'Returns ID value for cabar.env_settings based on env_setting. This is generated code.';


--
-- Name: env_settings__get_text(smallint); Type: FUNCTION; Schema: cabar; Owner: db_global_owner
--

CREATE FUNCTION env_settings__get_text(p_id smallint) RETURNS text
    AS $_$
SELECT env_setting FROM cabar.env_settings WHERE id = $1
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION cabar.env_settings__get_text(p_id smallint) OWNER TO db_global_owner;

--
-- Name: FUNCTION env_settings__get_text(p_id smallint); Type: COMMENT; Schema: cabar; Owner: db_global_owner
--

COMMENT ON FUNCTION env_settings__get_text(p_id smallint) IS 'Returns env_setting value for cabar.env_settings based on id. This is generated code.';


--
-- Name: patch(text, text); Type: FUNCTION; Schema: cabar; Owner: db_global_owner
--

CREATE FUNCTION patch(p_component text, p_patch_file text) RETURNS void
    AS $$
DECLARE
  fname CONSTANT text := 'cabar.patch';
  r_patch tools.schema_patches;
  r_component cabar.components;
  v_component_id cabar.component_schemas.component_id%TYPE := cabar.components__get_id( p_component );
BEGIN
  -- The NOT NULL will blow up if this fails anyway, so no sense in creating an assert for it
  IF v_component_id IS NULL THEN
    RAISE EXCEPTION '%: component "%" does not exist (cabar.components__get_id returned null)', fname, p_component;
  END IF;
  r_component := cabar.components__get( v_component_id );

  SELECT INTO r_patch *
      FROM tools.schema_patches
      WHERE patch_file = p_patch_file
      FOR UPDATE
  ;
  IF NOT FOUND THEN
      RAISE EXCEPTION 'No record for patch %', p_patch_file;
  END IF;
  
  /*
   * The component that's been passed in should always match what's in the
   * table... except we need to make an exception for backwards compatibility
   * (at least for now)
   */
  IF r_patch.component_id IS DISTINCT FROM r_component.id
    -- Our backwards-compatibility exception
    AND NOT ( r_patch.component_id IS NULL AND lower( p_component ) = 'cnuapp' )
  THEN
    RAISE EXCEPTION '%: component_id in tools.schema_patches ( = % / % ) != component passed in ( = % / % )'
      , fname
      , r_patch.component_id
      , r_component.component
      , r_component.id
      , p_component
    ;
  END IF;

  UPDATE tools.schema_patches SET applied_at = now() WHERE patch_file = p_patch_file;

  -- If you add anything to this list make sure to add it to cabar.env_settings
  PERFORM tools.session__set( 'cabar patch', 'search_path', coalesce( r_component.component_search_path, 'cnu, public' ) );
  PERFORM tools.session__set( 'cabar patch', 'role', r_component.component || '_owner' );
  PERFORM tools.session__set( 'cabar patch', 'client_min_messages', 'WARNING' );
  RAISE DEBUG '%: session variables: search_path=%, role=%, client_min_messages=%'
    , fname
    , tools.session__get( 'cabar patch', 'search_path' )
    , tools.session__get( 'cabar patch', 'role' )
    , tools.session__get( 'cabar patch', 'client_min_messages' )
  ;
  PERFORM cabar.env__reset();
END
$$
    LANGUAGE plpgsql;


ALTER FUNCTION cabar.patch(p_component text, p_patch_file text) OWNER TO db_global_owner;

--
-- Name: patch_insert(text, text, text[]); Type: FUNCTION; Schema: cabar; Owner: db_global_owner
--

CREATE FUNCTION patch_insert(p_patch_file text, p_component text, p_depends_on text[]) RETURNS void
    AS $$
DECLARE
  v_component_id cabar.components.id%TYPE := cabar.components__get_id( p_component );
BEGIN
  /* For now we need to allow for NULL components
  IF v_component_id IS NULL THEN
    RAISE EXCEPTION 'Component % does not exist', p_component;
  END IF;
  */
  LOOP
    BEGIN
      INSERT INTO tools.schema_patches( patch_file, component_id, depends_on ) VALUES( p_patch_file, v_component_id, p_depends_on );
      RETURN;
    EXCEPTION WHEN unique_violation THEN
        -- do nothing
    END;

    -- If we've already applied the patch just punt
    IF EXISTS( SELECT * FROM tools.schema_patches WHERE patch_file = p_patch_file AND applied_at IS NOT NULL ) THEN
        RETURN;
    END IF;

    UPDATE tools.schema_patches
      SET depends_on = p_depends_on
        , component_id = v_component_id
      WHERE patch_file = p_patch_file
    ;
    IF found THEN
        RETURN;
    END IF;
  END LOOP;
END;
$$
    LANGUAGE plpgsql SECURITY DEFINER
    SET search_path TO pg_catalog;


ALTER FUNCTION cabar.patch_insert(p_patch_file text, p_component text, p_depends_on text[]) OWNER TO db_global_owner;

SET search_path = code, pg_catalog;

--
-- Name: function(text, text, text, text, text, text, text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION function(p_function_name text, p_parameters text, p_returns text, p_options text, p_body text, p_comment text, p_grant_to text) RETURNS void
    AS $_$
DECLARE
    v_object_class text := 'function';
    v_function_head text := ' FUNCTION ' || p_function_name || '(
' || p_parameters || '
) ';

    sql text;
BEGIN

    PERFORM code_internal.create_object( p_function_name, 'FUNCTION', v_object_class, array[ ['function', p_function_name]
                                                                                                , ['params', p_parameters]
                                                                                                , ['returns', p_returns]
                                                                                                , ['options', p_options]
                                                                                                , ['body', p_body]
                                                                                            ] )
    ;

    /*
     * I'm not very happy about doing this stuff in here, but I can't think of
     * a way to add it to templates without seriously increasing their
     * complexity
     */
    IF p_comment IS NOT NULL THEN
        sql := 'COMMENT ON' || v_function_head || 'IS $comment_quote$' || p_comment || '$comment_quote$';
        PERFORM tools.exec(sql);
    END IF;
    IF p_grant_to IS NOT NULL THEN
        sql := 'REVOKE ALL ON' || v_function_head || 'FROM public';
        PERFORM tools.exec(sql);
        sql := 'GRANT EXECUTE ON' || v_function_head || 'TO ' || p_grant_to;
        PERFORM tools.exec(sql);
    END IF;
END;
$_$
    LANGUAGE plpgsql;


ALTER FUNCTION code.function(p_function_name text, p_parameters text, p_returns text, p_options text, p_body text, p_comment text, p_grant_to text) OWNER TO db_global_owner;

--
-- Name: FUNCTION function(p_function_name text, p_parameters text, p_returns text, p_options text, p_body text, p_comment text, p_grant_to text); Type: COMMENT; Schema: code; Owner: db_global_owner
--

COMMENT ON FUNCTION function(p_function_name text, p_parameters text, p_returns text, p_options text, p_body text, p_comment text, p_grant_to text) IS 'Creates a function.

Parameters:
    p_function_name Name of function (can be schema-qualified)
    p_parameters    Parameters for the function
    p_returns       Data type the function returns
    p_options       Options for the function (language, etc)
    p_body          Function body
    p_comment       A comment for the function (can be NULL)
    p_grant_to      Users/groups to grant EXECUTE to (can be NULL)

If p_comment is NULL, no comment will be created.

If p_grant_to is NULL, no grants will be performed. Otherwise, before granting
execute to the roles specified in p_grant_to, all rights for the function
will be revoked from public.
';


--
-- Name: get(text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION get(p_table_name text) RETURNS void
    AS $$
DECLARE
    v_object_class text := 'get';
    v_function_name text := p_table_name || '__' || v_object_class;

    v_table_full text := tools.full_name_table( p_table_name );
    v_schema text;
    v_table text;

BEGIN
    SELECT INTO v_schema, v_table * FROM tools.split_schema( v_table_full );

    -- Ensure that the table actually has an ID field and that it's the PK
    IF NOT EXISTS(
        SELECT *
            FROM pg_catalog.pg_index i
                JOIN pg_catalog.pg_class ci ON ci.oid = i.indexrelid
                JOIN pg_catalog.pg_attribute ai ON ai.attrelid = i.indexrelid
            WHERE i.indrelid = p_table_name::regclass
                AND ai.attname='id'
                AND i.indisprimary
                AND ai.attnum = 1
    ) THEN
        RAISE EXCEPTION 'Table % does not have an ID field in the first position of its primary key', p_table_name;
    END IF;

    PERFORM code_internal.create_object( v_function_name, 'FUNCTION', v_object_class, array[ ['schema', v_schema], ['table', v_table] ] );
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION code.get(p_table_name text) OWNER TO db_global_owner;

--
-- Name: FUNCTION get(p_table_name text); Type: COMMENT; Schema: code; Owner: db_global_owner
--

COMMENT ON FUNCTION get(p_table_name text) IS 'Creates a function that returns the row of a table based on its ID value.

Parameters:
    p_table_name Name of table to create the __get function for.

Result:
Creates the function %p_table_name%__get( p_id int ).

Note:
p_table_name must have a primary key that starts with ID
';


--
-- Name: get_id(text, text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION get_id(p_table_name text, p_lookup text) RETURNS void
    AS $$
DECLARE
    v_object_class text := 'get_id';
    v_function_name text := p_table_name || '__' || v_object_class;

    v_table_full text := tools.full_name_table( p_table_name );
    v_schema text;
    v_table text;

BEGIN
    SELECT INTO v_schema, v_table * FROM tools.split_schema( v_table_full );

    PERFORM code_internal.create_object( v_function_name, 'FUNCTION', v_object_class, array[ ['schema', v_schema], ['table', v_table], ['lookup', p_lookup] ] );
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION code.get_id(p_table_name text, p_lookup text) OWNER TO db_global_owner;

--
-- Name: FUNCTION get_id(p_table_name text, p_lookup text); Type: COMMENT; Schema: code; Owner: db_global_owner
--

COMMENT ON FUNCTION get_id(p_table_name text, p_lookup text) IS 'Creates a function that will return an ID value based on a lookup field. The lookup is done in a case-insensitive manor.

Paramaters:
    p_table_name Table to lookup from.
    p_lookup Text field to use for the lookup.

Result:
Creates the function %p_table_name%__get_id( p_%p_lookup% <type of lookup field> ).

NOTE:
p_table_name must have a unique index on lower( %p_lookup% )
';


--
-- Name: get_text(text, text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION get_text(p_table_name text, p_lookup text) RETURNS void
    AS $$
DECLARE
    v_object_class text := 'get_text';
    v_function_name text := p_table_name || '__' || v_object_class;

    v_table_full text := tools.full_name_table( p_table_name );
    v_schema text;
    v_table text;

BEGIN
    SELECT INTO v_schema, v_table * FROM tools.split_schema( v_table_full );

    -- Ensure that the table actually has an ID field and that it's the PK
    IF NOT EXISTS(
        SELECT *
            FROM pg_catalog.pg_index i
                JOIN pg_catalog.pg_class ci ON ci.oid = i.indexrelid
                JOIN pg_catalog.pg_attribute ai ON ai.attrelid = i.indexrelid
            WHERE i.indrelid = p_table_name::regclass
                AND ai.attname='id'
                AND i.indisprimary
                AND ai.attnum = 1
    ) THEN
        RAISE EXCEPTION 'Table % does not have an ID field in the first position of its primary key', p_table_name;
    END IF;

    PERFORM code_internal.create_object( v_function_name, 'FUNCTION', v_object_class, array[ ['schema', v_schema], ['table', v_table], ['lookup', p_lookup] ] );
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION code.get_text(p_table_name text, p_lookup text) OWNER TO db_global_owner;

--
-- Name: FUNCTION get_text(p_table_name text, p_lookup text); Type: COMMENT; Schema: code; Owner: db_global_owner
--

COMMENT ON FUNCTION get_text(p_table_name text, p_lookup text) IS 'Creates a function that will return a lookup value based on its ID.

Paramaters:
    p_table_name Table to lookup from.
    p_lookup Text field to use for the lookup.

Result:
Creates the function %p_table_name%__get_text( p_%p_lookup% <type of lookup field> ).

Note:
p_table_name must have a primary key that starts with ID
Arguable, all of this should just be a wrapper to %p_table_name%%__get.
';


--
-- Name: getsert(text, text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION getsert(p_table_name text, p_lookup text) RETURNS void
    AS $$
DECLARE
    v_object_class text := 'getsert';
    v_function_name text := p_table_name || '__' || v_object_class;

    v_table_full text := tools.full_name_table( p_table_name );
    v_schema text;
    v_table text;

BEGIN
    SELECT INTO v_schema, v_table * FROM tools.split_schema( v_table_full );

    PERFORM code_internal.create_object( v_function_name, 'FUNCTION', v_object_class, array[ ['schema', v_schema], ['table', v_table], ['lookup', p_lookup] ] );
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION code.getsert(p_table_name text, p_lookup text) OWNER TO db_global_owner;

--
-- Name: FUNCTION getsert(p_table_name text, p_lookup text); Type: COMMENT; Schema: code; Owner: db_global_owner
--

COMMENT ON FUNCTION getsert(p_table_name text, p_lookup text) IS 'Creates a function that will lookup an ID based on a text lookup value (p_lookup). If no record exists, one will be created.

Parameters:
    p_table_name Name of the table to lookup the value in
    p_lookup Name of the field to use for the lookup value

Results:
Creates function %p_table_name%__getsert( %p_lookup% with a type matching the p_lookup field in p_table_name ). The function returns an ID as an int.
Revokes all on the function from public and grants execute to cnuapp_role.
';


--
-- Name: id_automatic_update(text, text, text, text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION id_automatic_update(p_schema text, p_source_singular text, p_sources_plural text, p_target text) RETURNS text
    AS $$

DECLARE
    v_trigger_name CONSTANT text := 'tg_' || p_source_singular || '_update_' || p_target;
    v_full_trigger_name CONSTANT text := p_schema || '.' || v_trigger_name;
BEGIN
    RAISE INFO 'Creating status automatic update trigger %', v_full_trigger_name;

    PERFORM code_internal.create_object(
            v_full_trigger_name
            , 'TRIGGER'
            , 'id_automatic_update'
            , array[    ['schema', p_schema]
                        , ['source_singular', p_source_singular]
                        , ['sources_plural', p_sources_plural]
                        , ['target', p_target]
                    ]
        );
    
    RETURN v_full_trigger_name;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION code.id_automatic_update(p_schema text, p_source_singular text, p_sources_plural text, p_target text) OWNER TO db_global_owner;

--
-- Name: FUNCTION id_automatic_update(p_schema text, p_source_singular text, p_sources_plural text, p_target text); Type: COMMENT; Schema: code; Owner: db_global_owner
--

COMMENT ON FUNCTION id_automatic_update(p_schema text, p_source_singular text, p_sources_plural text, p_target text) IS '
Creates a trigger that will automatically update one _id field based on change to another one.

Parameters:
p_schema
Schema that the table with the status fields is in. The trigger function will
be created in this schema.

p_source_singular
Field whose updates will push changes to the target field (don''t add _id).

p_sources_plural
Plural version of field name (used for table name).

p_target
This is the field that will be automatically updated by changes in p_source_singular

Results:
Creates a trigger function that updates p_target based on p_source_singular. This
requires that there be a parent table for p_source_singular that is a static lookup
table with an additional field named %p_target%_id.  When %p_source_singular%_id is
inserted to or updated, %p_target%_id will be set to the value of %p_target%_id
in the lookup table for p_source_singular. If %p_target%_id is NULL in the lookup table,
the field in the table the trigger is applied to will not be set.

NOTE:
The trigger must be manually added.
';


--
-- Name: index__create(text, text, boolean, text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION index__create(p_index_name text, p_table_name text, p_is_unique boolean, p_definition text) RETURNS void
    AS $_$
DECLARE
  c_table_name CONSTANT text := p_table_name::regclass::text;
  c_index_name CONSTANT text := quote_ident(p_index_name);
BEGIN
  PERFORM tools.assert( 'not null', 'code.index__create p_index_name', p_index_name IS NOT NULL, 'p_index_name may not be NULL' );
  PERFORM tools.assert( 'not null', 'code.index__create ', p_table_name IS NOT NULL, 'p_table_name may not be NULL' );
  PERFORM tools.assert( 'not null', 'code.index__create p_is_unique', p_is_unique IS NOT NULL, 'p_is_unique may not be NULL' );
  PERFORM tools.assert( 'not null', 'code.index__create p_definition', p_definition IS NOT NULL, 'p_definition may not be NULL' );
  PERFORM tools.assert(
    'semicolon not allowed'
    , 'code.index__create'
    , p_definition !~ ';'
    , 'definition( = ' || tools.textcoalesce( p_definition ) || ' ) may not contain a semicolon'
  );

  PERFORM tools.exec( tools.parameter_replace(
      $$CREATE %is_unique%INDEX %index_name% ON %table_name%%definition%$$
      , array[
          [ 'is_unique', CASE WHEN p_is_unique THEN 'UNIQUE ' ELSE '' END ]
          , [ 'index_name', c_index_name ]
          , [ 'table_name', c_table_name  ] -- Cast to regclass to protect against injection and make sure table exists
          , [ 'definition', p_definition ]
        ]
      ) )
    WHERE NOT EXISTS(
          SELECT *
            FROM tools.indexes
            WHERE table_oid = c_table_name::regclass
              AND index_name = c_index_name
              AND is_valid
        )
  ;
END;
$_$
    LANGUAGE plpgsql;


ALTER FUNCTION code.index__create(p_index_name text, p_table_name text, p_is_unique boolean, p_definition text) OWNER TO db_global_owner;

--
-- Name: FUNCTION index__create(p_index_name text, p_table_name text, p_is_unique boolean, p_definition text); Type: COMMENT; Schema: code; Owner: db_global_owner
--

COMMENT ON FUNCTION index__create(p_index_name text, p_table_name text, p_is_unique boolean, p_definition text) IS 'Creates an index, but only if a valid index of the same name on the same table does not already exist';


--
-- Name: inheritance_ri(text, text, text, text, text, text, text, text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION inheritance_ri(p_schema text, p_parent_table text, p_primary_key text, p_type_plural text, p_type_singular text, p_table_name_field text, p_child_name_prefix text, p_child_name_suffix text) RETURNS text
    AS $$

DECLARE
    v_function_name CONSTANT text := p_schema || '.tg_' || p_parent_table || '_ri()';
BEGIN
    PERFORM code_internal.create_object(
            v_function_name
            , 'FUNCTION'
            , 'inheritance_ri'
            , array[    ['schema', p_schema]
                        , [ 'parent_table', p_parent_table ]
                        , [ 'primary_key', p_primary_key ]
                        , [ 'types_plural', p_type_plural ]
                        , [ 'type_singular', p_type_singular ]
                        , [ 'table_name_field', coalesce( p_table_name_field, p_type_singular ) ]
                        , [ 'child_name_prefix', coalesce( p_child_name_prefix, p_schema || '.' ) ]
                        , [ 'child_name_suffix', coalesce( p_child_name_suffix, 's' ) ]
                    ]
        )
    ;

    RETURN v_function_name;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION code.inheritance_ri(p_schema text, p_parent_table text, p_primary_key text, p_type_plural text, p_type_singular text, p_table_name_field text, p_child_name_prefix text, p_child_name_suffix text) OWNER TO db_global_owner;

--
-- Name: FUNCTION inheritance_ri(p_schema text, p_parent_table text, p_primary_key text, p_type_plural text, p_type_singular text, p_table_name_field text, p_child_name_prefix text, p_child_name_suffix text); Type: COMMENT; Schema: code; Owner: db_global_owner
--

COMMENT ON FUNCTION inheritance_ri(p_schema text, p_parent_table text, p_primary_key text, p_type_plural text, p_type_singular text, p_table_name_field text, p_child_name_prefix text, p_child_name_suffix text) IS '
Creates a trigger function that enforces insert and update referential
integrity against inherited tables that have a defined "type" column.

Parameters:
p_schema
Schema that the parent (inherited) table is in. The trigger funciton will be created in this schema as well.

p_parent_table
Table that has been inherited from that we wish to enforce RI against.

p_primary_key
Primary key of the parent table (only single field numeric keys are currently supported).

p_types_plural, p_type_singular
Singular and plural versions of the table that defines what the "type" of each
record is. (See below.) A %p_types_plural%__get() function must exist.

p_table_name_field
Field in the lookup table %schema%.%p_types_plural% that defines what the name
of a child table is. Set to p_type_singular if NULL.

p_child_name_prefix, p_child_name_suffix
These will be prepended and appended to the type that is returned by the type
field/function to determine the name of the table to find a record in. Set to
''%p_schema%.'' and ''s'' if NULL.

Returns:
Name of function created.

What are "types"?
The key to how this function works is having the ability to know what child
table a record is in based on what is set in a field in the inherited table.
Take this simplified example:

INSERT INTO payment_instruments.payment_instrument_types( id, payment_instrument_type ) VALUES( 1, ''debit_card'' );
CREATE TABLE payment_insturemnts.payment_instruments(
    payment_instrument_id int PRIMARY KEY
    , payment_instrument_type_id ... CHECK( payment_instrument_type_id NOT IN ( 1 ) )
);
CREATE TABLE payment_instruments.debit_cards(
    CHECK ( payment_instrument_type_id = 1 )
) INHERITS( payment_instruments );

In this case, the payment_instrument_type_id field is used to determine what type of payment instrument something is, and there are check constraints in place to ensure that debit cards must end up in the debit_cards table. So to create the RI trigger function, you would pass the following arguments to code.inheritance_ri:
p_schema => ''payment_instruments''
, p_parent_table => ''payment_instruments''
, p_primary_key => ''payment_instrument_id''
, p_types_plural => ''payment_instrument_types''
, p_type_singular => ''payment_instrument_type''
, p_table_name_field => NULL
, p_child_name_prefix => NULL
, p_child_name_suffix => NULL

p_table_name_field will default to p_type_singular, p_child_name_prefix will default
to '''', and p_child_name_suffix will default to ''s''.

That means that the function will use the function
payment_instruments.payment_instrument_types__get( payment_instrument_type_id )
(derived from %p_schema%.%p_types_plural%__get( %p_type_singular%_id ) ) to
obtain a type row, and it will look at the payment_instrument_type field
(derived from p_table_name_field, which defaults to p_type_singular) to determine
the base name of the child table. It will then pre-pend '''' to the base name
(p_child_name_prefix defaults to ''''), and append ''s'' (p_child_name_suffix
defaults to ''s''). That means that for a record where payment_instrument_type_id
is 1, it will look in the child table payment_instruments.debit_cards.
';


--
-- Name: inheritance_unique(text, text, text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION inheritance_unique(p_schema text, p_parent_table text, p_primary_key text) RETURNS text
    AS $$

DECLARE
    v_function_name CONSTANT text := p_schema || '.tg_' || p_parent_table || '_unique()';
BEGIN
    PERFORM code_internal.create_object(
            v_function_name
            , 'FUNCTION'
            , 'inheritance_unique'
            , array[    ['schema', p_schema]
                        , [ 'parent_table', p_parent_table ]
                        , [ 'primary_key', p_primary_key ]
                    ]
        )
    ;

    RETURN v_function_name;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION code.inheritance_unique(p_schema text, p_parent_table text, p_primary_key text) OWNER TO db_global_owner;

--
-- Name: FUNCTION inheritance_unique(p_schema text, p_parent_table text, p_primary_key text); Type: COMMENT; Schema: code; Owner: db_global_owner
--

COMMENT ON FUNCTION inheritance_unique(p_schema text, p_parent_table text, p_primary_key text) IS '
Creates a trigger function that tries to enforce uniqueness of primary keys
across a set of inherited tables. This is far from perfect because of a huge
race condition, but it''s better than nothing.

Parameters:
p_schema
Schema that the parent (inherited) table is in. The trigger funciton will be
created in this schema as well.

p_parent_table
Table that has been inherited from.

p_primary_key
Primary key of the parent table (only single field keys are currently supported).

Returns:
Name of function created.
';


--
-- Name: lookup_table_dynamic(text, text, text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION lookup_table_dynamic(p_schema text, p_plural text, p_singular text) RETURNS void
    AS $$
BEGIN
    PERFORM code_internal.create_object( p_schema || '.' || p_plural
                                , 'TABLE'
                                , 'lookup_table_dynamic'
                                , array[ ['schema', p_schema], ['plural', p_plural], ['singular', p_singular] ]
                            );
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION code.lookup_table_dynamic(p_schema text, p_plural text, p_singular text) OWNER TO db_global_owner;

--
-- Name: FUNCTION lookup_table_dynamic(p_schema text, p_plural text, p_singular text); Type: COMMENT; Schema: code; Owner: db_global_owner
--

COMMENT ON FUNCTION lookup_table_dynamic(p_schema text, p_plural text, p_singular text) IS 'Creates a dynamic lookup table and associated objects.

A dynamic lookup table is one that needs to have new data added to it by code.
Generally, it is better to use a static lookup table (see
code.lookup_table_static).

Parameters:
    p_schema Schema to put the table in.
    p_plural Pluralized name of table.
    p_singular Singular name of table.

Result:
Creates a table called p_plural in schema p_schema. The table has the fields:
id              serial      NOT NULL PRIMARY KEY
%p_singular%    text        NOT NULL
description     next

%p_singular% must be unique on a case-insensitive basis. The table is marked as a seed table.
SELECT access is granted to cnubase_role (no insert, update or delete privileges are granted; that access should be done via the getsert function).
getsert and get functions are created for the table.';


--
-- Name: lookup_table_static(text, text, text, text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION lookup_table_static(p_schema text, p_plural text, p_singular text, p_post_id text) RETURNS void
    AS $$
DECLARE
    v_full_table_name CONSTANT text := p_schema || '.' || p_plural;
BEGIN
    RAISE INFO 'Creating smallint lookup table %', v_full_table_name;
    PERFORM code_internal.create_object( 
                                v_full_table_name
                                , 'TABLE'
                                , 'lookup_table_static'
                                , array[ ['schema', p_schema]
                                        , ['plural', p_plural]
                                        , ['singular', p_singular]
                                        , ['post_id', coalesce( p_post_id, '' ) ]
                                        ]
                            );
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION code.lookup_table_static(p_schema text, p_plural text, p_singular text, p_post_id text) OWNER TO db_global_owner;

--
-- Name: lookup_table_static(text, text, text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION lookup_table_static(p_schema text, p_plural text, p_singular text) RETURNS void
    AS $_$
SELECT code.lookup_table_static( $1, $2, $3, NULL )
$_$
    LANGUAGE sql;


ALTER FUNCTION code.lookup_table_static(p_schema text, p_plural text, p_singular text) OWNER TO db_global_owner;

--
-- Name: FUNCTION lookup_table_static(p_schema text, p_plural text, p_singular text); Type: COMMENT; Schema: code; Owner: db_global_owner
--

COMMENT ON FUNCTION lookup_table_static(p_schema text, p_plural text, p_singular text) IS 'Creates a static lookup table and objects to go along with it.

The idea behind a static lookup table is that the data in it should only be
modified by a patch or something similar. It should not be dynamically
modified.

Parameters:
    p_schema Schema to put the table in.
    p_plural Pluralized name of table.
    p_singular Singular name of table.
    p_post_id (optional) Will be inserted verbatim into the table creation portion.

Result:
Creates a table called p_plural in schema p_schema. The table has the fields:
id              smallint    NOT NULL PRIMARY KEY
%p_singular%    text        NOT NULL
description     next

%p_singular% must be unique on a case-insensitive basis.
The table is marked as a seed table.
SELECT access is granted to cnubase_role (no insert, update or delete privileges are granted).
get_id, get_text and get functions are created for the table.';


--
-- Name: lookup_view(text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION lookup_view(p_table_full_name text) RETURNS void
    AS $$
DECLARE
    -- DO NOT get rid of this! tools.table_full_name prevents SQL injection
    v_full_table_name CONSTANT text := tools.table_full_name( p_table_full_name );
    v_full_view_name CONSTANT text := v_full_table_name || '_v';

    v_select_list_template text := 
         E'\n\t\t, (%schema%.%plural%__get(t.%singular%_id)).%singular%';
    v_select_list text := '';
    v_parameters text[];
    v_grantee text;
    v_schema text;
    v_table text;
BEGIN
    SELECT INTO v_schema, v_table * FROM tools.table_schema_and_name( v_full_table_name );
    RAISE INFO 'Creating denormalized lookup view %', v_full_view_name;
    -- Build up the select list using the parameters from lookup table creation
    FOR v_parameters IN 
        SELECT parameters FROM code_internal.objects_v
        WHERE object_class IN ( 
                'lookup_table_static', 'lookup_table_dynamic' 
            )
            AND full_name IN (
                SELECT tools.relation_full_name( confrelid::regclass::text )
                FROM pg_constraint
                WHERE conrelid = (v_full_table_name)::regclass
                    AND contype = 'f'
            )
    LOOP
        v_select_list := v_select_list 
            || tools.parameter_replace(v_select_list_template, v_parameters);
    END LOOP;

    v_parameters := ARRAY[  ['schema', v_schema]
                            , ['table', v_table]
                            , ['selectlist', v_select_list] ];

    -- We want to create the object with the same ownership as the parent table
    PERFORM code_internal.create_object(
        v_full_view_name
        , 'View'
        , 'denormalized_view'
        , v_parameters
    );

    PERFORM tools.exec( 'GRANT SELECT ON ' || v_full_view_name || ' TO ' ||
        array_to_string(
            array( SELECT * FROM tools.table_select_privileges( p_table_full_name ) )
            , ', '
        )
    );
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION code.lookup_view(p_table_full_name text) OWNER TO db_global_owner;

--
-- Name: role(text, text, text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION role(p_role_name text, p_in_roles text, p_comment text) RETURNS void
    AS $$
-- Created in bug_192860_code.role
DECLARE
c_role_name text := quote_ident( p_role_name );
BEGIN
	PERFORM tools.assert( 'code__role__single_role'
		, p_role_name !~ ','
		, 'You can not create multiple roles at once via code.role (no commas allowed)'
	);

  /*
   * Note: we can't run p_in_roles through quote_literal because it breaks
   * the ability to grant multiple roles at once. Anyone running this
   * function has to have su access already, so presumably we don't need to
   * worry about SQL injection. Still, it's a good idea to at least make sure
   * there's no ; in the string.
   */
  PERFORM tools.assert( 'code__role__no_semicolon_in_p_in_roles'
    -- It IS valid for p_in_roles to be NULL...
    , p_in_roles IS NULL OR p_in_roles !~ ';'
    , 'p_in_roles may not contain a semicolon'
  );

	IF NOT EXISTS( SELECT * FROM pg_roles WHERE rolname = c_role_name ) THEN
		PERFORM tools.exec_as(
      tools.parameter_replace(
        'CREATE ROLE %role_name% '
          || CASE WHEN p_in_roles IS NOT NULL THEN ' IN ROLE %in_roles%' ELSE '' END
          || ';' || CASE WHEN p_comment IS NOT NULL THEN ' COMMENT ON ROLE %role_name% IS %comment%' ELSE '' END
        , array[
          [ 'role_name', c_role_name ]
          , [ 'in_roles', coalesce( p_in_roles, '' ) ]
          , [ 'comment', pg_catalog.quote_literal( coalesce( p_comment, '' ) ) ] 
        ]
      )
      , 'su'
    );
	END IF;
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION code.role(p_role_name text, p_in_roles text, p_comment text) OWNER TO db_global_owner;

--
-- Name: role(text, text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION role(p_role_name text, p_in_roles text) RETURNS void
    AS $_$
SELECT code.role( $1, $2, NULL )
$_$
    LANGUAGE sql;


ALTER FUNCTION code.role(p_role_name text, p_in_roles text) OWNER TO db_global_owner;

--
-- Name: role(text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION role(p_role_name text) RETURNS void
    AS $_$
SELECT code.role( $1, NULL, NULL )
$_$
    LANGUAGE sql;


ALTER FUNCTION code.role(p_role_name text) OWNER TO db_global_owner;

--
-- Name: schema(text, text, text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION schema(p_schema_name text, p_owner text, p_comment text) RETURNS void
    AS $_$
-- Created by bug_192474__schema_as
-- note: exec_as will change the effective role to p_owner
SELECT tools.exec_as(
    tools.parameter_replace(
      'CREATE SCHEMA %schema_name% AUTHORIZATION %owner%; GRANT USAGE ON SCHEMA %schema_name% TO cnubase_role'
        || CASE WHEN $3 IS NOT NULL THEN '; COMMENT ON SCHEMA %schema_name% IS %comment%' ELSE '' END
      , array[
        [ 'schema_name', pg_catalog.quote_ident( $1 ) ]
        , [ 'owner', pg_catalog.quote_ident( $2 ) ]
        , [ 'comment', pg_catalog.quote_literal( coalesce( $3, '' ) ) ] 
      ] 
    )
  , 'su'
);
$_$
    LANGUAGE sql;


ALTER FUNCTION code.schema(p_schema_name text, p_owner text, p_comment text) OWNER TO db_global_owner;

--
-- Name: schema(text, text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION schema(schema_name text, p_owner text) RETURNS void
    AS $_$
-- Created by bug_192474__schema_as
  SELECT code.schema( $1, $2, NULL )
$_$
    LANGUAGE sql;


ALTER FUNCTION code.schema(schema_name text, p_owner text) OWNER TO db_global_owner;

--
-- Name: status_history(text, text, text, text, text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION status_history(p_schema text, p_table_plural text, p_table_singular text, p_primary_key text, p_status_name text) RETURNS void
    AS $_$

DECLARE
    v_status_table_name CONSTANT text := p_status_name || '_status_history';
    v_status_table_name_full CONSTANT text := p_schema || '.' || v_status_table_name;

    v_table_ri text := '';
    v_track_table_name_full CONSTANT text := p_schema || '.' || p_table_plural;
BEGIN
    IF p_status_name ~* 'status' THEN
        RAISE WARNING $$p_status_name ("%") contains the string 'status', resulting in a table name of %; are you sure this is what you want?$$
            , p_status_name
            , v_status_table_name
        ;
    ELSE
        RAISE INFO 'Creating status history table %', v_status_table_name_full;
    END IF;

    -- See if the table being tracked has a primary key. If it does, add RI to it
    IF EXISTS( SELECT *
                    FROM pg_index
                    WHERE indrelid = v_track_table_name_full::regclass
                        AND indisprimary
    ) THEN
        v_table_ri := 'REFERENCES ' || v_track_table_name_full || ' ON DELETE CASCADE ON UPDATE CASCADE';
    ELSE
        RAISE WARNING 'Table being tracked(%) does not have a primary key, no foreign key will be created!', v_track_table_name_full;
    END IF;

    PERFORM code_internal.create_object(
            v_status_table_name_full
            , 'TABLE'
            , 'status_history'
            , array[    ['schema', p_schema]
                        , ['table_plural', p_table_plural]
                        , ['primary_key', p_primary_key]
                        , ['table_pk_reference', CASE WHEN p_primary_key = 'id' THEN p_table_singular || '_id' ELSE p_primary_key END ]
                        , ['status_name', p_status_name]
                        , ['track_table_RI', v_table_ri]
                    ]
        );
END;

$_$
    LANGUAGE plpgsql;


ALTER FUNCTION code.status_history(p_schema text, p_table_plural text, p_table_singular text, p_primary_key text, p_status_name text) OWNER TO db_global_owner;

--
-- Name: FUNCTION status_history(p_schema text, p_table_plural text, p_table_singular text, p_primary_key text, p_status_name text); Type: COMMENT; Schema: code; Owner: db_global_owner
--

COMMENT ON FUNCTION status_history(p_schema text, p_table_plural text, p_table_singular text, p_primary_key text, p_status_name text) IS '
Creates a table and trigger to track changes in a status field.

Parameters:
p_schema
Schema that the table to track the status of is in. The history table and
trigger function will be created in this schema.

p_table_plural, p_table_singular
Plural and singular name of the table to track the status of.

p_primary_key
Primary key of the table to track the status of.

p_status_name
Name of the status to track. Note that you should *not* add _status to this
name.

Results:
Creates a status history table:
CREATE TABLE %p_schema%.%p_status_name%_status_history(
    id                          serial          PRIMARY KEY
    , *table_pk_reference*      int             NOT NULL REFERENCES %p_schema%.%p_table_plural%
    , changed_on                timestamptz     NOT NULL DEFAULT now()
    , %p_status_name%_status_id smallint        NOT NULL REFERENCES %p_schema%.%p_status_name%_statuses
    , changed_by                varchar(30)
);
If p_primary_key = ''id'' then *table_pk_reference* is %p_tale_singular%_id, otherwise it is %p_primary_key%
The table has an index on %p_table_singular%_id, %p_status_name%_status_id.
SELECT access is granted to cnubase_role.
A trigger function named %p_schema%.tg_%p_status_name%_status_history is created. An after insert/update trigger calling this function should be placed on the table to track the status of. The function takes an optional argument that specifies if the table has an updated_by field (default is ''true'').

NOTE:
The trigger must be manually added.
';


--
-- Name: tg_match_field(text, text, text, text, text, text, text); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION tg_match_field(p_schema text, p_target_table text, p_target_pk text, p_target_field text, p_fk_field text, p_match_field text, p_select text) RETURNS text
    AS $$

DECLARE
    v_trigger_name CONSTANT text := 'tg_match_' || p_fk_field || '__' || p_match_field;
    v_full_trigger_name CONSTANT text := p_schema || '.' || v_trigger_name;
    v_target_pk CONSTANT text := coalesce( p_target_pk, 'id' );
    v_select text;
BEGIN
    RAISE INFO 'Creating status automatic update trigger %', v_full_trigger_name;

    v_select := coalesce( p_select
        , p_target_field || ' FROM ' || p_schema || '.' || p_target_table || '
        WHERE ' || v_target_pk || ' = NEW.' || p_fk_field
    );

    PERFORM code_internal.create_object(
            v_full_trigger_name
            , 'TRIGGER'
            , 'tg_match_field'
            , array[    [ 'schema', p_schema ]
                            , [ 'tg_name', v_trigger_name ]
                            , [ 'target_table', p_schema || '.' || p_target_table ]
                            , [ 'target_pk', v_target_pk ]
                            , [ 'target_field', p_target_field ]
                            , [ 'fk_field', p_fk_field ]
                            , [ 'match_field', p_match_field ]
                            , [ 'select', v_select ]
                    ]
        );
    
    RETURN v_full_trigger_name;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION code.tg_match_field(p_schema text, p_target_table text, p_target_pk text, p_target_field text, p_fk_field text, p_match_field text, p_select text) OWNER TO db_global_owner;

--
-- Name: FUNCTION tg_match_field(p_schema text, p_target_table text, p_target_pk text, p_target_field text, p_fk_field text, p_match_field text, p_select text); Type: COMMENT; Schema: code; Owner: db_global_owner
--

COMMENT ON FUNCTION tg_match_field(p_schema text, p_target_table text, p_target_pk text, p_target_field text, p_fk_field text, p_match_field text, p_select text) IS '
Creates a trigger that will verify that a field in another table matches a
field in the table that the trigger is on.

Parameters:
p_schema
Schema that the target table is in. This is also the schema where the trigger
function will be created.

p_target_table
External table that we want to check (this is not the table you put the trigger on!)

p_target_pk
Primary key field for the target table. Defaults to id.

p_target_field
The field in p_target_table that we will be checking the value of.

p_fk_field
Field in the table that the trigger is on that allows us to join to p_target_table

p_match_field
The field in the table that the trigger is on that will be compared against p_target_field.

p_select
Over-rides the normal select that is used.

Returns:
Name of trigger

Results:
Creates a trigger function that verifies that the value in p_match_field
matches the value in p_target_field, which is a field in an ''external'' table.
The trigger finds the value of p_target_field by looking for a row in
p_target_table where p_target_pk = p_fk_field, IE:

SELECT %p_target_field%
    FROM %p_schema%.%p_target_table%
    WEHER %p_target_pk% = NEW.%p_fk_field%

You can over-ride the actual select using p_select, but you still have to
provide all the other values (omit the SELECT and ; from p_select).

NOTE:
The trigger must be manually added.
';


--
-- Name: valid_transitions(text, text, boolean, boolean); Type: FUNCTION; Schema: code; Owner: db_global_owner
--

CREATE FUNCTION valid_transitions(p_schema text, p_status_name text, p_insert_allowed boolean, p_delete_allowed boolean) RETURNS void
    AS $_$
DECLARE
  v_function_version CONSTANT smallint := 2;
  v_name_max_length CONSTANT pg_type.typlen%TYPE := ( SELECT typlen FROM pg_type WHERE oid = 'name'::regtype ) - 1;

  v_table_suffix CONSTANT text := '_valid_transitions';
  v_table_name CONSTANT text := p_status_name || v_table_suffix;
  v_full_table_name CONSTANT text := p_schema || '.' || v_table_name;
BEGIN
  -- Make sure we won't accidentally overflow something
  IF length( v_table_name ) > v_name_max_length THEN
    RAISE EXCEPTION 'p_status_name is too long, it is % characters, but it must be % characters or less.'
      , length( p_status_name )
      , v_name_max_length - length( '_status_valid_trans__u_curr__allow' )
    ;
  END IF;

  IF p_status_name ~ 'status' THEN
    RAISE WARNING $$p_status_name ("%") contains the string 'status', resulting in a table name of %; are you sure this is what you want?$$
      , p_status_name
      , v_table_name
    ;
  ELSE
      RAISE INFO 'Creating valid transitions table %', v_full_table_name;
  END IF;

  IF NOT EXISTS( SELECT * FROM pg_tables WHERE schemaname = p_schema AND tablename = p_status_name || '_statuses' ) THEN
    PERFORM code.lookup_table_static( p_schema, p_status_name || '_statuses', p_status_name || '_status' );
  END IF;

  -- Create the transitions table
  PERFORM code_internal.create_object( v_full_table_name
      , 'TABLE'
      , 'valid_transitions'
      , array[
            ['schema', p_schema]
            , ['status_name', p_status_name]
            , [ 'current_not_null', CASE WHEN NOT p_insert_allowed THEN 'NOT NULL' ELSE '' END ]
            , [ 'allowed_not_null', CASE WHEN NOT p_delete_allowed THEN 'NOT NULL' ELSE '' END ]
        ]
      , v_function_version
  );
END;
$_$
    LANGUAGE plpgsql;


ALTER FUNCTION code.valid_transitions(p_schema text, p_status_name text, p_insert_allowed boolean, p_delete_allowed boolean) OWNER TO db_global_owner;

--
-- Name: FUNCTION valid_transitions(p_schema text, p_status_name text, p_insert_allowed boolean, p_delete_allowed boolean); Type: COMMENT; Schema: code; Owner: db_global_owner
--

COMMENT ON FUNCTION valid_transitions(p_schema text, p_status_name text, p_insert_allowed boolean, p_delete_allowed boolean) IS 'Creates a table to store valid status transitions.

Parameters:
    p_schema
Schema to create the table in.

    p_status_name
Singular name of the status field this is for. This should not include ''status''
in it; ie: for the installment_status field, p_status_name should just be
installment.

    p_insert_allowed
Are inserts allowed in the table with this status field? If they are allowed,
then we allow for the status to transition *from* a NULL value.

    p_delete_allowed
Are deletes allowed in the table with this status field? If they are allowed,
then we allow for the status to transition *to* a NULL value.

Results:
Checks for a static lookup table named %p_status_name%_statuses. If it doesn''t exist, it''s created.
Creates table %p_shema%.%p_status_name%_status_valid_transitions(
    id                            serial        PRIMARY KEY
    , current_status_id            smallint    RI to lookup table
    , allowed_status_id            smallint    RI to lookup table
    , event_name                text
    , description                text
)
The table has a constraint that enforces current_status_id and allowed_status_id must be different.
There is a set of unique indexes that ensure each status transition is unique.
The table is marked as a seed table.
SELECT access is granted to cnubase_role (no INSERT, UPDATE, or DELETE).
A trigger function name tg_%p_status_name%_status_transitions is created. This
trigger should be added to any tables that use this status field as an after
insert or update or delete trigger.

NOTE:
The trigger must be manually added.
The status field in the base table must be called %p_status_name%_id.
';


SET search_path = code_internal, pg_catalog;

--
-- Name: create_object(text, text, text, text[]); Type: FUNCTION; Schema: code_internal; Owner: db_global_owner
--

CREATE FUNCTION create_object(p_object_name text, p_object_type text, p_object_class text, p_parameters text[]) RETURNS void
    AS $_$
SELECT code_internal.create_object( $1, $2, $3, $4, 1::smallint );
$_$
    LANGUAGE sql;


ALTER FUNCTION code_internal.create_object(p_object_name text, p_object_type text, p_object_class text, p_parameters text[]) OWNER TO db_global_owner;

--
-- Name: create_object(text, text, text, text[], smallint); Type: FUNCTION; Schema: code_internal; Owner: db_global_owner
--

CREATE FUNCTION create_object(p_object_name text, p_object_type text, p_object_class text, p_parameters text[], p_creation_function_version smallint) RETURNS void
    AS $$

DECLARE
  v_saved_min_messages CONSTANT text := current_setting('client_min_messages');

  v_template code_internal.templates;
  v_sql text;
BEGIN
  v_template := code_internal.template__get_latest( p_object_type, p_object_class );

  PERFORM code_internal.object__upsert( p_object_name, v_template.id, p_parameters, p_creation_function_version );
  FOR i IN array_lower( v_template.template, 1 ) .. array_upper( v_template.template, 1 ) LOOP
    v_sql := tools.parameter_replace( v_template.template[i], p_parameters );

    -- Lots of objects spit out annoying NOTICEs on creation, so squelch that
    RAISE DEBUG 'Creating object using dynamic SQL: %', v_sql;
    PERFORM set_config( 'client_min_messages', 'WARNING', true );
    EXECUTE v_sql; -- tools.exec is pointless with client_min_messages forced to warning, and using it could result in multiple log messages
    PERFORM set_config( 'client_min_messages', v_saved_min_messages, true );
  END LOOP;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION code_internal.create_object(p_object_name text, p_object_type text, p_object_class text, p_parameters text[], p_creation_function_version smallint) OWNER TO db_global_owner;

--
-- Name: FUNCTION create_object(p_object_name text, p_object_type text, p_object_class text, p_parameters text[], p_creation_function_version smallint); Type: COMMENT; Schema: code_internal; Owner: db_global_owner
--

COMMENT ON FUNCTION create_object(p_object_name text, p_object_type text, p_object_class text, p_parameters text[], p_creation_function_version smallint) IS 'Function that actually creates a set of objects.';


--
-- Name: description(text, text, text); Type: FUNCTION; Schema: code_internal; Owner: db_global_owner
--

CREATE FUNCTION description(p_full_name text, p_object_type text, p_description text) RETURNS text
    AS $$
DECLARE
    c_regex CONSTANT text := E'\\W*This is generated code.'; -- \\ is turned into \

    c_object_oid CONSTANT oid := tools.object__get_oid( p_full_name, p_object_type );
    v_description text := btrim( coalesce( p_description, obj_description( c_object_oid ) ) );
BEGIN
    IF EXISTS( SELECT * FROM code_internal.objects_v WHERE full_name = p_full_name AND lower(object_type) = lower(p_object_type) ) THEN
        RAISE DEBUG 'Object is generated';
        IF v_description ~* c_regex THEN
            RETURN v_description;
        ELSE
            RETURN CASE WHEN coalesce( v_description, '' ) = '' THEN '' ELSE v_description || ' ' END || 'This is generated code.';
        END IF;
    ELSE
        RAISE DEBUG 'Object is not generated';
        RETURN regexp_replace( v_description, c_regex, '', 'ig' );
    END IF;
END;
$$
    LANGUAGE plpgsql STABLE;


ALTER FUNCTION code_internal.description(p_full_name text, p_object_type text, p_description text) OWNER TO db_global_owner;

--
-- Name: FUNCTION description(p_full_name text, p_object_type text, p_description text); Type: COMMENT; Schema: code_internal; Owner: db_global_owner
--

COMMENT ON FUNCTION description(p_full_name text, p_object_type text, p_description text) IS 'Returns a description for an object. If p_description is null then the
object''s current description is used.

If the object was generated by metacode, the description returned will indicate
that it was. Otherwise, the function ensures there''s no mention of it being a
metacode object.';


--
-- Name: object__upsert(text, integer, text[]); Type: FUNCTION; Schema: code_internal; Owner: db_global_owner
--

CREATE FUNCTION object__upsert(p_full_name text, p_template_id integer, p_parameters text[]) RETURNS void
    AS $$
DECLARE
BEGIN
    LOOP
        UPDATE code_internal.objects
            SET template_id = p_template_id
            WHERE full_name = p_full_name
        ;
        IF found THEN
            RETURN;
        END IF;

        BEGIN
            RAISE DEBUG 'Inserting "%" into code_internal.objects', p_full_name;
            INSERT INTO code_internal.objects( full_name, template_id, parameters, updated_on ) VALUES( p_full_name, p_template_id, p_parameters, now() );
        EXCEPTION WHEN unique_violation THEN
            -- do nothing
        END;
    END LOOP;
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION code_internal.object__upsert(p_full_name text, p_template_id integer, p_parameters text[]) OWNER TO db_global_owner;

--
-- Name: object__upsert(text, integer, text[], smallint); Type: FUNCTION; Schema: code_internal; Owner: db_global_owner
--

CREATE FUNCTION object__upsert(p_full_name text, p_template_id integer, p_parameters text[], p_creation_function_version smallint) RETURNS void
    AS $$
DECLARE
BEGIN
  LOOP
    UPDATE code_internal.objects
        SET template_id = p_template_id
          , creation_function_version = p_creation_function_version
        WHERE full_name = p_full_name
    ;
    IF found THEN
        RETURN;
    END IF;

    BEGIN
      RAISE DEBUG 'Inserting "%" into code_internal.objects', p_full_name;
      INSERT INTO code_internal.objects( full_name, template_id, parameters, updated_on, creation_function_version )
        VALUES( p_full_name, p_template_id, p_parameters, now(), p_creation_function_version )
      ;
    EXCEPTION WHEN unique_violation THEN
      -- do nothing
    END;
  END LOOP;
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION code_internal.object__upsert(p_full_name text, p_template_id integer, p_parameters text[], p_creation_function_version smallint) OWNER TO db_global_owner;

--
-- Name: object_class; Type: TABLE; Schema: code_internal; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE object_class (
    id integer NOT NULL,
    object_type_id integer NOT NULL,
    object_class text NOT NULL,
    description text
);


ALTER TABLE code_internal.object_class OWNER TO db_global_owner;

--
-- Name: TABLE object_class; Type: COMMENT; Schema: code_internal; Owner: db_global_owner
--

COMMENT ON TABLE object_class IS 'This is a seed table.';


--
-- Name: object_class__get(integer, text); Type: FUNCTION; Schema: code_internal; Owner: db_global_owner
--

CREATE FUNCTION object_class__get(p_object_type_id integer, p_object_class text) RETURNS object_class
    AS $_$
SELECT * FROM object_class WHERE lower(object_class) = lower($2) AND object_type_id = $1
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION code_internal.object_class__get(p_object_type_id integer, p_object_class text) OWNER TO db_global_owner;

--
-- Name: object_class__get(text, text); Type: FUNCTION; Schema: code_internal; Owner: db_global_owner
--

CREATE FUNCTION object_class__get(p_object_type text, p_object_class text) RETURNS object_class
    AS $_$
SELECT * FROM object_class WHERE lower(object_class) = lower($2) AND object_type_id = code_internal.object_type__get_id( $1 )
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION code_internal.object_class__get(p_object_type text, p_object_class text) OWNER TO db_global_owner;

--
-- Name: object_class__get_id(text, text); Type: FUNCTION; Schema: code_internal; Owner: db_global_owner
--

CREATE FUNCTION object_class__get_id(p_object_type text, p_object_class text) RETURNS integer
    AS $_$
SELECT code_internal.object_class__getsert( code_internal.object_type__get_id( $1 ), $2 )
$_$
    LANGUAGE sql;


ALTER FUNCTION code_internal.object_class__get_id(p_object_type text, p_object_class text) OWNER TO db_global_owner;

--
-- Name: object_class__getsert(integer, text); Type: FUNCTION; Schema: code_internal; Owner: db_global_owner
--

CREATE FUNCTION object_class__getsert(p_object_type_id integer, p_object_class text) RETURNS integer
    AS $$
DECLARE
    v_id      code_internal.object_class.id%TYPE;
BEGIN
    IF p_object_type_id IS NULL THEN
        RAISE EXCEPTION 'p_object_type_id can not be NULL (perhaps you tried looking for an unknown object type?)';
    END IF;
    LOOP
        SELECT INTO v_id
                id
            FROM code_internal.object_class
                WHERE object_class = p_object_class
                    AND object_type_id = p_object_type_id
        ;
        IF found THEN
            RAISE DEBUG 'object_class "%" of type_id "%" has id %', p_object_class, p_object_type_id, v_id;
            RETURN v_id;
        END IF;

        BEGIN
            RAISE DEBUG 'Inserting "%" into object_class', p_object_class;
            INSERT INTO code_internal.object_class( object_type_id, object_class ) VALUES( p_object_type_id, p_object_class );
            RETURN tools.currval( 'code_internal.object_class', 'id' );
        EXCEPTION WHEN unique_violation THEN
            -- do nothing
        END;
    END LOOP;
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION code_internal.object_class__getsert(p_object_type_id integer, p_object_class text) OWNER TO db_global_owner;

--
-- Name: object_type__get_id(text); Type: FUNCTION; Schema: code_internal; Owner: db_global_owner
--

CREATE FUNCTION object_type__get_id(p_object_type text) RETURNS integer
    AS $_$
SELECT id FROM code_internal.object_type WHERE lower(object_type) = lower($1)
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION code_internal.object_type__get_id(p_object_type text) OWNER TO db_global_owner;

--
-- Name: template__add(text, text, integer, text[], text); Type: FUNCTION; Schema: code_internal; Owner: db_global_owner
--

CREATE FUNCTION template__add(p_object_type text, p_object_class text, p_version integer, p_template text[], p_description text) RETURNS integer
    AS $$
DECLARE
    v_object_type_id code_internal.object_type.id%TYPE := code_internal.object_type__get_id( p_object_type );
    v_object_class_id code_internal.object_class.id%TYPE;

    v_id code_internal.templates.id%TYPE;
BEGIN
    IF v_object_type_id IS NULL THEN
        RAISE EXCEPTION 'Unknown object type: %', p_object_type;
    END IF;

    -- Now that we know we have a valid type, do the getsert
    v_object_class_id := code_internal.object_class__getsert( v_object_type_id, p_object_class );

    INSERT INTO code_internal.templates( object_class_id, version, template, description )
        VALUES( v_object_class_id, p_version, p_template, p_description )
        RETURNING id
        INTO v_id
    ;

    RETURN v_id;
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION code_internal.template__add(p_object_type text, p_object_class text, p_version integer, p_template text[], p_description text) OWNER TO db_global_owner;

--
-- Name: template__add(text, text, integer, text, text); Type: FUNCTION; Schema: code_internal; Owner: db_global_owner
--

CREATE FUNCTION template__add(p_object_type text, p_object_class text, p_version integer, p_template text, p_description text) RETURNS integer
    AS $_$
SELECT code_internal.template__add( $1, $2, $3, array[$4], $5 )
$_$
    LANGUAGE sql;


ALTER FUNCTION code_internal.template__add(p_object_type text, p_object_class text, p_version integer, p_template text, p_description text) OWNER TO db_global_owner;

--
-- Name: templates; Type: TABLE; Schema: code_internal; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE templates (
    id integer NOT NULL,
    object_class_id integer NOT NULL,
    version smallint NOT NULL,
    template text[] NOT NULL,
    description text
);


ALTER TABLE code_internal.templates OWNER TO db_global_owner;

--
-- Name: TABLE templates; Type: COMMENT; Schema: code_internal; Owner: db_global_owner
--

COMMENT ON TABLE templates IS 'This is a seed table.';


--
-- Name: template__get_latest(text, text); Type: FUNCTION; Schema: code_internal; Owner: db_global_owner
--

CREATE FUNCTION template__get_latest(p_object_type text, p_object_class text) RETURNS templates
    AS $_$
SELECT *
    FROM code_internal.templates
    WHERE object_class_id = code_internal.object_class__get_id( $1, $2 )
    ORDER BY version DESC
    LIMIT 1
$_$
    LANGUAGE sql;


ALTER FUNCTION code_internal.template__get_latest(p_object_type text, p_object_class text) OWNER TO db_global_owner;

SET search_path = inheritance, pg_catalog;

--
-- Name: child_object__add(text, text, text, text, text[]); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text, p_exclude_tables text[]) RETURNS integer
    AS $$

DECLARE
  v_id inheritance.child_objects.id%TYPE;
  v_exclude_table_ids int[];
BEGIN
  INSERT INTO inheritance.child_objects( inherited_table_id, object_type_id, object_name_template, object_body_template )
    SELECT tools.tables__getsert( p_inherited_table ), inheritance.object_types__get_id( p_object_type )
        , p_object_name_template, p_object_body_template
  ;
  v_id = tools.currval( 'inheritance.child_objects', 'id' );

  IF p_exclude_tables IS NOT NULL AND tools.array_length( p_exclude_tables, 1 ) > 0 THEN
    v_exclude_table_ids := tools.tables__getsert( p_exclude_tables );
    INSERT INTO inheritance.child_objects_exclude_tables( child_object_id, exclude_table_id )
      SELECT v_id, e_id
        FROM tools.explode_array( tools.tables__getsert( p_exclude_tables ) ) e_id
        WHERE e_id IS NOT NULL
    ;
  END IF;

  RETURN v_id;
END;

$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION inheritance.child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text, p_exclude_tables text[]) OWNER TO db_global_owner;

--
-- Name: FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text, p_exclude_tables text[]); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text, p_exclude_tables text[]) IS 'Adds a dependency object for an inherited table.

Parameters:
p_inherited_table
Name of the parent table.

p_object_type
Type of object (ie: not null, check, trigger)

p_object_name_template
p_object_body_template
Template used to create the name and body of the object. This is used in
conjuction with the template field in the table inheritance.object_types. The
template field specifies where object_name_template and object_body_template
are inserted to create a runnable SQL command.

Object name and body are themselves templates because they can contain these
parameters:
%schema_name% name of schema table is in
%table_name% name of table
%table% full name of table (equivalent to %schema_name%.%table_name%)

p_exclude_tables 
A text array of child tables that should not have this object created. Note
that these children do not have to exist when this function is run.
';


--
-- Name: child_object__add(text, text, text, text, text); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text, p_exclude_tables text) RETURNS integer
    AS $_$
SELECT inheritance.child_object__add( $1, $2, $3, $4, tools.string_or_array( $5 ) )
$_$
    LANGUAGE sql;


ALTER FUNCTION inheritance.child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text, p_exclude_tables text) OWNER TO db_global_owner;

--
-- Name: FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text, p_exclude_tables text); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text, p_exclude_tables text) IS 'A version of child_object__add that accepts a string list for p_exclude_tables instead of an array. See the other version of child_object__add for more information.';


--
-- Name: child_object__add(text, text, text, text); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text) RETURNS integer
    AS $_$
SELECT inheritance.child_object__add( $1, $2, $3, $4, NULL )
$_$
    LANGUAGE sql;


ALTER FUNCTION inheritance.child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text) OWNER TO db_global_owner;

--
-- Name: FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text) IS 'A version of child_object__add that does not specify any exclude tables. See the other version of child_object__add for more information.';


--
-- Name: child_object__add_excluded_tables(integer, text[]); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION child_object__add_excluded_tables(p_child_object_id integer, p_exclude_tables text[]) RETURNS void
    AS $_$

INSERT INTO inheritance.child_objects_exclude_tables( child_object_id, exclude_table_id )
  SELECT $1, e_id FROM tools.explode_array( tools.tables__getsert( $2 ) ) e_id
  EXCEPT
  SELECT child_object_id, exclude_table_id FROM inheritance.child_objects_exclude_tables
;

$_$
    LANGUAGE sql SECURITY DEFINER;


ALTER FUNCTION inheritance.child_object__add_excluded_tables(p_child_object_id integer, p_exclude_tables text[]) OWNER TO db_global_owner;

--
-- Name: FUNCTION child_object__add_excluded_tables(p_child_object_id integer, p_exclude_tables text[]); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION child_object__add_excluded_tables(p_child_object_id integer, p_exclude_tables text[]) IS 'Adds tables to the exclusion list for an object. Any tables already on the list are ignored.';


--
-- Name: child_object__add_excluded_tables(integer, text); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION child_object__add_excluded_tables(p_child_object_id integer, p_exclude_tables text) RETURNS void
    AS $_$
SELECT inheritance.child_object__add_excluded_tables( $1, tools.string_or_array( $2 ) )
$_$
    LANGUAGE sql;


ALTER FUNCTION inheritance.child_object__add_excluded_tables(p_child_object_id integer, p_exclude_tables text) OWNER TO db_global_owner;

--
-- Name: FUNCTION child_object__add_excluded_tables(p_child_object_id integer, p_exclude_tables text); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION child_object__add_excluded_tables(p_child_object_id integer, p_exclude_tables text) IS 'Version of child_object__add_excluded_tables that accepts a string of excluded tables instead of an array.';


--
-- Name: child_object__add_excluded_tables(text, text, text, text[]); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION child_object__add_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text[]) RETURNS void
    AS $_$

SELECT inheritance.child_object__add_excluded_tables( inheritance.child_object__get_id( $1, $2, $3 ), $4 )
  WHERE tools.assert( 'inheritance__child_object__not_found'
    , inheritance.child_object__get_id( $1, $2, $3 ) IS NOT NULL
    , 'Child object for inherited table ' || $1 || ', object type ' || $2 || ', object name template ' || $3 || ' not found'
  )

$_$
    LANGUAGE sql;


ALTER FUNCTION inheritance.child_object__add_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text[]) OWNER TO db_global_owner;

--
-- Name: FUNCTION child_object__add_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text[]); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION child_object__add_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text[]) IS 'Version of child_object__add_excluded_tables that accepts strings instead of an ID.';


--
-- Name: child_object__add_excluded_tables(text, text, text, text); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION child_object__add_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text) RETURNS void
    AS $_$
SELECT inheritance.child_object__add_excluded_tables( $1, $2, $3, tools.string_or_array( $4 ) )
$_$
    LANGUAGE sql;


ALTER FUNCTION inheritance.child_object__add_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text) OWNER TO db_global_owner;

--
-- Name: FUNCTION child_object__add_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION child_object__add_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text) IS 'Version of child_object__add_excluded_tables that accepts strings instead of an ID and a string of excluded tables instead of an array.';


--
-- Name: child_objects; Type: TABLE; Schema: inheritance; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE child_objects (
    id integer NOT NULL,
    inherited_table_id integer NOT NULL,
    object_type_id smallint NOT NULL,
    object_name_template text NOT NULL,
    object_body_template text NOT NULL
);


ALTER TABLE inheritance.child_objects OWNER TO db_global_owner;

--
-- Name: TABLE child_objects; Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON TABLE child_objects IS 'Objects that we want to be placed on children of an inherited table. This is a seed table.';


--
-- Name: COLUMN child_objects.inherited_table_id; Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON COLUMN child_objects.inherited_table_id IS 'Parent table. This object will be applied to all the children of this parent.';


--
-- Name: child_object__get(text, text, text); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION child_object__get(p_inherited_table text, p_object_type text, p_object_name_template text) RETURNS child_objects
    AS $_$

SELECT *
  FROM inheritance.child_objects
  WHERE inherited_table_id = tools.tables__get_id( $1 )
    AND object_type_id = inheritance.object_types__get_id( $2 )
    AND object_name_template = $3

$_$
    LANGUAGE sql SECURITY DEFINER;


ALTER FUNCTION inheritance.child_object__get(p_inherited_table text, p_object_type text, p_object_name_template text) OWNER TO db_global_owner;

--
-- Name: FUNCTION child_object__get(p_inherited_table text, p_object_type text, p_object_name_template text); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION child_object__get(p_inherited_table text, p_object_type text, p_object_name_template text) IS 'Returns a inheritance.child_objects record.';


--
-- Name: child_object__get_id(text, text, text); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION child_object__get_id(p_inherited_table text, p_object_type text, p_object_name_template text) RETURNS integer
    AS $_$
SELECT id FROM inheritance.child_object__get( $1, $2, $3 )
$_$
    LANGUAGE sql;


ALTER FUNCTION inheritance.child_object__get_id(p_inherited_table text, p_object_type text, p_object_name_template text) OWNER TO db_global_owner;

--
-- Name: FUNCTION child_object__get_id(p_inherited_table text, p_object_type text, p_object_name_template text); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION child_object__get_id(p_inherited_table text, p_object_type text, p_object_name_template text) IS 'Returns the id of a inheritance.child_objects record.';


--
-- Name: child_object__remove_excluded_tables(integer, text[]); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION child_object__remove_excluded_tables(p_child_object_id integer, p_exclude_tables text[]) RETURNS void
    AS $_$

DELETE FROM inheritance.child_objects_exclude_tables
  WHERE child_object_id = $1 AND exclude_table_id = ANY( tools.tables__get_id( $2 ) ) 
;

$_$
    LANGUAGE sql SECURITY DEFINER;


ALTER FUNCTION inheritance.child_object__remove_excluded_tables(p_child_object_id integer, p_exclude_tables text[]) OWNER TO db_global_owner;

--
-- Name: FUNCTION child_object__remove_excluded_tables(p_child_object_id integer, p_exclude_tables text[]); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION child_object__remove_excluded_tables(p_child_object_id integer, p_exclude_tables text[]) IS 'Removes tables from the exclusion list for an object.';


--
-- Name: child_object__remove_excluded_tables(integer, text); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION child_object__remove_excluded_tables(p_child_object_id integer, p_exclude_tables text) RETURNS void
    AS $_$
SELECT inheritance.child_object__remove_excluded_tables( $1, tools.string_or_array( $2 ) )
$_$
    LANGUAGE sql;


ALTER FUNCTION inheritance.child_object__remove_excluded_tables(p_child_object_id integer, p_exclude_tables text) OWNER TO db_global_owner;

--
-- Name: FUNCTION child_object__remove_excluded_tables(p_child_object_id integer, p_exclude_tables text); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION child_object__remove_excluded_tables(p_child_object_id integer, p_exclude_tables text) IS 'Version of child_object__remove_excluded_tables that accepts a string of excluded tables instead of an array.';


--
-- Name: child_object__remove_excluded_tables(text, text, text, text[]); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION child_object__remove_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text[]) RETURNS void
    AS $_$

SELECT inheritance.child_object__remove_excluded_tables( inheritance.child_object__get_id( $1, $2, $3 ), $4 )
  WHERE tools.assert( 'inheritance__child_object__not_found'
    , inheritance.child_object__get_id( $1, $2, $3 ) IS NOT NULL
    , 'Child object for inherited table ' || $1 || ', object type ' || $2 || ', object name template ' || $3 || ' not found'
  )

$_$
    LANGUAGE sql;


ALTER FUNCTION inheritance.child_object__remove_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text[]) OWNER TO db_global_owner;

--
-- Name: FUNCTION child_object__remove_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text[]); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION child_object__remove_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text[]) IS 'Version of child_object__remove_excluded_tables that accepts strings instead of an ID.';


--
-- Name: child_object__remove_excluded_tables(text, text, text, text); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION child_object__remove_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text) RETURNS void
    AS $_$
SELECT inheritance.child_object__remove_excluded_tables( $1, $2, $3, tools.string_or_array( $4 ) )
$_$
    LANGUAGE sql;


ALTER FUNCTION inheritance.child_object__remove_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text) OWNER TO db_global_owner;

--
-- Name: FUNCTION child_object__remove_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION child_object__remove_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text) IS 'Version of child_object__remove_excluded_tables that accepts strings instead of an ID and a string of excluded tables instead of an array.';


--
-- Name: get_parents(oid); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION get_parents(p_child_table_oid oid, OUT parent_oid oid) RETURNS SETOF oid
    AS $_$

DECLARE
  v_oid oid;
BEGIN
  FOR v_oid IN SELECT inhparent FROM pg_catalog.pg_inherits WHERE inhrelid = $1::regclass LOOP
    parent_oid := v_oid;
    RETURN NEXT;
    -- For each one of our parents, we also want to return all of their parents
    FOR parent_oid IN SELECT * FROM inheritance.get_parents( v_oid ) LOOP
      RETURN NEXT;
    END LOOP;
  END LOOP;
END
$_$
    LANGUAGE plpgsql STABLE;


ALTER FUNCTION inheritance.get_parents(p_child_table_oid oid, OUT parent_oid oid) OWNER TO db_global_owner;

--
-- Name: FUNCTION get_parents(p_child_table_oid oid, OUT parent_oid oid); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION get_parents(p_child_table_oid oid, OUT parent_oid oid) IS 'Returns all parents of a child table, as well as the parents parents, etc';


--
-- Name: londiste_trigger_refresh(text); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION londiste_trigger_refresh(p_parent_table text, OUT child_table text) RETURNS SETOF text
    AS $$
DECLARE
  v_child text;
BEGIN
  FOR v_child IN 
    SELECT inhrelid::regclass::text 
        FROM pg_inherits 
        WHERE inhparent = p_parent_table::regclass 
  LOOP
    PERFORM tools.londiste_trigger_refresh( v_child );
    child_table := v_child;
    RETURN NEXT;

    FOR child_table IN 
        SELECT * FROM inheritance.londiste_trigger_refresh( v_child )
    LOOP
      RETURN NEXT;
    END LOOP;
  END LOOP;

  RETURN;
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION inheritance.londiste_trigger_refresh(p_parent_table text, OUT child_table text) OWNER TO db_global_owner;

--
-- Name: FUNCTION londiste_trigger_refresh(p_parent_table text, OUT child_table text); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION londiste_trigger_refresh(p_parent_table text, OUT child_table text) IS 'Calls tools.londiste_trigger_refresh for each table that inherits from p_parent_table, as well as any of their children.';


--
-- Name: object_types; Type: TABLE; Schema: inheritance; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE object_types (
    id smallint NOT NULL,
    object_type text NOT NULL,
    description text,
    template text NOT NULL,
    reset_template text
);


ALTER TABLE inheritance.object_types OWNER TO db_global_owner;

--
-- Name: TABLE object_types; Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON TABLE object_types IS 'Lookup table for object_type. The data in this table is considered to be static, meaning that it should only be changed via a patch. This is generated code. This is a seed table.';


--
-- Name: object_types__get(smallint); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION object_types__get(p_id smallint) RETURNS object_types
    AS $_$
SELECT * FROM inheritance.object_types WHERE id = $1
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION inheritance.object_types__get(p_id smallint) OWNER TO db_global_owner;

--
-- Name: FUNCTION object_types__get(p_id smallint); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION object_types__get(p_id smallint) IS 'Returns row from inheritance.object_types based on id. This is generated code.';


--
-- Name: object_types__get_id(text); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION object_types__get_id(p_object_type text) RETURNS smallint
    AS $_$
SELECT id FROM inheritance.object_types WHERE lower(object_type) = lower($1)
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION inheritance.object_types__get_id(p_object_type text) OWNER TO db_global_owner;

--
-- Name: FUNCTION object_types__get_id(p_object_type text); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION object_types__get_id(p_object_type text) IS 'Returns ID value for inheritance.object_types based on object_type. This is generated code.';


--
-- Name: object_types__get_text(smallint); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION object_types__get_text(p_id smallint) RETURNS text
    AS $_$
SELECT object_type FROM inheritance.object_types WHERE id = $1
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION inheritance.object_types__get_text(p_id smallint) OWNER TO db_global_owner;

--
-- Name: FUNCTION object_types__get_text(p_id smallint); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION object_types__get_text(p_id smallint) IS 'Returns object_type value for inheritance.object_types based on id. This is generated code.';


--
-- Name: process_all(); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION process_all(OUT child_table text) RETURNS SETOF text
    AS $$

DECLARE
  v_parent text;
BEGIN
  FOR v_parent IN SELECT DISTINCT inherited_table_full_name FROM inheritance.child_objects_v LOOP
    FOR child_table IN SELECT * FROM inheritance.process_children( v_parent ) LOOP
      RETURN NEXT;
    END LOOP;
  END LOOP;

  RETURN;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION inheritance.process_all(OUT child_table text) OWNER TO db_global_owner;

--
-- Name: FUNCTION process_all(OUT child_table text); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION process_all(OUT child_table text) IS 'Calls inheritance.process_children for each table that is registered in the inheritance framework.';


--
-- Name: process_child(text); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION process_child(p_child_table text) RETURNS text
    AS $$

DECLARE
  fname CONSTANT text := 'inheritance.process_child';
  v_child oid := p_child_table::regclass::oid;

  v_child_full text := tools.table_full_name( p_child_table );
  v_child_schema text;
  v_child_name text;

  r_object record;
  v_name_array text[][];
  sql text;
BEGIN
  /* We need to be SU to run the dangerous londiste stuff we're doing. That
   * means it's up to us to check that the calling user owns the tables
   * involved. We can't check parents yet, but we can at least check the child
   * now.
   */
  /* Need a way to check to see if user has access to SU
  IF ( SELECT pg_catalog.pg_get_userbyid( relowner ) FROM pg_class WHERE oid = v_child ) != session_user THEN
    RAISE EXCEPTION 'You must own the child table %', v_child_full;
  END IF;
  */

  IF NOT EXISTS( SELECT * FROM pg_inherits WHERE inhrelid = v_child ) THEN
    RAISE EXCEPTION '%: % is not an inherited table', fname, p_child_table;
  END IF;

  /* Having londiste triggers MIA could cause problems, so put them back. Note
   * that subscriber_pending_fkeys is only used during replication setup, so we
   * don't want to screw with it, but we should bomb if anything is in there
   * for our table.
   * MAKE SURE TO CALL drop_all_triggers at the end!
   */
  PERFORM tools.assert(
    'inheritance__process_child no pending fkeys'
    , NOT EXISTS( SELECT * FROM tools.londiste__pending_fkeys( v_child_full ) )
    , 'Unable to process table ' || v_child_full || ': there are pending foreign keys for it in londiste'
  );
  PERFORM tools.londiste__restore_all_triggers( v_child_full );

  SELECT INTO v_child_schema, v_child_name * FROM tools.table_schema_and_name( p_child_table );
  FOR r_object IN
  SELECT c.*
    FROM inheritance.child_objects_v c
      JOIN inheritance.get_parents( v_child ) p ON c.inherited_table_oid = p.parent_oid
    WHERE NOT EXISTS(
            SELECT *
              FROM inheritance.child_objects_exclude_tables e
                JOIN tools.tables_v t ON t.id = e.exclude_table_id
              WHERE e.child_object_id = c.id
                AND t.oid = v_child
          )
  LOOP
    /* Need a way to check to see if user has access to SU
    IF ( SELECT pg_catalog.pg_get_userbyid( relowner ) FROM pg_class WHERE oid = r_object.inherited_table_oid ) != session_user
      AND NOT pg_has_role(session_user, role, 'USAGE' )
    THEN
      RAISE EXCEPTION 'You must own the parent table % or be superuser', r_object.inherited_table_full_name;
    END IF;
    */

    PERFORM tools.assert(
      'inheritance__process_child no pending fkeys'
      , NOT EXISTS( SELECT * FROM tools.londiste__pending_fkeys( r_object.inherited_table_full_name  ) )
      , 'Unable to process table ' || r_object.inherited_table_full_name || ': there are pending foreign keys for it in londiste'
    );

    v_name_array := array[
      [ 'parent_schema', r_object.inherited_table_schema ]
      , [ 'parent_name', r_object.inherited_table_name ]
      , [ 'parent_full', r_object.inherited_table_full_name ]
      , [ 'child_schema', v_child_schema ]
      , [ 'child_name', v_child_name ]
      , [ 'child_full', v_child_full ]
    ];
    SELECT INTO sql template FROM inheritance.object_types__get( r_object.object_type_id );
    sql := tools.parameter_replace(
        tools.parameter_replace(
          sql
          , array[
            [ 'object_name_template', r_object.object_name_template ]
            , [ 'object_body_template', r_object.object_body_template ]
          ]
        )
        , v_name_array
    );
    BEGIN
      PERFORM tools.exec( sql );
    EXCEPTION
      WHEN duplicate_table THEN -- We get this for a duplicate unique constraint
        IF SQLERRM !~ tools.parameter_replace( r_object.object_name_template, v_name_array ) THEN
          RAISE EXCEPTION '%: Error %, %', fname, SQLSTATE, SQLERRM;
        END IF;
      WHEN duplicate_object THEN
        IF SQLERRM !~ tools.parameter_replace( r_object.object_name_template, v_name_array ) THEN
          RAISE EXCEPTION '%: Error %, %', fname, SQLSTATE, SQLERRM;
        END IF;
      WHEN invalid_table_definition THEN
        IF SQLERRM !~ 'multiple primary keys for table' THEN
          RAISE EXCEPTION '%: Error %, %', fname, SQLSTATE, SQLERRM;
        END IF;
      WHEN others THEN
        RAISE EXCEPTION '%: Error %, %', fname, SQLSTATE, SQLERRM;
    END;
  END LOOP;

  -- Remove the triggers again now that we're done
  PERFORM tools.londiste__drop_all_triggers( v_child_full );

  RETURN v_child::regclass::text;
END;

$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION inheritance.process_child(p_child_table text) OWNER TO db_global_owner;

--
-- Name: FUNCTION process_child(p_child_table text); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION process_child(p_child_table text) IS 'Applies all objects to a table that should be there based on the tables it inherits from.

Parameters:
p_child_table
Name of table to process.

This function will find all the tables that p_child_table inherits from. It
looks for matching records in inheritance.child_objects for those parents, and
applies those objects to p_child_table (assuming that p_child_table is not
 listed as an excluded table). If the object already exists no error is
thrown.

Note that this function will never remove an object.
';


--
-- Name: process_children(text); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION process_children(p_parent_table text, OUT child_table text) RETURNS SETOF text
    AS $$

DECLARE
  v_child text;
BEGIN
  FOR v_child IN SELECT inhrelid::regclass::text FROM pg_inherits WHERE inhparent = p_parent_table::regclass LOOP
    child_table := inheritance.process_child( v_child ); -- process_child just returns the table name
    RETURN NEXT;

    FOR child_table IN SELECT * FROM inheritance.process_children( v_child ) LOOP
      RETURN NEXT;
    END LOOP;
  END LOOP;

  RETURN;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION inheritance.process_children(p_parent_table text, OUT child_table text) OWNER TO db_global_owner;

--
-- Name: FUNCTION process_children(p_parent_table text, OUT child_table text); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION process_children(p_parent_table text, OUT child_table text) IS 'Calls inheritance.process_child for each table that inherits from p_parent_table, as well as any of their children.';


--
-- Name: reset_child(text, text); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION reset_child(p_child_table text, p_object_type text) RETURNS text
    AS $$

DECLARE
  fname CONSTANT text := 'inheritance.reset_child';
  v_child oid := p_child_table::regclass::oid;

  v_child_full text := tools.table_full_name( p_child_table );
  v_child_schema text;
  v_child_name text;

  r_object record;
  v_name_array text[][];
  sql text;
BEGIN
  /* We need to be SU to run the dangerous londiste stuff we're doing. That
   * means it's up to us to check that the calling user owns the tables
   * involved. We can't check parents yet, but we can at least check the child
   * now.
   */
  /* Need a way to check to see if user has access to SU
  IF ( SELECT pg_catalog.pg_get_userbyid( relowner ) FROM pg_class WHERE oid = v_child ) != session_user THEN
    RAISE EXCEPTION 'You must own the child table %', v_child_full;
  END IF;
  */

  IF NOT EXISTS( SELECT * FROM pg_inherits WHERE inhrelid = v_child ) THEN
    RAISE EXCEPTION '%: % is not an inherited table', fname, p_child_table;
  END IF;

  /* Having londiste triggers MIA could cause problems, so put them back. Note
   * that subscriber_pending_fkeys is only used during replication setup, so we
   * don't want to screw with it, but we should bomb if anything is in there
   * for our table.
   * MAKE SURE TO CALL drop_all_triggers at the end!
   */
  PERFORM tools.assert(
    'inheritance__process_child no pending fkeys'
    , NOT EXISTS( SELECT * FROM tools.londiste__pending_fkeys( v_child_full ) )
    , 'Unable to process table ' || v_child_full || ': there are pending foreign keys for it in londiste'
  );
  PERFORM tools.londiste__restore_all_triggers( v_child_full );

  SELECT INTO v_child_schema, v_child_name * FROM tools.table_schema_and_name( p_child_table );
  FOR r_object IN
  SELECT c.*
    FROM inheritance.child_objects_v c
      JOIN inheritance.get_parents( v_child ) p ON c.inherited_table_oid = p.parent_oid
    WHERE NOT EXISTS(
            SELECT *
              FROM inheritance.child_objects_exclude_tables e
                JOIN tools.tables_v t ON t.id = e.exclude_table_id
              WHERE e.child_object_id = c.id
                AND t.oid = v_child
          ) AND c.object_type = p_object_type
  LOOP
    /* Need a way to check to see if user has access to SU
    IF ( SELECT pg_catalog.pg_get_userbyid( relowner ) FROM pg_class WHERE oid = r_object.inherited_table_oid ) != session_user
      AND NOT pg_has_role(session_user, role, 'USAGE' )
    THEN
      RAISE EXCEPTION 'You must own the parent table % or be superuser', r_object.inherited_table_full_name;
    END IF;
    */

    PERFORM tools.assert(
      'inheritance__reset_child no pending fkeys'
      , NOT EXISTS( SELECT * FROM tools.londiste__pending_fkeys( r_object.inherited_table_full_name  ) )
      , 'Unable to reset table ' || r_object.inherited_table_full_name || ': there are pending foreign keys for it in londiste'
    );

    v_name_array := array[
      [ 'parent_schema', r_object.inherited_table_schema ]
      , [ 'parent_name', r_object.inherited_table_name ]
      , [ 'parent_full', r_object.inherited_table_full_name ]
      , [ 'child_schema', v_child_schema ]
      , [ 'child_name', v_child_name ]
      , [ 'child_full', v_child_full ]
    ];
    SELECT INTO sql reset_template FROM inheritance.object_types__get( r_object.object_type_id );
    sql := tools.parameter_replace(
        tools.parameter_replace(
          sql
          , array[
            [ 'object_name_template', r_object.object_name_template ]
            , [ 'object_body_template', r_object.object_body_template ]
          ]
        )
        , v_name_array
    );
    BEGIN
      PERFORM tools.exec( sql );
    EXCEPTION
      WHEN duplicate_table THEN -- We get this for a duplicate unique constraint
        IF SQLERRM !~ tools.parameter_replace( r_object.object_name_template, v_name_array ) THEN
          RAISE EXCEPTION '%: Error %, %', fname, SQLSTATE, SQLERRM;
        END IF;
      WHEN duplicate_object THEN
        IF SQLERRM !~ tools.parameter_replace( r_object.object_name_template, v_name_array ) THEN
          RAISE EXCEPTION '%: Error %, %', fname, SQLSTATE, SQLERRM;
        END IF;
      WHEN invalid_table_definition THEN
        IF SQLERRM !~ 'multiple primary keys for table' THEN
          RAISE EXCEPTION '%: Error %, %', fname, SQLSTATE, SQLERRM;
        END IF;
      WHEN others THEN
        RAISE EXCEPTION '%: Error %, %', fname, SQLSTATE, SQLERRM;
    END;
  END LOOP;

  -- Remove the triggers again now that we're done
  PERFORM tools.londiste__drop_all_triggers( v_child_full );

  RETURN v_child::regclass::text;
END;

$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION inheritance.reset_child(p_child_table text, p_object_type text) OWNER TO db_global_owner;

--
-- Name: FUNCTION reset_child(p_child_table text, p_object_type text); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION reset_child(p_child_table text, p_object_type text) IS 'Removes all objects on a table that should be there based on the tables it inherits from.

Parameters:
p_child_table
Name of table to reset.

This function will find all the tables that p_child_table inherits from. It
looks for matching records in inheritance.child_objects for those parents, and
removes those objects on p_child_table (assuming that p_child_table is not
 listed as an excluded table). If the object does not exist an error is
thrown.

Note that the object drops do not cascade, so if there are dependent objects
(as in the case of primary keys that have foreign keys referencing them) an
error will be thrown.
';


--
-- Name: reset_children(text, text); Type: FUNCTION; Schema: inheritance; Owner: db_global_owner
--

CREATE FUNCTION reset_children(p_parent_table text, p_object_type text, OUT child_table text) RETURNS SETOF text
    AS $$

DECLARE
  v_child text;
BEGIN
  FOR v_child IN SELECT inhrelid::regclass::text FROM pg_inherits WHERE inhparent = p_parent_table::regclass LOOP
    child_table := inheritance.reset_child( v_child, p_object_type ); -- reset_child just returns the table name
    RETURN NEXT;

    FOR child_table IN SELECT * FROM inheritance.reset_children( v_child, p_object_type ) LOOP
      RETURN NEXT;
    END LOOP;
  END LOOP;

  RETURN;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION inheritance.reset_children(p_parent_table text, p_object_type text, OUT child_table text) OWNER TO db_global_owner;

--
-- Name: FUNCTION reset_children(p_parent_table text, p_object_type text, OUT child_table text); Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON FUNCTION reset_children(p_parent_table text, p_object_type text, OUT child_table text) IS 'Calls inheritance.reset_child for each table that inherits from p_parent_table, as well as any of their children.';


SET search_path = object_monitor, pg_catalog;

--
-- Name: create_queues(text); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION create_queues(p_monitor_name text) RETURNS SETOF regclass
    AS $_$ 
    -- Create the queues
    SELECT r FROM (
            SELECT t.oid AS r
                , pgq.create_queue( 
                    'object_monitor.' || table_schema || '.' || table_name )
            FROM object_monitor.monitors_tables m
                INNER JOIN tools.tables_v t
                    ON m.table_id = t.id
            WHERE monitor_name = $1
        ) AS t2
    ;
$_$
    LANGUAGE sql;


ALTER FUNCTION object_monitor.create_queues(p_monitor_name text) OWNER TO db_global_owner;

--
-- Name: create_trigger(text); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION create_trigger(p_table_name text) RETURNS void
    AS $$ 
BEGIN
    -- Create the trigger
    BEGIN
        PERFORM tools.exec( 'CREATE TRIGGER tg_object_monitor AFTER '
            || 'INSERT OR UPDATE ON ' || p_table_name
            || ' FOR EACH ROW EXECUTE PROCEDURE object_monitor.tg_populate_queue()' )
        ;
    EXCEPTION WHEN duplicate_object THEN
        -- Do nothing, it's already done
    END;
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION object_monitor.create_trigger(p_table_name text) OWNER TO db_global_owner;

--
-- Name: drain_queue(text); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION drain_queue(p_table_name text) RETURNS SETOF integer
    AS $_$
    SELECT DISTINCT * FROM object_monitor.drain_queue_dups($1);
$_$
    LANGUAGE sql SECURITY DEFINER;


ALTER FUNCTION object_monitor.drain_queue(p_table_name text) OWNER TO db_global_owner;

--
-- Name: drain_queue(text, text, bigint); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION drain_queue(p_consumer_name text, p_table_name text, p_batch_count bigint) RETURNS SETOF integer
    AS $_$
    SELECT DISTINCT * FROM object_monitor.drain_queue_dups($1, $2, $3);
$_$
    LANGUAGE sql SECURITY DEFINER;


ALTER FUNCTION object_monitor.drain_queue(p_consumer_name text, p_table_name text, p_batch_count bigint) OWNER TO db_global_owner;

--
-- Name: drain_queue_dups(text); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION drain_queue_dups(p_table_name text) RETURNS SETOF integer
    AS $$
DECLARE
    v_batch_id int;
    v_data int;
BEGIN
    LOOP
        -- Loop through the available batches
        v_batch_id := pgq.next_batch( 'object_monitor.' || p_table_name,
            tools.session__get( 'object_monitor', txid_current()::text ) );
        EXIT WHEN v_batch_id IS NULL;
        -- Return the id inside each event in this batch
        FOR v_data IN 
            SELECT ev_data::int FROM pgq.get_batch_events( v_batch_id )
        LOOP
            RETURN NEXT v_data;
        END LOOP;
        -- Mark the batch finished
        PERFORM pgq.finish_batch( v_batch_id );
    END LOOP;
END;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION object_monitor.drain_queue_dups(p_table_name text) OWNER TO db_global_owner;

--
-- Name: drain_queue_dups(text, text, bigint); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION drain_queue_dups(p_consumer_name text, p_table_name text, p_batch_count bigint) RETURNS SETOF integer
    AS $$
DECLARE
    v_batch_id int;
    v_data int;
    v_loop_count int := 0;
BEGIN
    BEGIN
        LOOP
            -- Loop through the available batches
            v_batch_id := pgq.next_batch( 'object_monitor.' || p_table_name,
                p_consumer_name );
            IF v_batch_id IS NULL OR v_loop_count > p_batch_count THEN 
                RAISE EXCEPTION 'Success';
            END IF;
            -- Return the id inside each event in this batch
            FOR v_data IN 
                SELECT ev_data::int FROM pgq.get_batch_events( v_batch_id )
            LOOP
                RETURN NEXT v_data;
            END LOOP;
            -- Mark the batch finished
            PERFORM pgq.finish_batch( v_batch_id );
            v_loop_count := v_loop_count + 1;
        END LOOP;
    EXCEPTION WHEN raise_exception THEN
    END;
END;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION object_monitor.drain_queue_dups(p_consumer_name text, p_table_name text, p_batch_count bigint) OWNER TO db_global_owner;

--
-- Name: drop_queues(text); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION drop_queues(p_monitor_name text) RETURNS SETOF regclass
    AS $_$ 
    -- Drop the queues
    SELECT r FROM (
            SELECT t.oid AS r
                , pgq.drop_queue( 
                    'object_monitor.' || table_schema || '.' || table_name )
            FROM object_monitor.monitors_tables m
                INNER JOIN tools.tables_v t
                    ON m.table_id = t.id
            WHERE monitor_name = $1
                AND NOT EXISTS (
                        SELECT 1 FROM object_monitor.monitors_tables 
                        WHERE table_id = m.table_id
                            AND monitor_name != m.monitor_name
                    )
        ) AS t2
    ;
$_$
    LANGUAGE sql;


ALTER FUNCTION object_monitor.drop_queues(p_monitor_name text) OWNER TO db_global_owner;

--
-- Name: drop_trigger(text); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION drop_trigger(p_table_name text) RETURNS void
    AS $$ 
BEGIN
    -- Drop the trigger
    PERFORM tools.exec( 'DROP TRIGGER tg_object_monitor ON ' 
		|| p_table_name )
    ;
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION object_monitor.drop_trigger(p_table_name text) OWNER TO db_global_owner;

--
-- Name: finish_batches(text, text, bigint); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION finish_batches(p_consumer_name text, p_table_name text, p_batch_count bigint) RETURNS void
    AS $$
DECLARE
    v_batch_id int;
    v_loop_count int := 0;
BEGIN
    LOOP
        -- Loop through the available batches
        v_batch_id := pgq.next_batch( 'object_monitor.' || p_table_name,
            p_consumer_name );
        EXIT WHEN v_batch_id IS NULL OR v_loop_count > p_batch_count;
        -- Mark the batch finished
        PERFORM pgq.finish_batch( v_batch_id );
        v_loop_count := v_loop_count + 1;
    END LOOP;
END;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION object_monitor.finish_batches(p_consumer_name text, p_table_name text, p_batch_count bigint) OWNER TO db_global_owner;

--
-- Name: get_batch_count(text, text); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION get_batch_count(p_consumer_name text, p_table_name text) RETURNS integer
    AS $$
DECLARE
    v_batch_id bigint;
    v_batch_count int := 0;
BEGIN
    BEGIN
        LOOP
            v_batch_id := pgq.next_batch( 'object_monitor.' || p_table_name, 
                p_consumer_name );
            IF v_batch_id IS NULL THEN 
                RAISE EXCEPTION 'Success';
            END IF;
            PERFORM pgq.finish_batch( v_batch_id );
            v_batch_count := v_batch_count + 1;
        END LOOP;
    EXCEPTION WHEN raise_exception THEN
    END;

    RETURN v_batch_count;
END;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION object_monitor.get_batch_count(p_consumer_name text, p_table_name text) OWNER TO db_global_owner;

--
-- Name: monitor__add_tables(text, text[]); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION monitor__add_tables(p_monitor_name text, p_table_list text[]) RETURNS SETOF integer
    AS $_$ 
    SELECT * FROM object_monitor.process_monitor_create( $1, $2 );

    SELECT object_monitor.subscribe_consumer( monitor_name, consumer_name )
    FROM object_monitor.monitors_consumers
    WHERE monitor_name = $1
    ;
$_$
    LANGUAGE sql SECURITY DEFINER;


ALTER FUNCTION object_monitor.monitor__add_tables(p_monitor_name text, p_table_list text[]) OWNER TO db_global_owner;

--
-- Name: monitor__create(text, text[]); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION monitor__create(p_monitor_name text, p_table_list text[]) RETURNS SETOF regclass
    AS $_$ 
    /* Keep this version around for when we need the PK
    INSERT INTO tools.object_monitors 
            ( object_name, table_name, table_key ) 
        SELECT $1, t, (
            SELECT column_name 
            FROM information_schema.constraint_column_usage 
            WHERE (constraint_schema, constraint_name) = (
                SELECT constraint_schema, constraint_name 
                FROM information_schema.table_constraints 
                WHERE table_schema||'.'||table_name = t
                    AND constraint_type = 'PRIMARY KEY'
            )
        )
        FROM unnest($2) t
    ;
    */

    INSERT INTO object_monitor.monitors ( monitor_name ) VALUES ( $1 );

    SELECT * FROM object_monitor.process_monitor_create( $1, $2 );
$_$
    LANGUAGE sql SECURITY DEFINER;


ALTER FUNCTION object_monitor.monitor__create(p_monitor_name text, p_table_list text[]) OWNER TO db_global_owner;

--
-- Name: monitor__drop(text); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION monitor__drop(p_monitor_name text) RETURNS void
    AS $$ 
BEGIN
    -- Check for consumers
    IF EXISTS( SELECT * FROM object_monitor.monitors_consumers
                   WHERE monitor_name = p_monitor_name ) THEN 
        RAISE EXCEPTION 'Cannot drop monitor, consumers still attached';
    END IF;

    PERFORM * FROM object_monitor.process_monitor_drop( p_monitor_name );

    DELETE FROM object_monitor.monitors WHERE monitor_name = p_monitor_name;
END;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION object_monitor.monitor__drop(p_monitor_name text) OWNER TO db_global_owner;

--
-- Name: monitor__drop_tables(text, text[]); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION monitor__drop_tables(p_monitor_name text, p_table_list text[]) RETURNS void
    AS $$ 
BEGIN
    -- Unsubscribe everyone from the queue(s) that will be dropped
    PERFORM pgq.unregister_consumer( 
            'object_monitor.' || table_schema || '.' || table_name
            , c.consumer_name )
    FROM object_monitor.monitors_tables m
        INNER JOIN tools.tables_v t
            ON m.table_id = t.id
        INNER JOIN object_monitor.monitors_consumers c
            ON m.monitor_name = c.monitor_name
    WHERE m.monitor_name = p_monitor_name
        AND t.table_schema || '.' || t.table_name IN (
                SELECT unnest( p_table_list )
            )
        AND NOT EXISTS (
                SELECT 1 FROM object_monitor.monitors_tables mt
                    INNER JOIN object_monitor.monitors_consumers mc
                        ON mt.monitor_name = mc.monitor_name
                WHERE mt.table_id = m.table_id
                    AND mt.monitor_name != m.monitor_name
                    AND mc.consumer_name = c.consumer_name
            )
    ; 
    -- Drop the trigger(s)
    PERFORM object_monitor.drop_trigger( 
        t.table_schema || '.' || t.table_name )
    FROM object_monitor.monitors_tables m
        INNER JOIN tools.tables_v t
            ON m.table_id = t.id
    WHERE monitor_name = p_monitor_name
        AND t.table_schema || '.' || t.table_name IN (
                SELECT unnest( p_table_list )
            )
        AND NOT EXISTS (
                SELECT 1 FROM object_monitor.monitors_tables 
                WHERE table_id = m.table_id
                    AND monitor_name != m.monitor_name
            )
    ;
    -- Drop the queue(s)
    PERFORM r FROM (
            SELECT t.oid AS r
                , pgq.drop_queue( 
                    'object_monitor.' || table_schema || '.' || table_name )
            FROM object_monitor.monitors_tables m
                INNER JOIN tools.tables_v t
                    ON m.table_id = t.id
            WHERE monitor_name = p_monitor_name
                AND t.table_schema || '.' || t.table_name IN (
                        SELECT unnest( p_table_list )
                    )
                AND NOT EXISTS (
                        SELECT 1 FROM object_monitor.monitors_tables 
                        WHERE table_id = m.table_id
                            AND monitor_name != m.monitor_name
                    )
        ) AS t2
    ;
    -- Remove the table(s) from the monitor
    DELETE FROM object_monitor.monitors_tables 
    WHERE monitor_name = p_monitor_name
        AND table_id IN ( 
                SELECT unnest( tools.tables__getsert( p_table_list ) ) 
            )
    ;
END;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION object_monitor.monitor__drop_tables(p_monitor_name text, p_table_list text[]) OWNER TO db_global_owner;

--
-- Name: process_monitor_create(text, text[]); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION process_monitor_create(p_monitor_name text, p_table_list text[]) RETURNS SETOF regclass
    AS $_$ 
    -- This is done in the odd order so we can return the results of
    -- the queue call. It works because the queue name is a text variable.

    -- First the tables
    INSERT INTO object_monitor.monitors_tables
            ( monitor_name, table_id ) 
        SELECT $1, t FROM unnest( tools.tables__getsert($2) ) t
    ;
    -- Then the triggers
    SELECT object_monitor.create_trigger( 
        t.table_schema || '.' || t.table_name )
    FROM object_monitor.monitors_tables m
        INNER JOIN tools.tables_v t
            ON m.table_id = t.id
    WHERE monitor_name = $1
    ;
    -- Then the queues
    SELECT * FROM object_monitor.create_queues($1);
$_$
    LANGUAGE sql;


ALTER FUNCTION object_monitor.process_monitor_create(p_monitor_name text, p_table_list text[]) OWNER TO db_global_owner;

--
-- Name: process_monitor_drop(text); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION process_monitor_drop(p_monitor_name text) RETURNS void
    AS $_$ 
    -- First the triggers
    SELECT object_monitor.drop_trigger( 
        t.table_schema || '.' || t.table_name )
    FROM object_monitor.monitors_tables m
        INNER JOIN tools.tables_v t
            ON m.table_id = t.id
    WHERE monitor_name = $1
        AND NOT EXISTS (
                SELECT 1 FROM object_monitor.monitors_tables 
                WHERE table_id = m.table_id
                    AND monitor_name != m.monitor_name
            )
    ;
    -- Then the queues
    SELECT * FROM object_monitor.drop_queues($1);
    -- Then the tables
    DELETE FROM object_monitor.monitors_tables 
        WHERE monitor_name = $1
    ;
$_$
    LANGUAGE sql;


ALTER FUNCTION object_monitor.process_monitor_drop(p_monitor_name text) OWNER TO db_global_owner;

--
-- Name: register_consumer(text, text); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION register_consumer(p_monitor_name text, p_consumer_name text) RETURNS SETOF integer
    AS $_$ 
    -- Subscribe to the monitor
    INSERT INTO object_monitor.monitors_consumers 
        VALUES ( $1, $2 )
    ;

    -- Subscribe to the queues
    SELECT * FROM object_monitor.subscribe_consumer( $1, $2 );
$_$
    LANGUAGE sql SECURITY DEFINER;


ALTER FUNCTION object_monitor.register_consumer(p_monitor_name text, p_consumer_name text) OWNER TO db_global_owner;

--
-- Name: set_consumer(text); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION set_consumer(p_consumer_name text) RETURNS void
    AS $_$
    SELECT tools.session__set( 'object_monitor', txid_current()::text, $1 );
$_$
    LANGUAGE sql SECURITY DEFINER;


ALTER FUNCTION object_monitor.set_consumer(p_consumer_name text) OWNER TO db_global_owner;

--
-- Name: subscribe_consumer(text, text); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION subscribe_consumer(p_monitor_name text, p_consumer_name text) RETURNS SETOF integer
    AS $_$ 
    -- Subscribe to the queues
    SELECT pgq.register_consumer( 
            'object_monitor.' || table_schema || '.' || table_name
            , $2 )
    FROM object_monitor.monitors_tables m
        INNER JOIN tools.tables_v t
            ON m.table_id = t.id
    WHERE monitor_name = $1
    ;
$_$
    LANGUAGE sql;


ALTER FUNCTION object_monitor.subscribe_consumer(p_monitor_name text, p_consumer_name text) OWNER TO db_global_owner;

--
-- Name: tg_populate_queue(); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION tg_populate_queue() RETURNS trigger
    AS $$ 
BEGIN
    PERFORM pgq.insert_event( 
        'object_monitor.' || TG_TABLE_SCHEMA || '.' || TG_TABLE_NAME, 
        TG_OP, 
        NEW.id::text 
    );
    RETURN NEW;
END;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION object_monitor.tg_populate_queue() OWNER TO db_global_owner;

--
-- Name: unregister_consumer(text, text); Type: FUNCTION; Schema: object_monitor; Owner: db_global_owner
--

CREATE FUNCTION unregister_consumer(p_monitor_name text, p_consumer_name text) RETURNS SETOF integer
    AS $_$ 
    -- Unsubscribe from the monitor
    DELETE FROM object_monitor.monitors_consumers 
        WHERE ( monitor_name, consumer_name ) = ( $1, $2 )
    ;

    -- Unsubscribe from the queues
    SELECT pgq.unregister_consumer( 
            'object_monitor.' || table_schema || '.' || table_name
            , $2 )
    FROM object_monitor.monitors_tables m
        INNER JOIN tools.tables_v t
            ON m.table_id = t.id
    WHERE m.monitor_name = $1
        AND NOT EXISTS (
                SELECT 1 FROM object_monitor.monitors_tables mt
                    INNER JOIN object_monitor.monitors_consumers mc
                        ON mt.monitor_name = mc.monitor_name
                WHERE mt.table_id = m.table_id
                    AND mt.monitor_name != m.monitor_name
                    AND mc.consumer_name = $2
            )
    ;
$_$
    LANGUAGE sql SECURITY DEFINER;


ALTER FUNCTION object_monitor.unregister_consumer(p_monitor_name text, p_consumer_name text) OWNER TO db_global_owner;

SET search_path = pgq, pg_catalog;

--
-- Name: batch_event_sql(bigint); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION batch_event_sql(x_batch_id bigint) RETURNS text
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.batch_event_sql(1)
--      Creates SELECT statement that fetches events for this batch.
--
-- Parameters:
--      x_batch_id    - ID of a active batch.
--
-- Returns:
--      SQL statement.
-- ----------------------------------------------------------------------

-- ----------------------------------------------------------------------
-- Algorithm description:
--      Given 2 snapshots, sn1 and sn2 with sn1 having xmin1, xmax1
--      and sn2 having xmin2, xmax2 create expression that filters
--      right txid's from event table.
--
--      Simplest solution would be
--      > WHERE ev_txid >= xmin1 AND ev_txid <= xmax2
--      >   AND NOT txid_visible_in_snapshot(ev_txid, sn1)
--      >   AND txid_visible_in_snapshot(ev_txid, sn2)
--
--      The simple solution has a problem with long transactions (xmin1 very low).
--      All the batches that happen when the long tx is active will need
--      to scan all events in that range.  Here is 2 optimizations used:
--
--      1)  Use [xmax1..xmax2] for range scan.  That limits the range to
--      txids that actually happened between two snapshots.  For txids
--      in the range [xmin1..xmax1] look which ones were actually
--      committed between snapshots and search for them using exact
--      values using IN (..) list.
--
--      2) As most TX are short, there could be lot of them that were
--      just below xmax1, but were committed before xmax2.  So look
--      if there are ID's near xmax1 and lower the range to include
--      them, thus decresing size of IN (..) list.
-- ----------------------------------------------------------------------
declare
    rec             record;
    sql             text;
    tbl             text;
    arr             text;
    part            text;
    select_fields   text;
    retry_expr      text;
    batch           record;
begin
    select s.sub_last_tick, s.sub_next_tick, s.sub_id, s.sub_queue,
           txid_snapshot_xmax(last.tick_snapshot) as tx_start,
           txid_snapshot_xmax(cur.tick_snapshot) as tx_end,
           last.tick_snapshot as last_snapshot,
           cur.tick_snapshot as cur_snapshot
        into batch
        from pgq.subscription s, pgq.tick last, pgq.tick cur
        where s.sub_batch = x_batch_id
          and last.tick_queue = s.sub_queue
          and last.tick_id = s.sub_last_tick
          and cur.tick_queue = s.sub_queue
          and cur.tick_id = s.sub_next_tick;
    if not found then
        raise exception 'batch not found';
    end if;

    -- load older transactions
    arr := '';
    for rec in
        -- active tx-es in prev_snapshot that were committed in cur_snapshot
        select id1 from
            txid_snapshot_xip(batch.last_snapshot) id1 left join
            txid_snapshot_xip(batch.cur_snapshot) id2 on (id1 = id2)
        where id2 is null
        order by 1 desc
    loop
        -- try to avoid big IN expression, so try to include nearby
        -- tx'es into range
        if batch.tx_start - 100 <= rec.id1 then
            batch.tx_start := rec.id1;
        else
            if arr = '' then
                arr := rec.id1;
            else
                arr := arr || ',' || rec.id1;
            end if;
        end if;
    end loop;

    -- must match pgq.event_template
    select_fields := 'select ev_id, ev_time, ev_txid, ev_retry, ev_type,'
        || ' ev_data, ev_extra1, ev_extra2, ev_extra3, ev_extra4';
    retry_expr :=  ' and (ev_owner is null or ev_owner = '
        || batch.sub_id || ')';

    -- now generate query that goes over all potential tables
    sql := '';
    for rec in
        select xtbl from pgq.batch_event_tables(x_batch_id) xtbl
    loop
        tbl := rec.xtbl;
        -- this gets newer queries that definitely are not in prev_snapshot
        part := select_fields
            || ' from pgq.tick cur, pgq.tick last, ' || tbl || ' ev '
            || ' where cur.tick_id = ' || batch.sub_next_tick
            || ' and cur.tick_queue = ' || batch.sub_queue
            || ' and last.tick_id = ' || batch.sub_last_tick
            || ' and last.tick_queue = ' || batch.sub_queue
            || ' and ev.ev_txid >= ' || batch.tx_start
            || ' and ev.ev_txid <= ' || batch.tx_end
            || ' and txid_visible_in_snapshot(ev.ev_txid, cur.tick_snapshot)'
            || ' and not txid_visible_in_snapshot(ev.ev_txid, last.tick_snapshot)'
            || retry_expr;
        -- now include older tx-es, that were ongoing
        -- at the time of prev_snapshot
        if arr <> '' then
            part := part || ' union all '
                || select_fields || ' from ' || tbl || ' ev '
                || ' where ev.ev_txid in (' || arr || ')'
                || retry_expr;
        end if;
        if sql = '' then
            sql := part;
        else
            sql := sql || ' union all ' || part;
        end if;
    end loop;
    if sql = '' then
        raise exception 'could not construct sql for batch %', x_batch_id;
    end if;
    return sql || ' order by 1';
end;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION pgq.batch_event_sql(x_batch_id bigint) OWNER TO skytools;

--
-- Name: batch_event_tables(bigint); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION batch_event_tables(x_batch_id bigint) RETURNS SETOF text
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.batch_event_tables(1)
--
--     Returns set of table names where this batch events may reside.
--
-- Parameters:
--     x_batch_id    - ID of a active batch.
-- ----------------------------------------------------------------------
declare
    nr                    integer;
    tbl                   text;
    use_prev              integer;
    use_next              integer;
    batch                 record;
begin
    select
           txid_snapshot_xmin(last.tick_snapshot) as tx_min, -- absolute minimum
           txid_snapshot_xmax(cur.tick_snapshot) as tx_max, -- absolute maximum
           q.queue_data_pfx, q.queue_ntables,
           q.queue_cur_table, q.queue_switch_step1, q.queue_switch_step2
        into batch
        from pgq.tick last, pgq.tick cur, pgq.subscription s, pgq.queue q
        where cur.tick_id = s.sub_next_tick
          and cur.tick_queue = s.sub_queue
          and last.tick_id = s.sub_last_tick
          and last.tick_queue = s.sub_queue
          and s.sub_batch = x_batch_id
          and q.queue_id = s.sub_queue;
    if not found then
        raise exception 'Cannot find data for batch %', x_batch_id;
    end if;

    -- if its definitely not in one or other, look into both
    if batch.tx_max < batch.queue_switch_step1 then
        use_prev := 1;
        use_next := 0;
    elsif batch.queue_switch_step2 is not null
      and (batch.tx_min > batch.queue_switch_step2)
    then
        use_prev := 0;
        use_next := 1;
    else
        use_prev := 1;
        use_next := 1;
    end if;

    if use_prev then
        nr := batch.queue_cur_table - 1;
        if nr < 0 then
            nr := batch.queue_ntables - 1;
        end if;
        tbl := batch.queue_data_pfx || '_' || nr;
        return next tbl;
    end if;

    if use_next then
        tbl := batch.queue_data_pfx || '_' || batch.queue_cur_table;
        return next tbl;
    end if;

    return;
end;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION pgq.batch_event_tables(x_batch_id bigint) OWNER TO skytools;

--
-- Name: create_queue(text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION create_queue(i_queue_name text) RETURNS integer
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.create_queue(1)
--
--      Creates new queue with given name.
--
-- Returns:
--      0 - queue already exists
--      1 - queue created
-- ----------------------------------------------------------------------
declare
    tblpfx   text;
    tblname  text;
    idxpfx   text;
    idxname  text;
    sql      text;
    id       integer;
    tick_seq text;
    ev_seq text;
    n_tables integer;
begin
    if i_queue_name is null then
        raise exception 'Invalid NULL value';
    end if;

    -- check if exists
    perform 1 from pgq.queue where queue_name = i_queue_name;
    if found then
        return 0;
    end if;

    -- insert event
    id := nextval('pgq.queue_queue_id_seq');
    tblpfx := 'pgq.event_' || id;
    idxpfx := 'event_' || id;
    tick_seq := 'pgq.event_' || id || '_tick_seq';
    ev_seq := 'pgq.event_' || id || '_id_seq';
    insert into pgq.queue (queue_id, queue_name,
            queue_data_pfx, queue_event_seq, queue_tick_seq)
        values (id, i_queue_name, tblpfx, ev_seq, tick_seq);

    select queue_ntables into n_tables from pgq.queue
        where queue_id = id;

    -- create seqs
    execute 'CREATE SEQUENCE ' || tick_seq;
    execute 'CREATE SEQUENCE ' || ev_seq;

    -- create data tables
    execute 'CREATE TABLE ' || tblpfx || ' () '
            || ' INHERITS (pgq.event_template)';
    for i in 0 .. (n_tables - 1) loop
        tblname := tblpfx || '_' || i;
        idxname := idxpfx || '_' || i;
        execute 'CREATE TABLE ' || tblname || ' () '
                || ' INHERITS (' || tblpfx || ')';
        execute 'ALTER TABLE ' || tblname || ' ALTER COLUMN ev_id '
                || ' SET DEFAULT nextval(' || quote_literal(ev_seq) || ')';
        execute 'create index ' || idxname || '_txid_idx on '
                || tblname || ' (ev_txid)';
    end loop;

    perform pgq.grant_perms(i_queue_name);

    perform pgq.ticker(i_queue_name);

    return 1;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.create_queue(i_queue_name text) OWNER TO skytools;

--
-- Name: current_event_table(text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION current_event_table(x_queue_name text) RETURNS text
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.current_event_table(1)
--
--      Return active event table for particular queue.
--      Event can be added to it without going via functions,
--      e.g. by COPY.
--
-- Note:
--      The result is valid only during current transaction.
--
-- Permissions:
--      Actual insertion requires superuser access.
--
-- Parameters:
--      x_queue_name    - Queue name.
-- ----------------------------------------------------------------------
declare
    res text;
begin
    select queue_data_pfx || '_' || queue_cur_table into res
        from pgq.queue where queue_name = x_queue_name;
    if not found then
        raise exception 'Event queue not found';
    end if;
    return res;
end;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION pgq.current_event_table(x_queue_name text) OWNER TO skytools;

--
-- Name: drop_queue(text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION drop_queue(x_queue_name text) RETURNS integer
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.drop_queue(1)
--
--     Drop queue and all associated tables.
--     No consumers must be listening on the queue.
--
-- ----------------------------------------------------------------------
declare
    tblname  text;
    q record;
    num integer;
begin
    -- check ares
    if x_queue_name is null then
        raise exception 'Invalid NULL value';
    end if;

    -- check if exists
    select * into q from pgq.queue
        where queue_name = x_queue_name;
    if not found then
        raise exception 'No such event queue';
    end if;

    -- check if no consumers
    select count(*) into num from pgq.subscription
        where sub_queue = q.queue_id;
    if num > 0 then
        raise exception 'cannot drop queue, consumers still attached';
    end if;

    -- drop data tables
    for i in 0 .. (q.queue_ntables - 1) loop
        tblname := q.queue_data_pfx || '_' || i;
        execute 'DROP TABLE ' || tblname;
    end loop;
    execute 'DROP TABLE ' || q.queue_data_pfx;

    -- delete ticks
    delete from pgq.tick where tick_queue = q.queue_id;

    -- drop seqs
    -- FIXME: any checks needed here?
    execute 'DROP SEQUENCE ' || q.queue_tick_seq;
    execute 'DROP SEQUENCE ' || q.queue_event_seq;

    -- delete event
    delete from pgq.queue
        where queue_name = x_queue_name;

    return 1;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.drop_queue(x_queue_name text) OWNER TO skytools;

--
-- Name: event_failed(bigint, bigint, text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION event_failed(x_batch_id bigint, x_event_id bigint, x_reason text) RETURNS integer
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.event_failed(3)
--
--      Copies the event to failed queue so it can be looked at later.
--
-- Parameters:
--      x_batch_id      - ID of active batch.
--      x_event_id      - Event id
--      x_reason        - Text to associate with event.
--
-- Returns:
--     0 if event was already in queue, 1 otherwise.
-- ----------------------------------------------------------------------
begin
    insert into pgq.failed_queue (ev_failed_reason, ev_failed_time,
        ev_id, ev_time, ev_txid, ev_owner, ev_retry, ev_type, ev_data,
        ev_extra1, ev_extra2, ev_extra3, ev_extra4)
    select x_reason, now(),
           ev_id, ev_time, NULL, sub_id, coalesce(ev_retry, 0),
           ev_type, ev_data, ev_extra1, ev_extra2, ev_extra3, ev_extra4
      from pgq.get_batch_events(x_batch_id),
           pgq.subscription
     where sub_batch = x_batch_id
       and ev_id = x_event_id;
    if not found then
        raise exception 'event not found';
    end if;
    return 1;

-- dont worry if the event is already in queue
exception
    when unique_violation then
        return 0;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.event_failed(x_batch_id bigint, x_event_id bigint, x_reason text) OWNER TO skytools;

--
-- Name: event_retry(bigint, bigint, timestamp with time zone); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION event_retry(x_batch_id bigint, x_event_id bigint, x_retry_time timestamp with time zone) RETURNS integer
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.event_retry(3)
--
--     Put the event into retry queue, to be processed again later.
--
-- Parameters:
--      x_batch_id      - ID of active batch.
--      x_event_id      - event id
--      x_retry_time    - Time when the event should be put back into queue
--
-- Returns:
--     nothing
-- ----------------------------------------------------------------------
begin
    insert into pgq.retry_queue (ev_retry_after,
        ev_id, ev_time, ev_txid, ev_owner, ev_retry, ev_type, ev_data,
        ev_extra1, ev_extra2, ev_extra3, ev_extra4)
    select x_retry_time,
           ev_id, ev_time, NULL, sub_id, coalesce(ev_retry, 0) + 1,
           ev_type, ev_data, ev_extra1, ev_extra2, ev_extra3, ev_extra4
      from pgq.get_batch_events(x_batch_id),
           pgq.subscription
     where sub_batch = x_batch_id
       and ev_id = x_event_id;
    if not found then
        raise exception 'event not found';
    end if;
    return 1;

-- dont worry if the event is already in queue
exception
    when unique_violation then
        return 0;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.event_retry(x_batch_id bigint, x_event_id bigint, x_retry_time timestamp with time zone) OWNER TO skytools;

--
-- Name: event_retry(bigint, bigint, integer); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION event_retry(x_batch_id bigint, x_event_id bigint, x_retry_seconds integer) RETURNS integer
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.event_retry(3)
--
--     Put the event into retry queue, to be processed later again.
--
-- Parameters:
--      x_batch_id      - ID of active batch.
--      x_event_id      - event id
--      x_retry_seconds - Time when the event should be put back into queue
--
-- Returns:
--     nothing
-- ----------------------------------------------------------------------
declare
    new_retry  timestamptz;
begin
    new_retry := current_timestamp + ((x_retry_seconds || ' seconds')::interval);
    return pgq.event_retry(x_batch_id, x_event_id, new_retry);
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.event_retry(x_batch_id bigint, x_event_id bigint, x_retry_seconds integer) OWNER TO skytools;

--
-- Name: event_retry_raw(text, text, timestamp with time zone, bigint, timestamp with time zone, integer, text, text, text, text, text, text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION event_retry_raw(x_queue text, x_consumer text, x_retry_after timestamp with time zone, x_ev_id bigint, x_ev_time timestamp with time zone, x_ev_retry integer, x_ev_type text, x_ev_data text, x_ev_extra1 text, x_ev_extra2 text, x_ev_extra3 text, x_ev_extra4 text) RETURNS bigint
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.event_retry_raw(12)
--
--      Allows full control over what goes to retry queue.
--
-- Parameters:
--      x_queue         - name of the queue
--      x_consumer      - name of the consumer
--      x_retry_after   - when the event should be processed again
--      x_ev_id         - event id
--      x_ev_time       - creation time
--      x_ev_retry      - retry count
--      x_ev_type       - user data
--      x_ev_data       - user data
--      x_ev_extra1     - user data
--      x_ev_extra2     - user data
--      x_ev_extra3     - user data
--      x_ev_extra4     - user data
--
-- Returns:
--      Event ID.
-- ----------------------------------------------------------------------
declare
    q record;
    id bigint;
begin
    select sub_id, queue_event_seq into q
      from pgq.consumer, pgq.queue, pgq.subscription
     where queue_name = x_queue
       and co_name = x_consumer
       and sub_consumer = co_id
       and sub_queue = queue_id;
    if not found then
        raise exception 'consumer not registered';
    end if;

    id := x_ev_id;
    if id is null then
        id := nextval(q.queue_event_seq);
    end if;

    insert into pgq.retry_queue (ev_retry_after,
            ev_id, ev_time, ev_owner, ev_retry,
            ev_type, ev_data, ev_extra1, ev_extra2, ev_extra3, ev_extra4)
    values (x_retry_after, x_ev_id, x_ev_time, q.sub_id, x_ev_retry,
            x_ev_type, x_ev_data, x_ev_extra1, x_ev_extra2,
            x_ev_extra3, x_ev_extra4);

    return id;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.event_retry_raw(x_queue text, x_consumer text, x_retry_after timestamp with time zone, x_ev_id bigint, x_ev_time timestamp with time zone, x_ev_retry integer, x_ev_type text, x_ev_data text, x_ev_extra1 text, x_ev_extra2 text, x_ev_extra3 text, x_ev_extra4 text) OWNER TO skytools;

--
-- Name: failed_event_count(text, text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION failed_event_count(x_queue_name text, x_consumer_name text) RETURNS integer
    AS $$ 
-- ----------------------------------------------------------------------
-- Function: pgq.failed_event_count(2)
--
--      Get size of failed event queue.
--
-- Parameters:
--      x_queue_name        - Queue name
--      x_consumer_name     - Consumer name
--
-- Returns:
--      Number of failed events in failed event queue.
-- ----------------------------------------------------------------------
declare 
    ret integer;
begin 
    select count(1) into ret
      from pgq.failed_queue, pgq.consumer, pgq.queue, pgq.subscription
     where queue_name = x_queue_name
       and co_name = x_consumer_name
       and sub_queue = queue_id
       and sub_consumer = co_id
       and ev_owner = sub_id;
    return ret;
end; 
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.failed_event_count(x_queue_name text, x_consumer_name text) OWNER TO skytools;

--
-- Name: failed_event_delete(text, text, bigint); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION failed_event_delete(x_queue_name text, x_consumer_name text, x_event_id bigint) RETURNS integer
    AS $$ 
-- ----------------------------------------------------------------------
-- Function: pgq.failed_event_delete(3)
--
--      Delete specific event from failed event queue.
--
-- Parameters:
--      x_queue_name        - Queue name
--      x_consumer_name     - Consumer name
--      x_event_id          - Event ID
--
-- Returns:
--      nothing
-- ----------------------------------------------------------------------
declare 
    x_sub_id integer;
begin 
    select sub_id into x_sub_id
      from pgq.subscription, pgq.consumer, pgq.queue
     where queue_name = x_queue_name
       and co_name = x_consumer_name
       and sub_consumer = co_id
       and sub_queue = queue_id;
    if not found then
        raise exception 'no such queue/consumer';
    end if;

    delete from pgq.failed_queue
     where ev_owner = x_sub_id
       and ev_id = x_event_id;
    if not found then
        raise exception 'event not found';
    end if;

    return 1;
end; 
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.failed_event_delete(x_queue_name text, x_consumer_name text, x_event_id bigint) OWNER TO skytools;

--
-- Name: failed_queue; Type: TABLE; Schema: pgq; Owner: skytools; Tablespace: 
--

CREATE TABLE failed_queue (
    ev_failed_reason text,
    ev_failed_time timestamp with time zone NOT NULL,
    ev_id bigint NOT NULL,
    ev_time timestamp with time zone NOT NULL,
    ev_txid bigint,
    ev_owner integer NOT NULL,
    ev_retry integer,
    ev_type text,
    ev_data text,
    ev_extra1 text,
    ev_extra2 text,
    ev_extra3 text,
    ev_extra4 text
);


ALTER TABLE pgq.failed_queue OWNER TO skytools;

--
-- Name: failed_event_list(text, text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION failed_event_list(x_queue_name text, x_consumer_name text) RETURNS SETOF failed_queue
    AS $$ 
-- ----------------------------------------------------------------------
-- Function: pgq.failed_event_list(2)
--
--      Get list of all failed events for one consumer.
--
-- Parameters:
--      x_queue_name        - Queue name
--      x_consumer_name     - Consumer name
--
-- Returns:
--      List of failed events.
-- ----------------------------------------------------------------------
declare 
    rec pgq.failed_queue%rowtype; 
begin 
    for rec in
        select fq.*
          from pgq.failed_queue fq, pgq.consumer,
               pgq.queue, pgq.subscription
         where queue_name = x_queue_name
           and co_name = x_consumer_name
           and sub_consumer = co_id
           and sub_queue = queue_id
           and ev_owner = sub_id
        order by ev_id
    loop
        return next rec; 
    end loop; 
    return;
end; 
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.failed_event_list(x_queue_name text, x_consumer_name text) OWNER TO skytools;

--
-- Name: failed_event_list(text, text, integer, integer); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION failed_event_list(x_queue_name text, x_consumer_name text, x_count integer, x_offset integer) RETURNS SETOF failed_queue
    AS $$ 
-- ----------------------------------------------------------------------
-- Function: pgq.failed_event_list(4)
--
--      Get list of failed events, from offset and specific count.
--
-- Parameters:
--      x_queue_name        - Queue name
--      x_consumer_name     - Consumer name
--      x_count             - Max amount of events to fetch
--      x_offset            - From this offset
--
-- Returns:
--      List of failed events.
-- ----------------------------------------------------------------------
declare 
    rec pgq.failed_queue%rowtype; 
begin 
    for rec in
        select fq.*
          from pgq.failed_queue fq, pgq.consumer,
               pgq.queue, pgq.subscription
         where queue_name = x_queue_name
           and co_name = x_consumer_name
           and sub_consumer = co_id
           and sub_queue = queue_id
           and ev_owner = sub_id
        order by ev_id
        limit x_count
        offset x_offset
    loop
        return next rec; 
    end loop; 
    return;
end; 
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.failed_event_list(x_queue_name text, x_consumer_name text, x_count integer, x_offset integer) OWNER TO skytools;

--
-- Name: failed_event_retry(text, text, bigint); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION failed_event_retry(x_queue_name text, x_consumer_name text, x_event_id bigint) RETURNS bigint
    AS $$ 
-- ----------------------------------------------------------------------
-- Function: pgq.failed_event_retry(3)
--
--      Insert specific event from failed queue to main queue.
--
-- Parameters:
--      x_queue_name        - Queue name
--      x_consumer_name     - Consumer name
--      x_event_id          - Event ID
--
-- Returns:
--      nothing
-- ----------------------------------------------------------------------
declare 
    ret         bigint;
    x_sub_id    integer;
begin 
    select sub_id into x_sub_id
      from pgq.subscription, pgq.consumer, pgq.queue
     where queue_name = x_queue_name
       and co_name = x_consumer_name
       and sub_consumer = co_id
       and sub_queue = queue_id;
    if not found then
        raise exception 'no such queue/consumer';
    end if;

    select pgq.insert_event_raw(x_queue_name, ev_id, ev_time,
            ev_owner, ev_retry, ev_type, ev_data,
            ev_extra1, ev_extra2, ev_extra3, ev_extra4)
      into ret
      from pgq.failed_queue, pgq.consumer, pgq.queue
     where ev_owner = x_sub_id
       and ev_id = x_event_id;
    if not found then
        raise exception 'event not found';
    end if;

    perform pgq.failed_event_delete(x_queue_name, x_consumer_name, x_event_id);

    return ret;
end; 
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.failed_event_retry(x_queue_name text, x_consumer_name text, x_event_id bigint) OWNER TO skytools;

--
-- Name: finish_batch(bigint); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION finish_batch(x_batch_id bigint) RETURNS integer
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.finish_batch(1)
--
--      Closes a batch.  No more operations can be done with events
--      of this batch.
--
-- Parameters:
--      x_batch_id      - id of batch.
--
-- Returns:
--      If batch 1 if batch was found, 0 otherwise.
-- ----------------------------------------------------------------------
begin
    update pgq.subscription
        set sub_active = now(),
            sub_last_tick = sub_next_tick,
            sub_next_tick = null,
            sub_batch = null
        where sub_batch = x_batch_id;
    if not found then
        raise warning 'finish_batch: batch % not found', x_batch_id;
        return 0;
    end if;

    return 1;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.finish_batch(x_batch_id bigint) OWNER TO skytools;

--
-- Name: force_tick(text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION force_tick(i_queue_name text) RETURNS bigint
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.force_tick(2)
--
--      Simulate lots of events happening to force ticker to tick.
--
--      Should be called in loop, with some delay until last tick
--      changes or too much time is passed.
--
--      Such function is needed because paraller calls of pgq.ticker() are
--      dangerous, and cannot be protected with locks as snapshot
--      is taken before locking.
--
-- Parameters:
--      i_queue_name     - Name of the queue
--
-- Returns:
--      Currently last tick id.
-- ----------------------------------------------------------------------
declare
    q  record;
    t  record;
begin
    -- bump seq and get queue id
    select queue_id,
           setval(queue_event_seq, nextval(queue_event_seq)
                                   + queue_ticker_max_count * 2) as tmp
      into q from pgq.queue
     where queue_name = i_queue_name
       and not queue_external_ticker;
    if not found then
        raise exception 'queue not found or ticks not allowed';
    end if;

    -- return last tick id
    select tick_id into t from pgq.tick
     where tick_queue = q.queue_id
     order by tick_queue desc, tick_id desc limit 1;

    return t.tick_id;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.force_tick(i_queue_name text) OWNER TO skytools;

--
-- Name: get_batch_events(bigint); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION get_batch_events(x_batch_id bigint) RETURNS SETOF ret_batch_event
    AS $$ 
-- ----------------------------------------------------------------------
-- Function: pgq.get_batch_events(1)
--
--      Get all events in batch.
--
-- Parameters:
--      x_batch_id      - ID of active batch.
--
-- Returns:
--      List of events.
-- ----------------------------------------------------------------------
declare 
    rec pgq.ret_batch_event%rowtype; 
    sql text; 
begin 
    sql := pgq.batch_event_sql(x_batch_id); 
    for rec in execute sql loop
        return next rec; 
    end loop; 
    return;
end; 
$$
    LANGUAGE plpgsql;


ALTER FUNCTION pgq.get_batch_events(x_batch_id bigint) OWNER TO skytools;

--
-- Name: get_batch_info(bigint); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION get_batch_info(x_batch_id bigint) RETURNS ret_batch_info
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.get_batch_info(1)
--
--      Returns detailed info about a batch.
--
-- Parameters:
--      x_batch_id      - id of a active batch.
--
-- Returns:
--      Info
-- ----------------------------------------------------------------------
declare
    ret  pgq.ret_batch_info%rowtype;
begin
    select queue_name, co_name,
           prev.tick_time as batch_start,
           cur.tick_time as batch_end,
           sub_last_tick, sub_next_tick,
           current_timestamp - cur.tick_time as lag
        into ret
        from pgq.subscription, pgq.tick cur, pgq.tick prev,
             pgq.queue, pgq.consumer
        where sub_batch = x_batch_id
          and prev.tick_id = sub_last_tick
          and prev.tick_queue = sub_queue
          and cur.tick_id = sub_next_tick
          and cur.tick_queue = sub_queue
          and queue_id = sub_queue
          and co_id = sub_consumer;
    return ret;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.get_batch_info(x_batch_id bigint) OWNER TO skytools;

--
-- Name: get_consumer_info(); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION get_consumer_info() RETURNS SETOF ret_consumer_info
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.get_consumer_info(0)
--
--      Returns info about all consumers on all queues.
--
-- Returns:
--      See pgq.get_consumer_info(2)
-- ----------------------------------------------------------------------
declare
    ret  pgq.ret_consumer_info%rowtype;
    i    record;
begin
    for i in select queue_name from pgq.queue order by 1
    loop
        for ret in
            select * from pgq.get_consumer_info(i.queue_name)
        loop
            return next ret;
        end loop;
    end loop;
    return;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.get_consumer_info() OWNER TO skytools;

--
-- Name: get_consumer_info(text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION get_consumer_info(x_queue_name text) RETURNS SETOF ret_consumer_info
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.get_consumer_info(1)
--
--      Returns info about consumers on one particular queue.
--
-- Parameters:
--      x_queue_name    - Queue name
--
-- Returns:
--      See pgq.get_consumer_info(2)
-- ----------------------------------------------------------------------
declare
    ret  pgq.ret_consumer_info%rowtype;
    tmp record;
begin
    for tmp in
        select queue_name, co_name
          from pgq.queue, pgq.consumer, pgq.subscription
         where queue_id = sub_queue
           and co_id = sub_consumer
           and queue_name = x_queue_name
         order by 1, 2
    loop
        for ret in
            select * from pgq.get_consumer_info(tmp.queue_name, tmp.co_name)
        loop
            return next ret;
        end loop;
    end loop;
    return;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.get_consumer_info(x_queue_name text) OWNER TO skytools;

--
-- Name: get_consumer_info(text, text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION get_consumer_info(x_queue_name text, x_consumer_name text) RETURNS SETOF ret_consumer_info
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.get_consumer_info(2)
--
--      Get info about particular consumer on particular queue.
--
-- Parameters:
--      x_queue_name        - name of a queue.
--      x_consumer_name     - name of a consumer
--
-- Returns:
--      queue_name          - Queue name
--      consumer_name       - Consumer name
--      lag                 - How old are events the consumer is processing
--      last_seen           - When the consumer seen by pgq
--      last_tick           - Tick ID of last processed tick
--      current_batch       - Current batch ID, if one is active or NULL
--      next_tick           - If batch is active, then its final tick.
-- ----------------------------------------------------------------------
declare
    ret  pgq.ret_consumer_info%rowtype;
begin
    for ret in 
        select queue_name, co_name,
               current_timestamp - tick_time as lag,
               current_timestamp - sub_active as last_seen,
               sub_last_tick as last_tick,
               sub_batch as current_batch,
               sub_next_tick as next_tick
          from pgq.subscription, pgq.tick, pgq.queue, pgq.consumer
         where tick_id = sub_last_tick
           and queue_id = sub_queue
           and tick_queue = sub_queue
           and co_id = sub_consumer
           and queue_name = x_queue_name
           and co_name = x_consumer_name
         order by 1,2
    loop
        return next ret;
    end loop;
    return;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.get_consumer_info(x_queue_name text, x_consumer_name text) OWNER TO skytools;

--
-- Name: get_queue_info(); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION get_queue_info() RETURNS SETOF ret_queue_info
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.get_queue_info(0)
--
--      Get info about all queues.
--
-- Returns:
--      List of pgq.ret_queue_info records.
-- ----------------------------------------------------------------------
declare
    q     record;
    ret   pgq.ret_queue_info%rowtype;
begin
    for q in
        select queue_name from pgq.queue order by 1
    loop
        select * into ret from pgq.get_queue_info(q.queue_name);
        return next ret;
    end loop;
    return;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.get_queue_info() OWNER TO skytools;

--
-- Name: get_queue_info(text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION get_queue_info(qname text) RETURNS ret_queue_info
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.get_queue_info(1)
--
--      Get info about particular queue.
--
-- Returns:
--      One pgq.ret_queue_info record.
-- ----------------------------------------------------------------------
declare
    ret   pgq.ret_queue_info%rowtype;
begin
    select queue_name, queue_ntables, queue_cur_table,
           queue_rotation_period, queue_switch_time,
           queue_external_ticker,
           queue_ticker_max_count, queue_ticker_max_lag,
           queue_ticker_idle_period,
           (select current_timestamp - tick_time
              from pgq.tick where tick_queue = queue_id
             order by tick_queue desc, tick_id desc limit 1
            ) as ticker_lag
      into ret from pgq.queue where queue_name = qname;
    return ret;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.get_queue_info(qname text) OWNER TO skytools;

--
-- Name: grant_perms(text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION grant_perms(x_queue_name text) RETURNS integer
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.grant_perms(1)
--
--      Make event tables readable by public.
--
-- Parameters:
--      x_queue_name        - Name of the queue.
--
-- Returns:
--      nothing
-- ----------------------------------------------------------------------
declare
    q           record;
    i           integer;
    tbl_perms   text;
    seq_perms   text;
begin
    select * from pgq.queue into q
        where queue_name = x_queue_name;
    if not found then
        raise exception 'Queue not found';
    end if;

    if true then
        -- safe, all access must go via functions
        seq_perms := 'select';
        tbl_perms := 'select';
    else
        -- allow ordinery users to directly insert
        -- to event tables.  dangerous.
        seq_perms := 'select, update';
        tbl_perms := 'select, insert';
    end if;

    -- tick seq, normal users don't need to modify it
    execute 'grant ' || seq_perms
        || ' on ' || q.queue_tick_seq || ' to public';

    -- event seq
    execute 'grant ' || seq_perms
        || ' on ' || q.queue_event_seq || ' to public';
    
    -- parent table for events
    execute 'grant select on ' || q.queue_data_pfx || ' to public';

    -- real event tables
    for i in 0 .. q.queue_ntables - 1 loop
        execute 'grant ' || tbl_perms
            || ' on ' || q.queue_data_pfx || '_' || i
            || ' to public';
    end loop;

    return 1;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.grant_perms(x_queue_name text) OWNER TO skytools;

--
-- Name: insert_event(text, text, text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION insert_event(queue_name text, ev_type text, ev_data text) RETURNS bigint
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.insert_event(3)
--
--      Insert a event into queue.
--
-- Parameters:
--      queue_name      - Name of the queue
--      ev_type         - User-specified type for the event
--      ev_data         - User data for the event
--
-- Returns:
--      Event ID
-- ----------------------------------------------------------------------
begin
    return pgq.insert_event(queue_name, ev_type, ev_data, null, null, null, null);
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.insert_event(queue_name text, ev_type text, ev_data text) OWNER TO skytools;

--
-- Name: insert_event(text, text, text, text, text, text, text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION insert_event(queue_name text, ev_type text, ev_data text, ev_extra1 text, ev_extra2 text, ev_extra3 text, ev_extra4 text) RETURNS bigint
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.insert_event(7)
--
--      Insert a event into queue with all the extra fields.
--
-- Parameters:
--      queue_name      - Name of the queue
--      ev_type         - User-specified type for the event
--      ev_data         - User data for the event
--      ev_extra1       - Extra data field for the event
--      ev_extra2       - Extra data field for the event
--      ev_extra3       - Extra data field for the event
--      ev_extra4       - Extra data field for the event
--
-- Returns:
--      Event ID
-- ----------------------------------------------------------------------
begin
    return pgq.insert_event_raw(queue_name, null, now(), null, null,
            ev_type, ev_data, ev_extra1, ev_extra2, ev_extra3, ev_extra4);
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.insert_event(queue_name text, ev_type text, ev_data text, ev_extra1 text, ev_extra2 text, ev_extra3 text, ev_extra4 text) OWNER TO skytools;

--
-- Name: insert_event_raw(text, bigint, timestamp with time zone, integer, integer, text, text, text, text, text, text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION insert_event_raw(queue_name text, ev_id bigint, ev_time timestamp with time zone, ev_owner integer, ev_retry integer, ev_type text, ev_data text, ev_extra1 text, ev_extra2 text, ev_extra3 text, ev_extra4 text) RETURNS bigint
    AS '$libdir/pgq_lowlevel', 'pgq_insert_event_raw'
    LANGUAGE c;


ALTER FUNCTION pgq.insert_event_raw(queue_name text, ev_id bigint, ev_time timestamp with time zone, ev_owner integer, ev_retry integer, ev_type text, ev_data text, ev_extra1 text, ev_extra2 text, ev_extra3 text, ev_extra4 text) OWNER TO skytools;

--
-- Name: logtriga(); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION logtriga() RETURNS trigger
    AS '$libdir/pgq_triggers', 'pgq_logtriga'
    LANGUAGE c;


ALTER FUNCTION pgq.logtriga() OWNER TO skytools;

--
-- Name: logutriga(); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION logutriga() RETURNS trigger
    AS '$libdir/pgq_triggers', 'pgq_logutriga'
    LANGUAGE c;


ALTER FUNCTION pgq.logutriga() OWNER TO skytools;

--
-- Name: maint_retry_events(); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION maint_retry_events() RETURNS integer
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.maint_retry_events(0)
--
--      Moves retry events back to main queue.
--
--      It moves small amount at a time.  It should be called
--      until it returns 0
--
-- Returns:
--      Number of events processed.
-- ----------------------------------------------------------------------
declare
    cnt    integer;
    rec    record;
begin
    cnt := 0;
    for rec in
        select queue_name,
               ev_id, ev_time, ev_owner, ev_retry, ev_type, ev_data,
               ev_extra1, ev_extra2, ev_extra3, ev_extra4
          from pgq.retry_queue, pgq.queue, pgq.subscription
         where ev_retry_after <= current_timestamp
           and sub_id = ev_owner
           and queue_id = sub_queue
         order by ev_retry_after
         limit 10
    loop
        cnt := cnt + 1;
        perform pgq.insert_event_raw(rec.queue_name,
                    rec.ev_id, rec.ev_time, rec.ev_owner, rec.ev_retry,
                    rec.ev_type, rec.ev_data, rec.ev_extra1, rec.ev_extra2,
                    rec.ev_extra3, rec.ev_extra4);
        delete from pgq.retry_queue
         where ev_owner = rec.ev_owner
           and ev_id = rec.ev_id;
    end loop;
    return cnt;
end;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION pgq.maint_retry_events() OWNER TO skytools;

--
-- Name: maint_rotate_tables_step1(text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION maint_rotate_tables_step1(i_queue_name text) RETURNS integer
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.maint_rotate_tables_step1(1)
--
--      Rotate tables for one queue.
--
-- Parameters:
--      i_queue_name        - Name of the queue
--
-- Returns:
--      1 if rotation happened, otherwise 0.
-- ----------------------------------------------------------------------
declare
    badcnt          integer;
    cf              record;
    nr              integer;
    tbl             text;
    lowest_tick_id  int8;
    lowest_xmin     int8;
begin
    -- check if needed and load record
    select * from pgq.queue into cf
        where queue_name = i_queue_name
          and queue_rotation_period is not null
          and queue_switch_step2 is not null
          and queue_switch_time + queue_rotation_period < current_timestamp
        for update;
    if not found then
        return 0;
    end if;

    -- find lowest tick for that queue
    select min(sub_last_tick) into lowest_tick_id
      from pgq.subscription
     where sub_queue = cf.queue_id;

    -- if some consumer exists
    if lowest_tick_id is not null then
        -- is the slowest one still on previous table?
        select txid_snapshot_xmin(tick_snapshot) into lowest_xmin
          from pgq.tick
         where tick_queue = cf.queue_id
           and tick_id = lowest_tick_id;
        if lowest_xmin <= cf.queue_switch_step2 then
            return 0; -- skip rotation then
        end if;
    end if;

    -- nobody on previous table, we can rotate
    
    -- calc next table number and name
    nr := cf.queue_cur_table + 1;
    if nr = cf.queue_ntables then
        nr := 0;
    end if;
    tbl := cf.queue_data_pfx || '_' || nr;

    -- there may be long lock on the table from pg_dump,
    -- detect it and skip rotate then
    begin
        execute 'lock table ' || tbl || ' nowait';
        execute 'truncate ' || tbl;
    exception
        when lock_not_available then
            -- cannot truncate, skipping rotate
            return 0;
    end;

    -- remember the moment
    update pgq.queue
        set queue_cur_table = nr,
            queue_switch_time = current_timestamp,
            queue_switch_step1 = txid_current(),
            queue_switch_step2 = NULL
        where queue_id = cf.queue_id;

    -- Clean ticks by using step2 txid from previous rotation.
    -- That should keep all ticks for all batches that are completely
    -- in old table.  This keeps them for longer than needed, but:
    -- 1. we want the pgq.tick table to be big, to avoid Postgres
    --    accitentally switching to seqscans on that.
    -- 2. that way we guarantee to consumers that they an be moved
    --    back on the queue at least for one rotation_period.
    --    (may help in disaster recovery)
    delete from pgq.tick
        where tick_queue = cf.queue_id
          and txid_snapshot_xmin(tick_snapshot) < cf.queue_switch_step2;

    return 1;
end;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION pgq.maint_rotate_tables_step1(i_queue_name text) OWNER TO skytools;

--
-- Name: maint_rotate_tables_step2(); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION maint_rotate_tables_step2() RETURNS integer
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.maint_rotate_tables_step2(0)
--
--      Stores the txid when the rotation was visible.  It should be
--      called in separate transaction than pgq.maint_rotate_tables_step1()
-- ----------------------------------------------------------------------
begin
    update pgq.queue
       set queue_switch_step2 = txid_current()
     where queue_switch_step2 is null;
    return 1;
end;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION pgq.maint_rotate_tables_step2() OWNER TO skytools;

--
-- Name: maint_tables_to_vacuum(); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION maint_tables_to_vacuum() RETURNS SETOF text
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.maint_tables_to_vacuum(0)
--
--      Returns list of tablenames that need frequent vacuuming.
--
--      The goal is to avoid hardcoding them into maintenance process.
--
-- Returns:
--      List of table names.
-- ----------------------------------------------------------------------
declare
    row record;
begin
    return next 'pgq.subscription';
    return next 'pgq.consumer';
    return next 'pgq.queue';
    return next 'pgq.tick';
    return next 'pgq.retry_queue';

    -- include also txid, pgq_ext and londiste tables if they exist
    for row in
        select n.nspname as scm, t.relname as tbl
          from pg_class t, pg_namespace n
         where n.oid = t.relnamespace
           and n.nspname = 'txid' and t.relname = 'epoch'
        union all
        select n.nspname as scm, t.relname as tbl
          from pg_class t, pg_namespace n
         where n.oid = t.relnamespace
           and n.nspname = 'londiste' and t.relname = 'completed'
        union all
        select n.nspname as scm, t.relname as tbl
          from pg_class t, pg_namespace n
         where n.oid = t.relnamespace
           and n.nspname = 'pgq_ext'
           and t.relname in ('completed_tick', 'completed_batch', 'completed_event', 'partial_batch')
    loop
        return next row.scm || '.' || row.tbl;
    end loop;

    return;
end;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION pgq.maint_tables_to_vacuum() OWNER TO skytools;

--
-- Name: next_batch(text, text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION next_batch(x_queue_name text, x_consumer_name text) RETURNS bigint
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.next_batch(2)
--
--      Makes next block of events active.
--
--      If it returns NULL, there is no events available in queue.
--      Consumer should sleep a bith then.
--
-- Parameters:
--      x_queue_name        - Name of the queue
--      x_consumer_name     - Name of the consumer
--
-- Returns:
--      Batch ID or NULL if there are no more events available.
-- ----------------------------------------------------------------------
declare
    next_tick       bigint;
    batch_id        bigint;
    errmsg          text;
    sub             record;
begin
    select sub_queue, sub_consumer, sub_id, sub_last_tick, sub_batch into sub
        from pgq.queue q, pgq.consumer c, pgq.subscription s
        where q.queue_name = x_queue_name
          and c.co_name = x_consumer_name
          and s.sub_queue = q.queue_id
          and s.sub_consumer = c.co_id;
    if not found then
        errmsg := 'Not subscriber to queue: '
            || coalesce(x_queue_name, 'NULL')
            || '/'
            || coalesce(x_consumer_name, 'NULL');
        raise exception '%', errmsg;
    end if;

    -- has already active batch
    if sub.sub_batch is not null then
        return sub.sub_batch;
    end if;

    -- find next tick
    select tick_id into next_tick
        from pgq.tick
        where tick_id > sub.sub_last_tick
          and tick_queue = sub.sub_queue
        order by tick_queue asc, tick_id asc
        limit 1;
    if not found then
        -- nothing to do
        return null;
    end if;

    -- get next batch
    batch_id := nextval('pgq.batch_id_seq');
    update pgq.subscription
        set sub_batch = batch_id,
            sub_next_tick = next_tick,
            sub_active = now()
        where sub_queue = sub.sub_queue
          and sub_consumer = sub.sub_consumer;
    return batch_id;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.next_batch(x_queue_name text, x_consumer_name text) OWNER TO skytools;

--
-- Name: register_consumer(text, text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION register_consumer(x_queue_name text, x_consumer_id text) RETURNS integer
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.register_consumer(2)
--
--      Subscribe consumer on a queue.
--
--      From this moment forward, consumer will see all events in the queue.
--
-- Parameters:
--      x_queue_name        - Name of queue
--      x_consumer_name     - Name of consumer
--
-- Returns:
--      0  - if already registered
--      1  - if new registration
-- ----------------------------------------------------------------------
begin
    return pgq.register_consumer(x_queue_name, x_consumer_id, NULL);
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.register_consumer(x_queue_name text, x_consumer_id text) OWNER TO skytools;

--
-- Name: register_consumer(text, text, bigint); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION register_consumer(x_queue_name text, x_consumer_name text, x_tick_pos bigint) RETURNS integer
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.register_consumer(3)
--
--      Extended registration, allows to specify tick_id.
--
-- Note:
--      For usage in special situations.
--
-- Parameters:
--      x_queue_name        - Name of a queue
--      x_consumer_name     - Name of consumer
--      x_tick_pos          - Tick ID
--
-- Returns:
--      0/1 whether consumer has already registered.
-- ----------------------------------------------------------------------
declare
    tmp         text;
    last_tick   bigint;
    x_queue_id          integer;
    x_consumer_id integer;
    queue integer;
    sub record;
begin
    select queue_id into x_queue_id from pgq.queue
        where queue_name = x_queue_name;
    if not found then
        raise exception 'Event queue not created yet';
    end if;

    -- get consumer and create if new
    select co_id into x_consumer_id from pgq.consumer
        where co_name = x_consumer_name;
    if not found then
        insert into pgq.consumer (co_name) values (x_consumer_name);
        x_consumer_id := currval('pgq.consumer_co_id_seq');
    end if;

    -- if particular tick was requested, check if it exists
    if x_tick_pos is not null then
        perform 1 from pgq.tick
            where tick_queue = x_queue_id
              and tick_id = x_tick_pos;
        if not found then
            raise exception 'cannot reposition, tick not found: %', x_tick_pos;
        end if;
    end if;

    -- check if already registered
    select sub_last_tick, sub_batch into sub
        from pgq.subscription
        where sub_consumer = x_consumer_id
          and sub_queue  = x_queue_id;
    if found then
        if x_tick_pos is not null then
            if sub.sub_batch is not null then
                raise exception 'reposition while active not allowed';
            end if;
            -- update tick pos if requested
            update pgq.subscription
                set sub_last_tick = x_tick_pos
                where sub_consumer = x_consumer_id
                  and sub_queue = x_queue_id;
        end if;
        -- already registered
        return 0;
    end if;

    --  new registration
    if x_tick_pos is null then
        -- start from current tick
        select tick_id into last_tick from pgq.tick
            where tick_queue = x_queue_id
            order by tick_queue desc, tick_id desc
            limit 1;
        if not found then
            raise exception 'No ticks for this queue.  Please run ticker on database.';
        end if;
    else
        last_tick := x_tick_pos;
    end if;

    -- register
    insert into pgq.subscription (sub_queue, sub_consumer, sub_last_tick)
        values (x_queue_id, x_consumer_id, last_tick);
    return 1;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.register_consumer(x_queue_name text, x_consumer_name text, x_tick_pos bigint) OWNER TO skytools;

--
-- Name: sqltriga(); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION sqltriga() RETURNS trigger
    AS '$libdir/pgq_triggers', 'pgq_sqltriga'
    LANGUAGE c;


ALTER FUNCTION pgq.sqltriga() OWNER TO skytools;

--
-- Name: ticker(text, bigint); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION ticker(i_queue_name text, i_tick_id bigint) RETURNS bigint
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.ticker(2)
--
--     Insert a tick with a particular tick_id.
--
--     For external tickers.
--
-- Parameters:
--     i_queue_name     - Name of the queue
--     i_tick_id        - Id of new tick.
--
-- Returns:
--     Tick id.
-- ----------------------------------------------------------------------
begin
    insert into pgq.tick (tick_queue, tick_id)
    select queue_id, i_tick_id
        from pgq.queue
        where queue_name = i_queue_name
          and queue_external_ticker;
    if not found then
        raise exception 'queue not found';
    end if;
    return i_tick_id;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.ticker(i_queue_name text, i_tick_id bigint) OWNER TO skytools;

--
-- Name: ticker(text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION ticker(i_queue_name text) RETURNS bigint
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.ticker(1)
--
--     Insert a tick with a tick_id from sequence.
--
--     For pgqadm usage.
--
-- Parameters:
--     i_queue_name     - Name of the queue
--
-- Returns:
--     Tick id.
-- ----------------------------------------------------------------------
declare
    res bigint;
    ext boolean;
    seq text;
    q record;
begin
    select queue_id, queue_tick_seq, queue_external_ticker into q
        from pgq.queue where queue_name = i_queue_name;
    if not found then
        raise exception 'no such queue';
    end if;

    if q.queue_external_ticker then
        raise exception 'This queue has external tick source.';
    end if;

    insert into pgq.tick (tick_queue, tick_id)
        values (q.queue_id, nextval(q.queue_tick_seq));

    res = currval(q.queue_tick_seq);
    return res;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.ticker(i_queue_name text) OWNER TO skytools;

--
-- Name: ticker(); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION ticker() RETURNS bigint
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.ticker(0)
--
--     Creates ticks for all queues which dont have external ticker.
--
-- Returns:
--     Number of queues that were processed.
-- ----------------------------------------------------------------------
declare
    res bigint;
begin
    select count(pgq.ticker(queue_name)) into res 
        from pgq.queue where not queue_external_ticker;
    return res;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.ticker() OWNER TO skytools;

--
-- Name: unregister_consumer(text, text); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION unregister_consumer(x_queue_name text, x_consumer_name text) RETURNS integer
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.unregister_consumer(2)
--
--      Unsubscriber consumer from the queue.  Also consumer's failed
--      and retry events are deleted.
--
-- Parameters:
--      x_queue_name        - Name of the queue
--      x_consumer_name     - Name of the consumer
--
-- Returns:
--      nothing
-- ----------------------------------------------------------------------
declare
    x_sub_id integer;
begin
    select sub_id into x_sub_id
        from pgq.subscription, pgq.consumer, pgq.queue
        where sub_queue = queue_id
          and sub_consumer = co_id
          and queue_name = x_queue_name
          and co_name = x_consumer_name;
    if not found then
        raise exception 'consumer not registered on queue';
    end if;

    delete from pgq.retry_queue
        where ev_owner = x_sub_id;

    delete from pgq.failed_queue
        where ev_owner = x_sub_id;

    delete from pgq.subscription
        where sub_id = x_sub_id;

    return 1;
end;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION pgq.unregister_consumer(x_queue_name text, x_consumer_name text) OWNER TO skytools;

--
-- Name: version(); Type: FUNCTION; Schema: pgq; Owner: skytools
--

CREATE FUNCTION version() RETURNS text
    AS $$
-- ----------------------------------------------------------------------
-- Function: pgq.version(0)
--
--      Returns verison string for pgq.  ATM its SkyTools version
--      that is only bumped when PGQ database code changes.
-- ----------------------------------------------------------------------
begin
    return '2.1.8';
end;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION pgq.version() OWNER TO skytools;

SET search_path = replication, pg_catalog;

--
-- Name: provider__add_tables(text); Type: FUNCTION; Schema: replication; Owner: db_global_owner
--

CREATE FUNCTION provider__add_tables(p_set text) RETURNS void
    AS $_$
DECLARE
  v_set_id CONSTANT replication.sets.id%TYPE := replication.sets__get_id(  p_set  );
  v_set replication.sets%ROWTYPE; 
  v_full_name text;
BEGIN
  IF v_set_id IS NULL THEN
    RAISE EXCEPTION 'set % does not exist', p_set;
  END IF;

  v_set := replication.sets__get( v_set_id ); 

  -- Inclusive is the simple case, just a list of tables
  IF v_set.inclusive THEN
    FOR v_full_name IN
      SELECT full_name FROM replication.tables WHERE set_id = v_set_id
    LOOP
      BEGIN -- Catch errors from each individual call and continue
        EXECUTE $$SELECT londiste.provider_add_table('londiste.$$
          || v_set.set || $$', '$$ || v_full_name || $$')$$;
      EXCEPTION WHEN raise_exception THEN
          RAISE NOTICE 'sqlstate %, sqlerrm %', SQLSTATE, SQLERRM;
      END;
    END LOOP;

    RETURN; -- Note the return here
  END IF;

  -- Otherwise exclusive, add all except a list, and globally excluded stuff
  FOR v_full_name IN
    SELECT 
        schemaname || '.' || tablename AS full_name
    FROM 
        pg_tables
    WHERE 
        NOT EXISTS ( SELECT 1 FROM pg_user WHERE usename = schemaname )
        AND schemaname !~ ALL (
          SELECT schema_name FROM replication.schemas__exclude
        )
        AND schemaname || '.' || tablename NOT IN ( 
          SELECT full_name FROM replication.tables__exclude
        )
        AND schemaname || '.' || tablename NOT IN ( 
          SELECT full_name 
          FROM replication.tables rt
          JOIN replication.sets rs
            ON rt.set_id = rs.id
          WHERE rs.set = v_set.exclude_set
        )
        AND NOT EXISTS ( 
          SELECT 1 FROM londiste.provider_table 
            WHERE table_name = schemaname || '.' || tablename
        )
  LOOP
    BEGIN -- Catch errors from each individual call and continue
      EXECUTE $$SELECT londiste.provider_add_table('londiste.$$
        || v_set.set || $$', '$$ || v_full_name || $$')$$;
    EXCEPTION WHEN raise_exception THEN
        RAISE NOTICE 'sqlstate %, sqlerrm %', SQLSTATE, SQLERRM;
    END;
  END LOOP;

  RETURN;
END;
$_$
    LANGUAGE plpgsql;


ALTER FUNCTION replication.provider__add_tables(p_set text) OWNER TO db_global_owner;

--
-- Name: schemas__exclude; Type: TABLE; Schema: replication; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE schemas__exclude (
    id smallint NOT NULL,
    schema_name text NOT NULL,
    description text
);


ALTER TABLE replication.schemas__exclude OWNER TO db_global_owner;

--
-- Name: TABLE schemas__exclude; Type: COMMENT; Schema: replication; Owner: db_global_owner
--

COMMENT ON TABLE schemas__exclude IS 'Regex list of schema names to always exclude from replication on all databases and environments';


--
-- Name: schemas__exclude__get(smallint); Type: FUNCTION; Schema: replication; Owner: db_global_owner
--

CREATE FUNCTION schemas__exclude__get(p_id smallint) RETURNS schemas__exclude
    AS $_$
SELECT * FROM replication.schemas__exclude WHERE id = $1
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION replication.schemas__exclude__get(p_id smallint) OWNER TO db_global_owner;

--
-- Name: FUNCTION schemas__exclude__get(p_id smallint); Type: COMMENT; Schema: replication; Owner: db_global_owner
--

COMMENT ON FUNCTION schemas__exclude__get(p_id smallint) IS 'Returns row from replication.schemas__exclude based on id. This is generated code.';


--
-- Name: schemas__exclude__get_id(text); Type: FUNCTION; Schema: replication; Owner: db_global_owner
--

CREATE FUNCTION schemas__exclude__get_id(p_schema_name text) RETURNS smallint
    AS $_$
SELECT id FROM replication.schemas__exclude WHERE lower(schema_name) = lower($1)
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION replication.schemas__exclude__get_id(p_schema_name text) OWNER TO db_global_owner;

--
-- Name: FUNCTION schemas__exclude__get_id(p_schema_name text); Type: COMMENT; Schema: replication; Owner: db_global_owner
--

COMMENT ON FUNCTION schemas__exclude__get_id(p_schema_name text) IS 'Returns ID value for replication.schemas__exclude based on schema_name. This is generated code.';


--
-- Name: schemas__exclude__get_text(smallint); Type: FUNCTION; Schema: replication; Owner: db_global_owner
--

CREATE FUNCTION schemas__exclude__get_text(p_id smallint) RETURNS text
    AS $_$
SELECT schema_name FROM replication.schemas__exclude WHERE id = $1
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION replication.schemas__exclude__get_text(p_id smallint) OWNER TO db_global_owner;

--
-- Name: FUNCTION schemas__exclude__get_text(p_id smallint); Type: COMMENT; Schema: replication; Owner: db_global_owner
--

COMMENT ON FUNCTION schemas__exclude__get_text(p_id smallint) IS 'Returns schema_name value for replication.schemas__exclude based on id. This is generated code.';


--
-- Name: sets; Type: TABLE; Schema: replication; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE sets (
    id smallint NOT NULL,
    inclusive boolean NOT NULL,
    exclude_set text,
    set text NOT NULL,
    description text
);


ALTER TABLE replication.sets OWNER TO db_global_owner;

--
-- Name: TABLE sets; Type: COMMENT; Schema: replication; Owner: db_global_owner
--

COMMENT ON TABLE sets IS 'Lookup table for set. The data in this table is considered to be static, meaning that it should only be changed via a patch. This is generated code. This is a seed table.';


--
-- Name: sets__get(smallint); Type: FUNCTION; Schema: replication; Owner: db_global_owner
--

CREATE FUNCTION sets__get(p_id smallint) RETURNS sets
    AS $_$
SELECT * FROM replication.sets WHERE id = $1
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION replication.sets__get(p_id smallint) OWNER TO db_global_owner;

--
-- Name: FUNCTION sets__get(p_id smallint); Type: COMMENT; Schema: replication; Owner: db_global_owner
--

COMMENT ON FUNCTION sets__get(p_id smallint) IS 'Returns row from replication.sets based on id. This is generated code.';


--
-- Name: sets__get_id(text); Type: FUNCTION; Schema: replication; Owner: db_global_owner
--

CREATE FUNCTION sets__get_id(p_set text) RETURNS smallint
    AS $_$
SELECT id FROM replication.sets WHERE lower(set) = lower($1)
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION replication.sets__get_id(p_set text) OWNER TO db_global_owner;

--
-- Name: FUNCTION sets__get_id(p_set text); Type: COMMENT; Schema: replication; Owner: db_global_owner
--

COMMENT ON FUNCTION sets__get_id(p_set text) IS 'Returns ID value for replication.sets based on set. This is generated code.';


--
-- Name: sets__get_text(smallint); Type: FUNCTION; Schema: replication; Owner: db_global_owner
--

CREATE FUNCTION sets__get_text(p_id smallint) RETURNS text
    AS $_$
SELECT set FROM replication.sets WHERE id = $1
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION replication.sets__get_text(p_id smallint) OWNER TO db_global_owner;

--
-- Name: FUNCTION sets__get_text(p_id smallint); Type: COMMENT; Schema: replication; Owner: db_global_owner
--

COMMENT ON FUNCTION sets__get_text(p_id smallint) IS 'Returns set value for replication.sets based on id. This is generated code.';


--
-- Name: table__add(text, text); Type: FUNCTION; Schema: replication; Owner: db_global_owner
--

CREATE FUNCTION table__add(p_set text, p_full_name text) RETURNS void
    AS $$
DECLARE
  -- This is a trick to ensure the table exists
  v_oid CONSTANT oid := p_full_name::regclass::oid;
  v_set_id CONSTANT replication.sets.id%TYPE := replication.sets__get_id(  p_set  );
BEGIN
  IF v_set_id IS NULL THEN
    RAISE EXCEPTION 'set % does not exist', p_set;
  END IF;

  INSERT INTO replication.tables( set_id, full_name )
    VALUES( v_set_id, p_full_name )
  ;
EXCEPTION WHEN unique_violation THEN
  -- DO NOTHING
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION replication.table__add(p_set text, p_full_name text) OWNER TO db_global_owner;

--
-- Name: table__text_list(text); Type: FUNCTION; Schema: replication; Owner: db_global_owner
--

CREATE FUNCTION table__text_list(p_set text) RETURNS text
    AS $$
DECLARE
  v_set_id CONSTANT replication.sets.id%TYPE := replication.sets__get_id(  p_set  );
BEGIN
  IF v_set_id IS NULL THEN
    RAISE EXCEPTION 'set % does not exist', p_set;
  END IF;

  RETURN array_to_string(
    array( SELECT full_name FROM replication.tables WHERE set_id = v_set_id )
    , ' '
  );
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION replication.table__text_list(p_set text) OWNER TO db_global_owner;

--
-- Name: tables__exclude; Type: TABLE; Schema: replication; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE tables__exclude (
    id smallint NOT NULL,
    full_name text NOT NULL,
    description text
);


ALTER TABLE replication.tables__exclude OWNER TO db_global_owner;

--
-- Name: TABLE tables__exclude; Type: COMMENT; Schema: replication; Owner: db_global_owner
--

COMMENT ON TABLE tables__exclude IS 'List of schema qualified table names to always exclude from replication on all databases and environments';


--
-- Name: tables__exclude__get(smallint); Type: FUNCTION; Schema: replication; Owner: db_global_owner
--

CREATE FUNCTION tables__exclude__get(p_id smallint) RETURNS tables__exclude
    AS $_$
SELECT * FROM replication.tables__exclude WHERE id = $1
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION replication.tables__exclude__get(p_id smallint) OWNER TO db_global_owner;

--
-- Name: FUNCTION tables__exclude__get(p_id smallint); Type: COMMENT; Schema: replication; Owner: db_global_owner
--

COMMENT ON FUNCTION tables__exclude__get(p_id smallint) IS 'Returns row from replication.tables__exclude based on id. This is generated code.';


--
-- Name: tables__exclude__get_id(text); Type: FUNCTION; Schema: replication; Owner: db_global_owner
--

CREATE FUNCTION tables__exclude__get_id(p_full_name text) RETURNS smallint
    AS $_$
SELECT id FROM replication.tables__exclude WHERE lower(full_name) = lower($1)
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION replication.tables__exclude__get_id(p_full_name text) OWNER TO db_global_owner;

--
-- Name: FUNCTION tables__exclude__get_id(p_full_name text); Type: COMMENT; Schema: replication; Owner: db_global_owner
--

COMMENT ON FUNCTION tables__exclude__get_id(p_full_name text) IS 'Returns ID value for replication.tables__exclude based on full_name. This is generated code.';


--
-- Name: tables__exclude__get_text(smallint); Type: FUNCTION; Schema: replication; Owner: db_global_owner
--

CREATE FUNCTION tables__exclude__get_text(p_id smallint) RETURNS text
    AS $_$
SELECT full_name FROM replication.tables__exclude WHERE id = $1
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION replication.tables__exclude__get_text(p_id smallint) OWNER TO db_global_owner;

--
-- Name: FUNCTION tables__exclude__get_text(p_id smallint); Type: COMMENT; Schema: replication; Owner: db_global_owner
--

COMMENT ON FUNCTION tables__exclude__get_text(p_id smallint) IS 'Returns full_name value for replication.tables__exclude based on id. This is generated code.';


SET search_path = tools, pg_catalog;

--
-- Name: settings; Type: TABLE; Schema: tools; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE settings (
    setting_name text NOT NULL,
    b boolean,
    f double precision,
    i integer,
    t text,
    iv interval
);


ALTER TABLE tools.settings OWNER TO db_global_owner;

--
-- Name: TABLE settings; Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON TABLE settings IS 'This is a seed table.';


SET search_path = setting, pg_catalog;

--
-- Name: setting(text); Type: FUNCTION; Schema: setting; Owner: db_global_owner
--

CREATE FUNCTION setting(text) RETURNS SETOF tools.settings
    AS $_$
  SELECT * FROM tools.settings WHERE lower(setting_name) = lower($1)
$_$
    LANGUAGE sql STABLE SECURITY DEFINER;


ALTER FUNCTION setting.setting(text) OWNER TO db_global_owner;

--
-- Name: setting__set(text, text, text); Type: FUNCTION; Schema: setting; Owner: db_global_owner
--

CREATE FUNCTION setting__set(p_setting_name text, p_field text, p_new_value text) RETURNS void
    AS $_$
DECLARE
  c_field CONSTANT text := quote_ident( p_field );
  c_new_value CONSTANT text := quote_literal( p_new_value );
  sql text;
  rows int;
BEGIN
  RAISE DEBUG 'p_setting_name = %, p_field = %, p_new_value = %, c_field = %, c_new_value = %'
    , p_setting_name
    , p_field
    , p_new_value
    , c_field
    , c_new_value
  ;
  FOR i IN 1..10 LOOP
    -- Can't use tools.exec because it won't tell us if we found a record
    sql := 'UPDATE tools.settings SET ' || c_field || $$ = $$
      || c_new_value || $$ WHERE lower( setting_name ) = $$ || quote_literal( lower( p_setting_name ) )
    ;
    RAISE DEBUG 'executing %', sql;
    EXECUTE sql;
    -- Can't use FOUND because it's not set for EXECUTE
    GET DIAGNOSTICS rows = ROW_COUNT;
    IF rows > 0 THEN
        RETURN;
    END IF;
    BEGIN
        sql := $$INSERT INTO tools.settings( setting_name, $$ || c_field || $$ ) VALUES ( $$ || quote_literal( p_setting_name ) || $$, $$
          || c_new_value || $$ )$$
        ;
        PERFORM tools.exec( sql );
        RETURN;
    EXCEPTION WHEN unique_violation THEN
        -- do nothing, and loop to try the UPDATE again
    END;
  END LOOP;
  RAISE EXCEPTION 'E_END_OF_WORLD: the ship falls off the planet. You die.';
END;
$_$
    LANGUAGE plpgsql;


ALTER FUNCTION setting.setting__set(p_setting_name text, p_field text, p_new_value text) OWNER TO db_global_owner;

--
-- Name: setting__set(text, boolean); Type: FUNCTION; Schema: setting; Owner: db_global_owner
--

CREATE FUNCTION setting__set(p_setting_name text, p_new_value boolean) RETURNS void
    AS $_$
/*
 * Created in: bug_243812_v5_stats
 */
SELECT setting.setting__set( $1, 'b', $2::text )
$_$
    LANGUAGE sql;


ALTER FUNCTION setting.setting__set(p_setting_name text, p_new_value boolean) OWNER TO db_global_owner;

--
-- Name: setting__set(text, double precision); Type: FUNCTION; Schema: setting; Owner: db_global_owner
--

CREATE FUNCTION setting__set(p_setting_name text, p_new_value double precision) RETURNS void
    AS $_$
/*
 * Created in: bug_243812_v5_stats
 */
SELECT setting.setting__set( $1, 'f', $2::text )
$_$
    LANGUAGE sql;


ALTER FUNCTION setting.setting__set(p_setting_name text, p_new_value double precision) OWNER TO db_global_owner;

--
-- Name: setting__set(text, integer); Type: FUNCTION; Schema: setting; Owner: db_global_owner
--

CREATE FUNCTION setting__set(p_setting_name text, p_new_value integer) RETURNS void
    AS $_$
/*
 * Created in: bug_243812_v5_stats
 */
SELECT setting.setting__set( $1, 'i', $2::text )
$_$
    LANGUAGE sql;


ALTER FUNCTION setting.setting__set(p_setting_name text, p_new_value integer) OWNER TO db_global_owner;

--
-- Name: setting__set(text, text); Type: FUNCTION; Schema: setting; Owner: db_global_owner
--

CREATE FUNCTION setting__set(p_setting_name text, p_new_value text) RETURNS void
    AS $_$
/*
 * Created in: bug_243812_v5_stats
 */
SELECT setting.setting__set( $1, 't', $2::text )
$_$
    LANGUAGE sql;


ALTER FUNCTION setting.setting__set(p_setting_name text, p_new_value text) OWNER TO db_global_owner;

--
-- Name: setting__set(text, interval); Type: FUNCTION; Schema: setting; Owner: db_global_owner
--

CREATE FUNCTION setting__set(p_setting_name text, p_new_value interval) RETURNS void
    AS $_$
/*
 * Created in: bug_243812_v5_stats
 */
SELECT setting.setting__set( $1, 'iv', $2::text )
$_$
    LANGUAGE sql;


ALTER FUNCTION setting.setting__set(p_setting_name text, p_new_value interval) OWNER TO db_global_owner;

--
-- Name: setting_b(text); Type: FUNCTION; Schema: setting; Owner: db_global_owner
--

CREATE FUNCTION setting_b(p_setting_name text) RETURNS boolean
    AS $_$
/*
 * Created in: bug_243812_v5_stats
 */
SELECT b FROM tools.setting($1)$_$
    LANGUAGE sql STABLE SECURITY DEFINER;


ALTER FUNCTION setting.setting_b(p_setting_name text) OWNER TO db_global_owner;

--
-- Name: setting_f(text); Type: FUNCTION; Schema: setting; Owner: db_global_owner
--

CREATE FUNCTION setting_f(p_setting_name text) RETURNS double precision
    AS $_$
/*
 * Created in: bug_243812_v5_stats
 */
SELECT f FROM tools.setting($1)$_$
    LANGUAGE sql STABLE SECURITY DEFINER;


ALTER FUNCTION setting.setting_f(p_setting_name text) OWNER TO db_global_owner;

--
-- Name: setting_i(text); Type: FUNCTION; Schema: setting; Owner: db_global_owner
--

CREATE FUNCTION setting_i(p_setting_name text) RETURNS integer
    AS $_$
/*
 * Created in: bug_243812_v5_stats
 */
SELECT i FROM tools.setting($1)$_$
    LANGUAGE sql STABLE SECURITY DEFINER;


ALTER FUNCTION setting.setting_i(p_setting_name text) OWNER TO db_global_owner;

--
-- Name: setting_iv(text); Type: FUNCTION; Schema: setting; Owner: db_global_owner
--

CREATE FUNCTION setting_iv(p_setting_name text) RETURNS interval
    AS $_$
/*
 * Created in: bug_243812_v5_stats
 */
SELECT iv FROM tools.setting($1)$_$
    LANGUAGE sql STABLE SECURITY DEFINER;


ALTER FUNCTION setting.setting_iv(p_setting_name text) OWNER TO db_global_owner;

--
-- Name: setting_t(text); Type: FUNCTION; Schema: setting; Owner: db_global_owner
--

CREATE FUNCTION setting_t(p_setting_name text) RETURNS text
    AS $_$
/*
 * Created in: bug_243812_v5_stats
 */
SELECT t FROM tools.setting($1)$_$
    LANGUAGE sql STABLE SECURITY DEFINER;


ALTER FUNCTION setting.setting_t(p_setting_name text) OWNER TO db_global_owner;

SET search_path = test, pg_catalog;

--
-- Name: assert(text, bigint, bigint); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION assert(p_expression text, p_result bigint, p_expected bigint) RETURNS boolean
    AS $$

-- Modified by bug_192474__schema_as
BEGIN
  IF p_result IS DISTINCT FROM p_expected THEN
    RAISE EXCEPTION 'Expected "%" for expression %, got "%"'
      , p_expected, p_expression, p_result
    ;
  END IF;

  RETURN true;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION test.assert(p_expression text, p_result bigint, p_expected bigint) OWNER TO db_global_owner;

--
-- Name: assert(text, text, text); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION assert(p_expression text, p_result text, p_expected text) RETURNS boolean
    AS $$

-- Modified by bug_192474__schema_as
BEGIN
  IF p_result IS DISTINCT FROM p_expected THEN
    RAISE EXCEPTION 'Expected "%" for expression %, got "%"'
      , p_expected, p_expression, p_result
    ;
  END IF;

  RETURN true;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION test.assert(p_expression text, p_result text, p_expected text) OWNER TO db_global_owner;

--
-- Name: assert(text, double precision, double precision); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION assert(p_expression text, p_result double precision, p_expected double precision) RETURNS boolean
    AS $$

-- Modified by bug_192474__schema_as
BEGIN
  IF p_result IS DISTINCT FROM p_expected THEN
    RAISE EXCEPTION 'Expected "%" for expression %, got "%"'
      , p_expected, p_expression, p_result
    ;
  END IF;

  RETURN true;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION test.assert(p_expression text, p_result double precision, p_expected double precision) OWNER TO db_global_owner;

--
-- Name: assert(text, timestamp with time zone, timestamp with time zone); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION assert(p_expression text, p_result timestamp with time zone, p_expected timestamp with time zone) RETURNS boolean
    AS $$

-- Modified by bug_192474__schema_as
BEGIN
  IF p_result IS DISTINCT FROM p_expected THEN
    RAISE EXCEPTION 'Expected "%" for expression %, got "%"'
      , p_expected, p_expression, p_result
    ;
  END IF;

  RETURN true;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION test.assert(p_expression text, p_result timestamp with time zone, p_expected timestamp with time zone) OWNER TO db_global_owner;

--
-- Name: assert(text, interval, interval); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION assert(p_expression text, p_result interval, p_expected interval) RETURNS boolean
    AS $$

-- Modified by bug_192474__schema_as
BEGIN
  IF p_result IS DISTINCT FROM p_expected THEN
    RAISE EXCEPTION 'Expected "%" for expression %, got "%"'
      , p_expected, p_expression, p_result
    ;
  END IF;

  RETURN true;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION test.assert(p_expression text, p_result interval, p_expected interval) OWNER TO db_global_owner;

--
-- Name: assert(text, numeric, numeric); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION assert(p_expression text, p_result numeric, p_expected numeric) RETURNS boolean
    AS $$

-- Modified by bug_192474__schema_as
BEGIN
  IF p_result IS DISTINCT FROM p_expected THEN
    RAISE EXCEPTION 'Expected "%" for expression %, got "%"'
      , p_expected, p_expression, p_result
    ;
  END IF;

  RETURN true;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION test.assert(p_expression text, p_result numeric, p_expected numeric) OWNER TO db_global_owner;

--
-- Name: assert(text, boolean, boolean); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION assert(p_expression text, p_result boolean, p_expected boolean) RETURNS boolean
    AS $$

-- Modified by bug_192474__schema_as
BEGIN
  IF p_result IS DISTINCT FROM p_expected THEN
    RAISE EXCEPTION 'Expected "%" for expression %, got "%"'
      , p_expected, p_expression, p_result
    ;
  END IF;

  RETURN true;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION test.assert(p_expression text, p_result boolean, p_expected boolean) OWNER TO db_global_owner;

--
-- Name: assert(text, text[], text[]); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION assert(p_expression text, p_result text[], p_expected text[]) RETURNS boolean
    AS $$

-- Modified by bug_192474__schema_as
BEGIN
  IF p_result IS DISTINCT FROM p_expected THEN
    RAISE EXCEPTION 'Expected "%" for expression %, got "%"'
      , p_expected, p_expression, p_result
    ;
  END IF;

  RETURN true;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION test.assert(p_expression text, p_result text[], p_expected text[]) OWNER TO db_global_owner;

--
-- Name: assert(text, text); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION assert(p_condition text, p_result text) RETURNS boolean
    AS $_$

SELECT tools.exec(
  'SELECT test.assert( $condition_text$' || $1 || '$condition_text$, ' || $1 || ', ' || $2 || ')'
) IS NOT NULL

$_$
    LANGUAGE sql;


ALTER FUNCTION test.assert(p_condition text, p_result text) OWNER TO db_global_owner;

--
-- Name: FUNCTION assert(p_condition text, p_result text); Type: COMMENT; Schema: test; Owner: db_global_owner
--

COMMENT ON FUNCTION assert(p_condition text, p_result text) IS 'Asserts a condition.';


--
-- Name: assert(boolean, text); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION assert(p_condition boolean, p_message text) RETURNS boolean
    AS $$
BEGIN
  IF p_condition THEN
    RETURN true;
  ELSE
    RAISE EXCEPTION '%', p_message;
  END IF;
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION test.assert(p_condition boolean, p_message text) OWNER TO db_global_owner;

--
-- Name: assert(text, bytea, bytea); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION assert(p_expression text, p_result bytea, p_expected bytea) RETURNS boolean
    AS $$

-- Modified by bug_192474__schema_as
BEGIN
  IF p_result IS DISTINCT FROM p_expected THEN
    RAISE EXCEPTION 'Expected "%" for expression %, got "%"'
      , p_expected, p_expression, p_result
    ;
  END IF;

  RETURN true;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION test.assert(p_expression text, p_result bytea, p_expected bytea) OWNER TO db_global_owner;

--
-- Name: assert(text, oid, oid); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION assert(p_expression text, p_result oid, p_expected oid) RETURNS boolean
    AS $$

-- Modified by bug_192474__schema_as
BEGIN
  IF p_result IS DISTINCT FROM p_expected THEN
    RAISE EXCEPTION 'Expected "%" for expression %, got "%"'
      , p_expected, p_expression, p_result
    ;
  END IF;

  RETURN true;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION test.assert(p_expression text, p_result oid, p_expected oid) OWNER TO db_global_owner;

--
-- Name: assert(text, integer, integer); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION assert(p_expression text, p_result integer, p_expected integer) RETURNS boolean
    AS $$

-- Modified by bug_192474__schema_as
BEGIN
  IF p_result IS DISTINCT FROM p_expected THEN
    RAISE EXCEPTION 'Expected "%" for expression %, got "%"'
      , p_expected, p_expression, p_result
    ;
  END IF;

  RETURN true;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION test.assert(p_expression text, p_result integer, p_expected integer) OWNER TO db_global_owner;

--
-- Name: assert(text, smallint, smallint); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION assert(p_expression text, p_result smallint, p_expected smallint) RETURNS boolean
    AS $$

-- Modified by bug_192474__schema_as
BEGIN
  IF p_result IS DISTINCT FROM p_expected THEN
    RAISE EXCEPTION 'Expected "%" for expression %, got "%"'
      , p_expected, p_expression, p_result
    ;
  END IF;

  RETURN true;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION test.assert(p_expression text, p_result smallint, p_expected smallint) OWNER TO db_global_owner;

--
-- Name: assert(text, real, real); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION assert(p_expression text, p_result real, p_expected real) RETURNS boolean
    AS $$

-- Modified by bug_192474__schema_as
BEGIN
  IF p_result IS DISTINCT FROM p_expected THEN
    RAISE EXCEPTION 'Expected "%" for expression %, got "%"'
      , p_expected, p_expression, p_result
    ;
  END IF;

  RETURN true;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION test.assert(p_expression text, p_result real, p_expected real) OWNER TO db_global_owner;

--
-- Name: foreign_key__exists(text, text[], text, text[]); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION foreign_key__exists(p_fk_table text, p_fk_columns text[], p_key_table text, p_key_columns text[]) RETURNS void
    AS $$

BEGIN
    IF NOT tools.foreign_key__exists( p_fk_table, p_fk_columns, p_key_table, p_key_columns ) THEN
        RAISE EXCEPTION 'Foreign key to % (%) from % (%) does not exist'
            , p_fk_table
            , array_to_string( p_fk_columns, ', ' )
            , p_key_table
            , array_to_string( p_key_columns, ', ' )
        ;
    END IF;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION test.foreign_key__exists(p_fk_table text, p_fk_columns text[], p_key_table text, p_key_columns text[]) OWNER TO db_global_owner;

--
-- Name: FUNCTION foreign_key__exists(p_fk_table text, p_fk_columns text[], p_key_table text, p_key_columns text[]); Type: COMMENT; Schema: test; Owner: db_global_owner
--

COMMENT ON FUNCTION foreign_key__exists(p_fk_table text, p_fk_columns text[], p_key_table text, p_key_columns text[]) IS 'Throws an exception if a foreign key to p_fk_table( p_fk_columns ) from p_key_table( p_key_columns ) does not exist.';


--
-- Name: foreign_key__exists(text, text, text, text); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION foreign_key__exists(p_fk_table text, p_fk_columns text, p_key_table text, p_key_columns text) RETURNS void
    AS $$

BEGIN
    IF NOT tools.foreign_key__exists( p_fk_table, p_fk_columns, p_key_table, p_key_columns ) THEN
        RAISE EXCEPTION 'Foreign key to % (%) from % (%) does not exist'
            , p_fk_table
            , p_fk_columns
            , p_key_table
            , p_key_columns
        ;
    END IF;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION test.foreign_key__exists(p_fk_table text, p_fk_columns text, p_key_table text, p_key_columns text) OWNER TO db_global_owner;

--
-- Name: FUNCTION foreign_key__exists(p_fk_table text, p_fk_columns text, p_key_table text, p_key_columns text); Type: COMMENT; Schema: test; Owner: db_global_owner
--

COMMENT ON FUNCTION foreign_key__exists(p_fk_table text, p_fk_columns text, p_key_table text, p_key_columns text) IS 'Throws an exception if a foreign key to p_fk_table( p_fk_columns ) from p_key_table( p_key_columns ) does not exist.';


--
-- Name: primary_unique_constraint__exists(text, text[]); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION primary_unique_constraint__exists(p_table text, p_columns text[]) RETURNS boolean
    AS $_$

SELECT tools.assert( tools.primary_unique_constraint__exists( $1, $2 )
  , 'There is no primary key/unique constraint on table ' || quote_ident( $1 ) || ' for columns '
    || array_to_string( $2, ', ' )
);

$_$
    LANGUAGE sql;


ALTER FUNCTION test.primary_unique_constraint__exists(p_table text, p_columns text[]) OWNER TO db_global_owner;

--
-- Name: FUNCTION primary_unique_constraint__exists(p_table text, p_columns text[]); Type: COMMENT; Schema: test; Owner: db_global_owner
--

COMMENT ON FUNCTION primary_unique_constraint__exists(p_table text, p_columns text[]) IS 'Returns true if a specified constraint actually exists';


--
-- Name: primary_unique_constraint__exists(text, text); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION primary_unique_constraint__exists(p_table text, p_columns text) RETURNS boolean
    AS $_$

SELECT tools.assert( tools.primary_unique_constraint__exists( $1, $2 )
  , 'There is no primary key/unique constraint on table ' || quote_ident( $1 ) || ' for columns ' || $2
);

$_$
    LANGUAGE sql;


ALTER FUNCTION test.primary_unique_constraint__exists(p_table text, p_columns text) OWNER TO db_global_owner;

--
-- Name: FUNCTION primary_unique_constraint__exists(p_table text, p_columns text); Type: COMMENT; Schema: test; Owner: db_global_owner
--

COMMENT ON FUNCTION primary_unique_constraint__exists(p_table text, p_columns text) IS 'Returns true if a specified constraint actually exists';


--
-- Name: proxynow_sanity(); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION proxynow_sanity() RETURNS boolean
    AS $$
    SELECT tools.proxynow() = pg_catalog.now();
  $$
    LANGUAGE sql;


ALTER FUNCTION test.proxynow_sanity() OWNER TO db_global_owner;

--
-- Name: seed_object(text); Type: FUNCTION; Schema: test; Owner: db_global_owner
--

CREATE FUNCTION seed_object(p_table text) RETURNS boolean
    AS $_$
SELECT test.assert(
  EXISTS( SELECT * FROM tools.seed_objects_v WHERE relation_oid = $1::regclass )
  , tools.relation_full_name( $1 ) || ' is not marked as a seed object'
)
$_$
    LANGUAGE sql;


ALTER FUNCTION test.seed_object(p_table text) OWNER TO db_global_owner;

SET search_path = tools, pg_catalog;

--
-- Name: acosh(double precision); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION acosh(double precision) RETURNS double precision
    AS $_$
    SELECT ln( $1 + sqrt($1 - 1) * sqrt($1 + 1) ); 
  $_$
    LANGUAGE sql IMMUTABLE;


ALTER FUNCTION tools.acosh(double precision) OWNER TO db_global_owner;

--
-- Name: array_length(anyarray, integer); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION array_length(p_array anyarray, p_dimension integer) RETURNS integer
    AS $_$
    SELECT array_upper( $1, $2 ) - array_lower( $1, $2 ) + 1
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.array_length(p_array anyarray, p_dimension integer) OWNER TO db_global_owner;

--
-- Name: asinh(double precision); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION asinh(double precision) RETURNS double precision
    AS $_$
    SELECT ln( $1 + sqrt($1 ^ 2 + 1) ); 
  $_$
    LANGUAGE sql IMMUTABLE;


ALTER FUNCTION tools.asinh(double precision) OWNER TO db_global_owner;

--
-- Name: assert(character varying, boolean, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION assert(p_name character varying, p_condition boolean, p_message text) RETURNS boolean
    AS $_$
    SELECT tools.assert( $1, NULL, $2, $3 );
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.assert(p_name character varying, p_condition boolean, p_message text) OWNER TO db_global_owner;

--
-- Name: assert(boolean, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION assert(p_condition boolean, p_message text) RETURNS boolean
    AS $$
BEGIN
    IF tools.asserts__disabled() OR p_condition THEN
        RETURN true;
    ELSE
        RAISE EXCEPTION '%', p_message;
    END IF;
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.assert(p_condition boolean, p_message text) OWNER TO db_global_owner;

--
-- Name: assert(character varying, character varying, boolean, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION assert(p_assert_name character varying, p_sub_assert_name character varying, p_condition boolean, p_message text) RETURNS boolean
    AS $_$
DECLARE
  v_assert tools.asserts;
  v_enabled boolean;
  v_disabled_by text := '';
BEGIN
  -- Modified in bug_252483_fix_asserts_for_dump
  IF p_condition THEN
    /*
     * Special-case check that's a lot faster than a full assert check.
     *
     * NOTE: This has been hand-tuned. Running a direct SELECT instead of using
     * tools.setting() is almost 2x faster. Not calling lower for our fixed
     * string is good for another 10%.
     *
     * Also note that there is stuff that *could* be refactored out of this
     * code path but it would hurt performance to do so.
     *
     */

    IF NOT coalesce(
        ( SELECT b FROM tools.settings WHERE lower( setting_name ) = 'disable assert exists check for true conditions' )
        , tools.proxy_current_database() ~* '_prod'
      )
    THEN
      RAISE DEBUG $$disabled setting = %, tools.proxy_current_database( % ) ~* '_prod' = %$$
        , ( SELECT b FROM tools.settings WHERE lower( setting_name ) = lower( 'disable assert exists check for true conditions' ) )
        , tools.proxy_current_database()
        , tools.proxy_current_database() ~* '_prod'
      ;
      IF tools.assert__get( p_assert_name ) IS NULL AND EXISTS( SELECT * FROM tools.asserts ) THEN
        RAISE EXCEPTION 'Assert % does not exist', p_assert_name;
      END IF;
    END IF;

    RETURN true; -- Note the return here
  END IF;

  -- Fetch the assert
  v_assert := tools.assert__get( p_assert_name );

  -- Assert that we were handed a valid assert :)
  PERFORM tools.assert( 
      'assert_exists'
      , NOT ( v_assert IS NULL AND EXISTS( SELECT * FROM tools.asserts ) )
      , p_assert_name || ' is not a valid assert' 
    )
  ;

  v_enabled := true;
  -- Global assert enabled checks
  IF NOT v_assert.enabled THEN
    v_disabled_by := v_disabled_by || '; global';
    v_enabled := false;
  END IF;
  IF v_assert.level < tools.min_assert_level() THEN
    v_disabled_by := v_disabled_by || '; minimum assert level';
    v_enabled := false;
  END IF;

  -- Session assert enabled check
  IF NOT tools.assert__enabled_session( p_assert_name ) THEN
    v_disabled_by := v_disabled_by || '; session';
    v_enabled := false;
  END IF;

  IF p_sub_assert_name IS NOT NULL THEN
    -- Global sub-assert enabled check
    IF NOT tools.sub_assert__enabled_global( p_assert_name, p_sub_assert_name ) THEN
      v_disabled_by := v_disabled_by || '; sub-assert global';
      v_enabled := false;
    END IF;

    -- Session sub-assert enabled check
    IF NOT tools.sub_assert__enabled_session( p_assert_name, p_sub_assert_name ) THEN
      v_disabled_by := v_disabled_by || '; sub-assert session';
      v_enabled := false;
    END IF;
  END IF;

  -- If it's enabled, throw an exception
  IF v_enabled THEN
    -- Print a different error for sub-asserts
    IF p_sub_assert_name IS NULL THEN
      RAISE EXCEPTION '% (assert %)', p_message, p_assert_name;
    ELSE
      RAISE EXCEPTION '% (assert %, sub-assert %)', p_message, p_assert_name, p_sub_assert_name;
    END IF;
  END IF;

  -- Otherwise raise a warning, with a different one for sub-asserts
  v_disabled_by := substring( v_disabled_by from 3 );
  IF p_sub_assert_name IS NULL THEN
    RAISE WARNING 'ASSERT FAILURE WHILE DISABLED! Assert %, message %, disabled at these levels: %', p_assert_name, p_message, v_disabled_by;
  ELSE
    RAISE WARNING 'SUB-ASSERT FAILURE WHILE DISABLED! Assert %, sub-assert %, message %, disabled at these levels: %', p_assert_name, p_sub_assert_name, p_message, v_disabled_by;
  END IF;

  RETURN true;
END;
$_$
    LANGUAGE plpgsql SECURITY DEFINER
    SET search_path TO pg_catalog;


ALTER FUNCTION tools.assert(p_assert_name character varying, p_sub_assert_name character varying, p_condition boolean, p_message text) OWNER TO db_global_owner;

--
-- Name: assert__add(character varying, integer); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION assert__add(p_assert_name character varying, p_level integer) RETURNS void
    AS $_$
INSERT INTO tools.asserts( assert_name, level ) VALUES( $1, $2 )
$_$
    LANGUAGE sql SECURITY DEFINER;


ALTER FUNCTION tools.assert__add(p_assert_name character varying, p_level integer) OWNER TO db_global_owner;

--
-- Name: FUNCTION assert__add(p_assert_name character varying, p_level integer); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION assert__add(p_assert_name character varying, p_level integer) IS 'Adds an assert to the asserts table.';


--
-- Name: assert__delete(character varying, boolean); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION assert__delete(p_assert_name character varying, p_force boolean) RETURNS void
    AS $$

BEGIN
  DELETE FROM tools.asserts WHERE lower( assert_name ) = lower( p_assert_name );

  IF NOT FOUND AND NOT p_force THEN
    RAISE EXCEPTION 'Assert "%" not found', p_assert_name;
  END IF;
END;

$$
    LANGUAGE plpgsql SECURITY DEFINER
    SET search_path TO pg_catalog;


ALTER FUNCTION tools.assert__delete(p_assert_name character varying, p_force boolean) OWNER TO db_global_owner;

--
-- Name: FUNCTION assert__delete(p_assert_name character varying, p_force boolean); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION assert__delete(p_assert_name character varying, p_force boolean) IS 'Deletes an assert. Throws an error if the assert does not exist unless p_force is true.';


--
-- Name: assert__delete(character varying); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION assert__delete(p_assert_name character varying) RETURNS void
    AS $_$
SELECT tools.assert__delete( $1, false )
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.assert__delete(p_assert_name character varying) OWNER TO db_global_owner;

--
-- Name: FUNCTION assert__delete(p_assert_name character varying); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION assert__delete(p_assert_name character varying) IS 'Deletes an assert. Throws an error if the assert does not exist.';


--
-- Name: assert__enabled_global(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION assert__enabled_global(p_assert_name text) RETURNS boolean
    AS $_$
  -- Original in bug_252483_fix_asserts_for_dump
  SELECT enabled FROM tools.assert__get( $1 )
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION tools.assert__enabled_global(p_assert_name text) OWNER TO db_global_owner;

--
-- Name: assert__enabled_global__set(character varying, boolean); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION assert__enabled_global__set(p_assert_name character varying, p_enabled boolean) RETURNS void
    AS $_$
UPDATE tools.asserts SET enabled = $2 WHERE lower( assert_name ) = lower( $1 );
$_$
    LANGUAGE sql SECURITY DEFINER;


ALTER FUNCTION tools.assert__enabled_global__set(p_assert_name character varying, p_enabled boolean) OWNER TO db_global_owner;

--
-- Name: FUNCTION assert__enabled_global__set(p_assert_name character varying, p_enabled boolean); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION assert__enabled_global__set(p_assert_name character varying, p_enabled boolean) IS 'Enables or disables a specific assert globally. May only be run by cnuadmin.';


--
-- Name: assert__enabled_global__set(character varying, character varying, boolean); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION assert__enabled_global__set(p_assert_name character varying, p_sub_assert_name character varying, p_enabled boolean) RETURNS void
    AS $$

BEGIN
    IF p_enabled THEN
        DELETE FROM tools.sub_asserts 
            WHERE ( assert_name, sub_assert_name ) = 
                ( p_assert_name, lower( p_sub_assert_name ) )
        ;
    ELSE
        BEGIN
            INSERT INTO tools.sub_asserts 
                ( assert_name, sub_assert_name )
            VALUES
                ( p_assert_name, lower( p_sub_assert_name ) )
            ;
        EXCEPTION WHEN unique_violation THEN
            -- Do nothing, it's already on
        END;
    END IF;
END;

$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION tools.assert__enabled_global__set(p_assert_name character varying, p_sub_assert_name character varying, p_enabled boolean) OWNER TO db_global_owner;

--
-- Name: FUNCTION assert__enabled_global__set(p_assert_name character varying, p_sub_assert_name character varying, p_enabled boolean); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION assert__enabled_global__set(p_assert_name character varying, p_sub_assert_name character varying, p_enabled boolean) IS 'Enables or disables a specific assert globally. May only be run by cnuadmin.';


--
-- Name: assert__enabled_session(character varying); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION assert__enabled_session(p_assert_name character varying) RETURNS boolean
    AS $$
BEGIN
    RETURN COALESCE( tools.session__get( 'disabledasserts'
                    , lower( p_assert_name ) ), 'true' );
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.assert__enabled_session(p_assert_name character varying) OWNER TO db_global_owner;

--
-- Name: FUNCTION assert__enabled_session(p_assert_name character varying); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION assert__enabled_session(p_assert_name character varying) IS 'Returns whether a specified assert is enabled globally or not.';


--
-- Name: assert__enabled_session(character varying, character varying); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION assert__enabled_session(p_assert_name character varying, p_sub_assert_name character varying) RETURNS boolean
    AS $_$
  -- Original in bug_90886_session_hash_tools
  -- Modified in bug_252483_fix_asserts_for_dump
  SELECT tools.assert__enabled_session( $1 ) AND tools.sub_assert__enabled_session( $1, $2 )
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.assert__enabled_session(p_assert_name character varying, p_sub_assert_name character varying) OWNER TO db_global_owner;

--
-- Name: assert__enabled_session__set(character varying, boolean); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION assert__enabled_session__set(p_assert_name character varying, p_enabled boolean) RETURNS void
    AS $$
BEGIN
    PERFORM tools.assert__enabled_session__set( lower( p_assert_name ), NULL, p_enabled );
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.assert__enabled_session__set(p_assert_name character varying, p_enabled boolean) OWNER TO db_global_owner;

--
-- Name: FUNCTION assert__enabled_session__set(p_assert_name character varying, p_enabled boolean); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION assert__enabled_session__set(p_assert_name character varying, p_enabled boolean) IS 'Enables or disables a specific assert globally. May only be run by cnuadmin.';


--
-- Name: assert__enabled_session__set(character varying, character varying, boolean); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION assert__enabled_session__set(p_assert_name character varying, p_sub_assert_name character varying, p_enabled boolean) RETURNS void
    AS $$
DECLARE
    v_assert_name text := lower( p_assert_name 
        || COALESCE('.' || p_sub_assert_name, '') );
BEGIN
    IF p_enabled THEN
        PERFORM tools.session__set( 'disabledasserts', v_assert_name, NULL );
    ELSE
        PERFORM tools.session__set( 'disabledasserts', v_assert_name, 'false' );
    END IF;
END;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION tools.assert__enabled_session__set(p_assert_name character varying, p_sub_assert_name character varying, p_enabled boolean) OWNER TO db_global_owner;

--
-- Name: asserts; Type: TABLE; Schema: tools; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE asserts (
    assert_name character varying(1000) NOT NULL,
    enabled boolean DEFAULT true NOT NULL,
    level smallint NOT NULL,
    CONSTRAINT asserts__assert_name_must_not_contain_periods CHECK (((assert_name)::text !~~ '%.%'::text))
);


ALTER TABLE tools.asserts OWNER TO db_global_owner;

--
-- Name: TABLE asserts; Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON TABLE asserts IS 'This is a seed table.';


--
-- Name: COLUMN asserts.assert_name; Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON COLUMN asserts.assert_name IS 'Assert names can not contain periods due to the way we handle allowing asserts to be disabled on a per-session basis.
TODO: Switch to case-insensitive text.';


--
-- Name: assert__get(character varying); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION assert__get(p_assert_name character varying) RETURNS asserts
    AS $_$
SELECT * FROM tools.asserts WHERE lower( assert_name ) = lower( $1 );
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.assert__get(p_assert_name character varying) OWNER TO db_global_owner;

--
-- Name: FUNCTION assert__get(p_assert_name character varying); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION assert__get(p_assert_name character varying) IS 'Returns a specified assert.';


--
-- Name: asserts__disabled(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION asserts__disabled() RETURNS boolean
    AS $$
SELECT coalesce( tools.setting_b( 'Asserts disabled' ), false ) 
$$
    LANGUAGE sql STABLE;


ALTER FUNCTION tools.asserts__disabled() OWNER TO db_global_owner;

--
-- Name: asserts__enabled(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION asserts__enabled() RETURNS boolean
    AS $$
SELECT NOT tools.asserts__disabled()
$$
    LANGUAGE sql STABLE;


ALTER FUNCTION tools.asserts__enabled() OWNER TO db_global_owner;

--
-- Name: atanh(double precision); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION atanh(double precision) RETURNS double precision
    AS $_$
    SELECT 0.5 * ln( (1 + $1) / (1 - $1) );
  $_$
    LANGUAGE sql IMMUTABLE;


ALTER FUNCTION tools.atanh(double precision) OWNER TO db_global_owner;

--
-- Name: attribute_positions(text, text[]); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION attribute_positions(p_relation_name text, p_attribute_names text[]) RETURNS smallint[]
    AS $$
DECLARE
    -- If we're given an invalid relation, this will throw an exception for us
    v_relation_oid CONSTANT oid := p_relation_name::regclass::oid;
    i int;
    v_attnum pg_attribute.attnum%TYPE;
    v_positions smallint[];
BEGIN
    FOR i IN array_lower( p_attribute_names, 1 ) .. array_upper( p_attribute_names, 1 )
    LOOP
        SELECT INTO v_attnum
                attnum
            FROM pg_attribute
            WHERE attrelid = v_relation_oid
                AND NOT attisdropped
                AND attname = p_attribute_names[i]
        ;
        IF NOT FOUND THEN
            RAISE EXCEPTION 'attribute "%" not found', p_attribute_names[i];
        END IF;

        v_positions[i] := v_attnum;
    END LOOP;

    RETURN v_positions;
END;
$$
    LANGUAGE plpgsql STABLE;


ALTER FUNCTION tools.attribute_positions(p_relation_name text, p_attribute_names text[]) OWNER TO db_global_owner;

--
-- Name: FUNCTION attribute_positions(p_relation_name text, p_attribute_names text[]); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION attribute_positions(p_relation_name text, p_attribute_names text[]) IS 'Returns the position of a set of attributes in a relation.';


--
-- Name: attribute_positions(text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION attribute_positions(p_relation_name text, p_attribute_names text) RETURNS smallint[]
    AS $_$
SELECT tools.attribute_positions( $1, tools.string_or_array( $2 ) )
$_$
    LANGUAGE sql STABLE;


ALTER FUNCTION tools.attribute_positions(p_relation_name text, p_attribute_names text) OWNER TO db_global_owner;

--
-- Name: FUNCTION attribute_positions(p_relation_name text, p_attribute_names text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION attribute_positions(p_relation_name text, p_attribute_names text) IS 'String version of code.attribute_positions';


--
-- Name: build_security_view(text, text, text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION build_security_view(p_hashname text, p_schema_name text, p_table_name text, p_secure_columns text) RETURNS void
    AS $_$
SELECT tools.build_security_view( $1, $2, $3, $4, '' )
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.build_security_view(p_hashname text, p_schema_name text, p_table_name text, p_secure_columns text) OWNER TO db_global_owner;

--
-- Name: check(boolean, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION "check"(p_condition boolean, p_message text) RETURNS boolean
    AS $$
BEGIN
    IF tools.checks__disabled() OR p_condition THEN
        RETURN true;
    ELSE
        RAISE EXCEPTION '%', p_message;
    END IF;
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools."check"(p_condition boolean, p_message text) OWNER TO db_global_owner;

--
-- Name: checks__disabled(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION checks__disabled() RETURNS boolean
    AS $$
SELECT coalesce( tools.setting_b( 'Checks disabled' ), false ) 
$$
    LANGUAGE sql STABLE;


ALTER FUNCTION tools.checks__disabled() OWNER TO db_global_owner;

--
-- Name: checks__enabled(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION checks__enabled() RETURNS boolean
    AS $$
SELECT NOT tools.checks__disabled()
$$
    LANGUAGE sql STABLE;


ALTER FUNCTION tools.checks__enabled() OWNER TO db_global_owner;

--
-- Name: comment__set(text, text, text, text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION comment__set(p_full_name text, p_schema_name text, p_object_name text, p_object_type text, p_comment text) RETURNS void
    AS $_$
DECLARE
    c_comment_ident CONSTANT text := '$really_obtuse_comment_identifier$';
    c_allowed_types CONSTANT text[] := array[ 'table', 'conversion', 'database', 'domain', 'index', 'role', 'schema', 'sequence', 'tablespace', 'type', 'view' ];

    c_full_name CONSTANT text := coalesce( p_full_name, quote_ident(p_schema_name) || '.' || quote_ident(p_object_name) );
    c_object_type CONSTANT text := lower(p_object_type);
    v_comment text := p_comment;
BEGIN
    RAISE DEBUG 'p_full_name = %
        , p_schema_name = %
        , p_object_name = %
        , p_object_type = %
        , p_comment = %'
        , p_full_name
        , p_schema_name
        , p_object_name
        , p_object_type
        , p_comment
    ;
    IF c_object_type != ALL( c_allowed_types ) THEN
        RAISE EXCEPTION '"%" is not an allowed object type (must be one of %)', v_object_type, array_to_string( c_allowed_types, ', ' );
    END IF;

    IF c_object_type IN ('table', 'sequence') THEN
        v_comment := tools.seed_description( c_full_name, v_comment );
    END IF;

    v_comment := code_internal.description( c_full_name, p_object_type, v_comment );
    PERFORM tools.exec( 'COMMENT ON ' || upper(p_object_type) || ' ' || c_full_name || ' IS ' || c_comment_ident || coalesce(v_comment, '') || c_comment_ident );
END
$_$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.comment__set(p_full_name text, p_schema_name text, p_object_name text, p_object_type text, p_comment text) OWNER TO db_global_owner;

--
-- Name: FUNCTION comment__set(p_full_name text, p_schema_name text, p_object_name text, p_object_type text, p_comment text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION comment__set(p_full_name text, p_schema_name text, p_object_name text, p_object_type text, p_comment text) IS 'Sets the comment on an object.

If p_full_name is not specified, p_scehma_name and p_object_name are used instead.';


--
-- Name: cosh(double precision); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION cosh(double precision) RETURNS double precision
    AS $_$
    SELECT 0.5 * ( exp($1) + exp(-$1) ); 
  $_$
    LANGUAGE sql IMMUTABLE;


ALTER FUNCTION tools.cosh(double precision) OWNER TO db_global_owner;

--
-- Name: current_environment(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION current_environment() RETURNS text
    AS $_$
DECLARE
    v_env text;
BEGIN
    SELECT INTO v_env 
        CASE 
            WHEN current_database() ~ '_(prod|dev|int)$' THEN 'us' 
            ELSE substring( current_database() FROM '_([^_]*)$' )
        END
    ;

    IF v_env = 'uk' THEN 
        v_env := 'gb';
    END IF;

    PERFORM tools.assert(
        'current_environment__valid'
        , v_env IN ( 'us', 'gb', 'jv', 'au', 'ca', 'pi' )
        , 'Could not determine environment from ending of database name'
    );

    RETURN v_env;
END;
$_$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.current_environment() OWNER TO db_global_owner;

--
-- Name: currval(text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION currval(p_table text, p_column text) RETURNS bigint
    AS $_$

SELECT currval( pg_get_serial_sequence( $1, $2 ) )
  WHERE tools.assert( 'tools__currval__no_sequence'
            , pg_get_serial_sequence( $1, $2 ) IS NOT NULL
            , 'column ' || pg_catalog.quote_ident($2) || ' of table ' || pg_catalog.quote_ident($1) || ' does not have a sequence associated with it'
          )

$_$
    LANGUAGE sql;


ALTER FUNCTION tools.currval(p_table text, p_column text) OWNER TO db_global_owner;

--
-- Name: FUNCTION currval(p_table text, p_column text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION currval(p_table text, p_column text) IS 'Returns the current value of the sequence being used by p_table.p_column';


--
-- Name: currval(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION currval(p_table text) RETURNS bigint
    AS $_$
SELECT tools.currval( $1, 'id' )
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.currval(p_table text) OWNER TO db_global_owner;

--
-- Name: FUNCTION currval(p_table text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION currval(p_table text) IS 'Returns the current value of the sequence being used by p_table.id';


--
-- Name: exec(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION exec(sql text) RETURNS void
    AS $$
BEGIN                                              
    RAISE DEBUG 'Executing dynamic sql: %', sql;   
    EXECUTE sql;                                   
END;   
$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.exec(sql text) OWNER TO db_global_owner;

--
-- Name: FUNCTION exec(sql text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION exec(sql text) IS 'Executes an arbitrary SQL command.';


--
-- Name: exec(text, boolean); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION exec(sql text, echo boolean) RETURNS text
    AS $$
BEGIN                                              
    RAISE DEBUG 'Executing dynamic sql: %', sql;   
    EXECUTE sql;                                   

    IF echo THEN
        RETURN sql;
    ELSE
        RETURN NULL;
    END IF;
END;   
$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.exec(sql text, echo boolean) OWNER TO db_global_owner;

--
-- Name: exec_as(text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION exec_as(sql text, p_user text) RETURNS void
    AS $$
DECLARE
    v_current_user text := current_user;
BEGIN
    PERFORM set_config( 'role', p_user, true );
    EXECUTE tools.exec( sql );
    PERFORM set_config( 'role', v_current_user, true );
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.exec_as(sql text, p_user text) OWNER TO db_global_owner;

--
-- Name: FUNCTION exec_as(sql text, p_user text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION exec_as(sql text, p_user text) IS 'Execute an arbitrary SQL command using alternate privileges.';


--
-- Name: exec_as(text, text, boolean); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION exec_as(sql text, p_user text, echo boolean) RETURNS text
    AS $$
DECLARE
    v_current_user text := current_user;
    v_output text;
BEGIN
    PERFORM set_config( 'role', p_user, true );
    v_output := tools.exec( sql, echo );
    PERFORM set_config( 'role', v_current_user, true );
    RETURN v_output;
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.exec_as(sql text, p_user text, echo boolean) OWNER TO db_global_owner;

--
-- Name: FUNCTION exec_as(sql text, p_user text, echo boolean); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION exec_as(sql text, p_user text, echo boolean) IS 'Execute an arbitrary SQL command using alternate privileges.';


--
-- Name: exec_time(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION exec_time(sql text) RETURNS interval
    AS $$
DECLARE
  v_start timestamptz;
  v_end timestamptz;
BEGIN                                              
  RAISE DEBUG 'Executing dynamic sql: %', sql;   

  v_start := clock_timestamp();
  EXECUTE sql;                                   
  v_end := clock_timestamp();

  RETURN v_end - v_start;
END;   
$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.exec_time(sql text) OWNER TO db_global_owner;

--
-- Name: explode_array(anyarray); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION explode_array(in_array anyarray) RETURNS SETOF anyelement
    AS $_$
    SELECT ($1)[s] FROM generate_series(
        array_lower($1, 1), array_upper($1, 1)) AS s;
$_$
    LANGUAGE sql IMMUTABLE;


ALTER FUNCTION tools.explode_array(in_array anyarray) OWNER TO db_global_owner;

--
-- Name: foreign_key__exists(text, text[], text, text[]); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION foreign_key__exists(p_fk_table text, p_fk_columns text[], p_key_table text, p_key_columns text[]) RETURNS boolean
    AS $_$

SELECT count(*) > 0 FROM tools.foreign_key__get( $1, $2, $3, $4 )

$_$
    LANGUAGE sql;


ALTER FUNCTION tools.foreign_key__exists(p_fk_table text, p_fk_columns text[], p_key_table text, p_key_columns text[]) OWNER TO db_global_owner;

--
-- Name: FUNCTION foreign_key__exists(p_fk_table text, p_fk_columns text[], p_key_table text, p_key_columns text[]); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION foreign_key__exists(p_fk_table text, p_fk_columns text[], p_key_table text, p_key_columns text[]) IS 'Returns true if a foreign key actually exists';


--
-- Name: foreign_key__exists(text, text, text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION foreign_key__exists(p_fk_table text, p_fk_columns text, p_key_table text, p_key_columns text) RETURNS boolean
    AS $_$

SELECT count(*) > 0 FROM tools.foreign_key__get( $1, $2, $3, $4 )

$_$
    LANGUAGE sql;


ALTER FUNCTION tools.foreign_key__exists(p_fk_table text, p_fk_columns text, p_key_table text, p_key_columns text) OWNER TO db_global_owner;

--
-- Name: FUNCTION foreign_key__exists(p_fk_table text, p_fk_columns text, p_key_table text, p_key_columns text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION foreign_key__exists(p_fk_table text, p_fk_columns text, p_key_table text, p_key_columns text) IS 'Returns true if a foreign key actually exists';


--
-- Name: foreign_key__get(text, text[], text, text[]); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION foreign_key__get(p_fk_table text, p_fk_columns text[], p_key_table text, p_key_columns text[]) RETURNS SETOF pg_constraint
    AS $$

DECLARE
  v_fk_length CONSTANT int := tools.array_length( p_fk_columns, 1 );
  v_key_length CONSTANT int := tools.array_length( p_key_columns, 1 );

  v_fk_positions CONSTANT smallint[] := tools.attribute_positions( p_fk_table, p_fk_columns );
  v_key_positions CONSTANT smallint[] := tools.attribute_positions( p_key_table, p_key_columns );
  v_return pg_catalog.pg_constraint;
BEGIN
  IF v_fk_length <> v_key_length THEN 
    RAISE EXCEPTION 'Number of FK columns (%) must equal number of key columns (%)', v_fk_length, v_key_length;
  END IF;
  FOR v_return IN
  SELECT *
    FROM pg_constraint
    WHERE contype = 'f'
      AND confrelid = p_fk_table::regclass
      AND confkey = v_fk_positions
      AND conrelid = p_key_table::regclass
      AND conkey = v_key_positions
  LOOP
    RETURN NEXT v_return;
  END LOOP;
  RETURN;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.foreign_key__get(p_fk_table text, p_fk_columns text[], p_key_table text, p_key_columns text[]) OWNER TO db_global_owner;

--
-- Name: FUNCTION foreign_key__get(p_fk_table text, p_fk_columns text[], p_key_table text, p_key_columns text[]); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION foreign_key__get(p_fk_table text, p_fk_columns text[], p_key_table text, p_key_columns text[]) IS 'Returns the pg_constraint records for foreign keys to p_fk_table( p_fk_columns ) from p_key_table( p_key_columns ).';


--
-- Name: foreign_key__get(text, text, text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION foreign_key__get(p_fk_table text, p_fk_columns text, p_key_table text, p_key_columns text) RETURNS SETOF pg_constraint
    AS $_$
SELECT * FROM tools.foreign_key__get( $1, tools.string_or_array( $2 ), $3, tools.string_or_array( $4 ) )
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.foreign_key__get(p_fk_table text, p_fk_columns text, p_key_table text, p_key_columns text) OWNER TO db_global_owner;

--
-- Name: FUNCTION foreign_key__get(p_fk_table text, p_fk_columns text, p_key_table text, p_key_columns text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION foreign_key__get(p_fk_table text, p_fk_columns text, p_key_table text, p_key_columns text) IS 'String-based version of tools.goreign_key__get()';


--
-- Name: full_name_relation(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION full_name_relation(p_relation_name text) RETURNS text
    AS $_$SELECT tools.relation_full_name( $1 )$_$
    LANGUAGE sql;


ALTER FUNCTION tools.full_name_relation(p_relation_name text) OWNER TO db_global_owner;

--
-- Name: full_name_table(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION full_name_table(p_table_name text) RETURNS text
    AS $_$SELECT tools.table_full_name( $1 )$_$
    LANGUAGE sql;


ALTER FUNCTION tools.full_name_table(p_table_name text) OWNER TO db_global_owner;

--
-- Name: is_empty_or_null(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION is_empty_or_null(t text) RETURNS boolean
    AS $_$
-- Created by bug_192474__schema_as
SELECT coalesce( tools.trim_whitespace( $1 ), '' ) = ''
$_$
    LANGUAGE sql IMMUTABLE;


ALTER FUNCTION tools.is_empty_or_null(t text) OWNER TO db_global_owner;

--
-- Name: FUNCTION is_empty_or_null(t text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION is_empty_or_null(t text) IS 'Returns true if t is NULL, empty, or contains nothing but whitespace';


--
-- Name: londiste__drop_all_triggers(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION londiste__drop_all_triggers(p_table text) RETURNS void
    AS $$

DECLARE
  v_table CONSTANT text := tools.full_name_table( p_table );
BEGIN
  IF tools.londiste_subscriber( v_table ) THEN
    PERFORM londiste.subscriber_drop_all_table_triggers( v_table );

    -- We may have just yanked a londiste_no_dml trigger. If we did, put it back in place.
    PERFORM londiste.subscriber_restore_table_trigger(table_name, trigger_name)
      FROM londiste.subscriber_pending_triggers
      WHERE trigger_def ~ ANY ( ARRAY[
            'EXECUTE PROCEDURE tools.tg_londiste_no_dml()'
            , 'EXECUTE PROCEDURE object_monitor.tg_populate_queue()'
            , 'EXECUTE PROCEDURE pgq.sqltriga()'
            , 'EXECUTE PROCEDURE pgq.logtriga()'
            , 'EXECUTE PROCEDURE pgq.logutriga()'
        ] )
    ;
  END IF;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.londiste__drop_all_triggers(p_table text) OWNER TO db_global_owner;

--
-- Name: FUNCTION londiste__drop_all_triggers(p_table text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION londiste__drop_all_triggers(p_table text) IS 'Drops all triggers on a table, but only if it is being replicated.
  This function is dangerous; you *MUST* call tools.londiste__drop_all_triggers
  after this is done to put londiste back into a sane state! This is why this
  function is restricted to cnuadmin. It SHOULD be restricted to SU, but that
  would mean inheritance.process_child() would need to be security definer,
  which is probably even worse.';


--
-- Name: londiste__no_dml(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION londiste__no_dml() RETURNS void
    AS $$
BEGIN
  DELETE FROM londiste.subscriber_pending_triggers where trigger_name = 'londiste_no_dml';
  PERFORM tools.exec( CASE WHEN trigger_exists THEN 'DROP TRIGGER ' || tgname || ' ON ' || full_name                                                                                            
          ELSE 'CREATE TRIGGER londiste_no_dml AFTER INSERT OR DELETE OR UPDATE ON ' || full_name
              || ' FOR EACH STATEMENT EXECUTE PROCEDURE tools.tg_londiste_no_dml()'
          END, true )                                                                                                 
      FROM (SELECT full_name, tgname, tg.tgrelid IS NOT NULL AS trigger_exists, s.table_name IS NOT NULL AS subscriber
              FROM (SELECT schemaname as schema_name, tablename as table_name, quote_ident(schemaname) || '.' || quote_ident(tablename) as full_name
                      FROM pg_tables
                      ) t                                                                                   
                  LEFT JOIN pg_trigger tg ON tg.tgrelid=t.full_name::regclass AND tgname ~ 'londiste_no_dml'
                  LEFT JOIN londiste.subscriber_table s ON s.table_name = replace( full_name, 'public.', '' ) AND merge_state = 'ok'
              ) a                                     
      WHERE trigger_exists IS DISTINCT FROM subscriber
  ;
END;
$$
    LANGUAGE plpgsql
    SET statement_timeout TO '60000';


ALTER FUNCTION tools.londiste__no_dml() OWNER TO db_global_owner;

--
-- Name: londiste__pending_fkeys(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION londiste__pending_fkeys(p_child text, OUT from_table text, OUT to_table text, OUT fkey_name text, OUT fkey_def text) RETURNS SETOF record
    AS $_$

    SELECT from_table, to_table, fkey_name, fkey_def FROM londiste.subscriber_get_table_pending_fkeys( $1 )

$_$
    LANGUAGE sql SECURITY DEFINER;


ALTER FUNCTION tools.londiste__pending_fkeys(p_child text, OUT from_table text, OUT to_table text, OUT fkey_name text, OUT fkey_def text) OWNER TO db_global_owner;

--
-- Name: FUNCTION londiste__pending_fkeys(p_child text, OUT from_table text, OUT to_table text, OUT fkey_name text, OUT fkey_def text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION londiste__pending_fkeys(p_child text, OUT from_table text, OUT to_table text, OUT fkey_name text, OUT fkey_def text) IS 'Wrapper function that allows cnuadmin to run londiste.subscriber_get_table_pending_fkeys.';


--
-- Name: londiste__restore_all_triggers(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION londiste__restore_all_triggers(p_table text) RETURNS void
    AS $_$

DECLARE
  v_table CONSTANT text := tools.full_name_table( p_table );
BEGIN
  IF tools.londiste_subscriber( v_table ) THEN
    PERFORM londiste.subscriber_restore_all_table_triggers( tools.full_name_table( $1 ) );
  END IF;
END;

$_$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.londiste__restore_all_triggers(p_table text) OWNER TO db_global_owner;

--
-- Name: FUNCTION londiste__restore_all_triggers(p_table text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION londiste__restore_all_triggers(p_table text) IS 'Restores all triggers on a table, but only if it is being replicated.
  This function is dangerous; you *MUST* call tools.londiste__drop_all_triggers
  after this is done to put londiste back into a sane state! This is why this
  function is restricted to cnuadmin. It SHOULD be restricted to SU, but that
  would mean inheritance.process_child() would need to be security definer,
  which is probably even worse.';


--
-- Name: londiste_drop_slave_trigger(text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION londiste_drop_slave_trigger(p_table_name text, p_trigger_name text) RETURNS bigint
    AS $$
DECLARE
    v_current_search_path text := current_setting('search_path');
    v_ret bigint;
BEGIN
    /*
     * If we don't strip the search path the trigger definition captured by subscriber_drop_table_trigger
     * won't contain any schemas that were in search_path. That means that if search_path changes and we
     * try to remove the table from replication we can get an error.
     */
    PERFORM set_config('search_path', 'pg_catalog', false);

    SELECT INTO v_ret sum( londiste.subscriber_drop_table_trigger( table_name, p_trigger_name ) )
        FROM londiste.subscriber_table
        WHERE table_name = CASE WHEN strpos( p_table_name, '.' ) = 0
                                    THEN 'cnu.' || p_table_name
                                ELSE p_table_name
                            END
    ;

    PERFORM set_config('search_path', v_current_search_path, false);

    RETURN v_ret;
END;
$$
    LANGUAGE plpgsql SECURITY DEFINER;


ALTER FUNCTION tools.londiste_drop_slave_trigger(p_table_name text, p_trigger_name text) OWNER TO db_global_owner;

--
-- Name: londiste_subscriber(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION londiste_subscriber(p_table_name text) RETURNS boolean
    AS $_$
SELECT EXISTS( SELECT *
                    FROM londiste.subscriber_table
                    WHERE table_name = tools.full_name_table( $1 )
                )
    -- This oddity is here to throw an error if p_table_name doesn't actually exist or isn't a table
    WHERE tools.full_name_table( $1 ) IS NOT NULL
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.londiste_subscriber(p_table_name text) OWNER TO db_global_owner;

--
-- Name: londiste_trigger_refresh(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION londiste_trigger_refresh(p_table_name text) RETURNS bigint
    AS $_$
SELECT count( londiste.provider_refresh_trigger( queue_name, table_name ) )
    FROM londiste.provider_table
    WHERE table_name = CASE WHEN strpos( $1, '.' ) = 0
                                THEN 'cnu.' || $1
                            ELSE $1
                        END
;
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.londiste_trigger_refresh(p_table_name text) OWNER TO db_global_owner;

--
-- Name: min_assert_level(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION min_assert_level() RETURNS integer
    AS $$
SELECT CASE WHEN current_database() LIKE '%_prod'
            THEN 0
        ELSE coalesce( tools.setting_i( 'Minimum assert level' ), 0 )
        END
$$
    LANGUAGE sql;


ALTER FUNCTION tools.min_assert_level() OWNER TO db_global_owner;

--
-- Name: next_stats_date(text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION next_stats_date(p_stats_table text, p_stats_view text) RETURNS date
    AS $_$SELECT tools.next_stats_date( $1, $2, 0 )$_$
    LANGUAGE sql STABLE SECURITY DEFINER;


ALTER FUNCTION tools.next_stats_date(p_stats_table text, p_stats_view text) OWNER TO db_global_owner;

--
-- Name: object__get_oid(text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION object__get_oid(p_object_name text, p_object_type text) RETURNS oid
    AS $_$
DECLARE
    r_type tools.object_types;
    v_relation_type text;
    sql text;
    v_oid oid;
BEGIN
    SELECT INTO r_type * FROM tools.object_types WHERE lower(object_type) = lower(p_object_type);
    IF NOT FOUND THEN
        RAISE EXCEPTION 'Unknown object type: %', p_object_type;
    END IF;

    IF r_type.cast_name = 'regclass' THEN
        v_oid := p_object_name::regclass::oid;
        SELECT INTO v_relation_type lower( tools.relation_type( relkind ) ) FROM pg_class WHERE oid = v_oid;

        PERFORM tools.assert( 'tools__object__get_oid__relkind_check'
                , v_relation_type = r_type.object_type
                , 'Object ' || p_object_name || ' is of type ' || v_relation_type || ', not ' || r_type.object_type
            );

        RETURN v_oid;

    ELSEIF r_type.cast_name = 'regtype' THEN
        RETURN p_object_name::regtype::oid;
    ELSEIF r_type.cast_name = 'regprocedure' THEN
        RETURN p_object_name::regprocedure::oid;
    ELSEIF r_type.cast_name = 'regoperator' THEN
        RETURN p_object_name::regoperator::oid;
    ELSE
        sql = 'SELECT oid
                FROM pg_catalog.' || quote_ident( lower( r_type.catalog_table ) ) || '
                WHERE ' || coalesce( r_type.name_prefix, substring( lower( r_type.catalog_table ) from 4 for 3 ) ) || 'name'
                    || $$ = '$$ || p_object_name || $$'$$
        ;
        RAISE DEBUG 'Executing dynamic query % into v_oid', sql;
        EXECUTE sql INTO v_oid;
        RETURN v_oid;
    END IF;
END
$_$
    LANGUAGE plpgsql STABLE;


ALTER FUNCTION tools.object__get_oid(p_object_name text, p_object_type text) OWNER TO db_global_owner;

--
-- Name: FUNCTION object__get_oid(p_object_name text, p_object_type text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION object__get_oid(p_object_name text, p_object_type text) IS 'Returns the OID for an object based on its type.';


--
-- Name: parameter_replace(text, text[]); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION parameter_replace(p_template text, p_parameters text[]) RETURNS text
    AS $$
DECLARE
    v_name text := 'tools.parameter_replace';
    v_out text := p_template;
BEGIN
    PERFORM tools.assert( array_lower( p_parameters, 2 ) = 1, 'Lower bound of second dimension of p_parameters must be 1' );
    PERFORM tools.assert( array_upper( p_parameters, 2 ) = 2, 'Upper bound of second dimension of p_parameters must be 2' );

    FOR i IN array_lower( p_parameters, 1 ) .. array_upper( p_parameters, 1 ) LOOP
        RAISE DEBUG '%: Replacing % with %', v_name, '%' || p_parameters[i][1] || '%', p_parameters[i][2];
        v_out := replace( v_out, '%' || p_parameters[i][1] || '%', p_parameters[i][2] );
    END LOOP;

    RETURN v_out;
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.parameter_replace(p_template text, p_parameters text[]) OWNER TO db_global_owner;

--
-- Name: patch(text, boolean); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION patch(p_patch_file text, p_no_error boolean) RETURNS void
    AS $_$SELECT cabar.patch( 'cnuapp', $1 )$_$
    LANGUAGE sql;


ALTER FUNCTION tools.patch(p_patch_file text, p_no_error boolean) OWNER TO db_global_owner;

--
-- Name: patch(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION patch(p_patch_file text) RETURNS void
    AS $_$
SELECT tools.patch( $1, false )
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.patch(p_patch_file text) OWNER TO db_global_owner;

--
-- Name: patch_get_patch_plus_deps(text, integer, boolean); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION patch_get_patch_plus_deps(p_patch_file text, p_level integer, p_include_applied boolean, OUT patch_file text, OUT level integer) RETURNS SETOF record
    AS $$
DECLARE
    v_patch     tools.schema_patches;
    i int;
    rec record;
BEGIN
    RAISE DEBUG 'patch_get_patch_plus_deps( %, % )', p_patch_file, p_level;

    SELECT INTO v_patch *
        FROM tools.schema_patches p
        WHERE p.patch_file = p_patch_file
    ;
    IF NOT FOUND THEN
        RAISE EXCEPTION 'Patch file % not found in tools.schema_patches', p_patch_file;
    END IF;

    patch_file := p_patch_file;
    level := p_level;
    RETURN NEXT;

    IF v_patch.depends_on IS NOT NULL
      AND ( v_patch.applied_at IS NULL OR p_include_applied )
    THEN
        FOR i IN array_lower(v_patch.depends_on, 1) .. array_upper(v_patch.depends_on, 1) LOOP
            FOR rec IN SELECT * FROM tools.patch_get_patch_plus_deps( v_patch.depends_on[i], p_level - 1, p_include_applied ) LOOP
                patch_file := rec.patch_file;
                level := rec.level;
                RETURN NEXT;
            END LOOP;
        END LOOP;
    END IF;

    RETURN;
END;
$$
    LANGUAGE plpgsql STABLE;


ALTER FUNCTION tools.patch_get_patch_plus_deps(p_patch_file text, p_level integer, p_include_applied boolean, OUT patch_file text, OUT level integer) OWNER TO db_global_owner;

--
-- Name: patch_get_patches(text[]); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION patch_get_patches(p_patch_files text[], OUT patch_file text, OUT level integer) RETURNS SETOF record
    AS $_$
  SELECT * FROM tools.patch_get_patches( $1, false )
$_$
    LANGUAGE sql STABLE;


ALTER FUNCTION tools.patch_get_patches(p_patch_files text[], OUT patch_file text, OUT level integer) OWNER TO db_global_owner;

--
-- Name: patch_get_patches(text[], boolean); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION patch_get_patches(p_patch_files text[], p_include_applied boolean, OUT patch_file text, OUT level integer) RETURNS SETOF record
    AS $$
DECLARE
  v_patch     tools.schema_patches;
  i int;
  rec record;
BEGIN
  FOR i IN array_lower(p_patch_files, 1) .. array_upper(p_patch_files, 1) LOOP
    SELECT INTO v_patch *
      FROM tools.schema_patches p
      WHERE p.patch_file = p_patch_files[i]
    ;
    IF NOT FOUND THEN
      RAISE EXCEPTION 'Patch file % not found in tools.schema_patches', p_patch_files[i];
    END IF;

    -- Note that we need to reference the table otherwise we get parameter conflicts
    FOR rec IN SELECT p.patch_file, min(p.level)
                  FROM tools.patch_get_patch_plus_deps( v_patch.patch_file, 0, p_include_applied ) p
                  GROUP BY p.patch_file
    LOOP
      patch_file := rec.patch_file;
      level := rec.min;
      RETURN NEXT;
    END LOOP;
  END LOOP;

  RETURN;
END;
$$
    LANGUAGE plpgsql STABLE;


ALTER FUNCTION tools.patch_get_patches(p_patch_files text[], p_include_applied boolean, OUT patch_file text, OUT level integer) OWNER TO db_global_owner;

--
-- Name: patch_insert(text, text[]); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION patch_insert(p_patch_file text, p_depends_on text[]) RETURNS void
    AS $$
BEGIN
  PERFORM cabar.patch_insert( p_patch_file, 'cnuapp', p_depends_on );
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.patch_insert(p_patch_file text, p_depends_on text[]) OWNER TO db_global_owner;

--
-- Name: primary_unique_constraint__exists(text, text[]); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION primary_unique_constraint__exists(p_table text, p_columns text[]) RETURNS boolean
    AS $_$

SELECT count(*) > 0 FROM tools.primary_unique_constraint__get( $1, $2 )

$_$
    LANGUAGE sql;


ALTER FUNCTION tools.primary_unique_constraint__exists(p_table text, p_columns text[]) OWNER TO db_global_owner;

--
-- Name: FUNCTION primary_unique_constraint__exists(p_table text, p_columns text[]); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION primary_unique_constraint__exists(p_table text, p_columns text[]) IS 'Returns true if a specified constraint actually exists';


--
-- Name: primary_unique_constraint__exists(text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION primary_unique_constraint__exists(p_table text, p_columns text) RETURNS boolean
    AS $_$

SELECT count(*) > 0 FROM tools.primary_unique_constraint__get( $1, $2 )

$_$
    LANGUAGE sql;


ALTER FUNCTION tools.primary_unique_constraint__exists(p_table text, p_columns text) OWNER TO db_global_owner;

--
-- Name: FUNCTION primary_unique_constraint__exists(p_table text, p_columns text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION primary_unique_constraint__exists(p_table text, p_columns text) IS 'Returns true if a specified constraint actually exists';


--
-- Name: primary_unique_constraint__get(text, text[]); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION primary_unique_constraint__get(p_table text, p_columns text[]) RETURNS SETOF pg_constraint
    AS $_$

SELECT *
  FROM pg_constraint
  WHERE conrelid = $1::regclass
    AND conkey = tools.attribute_positions( $1, $2 )
    AND contype IN ( 'p', 'u' )

$_$
    LANGUAGE sql;


ALTER FUNCTION tools.primary_unique_constraint__get(p_table text, p_columns text[]) OWNER TO db_global_owner;

--
-- Name: FUNCTION primary_unique_constraint__get(p_table text, p_columns text[]); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION primary_unique_constraint__get(p_table text, p_columns text[]) IS 'Returns a constraint record for a primary key or unique constraint';


--
-- Name: primary_unique_constraint__get(text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION primary_unique_constraint__get(p_table text, p_columns text) RETURNS SETOF pg_constraint
    AS $_$

SELECT *
  FROM pg_constraint
  WHERE conrelid = $1::regclass
    AND conkey = tools.attribute_positions( $1, $2 )
    AND contype IN ( 'p', 'u' )

$_$
    LANGUAGE sql;


ALTER FUNCTION tools.primary_unique_constraint__get(p_table text, p_columns text) OWNER TO db_global_owner;

--
-- Name: FUNCTION primary_unique_constraint__get(p_table text, p_columns text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION primary_unique_constraint__get(p_table text, p_columns text) IS 'Returns a constraint record for a primary key or unique constraint';


--
-- Name: proxy_current_database(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION proxy_current_database() RETURNS name
    AS $$
  -- Original in bug_252483_fix_asserts_for_dump
  SELECT current_database()
$$
    LANGUAGE sql IMMUTABLE;


ALTER FUNCTION tools.proxy_current_database() OWNER TO db_global_owner;

--
-- Name: proxynow(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION proxynow() RETURNS timestamp with time zone
    AS $$
    SELECT now();
  $$
    LANGUAGE sql;


ALTER FUNCTION tools.proxynow() OWNER TO db_global_owner;

--
-- Name: raise(text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION raise(p_level text, p_message text) RETURNS boolean
    AS $$
BEGIN
    IF upper( p_level ) = 'DEBUG' THEN
        RAISE DEBUG '%', p_message;
    ELSEIF upper( p_level ) = 'LOG' THEN
        RAISE LOG '%', p_message;
    ELSEIF upper( p_level ) = 'INFO' THEN
        RAISE INFO '%', p_message;
    ELSEIF upper( p_level ) = 'NOTICE' THEN
        RAISE NOTICE '%', p_message;
    ELSEIF upper( p_level ) = 'WARNING' THEN
        RAISE WARNING '%', p_message;
    ELSEIF upper( p_level ) = 'EXCEPTION' THEN
        RAISE EXCEPTION '%', p_message;
    ELSE
        RAISE EXCEPTION 'Invalid level % passed into tools.raise';
    END IF;

    RETURN true;
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.raise(p_level text, p_message text) OWNER TO db_global_owner;

--
-- Name: raise_exception(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION raise_exception(p_message text) RETURNS boolean
    AS $$
BEGIN
    RAISE EXCEPTION '%', p_message;
    RETURN false;
END;
$$
    LANGUAGE plpgsql IMMUTABLE;


ALTER FUNCTION tools.raise_exception(p_message text) OWNER TO db_global_owner;

--
-- Name: reject_update(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION reject_update() RETURNS trigger
    AS $_$
DECLARE
    vq_field text;
    sql text;
BEGIN
    -- Triggers calling this function must be written so that they only fire if the field in question changes!

    -- TODO: Make this support passing in multiple fields

    PERFORM tools.assert( 'tools.reject_update NARGS', TG_NARGS = 1, 'cnu.reject_update must be passed exactly one field' );

    vq_field := quote_ident( TG_ARGV[0] );
    sql := $$SELECT tools.assert( NOT NEW.$$ || vq_field || ' = OLD.' || vq_field || $$
            , 'Updates not allowed on $$ || TG_RELNAME || '.' || vq_field || $$')$$
    ;

    PERFORM tools.exec(sql);
END;
$_$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.reject_update() OWNER TO db_global_owner;

--
-- Name: relation_full_name(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION relation_full_name(p_relation_name text) RETURNS text
    AS $_$SELECT schema_name || '.' || relation_name FROM tools.relation_schema_and_name( $1 )$_$
    LANGUAGE sql;


ALTER FUNCTION tools.relation_full_name(p_relation_name text) OWNER TO db_global_owner;

--
-- Name: relation_schema_and_name(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION relation_schema_and_name(p_relation_name text, OUT schema_name text, OUT relation_name text) RETURNS record
    AS $_$
SELECT quote_ident(nspname),  quote_ident(relname)
  FROM pg_class c
    JOIN pg_namespace n ON n.oid = c.relnamespace
  WHERE c.oid = $1::regclass
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.relation_schema_and_name(p_relation_name text, OUT schema_name text, OUT relation_name text) OWNER TO db_global_owner;

--
-- Name: relation_type("char"); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION relation_type(p_relkind "char") RETURNS text
    AS $_$
SELECT CASE $1
        WHEN 'S' THEN 'Sequence'
        WHEN 'r' THEN 'Table'
        WHEN 'i' THEN 'Index'
        WHEN 'v' THEN 'View'
        WHEN 'c' THEN 'Composite Type'
        WHEN 't' THEN 'TOAST Table'
        ELSE CASE WHEN tools.raise_exception( 'Unknown relkind ' || $1 ) THEN 'boom!' ELSE 'OMGWTFBBQ?!!?!' END
    END AS object_type
$_$
    LANGUAGE sql IMMUTABLE STRICT;


ALTER FUNCTION tools.relation_type(p_relkind "char") OWNER TO db_global_owner;

--
-- Name: FUNCTION relation_type(p_relkind "char"); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION relation_type(p_relkind "char") IS 'Given a pg_class.relkind entry, returns a text description';


--
-- Name: secure_table(text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION secure_table(p_table_name text, p_secure_columns text) RETURNS void
    AS $_$
    SELECT tools.secure_table($1, $2, '', 'cnu', true)
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.secure_table(p_table_name text, p_secure_columns text) OWNER TO db_global_owner;

--
-- Name: FUNCTION secure_table(p_table_name text, p_secure_columns text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION secure_table(p_table_name text, p_secure_columns text) IS 'Wrapper for tools.secure_table(p_table_name text, p_secure_columns text, p_exclude_columns text, p_hashname text, p_id boolean). Assumes p_hashname of ''cnu'' and p_id is true.';


--
-- Name: seed_description(text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION seed_description(p_full_name text, p_description text) RETURNS text
    AS $$
DECLARE
    c_regex CONSTANT text := chr(10) || E'?\\W*This is a seed [^\. ]*.';

    c_object_oid CONSTANT oid := p_full_name::regclass::oid;
    v_object_type text;
    v_description text := btrim( coalesce( p_description, obj_description(c_object_oid) ) );
BEGIN
    SELECT INTO v_object_type lower(relation_type) FROM tools.seed_objects_v WHERE relation_oid = c_object_oid;
    IF NOT FOUND THEN
        RETURN regexp_replace( v_description, c_regex, '', 'ig' );
    ELSE
        IF v_description ~* c_regex THEN
            RETURN v_description;
        ELSE
            RETURN CASE WHEN coalesce( v_description, '' ) = '' THEN '' ELSE v_description || ' ' END || 'This is a seed ' || v_object_type || '.';
        END IF;
    END IF;
END;
$$
    LANGUAGE plpgsql STABLE;


ALTER FUNCTION tools.seed_description(p_full_name text, p_description text) OWNER TO db_global_owner;

--
-- Name: FUNCTION seed_description(p_full_name text, p_description text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION seed_description(p_full_name text, p_description text) IS 'Given a regclass object and an optional description, returns what the
object''s description should be.

If p_description isn''t supplied, the objects current description is used. The
function then ensures that the description mentions that the object is a seed
if it is. If the object is not a seed, any mention of it being seed is
removed.';


--
-- Name: seed_object(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION seed_object(p_table text) RETURNS boolean
    AS $_$
SELECT EXISTS( SELECT * FROM tools.seed_objects_v WHERE relation_oid = $1::regclass )
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.seed_object(p_table text) OWNER TO db_global_owner;

--
-- Name: seed_object__add(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION seed_object__add(p_relation_name text) RETURNS void
    AS $_$SELECT tools.seed_object__add( $1, NULL )$_$
    LANGUAGE sql;


ALTER FUNCTION tools.seed_object__add(p_relation_name text) OWNER TO db_global_owner;

--
-- Name: FUNCTION seed_object__add(p_relation_name text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION seed_object__add(p_relation_name text) IS 'Marks a table or sequence as being seed.';


--
-- Name: seed_object__add(text, integer); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION seed_object__add(p_relation_name text, p_dump_order integer) RETURNS void
    AS $_$
DECLARE
  v_dummy int;
  /*
   * If we get handed a relation that doesn't exist this cast will throw an exception for us
   * Do not remove this without changing the SQL that we're building below, otherwise we 
   * risk injection
   */
  c_relation_oid CONSTANT oid := p_relation_name::regclass::oid;
  c_dump_order CONSTANT int := coalesce( p_dump_order, 0 );
  v_schema_name text;
  v_relation_name text;
  v_relation_type text;
BEGIN
  PERFORM tools.assert( 'tools__seed_object__add__dump_order_out_of_range'
    , c_dump_order BETWEEN -50 AND 50
    , 'Unable to mark ' || v_relation_name || ' as a seed ' || lower( v_relation_type ) || ': p_dump_order( = '
      || tools.textcoalesce( p_dump_order ) || ' ) must be NULL or between -50 and 50'
  );

  SELECT INTO v_schema_name, v_relation_name, v_relation_type
      quote_ident(nspname), quote_ident(relname)
      , tools.relation_type( c.relkind ) AS relation_type
    FROM pg_namespace n
      JOIN pg_class c ON n.oid = c.relnamespace
    WHERE c.oid = c_relation_oid
  ;
  PERFORM tools.assert( 'tools__seed_object__add__no_system_schema'
    , v_schema_name !~ '^pg_'
    , 'Unable to mark ' || v_relation_name || ' as a seed ' || lower( v_relation_type ) || ' because it exists in a system schema ('
      || v_schema_name || ')'
  );

  IF EXISTS( SELECT * FROM pg_user WHERE usename = v_schema_name ) THEN
    RAISE EXCEPTION '% % can not be marked as a seed % because it exists in a user schema (%)'
      , v_relation_type, v_relation_name, lower(v_relation_type), v_schema_name
    ;
  END IF;

  -- If the object is already a seed grab a lock so no one else can modify it until we're done
  v_dummy := NULL;
  FOR i IN 1..10 LOOP
    BEGIN
      INSERT INTO tools.seed_objects( schema_name, object_name, dump_order ) VALUES ( v_schema_name, v_relation_name, c_dump_order );
      v_dummy := 1;
      EXIT;
    EXCEPTION WHEN unique_violation THEN
    END;

    UPDATE tools.seed_objects
      SET dump_order = c_dump_order
      WHERE schema_name = v_schema_name
        AND object_name = v_relation_name
    ;
    IF FOUND THEN
      v_dummy := 1;
      EXIT;
    END IF;
  END LOOP;
  IF v_dummy IS NULL THEN
    RAISE EXCEPTION 'Overflow in lock loop';
  END IF;

  PERFORM tools.comment__set( NULL, v_schema_name, v_relation_name, v_relation_type, NULL );

  /* It's pointless to mark sequences as seeds if they're in pg_depend, because pg_dump will dump those for us.
   * What we do care about is the case where a sequence isn't marked as being dependent. Of course, we should only
   * worry about this if we're marking a seed table.
   */
  IF v_relation_type = 'Table' THEN
    PERFORM tools.seed_object__add( sequence_name, c_dump_order )
        -- Get everything that looks like a sequence default
        FROM ( SELECT adrelid, adnum, substring( pg_catalog.pg_get_expr(adbin, adrelid) from $$nextval\({1,2}\s*'([^']*)$$ ) AS sequence_name
              FROM pg_catalog.pg_attrdef
              WHERE adrelid = c_relation_oid
            ) a
        WHERE sequence_name IS NOT NULL
          -- We only want sequences that don't have a pg_depend entry. This subselect is from pg_dump.c
          AND NOT EXISTS( SELECT *
              FROM pg_depend d
              WHERE d.refclassid = 'pg_class'::regclass
                AND d.refobjid = a.adrelid
                AND d.classid = 'pg_class'::regclass
                AND d.objid = sequence_name::regclass
                AND d.objsubid = 0
                AND d.deptype = 'a'
          )
    ;
  END IF;
END;
$_$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.seed_object__add(p_relation_name text, p_dump_order integer) OWNER TO db_global_owner;

--
-- Name: seed_object__remove(text, text, boolean); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION seed_object__remove(p_schema_name text, p_relation_name text, p_force boolean) RETURNS void
    AS $$

DECLARE
  v_force boolean := COALESCE(p_force,false);
BEGIN
  DELETE FROM tools.seed_objects
    WHERE schema_name = p_schema_name
      AND object_name = p_relation_name
  ;
  IF NOT v_force THEN
    PERFORM tools.assert(
      'seed_object_entry__exists'
      , FOUND
      , 'Table ' || p_schema_name || '.' || p_relation_name || ' not found in tools.seed_objects!'
    );
  END IF;
END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.seed_object__remove(p_schema_name text, p_relation_name text, p_force boolean) OWNER TO db_global_owner;

--
-- Name: FUNCTION seed_object__remove(p_schema_name text, p_relation_name text, p_force boolean); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION seed_object__remove(p_schema_name text, p_relation_name text, p_force boolean) IS 'Removes an object from the list of seed objects.
    
Parameters:
p_schema_name
Name of schema that contains p_relation_name

p_relation_name
Name of relation to remove from list of seed objects.

p_force
If p_force is false and no rows are found, an exception is raised.
';


--
-- Name: seed_object__remove(text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION seed_object__remove(p_schema_name text, p_relation_name text) RETURNS void
    AS $_$

SELECT tools.seed_object__remove($1,$2, false);

$_$
    LANGUAGE sql;


ALTER FUNCTION tools.seed_object__remove(p_schema_name text, p_relation_name text) OWNER TO db_global_owner;

--
-- Name: FUNCTION seed_object__remove(p_schema_name text, p_relation_name text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION seed_object__remove(p_schema_name text, p_relation_name text) IS 'Removes an object from the list of seed objects.

Parameters:
p_schema_name
Name of schema that contains p_relation_name

p_relation_name
Name of relation to remove from list of seed objects.
';


--
-- Name: seed_table__add(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION seed_table__add(p_relation_name text) RETURNS void
    AS $_$SELECT tools.seed_object__add( $1 )$_$
    LANGUAGE sql;


ALTER FUNCTION tools.seed_table__add(p_relation_name text) OWNER TO db_global_owner;

--
-- Name: FUNCTION seed_table__add(p_relation_name text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION seed_table__add(p_relation_name text) IS 'DEPRECATED, use tools.seed_object_add instead. Marks a table as being seed.';


--
-- Name: session__clean(text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION session__clean(mode text, source_keys text) RETURNS void
    AS $_X$
	BEGIN { strict->import(); }
	
	my $mode = shift;
	my $key_string = shift;
	my (@source_keys, @target_keys);
	
	# parse the string and form the keys list.
	foreach my $key (split (/,*\s/, $key_string)) {
		push @source_keys, $key;
	}
	
	if (lc($mode) eq "delete") {
		@target_keys = @source_keys;
	} elsif (lc($mode) eq "keep") {
		# get only those keys that are not in the list of source_keys
		my %source_hash = map {$_ => 1} @source_keys;
		@target_keys = grep $source_hash{$_} != 1, (keys %_SHARED);
	} else {
		die "Invalid value for the \'mode\' argument: $mode";
	}
	
	# remove elements from the hash
	foreach (@target_keys) {
		delete $_SHARED{$_};
	}	
$_X$
    LANGUAGE plperl SECURITY DEFINER;


ALTER FUNCTION tools.session__clean(mode text, source_keys text) OWNER TO db_global_owner;

--
-- Name: session__get(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION session__get(p_key text) RETURNS text
    AS $_X$
	BEGIN { strict->import(); }
	
	my $p_key = shift;
	my $value = $_SHARED{$p_key};
	if (ref $value) {
		die sprintf "%s{%s} is a reference", '$_SHARED', "$p_key";
	} 
	return $value;
$_X$
    LANGUAGE plperl SECURITY DEFINER;


ALTER FUNCTION tools.session__get(p_key text) OWNER TO db_global_owner;

--
-- Name: session__get(text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION session__get(p_key1 text, p_key2 text) RETURNS text
    AS $_X$
	BEGIN { strict->import(); }
	
	my $p_key1 = shift;
	my $p_key2 = shift;
	
	my $hashref = $_SHARED{$p_key1};
	return undef if (!defined $hashref);
	if (ref $hashref eq "HASH") {
		return $_SHARED{$p_key1}{$p_key2} unless ref $_SHARED{$p_key1}{$p_key2};
		die sprintf "%s{%s}{%s} is a reference", '%_SHARED', "$p_key1", "$p_key2";
	} else {
		die sprintf "%s{%s} is not a hash reference", '$_SHARED', "$p_key1";
	}
$_X$
    LANGUAGE plperl SECURITY DEFINER;


ALTER FUNCTION tools.session__get(p_key1 text, p_key2 text) OWNER TO db_global_owner;

--
-- Name: session__get(text, text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION session__get(p_key1 text, p_key2 text, p_key3 text) RETURNS text
    AS $_X$
	BEGIN { strict->import(); }
	
	my $p_key1 = shift;
	my $p_key2 = shift;
	my $p_key3 = shift;
	
	return undef if (!defined $_SHARED{$p_key1});
	my $hashref = $_SHARED{$p_key1}{$p_key2};
	return undef if (!defined $hashref);
	if (ref $hashref eq "HASH") {
		# note that unlike 2 arguments _get we don't check whether the result is a reference
		# due to the fact that we don't provide an API for creating 3-level nested hashes.
		return $_SHARED{$p_key1}{$p_key2}{$p_key3};
	} else {
		die sprintf "%s{%s}{%s} is not a hash reference", '$_SHARED', "$p_key1", "$p_key2";
	}
$_X$
    LANGUAGE plperl SECURITY DEFINER;


ALTER FUNCTION tools.session__get(p_key1 text, p_key2 text, p_key3 text) OWNER TO db_global_owner;

--
-- Name: session__list(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION session__list() RETURNS text
    AS $_X$
	BEGIN { strict->import(); }
	
	my $result = "";
	
	foreach my $key (sort keys %_SHARED) {
		if (! ref $_SHARED{$key}) {
			$result.= $key." => ".$_SHARED{$key}."\n";
		} elsif (ref $_SHARED{$key} eq "HASH") {
			foreach my $key2 (sort keys %{$_SHARED{$key}}) {
				if (! ref $_SHARED{$key}{$key2}) {
					$result.= $key. ", ".$key2." => ".$_SHARED{$key}{$key2}."\n";
				} else {
					foreach my $key3 (sort keys %{$_SHARED{$key}{$key2}}) {
						$result.= $key.", ".$key2.", ".$key3." => ".$_SHARED{$key}{$key2}{$key3}."\n";
					}
				}
			}
		} else {
			die sprintf "Reference %s{%s} is not a hash reference", '$_SHARED', $key;
		}
	}
	return $result;
$_X$
    LANGUAGE plperl SECURITY DEFINER;


ALTER FUNCTION tools.session__list() OWNER TO db_global_owner;

--
-- Name: session__reset(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION session__reset() RETURNS void
    AS $$
	BEGIN { strict->import(); }
	
	%_SHARED = ();
	
$$
    LANGUAGE plperl SECURITY DEFINER;


ALTER FUNCTION tools.session__reset() OWNER TO db_global_owner;

--
-- Name: session__set(text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION session__set(p_key text, value text) RETURNS void
    AS $_X$
	BEGIN { strict->import(); }
	
	my $p_key = shift;
	my $value = shift;
	
	# remove hash element if NULL is passed as a value
	if (!defined $value) {
		delete $_SHARED{$p_key};
		return;
	}
	
	# perform assignment only if the hash slot holds a value and not a hash reference
	if (exists $_SHARED{$p_key} && ref $_SHARED{$p_key}) {
		die sprintf '%s{%s} is a reference', '%_SHARED', "$p_key";
	}
	
	$_SHARED{$p_key} = $value;
$_X$
    LANGUAGE plperl SECURITY DEFINER;


ALTER FUNCTION tools.session__set(p_key text, value text) OWNER TO db_global_owner;

--
-- Name: session__set(text, text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION session__set(p_key1 text, p_key2 text, value text) RETURNS void
    AS $_X$
	BEGIN { strict->import(); }
	
	my $p_key1 = shift;
	my $p_key2 = shift;
	my $value = shift;
	
	if (exists $_SHARED{$p_key1} && ref $_SHARED{$p_key1} ne "HASH") {	
		die sprintf '%s{%s} is not a hash reference', '%_SHARED', "$p_key1";
	}
	
	# remove hash element if NULL is passed as a value
	if (!defined $value) {
		delete $_SHARED{$p_key1}{$p_key2};
		return;
	}
	
	# perform assignment only if the hash slot holds a value and not a hash reference
	if (exists $_SHARED{$p_key1}{$p_key2} && ref $_SHARED{$p_key1}{$p_key2}) {	
		die sprintf '%s{%s}{%s} is a reference', '%_SHARED', "$p_key1", "$p_key2";
	}
	
	if (!exists $_SHARED{$p_key1}) {
		$_SHARED{$p_key1} = {$p_key2 => $value};
	} else {
		$_SHARED{$p_key1}{$p_key2} = $value ;
	}
$_X$
    LANGUAGE plperl SECURITY DEFINER;


ALTER FUNCTION tools.session__set(p_key1 text, p_key2 text, value text) OWNER TO db_global_owner;

--
-- Name: session__set(text, text, text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION session__set(p_key1 text, p_key2 text, p_key3 text, value text) RETURNS void
    AS $_X$
	BEGIN { strict->import(); }
	
	my $p_key1 = shift;
	my $p_key2 = shift;
	my $p_key3 = shift;
	my $value = shift;
	
	if (exists $_SHARED{$p_key1}{$p_key2} && ref $_SHARED{$p_key1}{$p_key2} ne "HASH") {	
		die sprintf '%s{%s}{%s} is not a hash reference', '%_SHARED', "$p_key1", "$p_key2";
	}
	
	# remove hash element if NULL is passed as a value
	if (!defined $value) {
		delete $_SHARED{$p_key1}{$p_key2}{$p_key3};
		return;
	} elsif (!exists $_SHARED{$p_key1}) {
		$_SHARED{$p_key1} = {$p_key2 => {$p_key3 => $value}};
	} elsif (!exists $_SHARED{$p_key1}{$p_key2}) {
		$_SHARED{$p_key1}{$p_key2} = {$p_key3 => $value};
	} else {
		$_SHARED{$p_key1}{$p_key2}{$p_key3} = $value;
	}
$_X$
    LANGUAGE plperl SECURITY DEFINER;


ALTER FUNCTION tools.session__set(p_key1 text, p_key2 text, p_key3 text, value text) OWNER TO db_global_owner;

--
-- Name: setting(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION setting(text) RETURNS SETOF settings
    AS $_$SELECT * FROM setting.setting($1)$_$
    LANGUAGE sql STABLE SECURITY DEFINER;


ALTER FUNCTION tools.setting(text) OWNER TO db_global_owner;

--
-- Name: setting_b(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION setting_b(text) RETURNS boolean
    AS $_$SELECT b FROM tools.setting($1)$_$
    LANGUAGE sql STABLE SECURITY DEFINER;


ALTER FUNCTION tools.setting_b(text) OWNER TO db_global_owner;

--
-- Name: setting_f(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION setting_f(text) RETURNS double precision
    AS $_$SELECT f FROM tools.setting($1)$_$
    LANGUAGE sql STABLE SECURITY DEFINER;


ALTER FUNCTION tools.setting_f(text) OWNER TO db_global_owner;

--
-- Name: setting_i(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION setting_i(text) RETURNS integer
    AS $_$SELECT i FROM tools.setting($1)$_$
    LANGUAGE sql STABLE SECURITY DEFINER;


ALTER FUNCTION tools.setting_i(text) OWNER TO db_global_owner;

--
-- Name: setting_t(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION setting_t(text) RETURNS text
    AS $_$SELECT t FROM tools.setting($1)$_$
    LANGUAGE sql STABLE SECURITY DEFINER;


ALTER FUNCTION tools.setting_t(text) OWNER TO db_global_owner;

--
-- Name: sinh(double precision); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION sinh(double precision) RETURNS double precision
    AS $_$
    SELECT 0.5 * ( exp($1) - exp(-$1) ); 
  $_$
    LANGUAGE sql IMMUTABLE;


ALTER FUNCTION tools.sinh(double precision) OWNER TO db_global_owner;

--
-- Name: split_schema(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION split_schema(p_full_name text, OUT p_schema_name text, OUT p_object_name text) RETURNS record
    AS $$
BEGIN
    p_schema_name := regexp_replace( p_full_name, E'\\..*', '' );
    p_object_name := regexp_replace( p_full_name, E'.*\\.', '' );

    IF p_schema_name = p_object_name THEN
        p_schema_name := NULL;
    END IF;
    RETURN;
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.split_schema(p_full_name text, OUT p_schema_name text, OUT p_object_name text) OWNER TO db_global_owner;

--
-- Name: FUNCTION split_schema(p_full_name text, OUT p_schema_name text, OUT p_object_name text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION split_schema(p_full_name text, OUT p_schema_name text, OUT p_object_name text) IS 'Splits an object name into schema and object components.

    p_full_name         text
    Object name to split.

    OUT p_schema_name text
    Schema of object. Will be null if a schema was not specified.

    OUT p_object_name text
    Name of object.

Note that the function currently does no error checking.';


--
-- Name: stat_activity(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION stat_activity() RETURNS SETOF pg_stat_activity
    AS $$
-- decibel@workbook=# select array_to_string(array(select column_name::text from information_schema.columns where table_name='pg_stat_activity'), ', ');
SELECT datid, datname, procpid, usesysid, usename
    , CASE WHEN usename = session_user
          OR pg_catalog.pg_has_role( session_user, 'developer_role', 'MEMBER' )
          OR pg_catalog.pg_has_role( session_user, 'ops_support_role', 'MEMBER' )

          -- Note that usename is NOT affected by SET ROLE or SET SESSION_AUTHORIZATION, so we need to check the pid.
          OR procpid = pg_catalog.pg_backend_pid()
        THEN current_query
      ELSE CASE WHEN current_query IN ( '<IDLE>', '<IDLE> in transaction' ) THEN current_query END
      END
    , waiting, xact_start, query_start, backend_start, client_addr, client_port
  FROM pg_catalog.pg_stat_activity
;
$$
    LANGUAGE sql STABLE SECURITY DEFINER
    SET search_path TO pg_catalog, pg_temp;


ALTER FUNCTION tools.stat_activity() OWNER TO db_global_owner;

--
-- Name: string_or_array(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION string_or_array(p_string text) RETURNS text[]
    AS $$

BEGIN
  -- If the string looks like it might be an array or just has commas, fall-back to the array version
  IF p_string LIKE '{%}' OR p_string ~ ','
  THEN
    IF substring(version() from E'PostgreSQL \([^ ]+\)') >= '8.3' THEN
      IF p_string LIKE '{%}' THEN
        RETURN p_string::text[];
      ELSE
        RETURN ('{' || p_string || '}')::text[];
      END IF;
    ELSE
      RETURN string_to_array( regexp_replace( p_string, '[{} ]',  '', 'g' ), ',' );
    END IF;
  ELSE
    RETURN array[ p_string ];
  END IF;
END;

$$
    LANGUAGE plpgsql IMMUTABLE;


ALTER FUNCTION tools.string_or_array(p_string text) OWNER TO db_global_owner;

--
-- Name: FUNCTION string_or_array(p_string text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION string_or_array(p_string text) IS 'Converts a string into an array.';


--
-- Name: sub_assert__enabled_global(text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION sub_assert__enabled_global(p_assert_name text, p_sub_assert_name text) RETURNS boolean
    AS $_$
  -- Original in bug_252483_fix_asserts_for_dump
  SELECT NOT EXISTS(
                SELECT * FROM tools.sub_asserts 
                  WHERE lower( assert_name ) = lower( $1 )
                    AND lower( sub_assert_name ) = lower( $2 )
              )
$_$
    LANGUAGE sql STABLE STRICT;


ALTER FUNCTION tools.sub_assert__enabled_global(p_assert_name text, p_sub_assert_name text) OWNER TO db_global_owner;

--
-- Name: sub_assert__enabled_session(text, text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION sub_assert__enabled_session(p_assert_name text, p_sub_assert_name text) RETURNS boolean
    AS $_$
  -- Original in bug_252483_fix_asserts_for_dump
  SELECT COALESCE( tools.session__get( 'disabledasserts', lower( $1 || '.' || $2 ) ), 'true' )::boolean;
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.sub_assert__enabled_session(p_assert_name text, p_sub_assert_name text) OWNER TO db_global_owner;

--
-- Name: table_full_name(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION table_full_name(p_table_name text) RETURNS text
    AS $_$SELECT schema_name || '.' || table_name FROM tools.table_schema_and_name( $1 )$_$
    LANGUAGE sql;


ALTER FUNCTION tools.table_full_name(p_table_name text) OWNER TO db_global_owner;

--
-- Name: table_schema_and_name(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION table_schema_and_name(p_table_name text, OUT schema_name text, OUT table_name text) RETURNS record
    AS $_$
SELECT quote_ident(nspname),  quote_ident(relname)
  FROM pg_class c
    JOIN pg_namespace n ON n.oid = c.relnamespace
  WHERE c.oid = $1::regclass
    AND tools.assert( relkind = 'r', 'Relation ' || $1 || ' is not a table' )
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.table_schema_and_name(p_table_name text, OUT schema_name text, OUT table_name text) OWNER TO db_global_owner;

--
-- Name: table_select_privileges(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION table_select_privileges(p_table text) RETURNS SETOF text
    AS $_$
SELECT grantee
    FROM information_schema.table_privileges p
        JOIN tools.table_schema_and_name( $1 ) t ON ( p.table_schema = t.schema_name AND p.table_name = t.table_name )
    WHERE privilege_type = 'SELECT' 
$_$
    LANGUAGE sql STABLE SECURITY DEFINER;


ALTER FUNCTION tools.table_select_privileges(p_table text) OWNER TO db_global_owner;

--
-- Name: tables__get_id(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION tables__get_id(p_table text) RETURNS integer
    AS $$

DECLARE
  v_return tools.tables.id%TYPE;

  v_table_schema text;
  v_table_name text;
BEGIN
  SELECT INTO v_table_schema, v_table_name * FROM tools.table_schema_and_name( p_table );

  SELECT INTO v_return *
    FROM tools.tables
    WHERE table_schema = v_table_schema
      AND table_name = v_table_name
  ;

  RETURN v_return;
END;

$$
    LANGUAGE plpgsql STRICT;


ALTER FUNCTION tools.tables__get_id(p_table text) OWNER TO db_global_owner;

--
-- Name: FUNCTION tables__get_id(p_table text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION tables__get_id(p_table text) IS 'Returns the id of a schema/table record from tools.tables.';


--
-- Name: tables__get_id(text[]); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION tables__get_id(p_table text[]) RETURNS integer[]
    AS $_$
SELECT array( SELECT tools.tables__get_id( t ) FROM tools.explode_array( $1 ) t )
$_$
    LANGUAGE sql STRICT;


ALTER FUNCTION tools.tables__get_id(p_table text[]) OWNER TO db_global_owner;

--
-- Name: FUNCTION tables__get_id(p_table text[]); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION tables__get_id(p_table text[]) IS 'Returns the ids of a set of schema/table records from tools.tables.';


--
-- Name: tables__getsert(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION tables__getsert(p_table text) RETURNS integer
    AS $$

DECLARE
  v_id tools.tables.id%TYPE;

  v_table_schema text;
  v_table_name text;
BEGIN
  SELECT INTO v_table_schema, v_table_name * FROM tools.table_schema_and_name( p_table );

  FOR i IN 1 .. 10 LOOP
    SELECT INTO v_id id
      FROM tools.tables
      WHERE table_schema = v_table_schema
        AND table_name = v_table_name
    ;

    IF FOUND THEN
      RETURN v_id;
    END IF;

    BEGIN
      INSERT INTO tools.tables( table_schema, table_name ) VALUES( v_table_schema, v_table_name );
      RETURN tools.currval( 'tools.tables' );
    EXCEPTION WHEN unique_violation THEN
      -- Do nothing
    END;
  END LOOP;
  PERFORM tools.assert( 'inheritance__tables__getsert loop overflow', FALSE, 'KABOOM!' );
END;

$$
    LANGUAGE plpgsql STRICT SECURITY DEFINER;


ALTER FUNCTION tools.tables__getsert(p_table text) OWNER TO db_global_owner;

--
-- Name: FUNCTION tables__getsert(p_table text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION tables__getsert(p_table text) IS 'Returns the id of a schema/table record from tools.tables, creating it if it does not exist';


--
-- Name: tables__getsert(text[]); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION tables__getsert(p_tables text[]) RETURNS integer[]
    AS $$

DECLARE
  v_table_ids int[];
BEGIN
  FOR i IN array_lower( p_tables, 1 ) .. array_upper( p_tables, 1 ) LOOP
    v_table_ids[i] := tools.tables__getsert( p_tables[i] );
  END LOOP;
  RETURN v_table_ids;
END;

$$
    LANGUAGE plpgsql STRICT SECURITY DEFINER;


ALTER FUNCTION tools.tables__getsert(p_tables text[]) OWNER TO db_global_owner;

--
-- Name: FUNCTION tables__getsert(p_tables text[]); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION tables__getsert(p_tables text[]) IS 'Returns the id of a schema/table record from tools.tables, creating it if it does not exist';


--
-- Name: tanh(double precision); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION tanh(double precision) RETURNS double precision
    AS $_$
    SELECT tools.sinh($1) / tools.cosh($1);
  $_$
    LANGUAGE sql IMMUTABLE;


ALTER FUNCTION tools.tanh(double precision) OWNER TO db_global_owner;

--
-- Name: textcoalesce(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION textcoalesce(text) RETURNS text
    AS $_$
SELECT coalesce( $1::text, '<NULL>' )
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.textcoalesce(text) OWNER TO db_global_owner;

--
-- Name: FUNCTION textcoalesce(text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION textcoalesce(text) IS 'Casts the input to text and then coalesces that with ''<NULL>''.';


--
-- Name: textcoalesce(bigint); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION textcoalesce(bigint) RETURNS text
    AS $_$
SELECT coalesce( $1::text, '<NULL>' )
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.textcoalesce(bigint) OWNER TO db_global_owner;

--
-- Name: FUNCTION textcoalesce(bigint); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION textcoalesce(bigint) IS 'Casts the input to text and then coalesces that with ''<NULL>''.';


--
-- Name: textcoalesce(numeric); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION textcoalesce(numeric) RETURNS text
    AS $_$
SELECT coalesce( $1::text, '<NULL>' )
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.textcoalesce(numeric) OWNER TO db_global_owner;

--
-- Name: FUNCTION textcoalesce(numeric); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION textcoalesce(numeric) IS 'Casts the input to text and then coalesces that with ''<NULL>''.';


--
-- Name: textcoalesce(timestamp with time zone); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION textcoalesce(timestamp with time zone) RETURNS text
    AS $_$
SELECT coalesce( $1::text, '<NULL>' )
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.textcoalesce(timestamp with time zone) OWNER TO db_global_owner;

--
-- Name: FUNCTION textcoalesce(timestamp with time zone); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION textcoalesce(timestamp with time zone) IS 'Casts the input to text and then coalesces that with ''<NULL>''.';


--
-- Name: textcoalesce(timestamp without time zone); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION textcoalesce(timestamp without time zone) RETURNS text
    AS $_$
SELECT coalesce( $1::text, '<NULL>' )
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.textcoalesce(timestamp without time zone) OWNER TO db_global_owner;

--
-- Name: FUNCTION textcoalesce(timestamp without time zone); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION textcoalesce(timestamp without time zone) IS 'Casts the input to text and then coalesces that with ''<NULL>''.';


--
-- Name: textcoalesce(date); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION textcoalesce(date) RETURNS text
    AS $_$
SELECT coalesce( $1::text, '<NULL>' )
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.textcoalesce(date) OWNER TO db_global_owner;

--
-- Name: FUNCTION textcoalesce(date); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION textcoalesce(date) IS 'Casts the input to text and then coalesces that with ''<NULL>''.';


--
-- Name: textcoalesce(double precision); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION textcoalesce(double precision) RETURNS text
    AS $_$
SELECT coalesce( $1::text, '<NULL>' )
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.textcoalesce(double precision) OWNER TO db_global_owner;

--
-- Name: FUNCTION textcoalesce(double precision); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION textcoalesce(double precision) IS 'Casts the input to text and then coalesces that with ''<NULL>''.';


--
-- Name: textcoalesce(interval); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION textcoalesce(interval) RETURNS text
    AS $_$
SELECT coalesce( $1::text, '<NULL>' )
$_$
    LANGUAGE sql;


ALTER FUNCTION tools.textcoalesce(interval) OWNER TO db_global_owner;

--
-- Name: FUNCTION textcoalesce(interval); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION textcoalesce(interval) IS 'Casts the input to text and then coalesces that with ''<NULL>''.';


--
-- Name: tg_allow_on_condition(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION tg_allow_on_condition() RETURNS trigger
    AS $_X$
	BEGIN { strict->import(); }
	
	use lib "/opt/local/share/postgresql83/contrib";
	use check_helper;
	
	# should be called as a row-level trigger
	die "should be called as a row-level trigger" unless ($_TD->{level} eq 'ROW');
	
	# check for the correct number of arguments
	die "$_TD->{name}, wrong number of arguments" unless ($_TD->{argc} >= 3);
  ### Changed from 1 to 3 ----------------------------------------------^
	
  ### Added for assert framework
  my $assert = shift @{$_TD->{args}};
  my $error_text = shift @{$_TD->{args}};

	my $condition = shift @{$_TD->{args}};
	my @args = @{$_TD->{args}};
	
	my $ch = new check_helper($_TD);
	my $result;
	
	eval {
		$result = $ch->check_condition($condition, @args);
	};
	die $@ if ($@);
	
	if (!$result) {
    ### BEGIN CHANGES
    #my $query = "SELECT tools.assert( '$assert', false, " . '$s_exec_delim$' . $ch->{errmsg} . '$s_exec_delim$' . " )";
    my $query = "SELECT tools.assert( '$assert', false, " . '$s_exec_delim$' . $error_text . '$s_exec_delim$' . " )";
    elog(DEBUG, "query: $query");
    my $rv = spi_exec_query($query);
    
    # This should work, but I grow tired of banging my head against it
    #my $rv = spi_exec_prepared( spi_prepare( "SELECT tools.assert( '$assert', false, '" . '$1' . "' )", 'text' ), $ch->{errmsg} );
		#die $ch->{errmsg};
    ### END CHANGES
	}
	
	return;

$_X$
    LANGUAGE plperlu;


ALTER FUNCTION tools.tg_allow_on_condition() OWNER TO db_global_owner;

--
-- Name: tg_check_updates(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION tg_check_updates() RETURNS trigger
    AS $_X$
	BEGIN { strict->import(); }
	
	use lib "/opt/local/share/postgresql83/contrib";
	use check_helper;
	
	# This should be a row level trigger */
	die 'should be called as a row level trigger' unless ($_TD->{level} eq 'ROW');

	# Do nothing if event is not UPDATE 
	return if ($_TD->{event} ne 'UPDATE');
	
	die "$_TD->{name} takes at least 3 arguments"
	unless ($_TD->{argc} >= 3);
  ### Changed from 2 to 3 ^        ^
	
  ### Added for assert framework
  my $assert = shift @{$_TD->{args}};
	
	my ($options_arg, $condition);
	my @deny_columns = ();
	my @condition_columns = ();
	my $state = 0;
			
	foreach (@{$_TD->{args}}) {
		if ($state == 0) {
			# get the options for check_columns
			$options_arg = uc($_);
			$state++;
			next;
		}
		elsif ($state == 1) {
			# get the list of columns for check_columns.
			# Read the values until they are valid column names.
			if (defined $_TD->{new}{$_}) {
				push @deny_columns, $_;
				next;
			} else {
				$state++;
			}
		}
		if ($state == 2) {
			# get the condition to check
			$condition = $_;
			$state++;
		} elsif ($state == 3) {
			# get the columns for check_condition
			push @condition_columns, $_;
		} else {
			# shouldn't be there
			die "Invalid state: $state";
		}
	}
	
	# create a new instance of the 'helper' class.
	my $ch = new check_helper($_TD);
	my $result;
	my $errmsg = "";
	
	eval {
		# check whether this update touches columns that shouldn't change
		$result = $ch->check_columns($options_arg, @deny_columns);
		if ($result == 0) {
			# this update should be denied, however, we allow it if the
			# condition passed will be satisfied.
			$errmsg = $ch->{errmsg};
			$result = $ch->check_condition($condition, @condition_columns);
		}
	};
	die $@ if ($@);
	
	if (!$result) {
    ### BEGIN CHANGES
    my $rv = spi_exec_query("SELECT tools.assert( '$assert', false, '$errmsg')");
		#die $errmsg;
    ### END CHANGES
	}
	
	return;``
$_X$
    LANGUAGE plperlu;


ALTER FUNCTION tools.tg_check_updates() OWNER TO db_global_owner;

--
-- Name: tg_datetime_constraint(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION tg_datetime_constraint() RETURNS trigger
    AS $_X$
	BEGIN { strict->import(); }
	
	# should be called as a row-level trigger
	die "should be called as a row-level trigger" unless ($_TD->{level} eq 'ROW');
	
	# check for the correct number of arguments
	die "$_TD->{name}, wrong number of arguments" unless ($_TD->{argc} >= 2 && $_TD->{argc} <= 4);
	
	# do nothing on delete
	return if ($_TD->{event} ne 'INSERT' && $_TD->{event} ne 'UPDATE');
		
	my ($column, $op, $offset, $truncateto) = @{$_TD->{args}};
	
	# do nothing on update if the target column has not been updated
	return if ($_TD->{event} eq 'UPDATE' && $_TD->{new}{$column} eq $_TD->{old}{$column});
	
	my $value_to_check = $_TD->{new}{$column};
	if (defined $truncateto) {
		# Note: PG complains in unadorned date_trunc second argument;
		# need to apply cast to the appropriate type, for which we
		# need an extra trip to the catalogs :-(
		my $sql_string = sprintf "select format_type(atttypid, atttypmod) as coltype
			from pg_attribute where attrelid = '%s.%s'::regclass and attname = '%s'",
			$_TD->{table_schema}, $_TD->{table_name}, $column;
		my $result = spi_exec_query($sql_string);
		my $coltype = $result->{rows}[0]->{coltype};
		$value_to_check = sprintf "date_trunc('%s', '%s'::%s)", $truncateto, $value_to_check, $coltype;
	} else {
		$value_to_check = "'$value_to_check'";
	}
	
	# if offset is just an integer - assume days
	if (! defined $offset) {
		$offset = "0 days";
	} elsif ($offset =~ m/^\s*[+-]?\d+\s*$/) {
		$offset = $offset." days"
	}

	my $now_plus_offset;
	if (defined $truncateto) {
		$now_plus_offset = sprintf "date_trunc('%s', now() + '%s')", $truncateto, $offset;
	} else {
		$now_plus_offset = sprintf "now() + '%s'", $offset;
	}
	
	my $sql_string = sprintf 'SELECT %s %s %s AS result', $value_to_check, $op, $now_plus_offset;
	

##### BEGIN CNU SPECIFIC CODE (CMD version did not use tools.assert()) #####

    # always the same assert name, use sub-assert functionality to disable
    my $assert_name = "tg_datetime_constraint";
    my $sub_assert_name = sprintf "%s.%s.%s", 
        $_TD->{table_schema}, $_TD->{table_name}, $column;

    # build up assertion
    my $assert_condition = sprintf "%s %s %s", 
        $value_to_check, $op, $now_plus_offset;

    # build up the error message
    my $assert_message = sprintf "E'%s.%s.%s must be %s ' || %s",
        $_TD->{table_schema}, $_TD->{table_name}, $column, $op, 
        $now_plus_offset;

    # call tools.assert() to evaluate our condition so it can be disabled
    my $sql_string = sprintf "SELECT tools.assert( '%s', '%s', %s, %s )",
        $assert_name, $sub_assert_name, $assert_condition, $assert_message;

##### END CNU SPECIFIC CODE #####

    elog(DEBUG, $sql_string);

    my $result = spi_exec_query($sql_string);

    return;
$_X$
    LANGUAGE plperl;


ALTER FUNCTION tools.tg_datetime_constraint() OWNER TO db_global_owner;

--
-- Name: tg_deny_updates(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION tg_deny_updates() RETURNS trigger
    AS $_X$

	BEGIN { strict->import(); }

	use lib "/opt/local/share/postgresql83/contrib";
	use check_helper;
	
	my @column_names;

	# This should be a row level trigger */
	die 'should be called as a row level trigger' unless ($_TD->{level} eq 'ROW');

	# Do nothing if event is not UPDATE 
	return if ($_TD->{event} ne 'UPDATE');

	die "$_TD->{name}, missing options list argument"
	unless ($_TD->{argc} >= 2);
  ### Changed from 1 to 2 ^
	
	my $options_arg = uc(shift @{$_TD->{args}});
    
  ### Added for assert framework
  my $assert = shift @{$_TD->{args}};

	my $oldtuple = $_TD->{old};
	my $newtuple = $_TD->{new};
	
	if ($_TD->{argc} > 1) {
		@column_names = @{$_TD->{args}};
	} else {
		@column_names = ();
	}
	
	my $ch = new check_helper($_TD);
	my $result;
	
	eval {
		$result = $ch->check_columns($options_arg, @column_names);
	};
	die $@ if ($@);
	
	if (!$result) {
    ### BEGIN CHANGES
    my $rv = spi_exec_query("SELECT tools.assert( '$assert', false, '$ch->{errmsg}')");
		#die $ch->{errmsg};
    ### END CHANGES
	}

	return;
$_X$
    LANGUAGE plperlu;


ALTER FUNCTION tools.tg_deny_updates() OWNER TO db_global_owner;

--
-- Name: tg_disallow(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION tg_disallow() RETURNS trigger
    AS $_$
BEGIN
    -- If this is taken out then we need to deal with the return differently for statement vs row-level triggers
    -- Don't use tools.assert here because we don't want this check disabled
    IF TG_LEVEL <> 'STATEMENT' THEN
        RAISE EXCEPTION $$You should use a statement-level trigger (trigger %, table %)$$, TG_NAME, TG_RELID::regclass;
    END IF;

    IF tools.checks__enabled() THEN
        RAISE EXCEPTION $$%s are not allowed on table %$$, TG_OP, TG_RELNAME;
    END IF;

    RETURN NULL;
END;
$_$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.tg_disallow() OWNER TO db_global_owner;

--
-- Name: tg_londiste_no_dml(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION tg_londiste_no_dml() RETURNS trigger
    AS $_$
DECLARE
    v_text text;
BEGIN
    PERFORM tools.assert(TG_WHEN = 'AFTER', TG_NAME || ' needs to be an AFTER trigger');
    IF TG_LEVEL != 'STATEMENT' THEN
        RAISE WARNING $$There's no sense in making % a row level trigger; make it statement level instead.$$, TG_NAME;
    END IF;

    IF session_user != 'skytools' THEN
        IF TG_NARGS = 0 THEN
            RAISE EXCEPTION 'DML is not allowed on replicated table %', TG_RELNAME;
        ELSEIF TG_NARGS = 1 THEN
            v_text = replace(TG_ARGV[0], '%TABLE_NAME%', TG_RELNAME);
            RAISE EXCEPTION '%', v_text;
        ELSE
            RAISE EXCEPTION 'Invalid number of arguments (%) supplied in trigger % on table %', TG_NARGS, TG_NAME, TG_RELNAME;
        END IF;
    END IF;

    RETURN NULL;
END;
$_$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.tg_londiste_no_dml() OWNER TO db_global_owner;

--
-- Name: tg_schema_patches(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION tg_schema_patches() RETURNS trigger
    AS $_$
DECLARE
    v_text  text;
BEGIN
    PERFORM tools.assert( TG_WHEN = 'AFTER', TG_NAME || ' needs to be an after trigger');

    -- Disallow deletes
    IF TG_OP = 'DELETE' THEN
        RAISE EXCEPTION 'DELETE not allowed on %', TG_RELNAME;
    END IF;
    
    IF TG_OP = 'UPDATE' THEN
        -- If the patch was marked as applied, don't allow it to be changed
        IF OLD.applied_at IS NOT NULL THEN
            RAISE EXCEPTION $$A patch may not be changed once it's been applied (patch file %)$$, NEW.patch_file;
        -- If we are marking a patch applied, make sure it's dependencies are also applied
        ELSIF OLD.applied_at IS NULL AND NEW.applied_at IS NOT NULL THEN
            SELECT INTO v_text
                nullif(
                    array_to_string(
                        array(  SELECT patch_file
                                    FROM tools.schema_patches
                                    WHERE patch_file = ANY( NEW.depends_on )
                                        AND applied_at IS NULL
                                )
                        , ', '
                    )
                    , ''
                )
            ;
            IF v_text IS NOT NULL THEN
                RAISE EXCEPTION 'Unable to mark patch % as applied; the following dependent patches have not been applied yet: %', NEW.patch_file, v_text;
            END IF;
        END IF;
    END IF;

    RETURN NULL;
END;
$_$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.tg_schema_patches() OWNER TO db_global_owner;

--
-- Name: tg_tests_not_on_prod(); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION tg_tests_not_on_prod() RETURNS trigger
    AS $$
BEGIN
    PERFORM tools.assert( TG_WHEN = 'AFTER', TG_NAME || ' must be an after trigger' );

    IF current_database() ~* 'prod|slave' THEN
        RAISE EXCEPTION 'Tests may not be run on prod or slave databases (current database: %)', current_database();
    END IF;

    RETURN NULL;
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.tg_tests_not_on_prod() OWNER TO db_global_owner;

--
-- Name: FUNCTION tg_tests_not_on_prod(); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION tg_tests_not_on_prod() IS 'Disallow tests on databases with ''prod'' or ''slave'' in the name by disallowing DML on the tools.tests table';


--
-- Name: trigger_sanity(text, text, text, text, text, text, text[]); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION trigger_sanity(p_tg_name text, p_tg_when text, p_tg_level text, p_tg_op text, p_good_when text, p_good_level text, p_good_op text[]) RETURNS void
    AS $$
DECLARE
  v_when CONSTANT text := upper( p_good_when );
  v_level CONSTANT text := upper( p_good_level );
  v_op CONSTANT text[] := tools.upper( p_good_op );
BEGIN
	RAISE DEBUG 'tools.trigger_sanity: p_tg_name = %, p_tg_when = %, p_tg_level = %, p_tg_op = %
    , p_good_when = %, p_good_level = %, p_good_op = %'
    , p_tg_name , p_tg_when , p_tg_level , p_tg_op
    , p_good_when , p_good_level , p_good_op
	;	

  PERFORM tools.assert( 'tools__trigger_sanity__good_when'
      , v_when IN ('BEFORE', 'AFTER')
      , 'p_good_when ( = ' || tools.textcoalesce( p_good_when ) || ' ) must be either "BEFORE" or "AFTER"'
    )
  ;
  PERFORM tools.assert( 'tools__trigger_sanity__good_level'
      , v_level IN ('STATEMENT', 'ROW')
      , 'p_good_level ( = ' || tools.textcoalesce( p_good_level ) || ' ) must be either "STATEMENT" or "ROW"'
    )
  ;
  PERFORM tools.assert( 'tools__trigger_sanity__good_op'
      , v_op <@ array[ 'INSERT', 'UPDATE', 'DELETE' ]
			-- Postgres has trouble trying to concat text with the array value here, so just punt
      , 'p_good_op must contain only "INSERT", "UPDATE", and/or "DELETE"'
    )
  ;

  -- TODO per-table/trigger
	PERFORM tools.assert( 'tools__trigger_sanity__when', p_tg_when = v_when, p_tg_name || ' must be a'
		|| CASE WHEN v_when = 'AFTER' THEN 'n' END || ' ' || lower( v_when ) || ' trigger'
	);
  PERFORM tools.assert( 'tools__trigger_sanity__level', p_tg_level = v_level, p_tg_name || ' must be a per-'
		|| lower( v_level ) || ' trigger'
	);
  PERFORM tools.assert( 'tools__trigger_sanity__op', p_tg_op = ANY( v_op ), p_tg_name || ' must be a'
		|| CASE WHEN v_op[1] != 'DELETE' THEN 'n' END || ' '
		|| array_to_string( v_op, ' or ' ) || ' trigger'
	);
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION tools.trigger_sanity(p_tg_name text, p_tg_when text, p_tg_level text, p_tg_op text, p_good_when text, p_good_level text, p_good_op text[]) OWNER TO db_global_owner;

--
-- Name: trim_whitespace(text); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION trim_whitespace(t text) RETURNS text
    AS $_$
-- Created by bug_192474__schema_as
SELECT regexp_replace( $1, E'***:^\\s+|\\s+$', '', 'g' )
$_$
    LANGUAGE sql IMMUTABLE STRICT;


ALTER FUNCTION tools.trim_whitespace(t text) OWNER TO db_global_owner;

--
-- Name: FUNCTION trim_whitespace(t text); Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON FUNCTION trim_whitespace(t text) IS 'Trims all whitespace from the beginning and end of a string.';


--
-- Name: upper(text[]); Type: FUNCTION; Schema: tools; Owner: db_global_owner
--

CREATE FUNCTION upper(p_in text[]) RETURNS text[]
    AS $$
DECLARE
  v_out text[];
BEGIN
  -- TODO allow to work with multi-dimensional arrays
  FOR i IN array_lower( p_in, 1 ) .. array_upper( p_in, 1 ) LOOP
    v_out[i] := upper( p_in[i] );
  END LOOP;

  RETURN v_out;
END;
$$
    LANGUAGE plpgsql IMMUTABLE STRICT;


ALTER FUNCTION tools.upper(p_in text[]) OWNER TO db_global_owner;

SET search_path = cabar, pg_catalog;

--
-- Name: component_schemas; Type: TABLE; Schema: cabar; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE component_schemas (
    component_id smallint NOT NULL,
    schema_name text NOT NULL,
    CONSTRAINT component_schemas__schema_name_must_be_lowercase CHECK ((schema_name = lower(schema_name)))
);


ALTER TABLE cabar.component_schemas OWNER TO db_global_owner;

--
-- Name: TABLE component_schemas; Type: COMMENT; Schema: cabar; Owner: db_global_owner
--

COMMENT ON TABLE component_schemas IS 'What schemas belong to a given component. DML access is intentionally restricted to db_global role since not any old role should be able to create a new component. Theoretically, we should probably have a separate cabar component though. This is a seed table.';


--
-- Name: COLUMN component_schemas.schema_name; Type: COMMENT; Schema: cabar; Owner: db_global_owner
--

COMMENT ON COLUMN component_schemas.schema_name IS 'Must be lowercase.';


--
-- Name: component_schemas_v; Type: VIEW; Schema: cabar; Owner: db_global_owner
--

CREATE VIEW component_schemas_v AS
    SELECT t.component_id, t.schema_name, (components__get(t.component_id)).component AS component FROM component_schemas t;


ALTER TABLE cabar.component_schemas_v OWNER TO db_global_owner;

SET search_path = tools, pg_catalog;

--
-- Name: space; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW space AS
    SELECT a.oid, a.full_name, a.rows, a.heap_bytes, a.index_bytes, a.toast_bytes, a.total_bytes, pg_size_pretty(a.heap_bytes) AS heap_size, pg_size_pretty(a.index_bytes) AS index_size, pg_size_pretty(a.toast_bytes) AS toast_size, pg_size_pretty(a.total_bytes) AS total_size FROM (SELECT a.oid, a.full_name, a.reltuples AS rows, a.heap_bytes, a.index_bytes, ((a.total_bytes - a.index_bytes) - a.heap_bytes) AS toast_bytes, a.total_bytes FROM (SELECT t.oid, t.relname, t.relnamespace, t.reltype, t.relowner, t.relam, t.relfilenode, t.reltablespace, t.relpages, t.reltuples, t.reltoastrelid, t.reltoastidxid, t.relhasindex, t.relisshared, t.relkind, t.relnatts, t.relchecks, t.reltriggers, t.relukeys, t.relfkeys, t.relrefs, t.relhasoids, t.relhaspkey, t.relhasrules, t.relhassubclass, t.relacl, t.full_name, t.index_bytes, pg_relation_size(t.oid) AS heap_bytes, pg_total_relation_size(t.oid) AS total_bytes FROM (SELECT pg_class.oid, pg_class.relname, pg_class.relnamespace, pg_class.reltype, pg_class.relowner, pg_class.relam, pg_class.relfilenode, pg_class.reltablespace, pg_class.relpages, pg_class.reltuples, pg_class.reltoastrelid, pg_class.reltoastidxid, pg_class.relhasindex, pg_class.relisshared, pg_class.relkind, pg_class.relnatts, pg_class.relchecks, pg_class.reltriggers, pg_class.relukeys, pg_class.relfkeys, pg_class.relrefs, pg_class.relhasoids, pg_class.relhaspkey, pg_class.relhasrules, pg_class.relhassubclass, pg_class.relacl, (((n.nspname)::text || '.'::text) || (pg_class.relname)::text) AS full_name, COALESCE((SELECT (sum(pg_relation_size(i.indexrelid)))::bigint AS sum FROM pg_index i WHERE (i.indrelid = pg_class.oid))) AS index_bytes FROM (pg_class JOIN pg_namespace n ON ((n.oid = pg_class.relnamespace))) WHERE (pg_class.relkind = 'r'::"char")) t) a) a;


ALTER TABLE tools.space OWNER TO db_global_owner;

SET search_path = code, pg_catalog;

--
-- Name: functions; Type: VIEW; Schema: code; Owner: db_global_owner
--

CREATE VIEW functions AS
    SELECT p.proname AS "Function Name", obj_description(p.oid, 'pg_proc'::name) AS "Description" FROM (pg_proc p LEFT JOIN pg_namespace n ON ((n.oid = p.pronamespace))) WHERE (n.nspname = 'code'::name);


ALTER TABLE code.functions OWNER TO db_global_owner;

--
-- Name: VIEW functions; Type: COMMENT; Schema: code; Owner: db_global_owner
--

COMMENT ON VIEW functions IS 'Lists functions that are available for creating database code.';


SET search_path = code_internal, pg_catalog;

--
-- Name: object_class_id_seq; Type: SEQUENCE; Schema: code_internal; Owner: db_global_owner
--

CREATE SEQUENCE object_class_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


ALTER TABLE code_internal.object_class_id_seq OWNER TO db_global_owner;

--
-- Name: object_class_id_seq; Type: SEQUENCE OWNED BY; Schema: code_internal; Owner: db_global_owner
--

ALTER SEQUENCE object_class_id_seq OWNED BY object_class.id;


--
-- Name: object_class_id_seq; Type: SEQUENCE SET; Schema: code_internal; Owner: db_global_owner
--

SELECT pg_catalog.setval('object_class_id_seq', 14, true);


--
-- Name: object_type; Type: TABLE; Schema: code_internal; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE object_type (
    id integer NOT NULL,
    object_type text NOT NULL
);


ALTER TABLE code_internal.object_type OWNER TO db_global_owner;

--
-- Name: TABLE object_type; Type: COMMENT; Schema: code_internal; Owner: db_global_owner
--

COMMENT ON TABLE object_type IS 'This is a seed table.';


--
-- Name: object_type_id_seq; Type: SEQUENCE; Schema: code_internal; Owner: db_global_owner
--

CREATE SEQUENCE object_type_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


ALTER TABLE code_internal.object_type_id_seq OWNER TO db_global_owner;

--
-- Name: object_type_id_seq; Type: SEQUENCE OWNED BY; Schema: code_internal; Owner: db_global_owner
--

ALTER SEQUENCE object_type_id_seq OWNED BY object_type.id;


--
-- Name: object_type_id_seq; Type: SEQUENCE SET; Schema: code_internal; Owner: db_global_owner
--

SELECT pg_catalog.setval('object_type_id_seq', 4, true);


--
-- Name: objects; Type: TABLE; Schema: code_internal; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE objects (
    full_name text NOT NULL,
    template_id integer NOT NULL,
    parameters text[] NOT NULL,
    created_on timestamp without time zone DEFAULT now() NOT NULL,
    updated_on timestamp without time zone NOT NULL,
    creation_function_version smallint DEFAULT 1 NOT NULL
);


ALTER TABLE code_internal.objects OWNER TO db_global_owner;

--
-- Name: TABLE objects; Type: COMMENT; Schema: code_internal; Owner: db_global_owner
--

COMMENT ON TABLE objects IS 'This is a list of objects that were created by code_internal (instead of by hand). This is a seed table.';


--
-- Name: COLUMN objects.creation_function_version; Type: COMMENT; Schema: code_internal; Owner: db_global_owner
--

COMMENT ON COLUMN objects.creation_function_version IS 'What version of a code creation function (the functions in the code schema) was used to create an object. Similar to code_intrenal.templates.version.';


--
-- Name: objects_v; Type: VIEW; Schema: code_internal; Owner: db_global_owner
--

CREATE VIEW objects_v AS
    SELECT o.full_name, o.template_id, o.parameters, o.created_on, o.updated_on, typ.object_type, c.object_class FROM (((objects o LEFT JOIN templates tmp ON ((tmp.id = o.template_id))) LEFT JOIN object_class c ON ((tmp.object_class_id = c.id))) LEFT JOIN object_type typ ON ((c.object_type_id = typ.id)));


ALTER TABLE code_internal.objects_v OWNER TO db_global_owner;

--
-- Name: templates_id_seq; Type: SEQUENCE; Schema: code_internal; Owner: db_global_owner
--

CREATE SEQUENCE templates_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


ALTER TABLE code_internal.templates_id_seq OWNER TO db_global_owner;

--
-- Name: templates_id_seq; Type: SEQUENCE OWNED BY; Schema: code_internal; Owner: db_global_owner
--

ALTER SEQUENCE templates_id_seq OWNED BY templates.id;


--
-- Name: templates_id_seq; Type: SEQUENCE SET; Schema: code_internal; Owner: db_global_owner
--

SELECT pg_catalog.setval('templates_id_seq', 14, true);


--
-- Name: templates_v; Type: VIEW; Schema: code_internal; Owner: db_global_owner
--

CREATE VIEW templates_v AS
    SELECT tmp.id, tmp.object_class_id, tmp.version, tmp.template, tmp.description, typ.object_type, c.object_class FROM ((templates tmp LEFT JOIN object_class c ON ((tmp.object_class_id = c.id))) LEFT JOIN object_type typ ON ((c.object_type_id = typ.id)));


ALTER TABLE code_internal.templates_v OWNER TO db_global_owner;

SET search_path = inheritance, pg_catalog;

--
-- Name: child_objects_exclude_tables; Type: TABLE; Schema: inheritance; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE child_objects_exclude_tables (
    child_object_id integer NOT NULL,
    exclude_table_id integer NOT NULL
);


ALTER TABLE inheritance.child_objects_exclude_tables OWNER TO db_global_owner;

--
-- Name: TABLE child_objects_exclude_tables; Type: COMMENT; Schema: inheritance; Owner: db_global_owner
--

COMMENT ON TABLE child_objects_exclude_tables IS 'Child tables that should not have an object applied. This is a seed table.';


SET search_path = tools, pg_catalog;

--
-- Name: tables; Type: TABLE; Schema: tools; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE tables (
    id integer NOT NULL,
    table_schema text NOT NULL,
    table_name text NOT NULL
);


ALTER TABLE tools.tables OWNER TO db_global_owner;

--
-- Name: TABLE tables; Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON TABLE tables IS 'An immutable list of tables (since we can not depond on OIDs being constaint). This table is owned by SU on purpose; use the interface functions to interact with it.';


--
-- Name: tables_v; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW tables_v AS
    SELECT t.id, t.table_schema, t.table_name, c.oid FROM (tables t LEFT JOIN (pg_class c JOIN pg_namespace n ON ((n.oid = c.relnamespace))) ON ((((c.relname)::text = t.table_name) AND ((n.nspname)::text = t.table_schema))));


ALTER TABLE tools.tables_v OWNER TO db_global_owner;

--
-- Name: VIEW tables_v; Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON VIEW tables_v IS 'This view does not use the regclass cast because a table might not exist anymore. The join to pg_namespace is a regular join because you can end up with tables that do not have a schema.';


SET search_path = inheritance, pg_catalog;

--
-- Name: child_objects_v; Type: VIEW; Schema: inheritance; Owner: db_global_owner
--

CREATE VIEW child_objects_v AS
    SELECT c.id, c.inherited_table_id, c.object_type_id, c.object_name_template, c.object_body_template, t.table_schema AS inherited_table_schema, t.table_name AS inherited_table_name, ((t.table_schema || '.'::text) || t.table_name) AS inherited_table_full_name, t.oid AS inherited_table_oid, ot.object_type FROM ((child_objects c LEFT JOIN tools.tables_v t ON ((t.id = c.inherited_table_id))) LEFT JOIN object_types ot ON ((ot.id = c.object_type_id)));


ALTER TABLE inheritance.child_objects_v OWNER TO db_global_owner;

--
-- Name: child_objects_exclude_tables_v; Type: VIEW; Schema: inheritance; Owner: db_global_owner
--

CREATE VIEW child_objects_exclude_tables_v AS
    SELECT c.id, c.inherited_table_id, c.object_type_id, c.object_name_template, c.object_body_template, c.inherited_table_schema, c.inherited_table_name, c.inherited_table_full_name, c.inherited_table_oid, c.object_type, e.child_object_id, e.exclude_table_id, t.table_schema AS excluded_table_schema, t.table_name AS excluded_table_name, ((t.table_schema || '.'::text) || t.table_name) AS excluded_table_full_name, t.oid AS excluded_table_oid FROM ((child_objects_v c JOIN child_objects_exclude_tables e ON ((e.child_object_id = c.id))) LEFT JOIN tools.tables_v t ON ((t.id = e.exclude_table_id)));


ALTER TABLE inheritance.child_objects_exclude_tables_v OWNER TO db_global_owner;

--
-- Name: child_objects_id_seq; Type: SEQUENCE; Schema: inheritance; Owner: db_global_owner
--

CREATE SEQUENCE child_objects_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


ALTER TABLE inheritance.child_objects_id_seq OWNER TO db_global_owner;

--
-- Name: child_objects_id_seq; Type: SEQUENCE OWNED BY; Schema: inheritance; Owner: db_global_owner
--

ALTER SEQUENCE child_objects_id_seq OWNED BY child_objects.id;


--
-- Name: child_objects_id_seq; Type: SEQUENCE SET; Schema: inheritance; Owner: db_global_owner
--

SELECT pg_catalog.setval('child_objects_id_seq', 61, true);


SET search_path = object_monitor, pg_catalog;

--
-- Name: monitors; Type: TABLE; Schema: object_monitor; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE monitors (
    monitor_name text NOT NULL
);


ALTER TABLE object_monitor.monitors OWNER TO db_global_owner;

--
-- Name: TABLE monitors; Type: COMMENT; Schema: object_monitor; Owner: db_global_owner
--

COMMENT ON TABLE monitors IS 'This is a seed table.';


--
-- Name: monitors_consumers; Type: TABLE; Schema: object_monitor; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE monitors_consumers (
    monitor_name text NOT NULL,
    consumer_name text NOT NULL
);


ALTER TABLE object_monitor.monitors_consumers OWNER TO db_global_owner;

--
-- Name: TABLE monitors_consumers; Type: COMMENT; Schema: object_monitor; Owner: db_global_owner
--

COMMENT ON TABLE monitors_consumers IS 'This is a seed table.';


--
-- Name: monitors_tables; Type: TABLE; Schema: object_monitor; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE monitors_tables (
    monitor_name text NOT NULL,
    table_id integer NOT NULL
);


ALTER TABLE object_monitor.monitors_tables OWNER TO db_global_owner;

--
-- Name: TABLE monitors_tables; Type: COMMENT; Schema: object_monitor; Owner: db_global_owner
--

COMMENT ON TABLE monitors_tables IS 'This is a seed table.';


SET search_path = pgq, pg_catalog;

--
-- Name: batch_id_seq; Type: SEQUENCE; Schema: pgq; Owner: skytools
--

CREATE SEQUENCE batch_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


ALTER TABLE pgq.batch_id_seq OWNER TO skytools;

--
-- Name: batch_id_seq; Type: SEQUENCE SET; Schema: pgq; Owner: skytools
--

SELECT pg_catalog.setval('batch_id_seq', 1, false);


--
-- Name: consumer; Type: TABLE; Schema: pgq; Owner: skytools; Tablespace: 
--

CREATE TABLE consumer (
    co_id integer NOT NULL,
    co_name text DEFAULT 'fooz'::text NOT NULL
);


ALTER TABLE pgq.consumer OWNER TO skytools;

--
-- Name: consumer_co_id_seq; Type: SEQUENCE; Schema: pgq; Owner: skytools
--

CREATE SEQUENCE consumer_co_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


ALTER TABLE pgq.consumer_co_id_seq OWNER TO skytools;

--
-- Name: consumer_co_id_seq; Type: SEQUENCE OWNED BY; Schema: pgq; Owner: skytools
--

ALTER SEQUENCE consumer_co_id_seq OWNED BY consumer.co_id;


--
-- Name: consumer_co_id_seq; Type: SEQUENCE SET; Schema: pgq; Owner: skytools
--

SELECT pg_catalog.setval('consumer_co_id_seq', 1, false);


--
-- Name: event_template; Type: TABLE; Schema: pgq; Owner: skytools; Tablespace: 
--

CREATE TABLE event_template (
    ev_id bigint NOT NULL,
    ev_time timestamp with time zone NOT NULL,
    ev_txid bigint DEFAULT txid_current() NOT NULL,
    ev_owner integer,
    ev_retry integer,
    ev_type text,
    ev_data text,
    ev_extra1 text,
    ev_extra2 text,
    ev_extra3 text,
    ev_extra4 text
);


ALTER TABLE pgq.event_template OWNER TO skytools;

--
-- Name: queue; Type: TABLE; Schema: pgq; Owner: skytools; Tablespace: 
--

CREATE TABLE queue (
    queue_id integer NOT NULL,
    queue_name text NOT NULL,
    queue_ntables integer DEFAULT 3 NOT NULL,
    queue_cur_table integer DEFAULT 0 NOT NULL,
    queue_rotation_period interval DEFAULT '02:00:00'::interval NOT NULL,
    queue_switch_step1 bigint DEFAULT txid_current() NOT NULL,
    queue_switch_step2 bigint DEFAULT txid_current(),
    queue_switch_time timestamp with time zone DEFAULT now() NOT NULL,
    queue_external_ticker boolean DEFAULT false NOT NULL,
    queue_ticker_max_count integer DEFAULT 500 NOT NULL,
    queue_ticker_max_lag interval DEFAULT '00:00:03'::interval NOT NULL,
    queue_ticker_idle_period interval DEFAULT '00:01:00'::interval NOT NULL,
    queue_data_pfx text NOT NULL,
    queue_event_seq text NOT NULL,
    queue_tick_seq text NOT NULL
);


ALTER TABLE pgq.queue OWNER TO skytools;

--
-- Name: queue_queue_id_seq; Type: SEQUENCE; Schema: pgq; Owner: skytools
--

CREATE SEQUENCE queue_queue_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


ALTER TABLE pgq.queue_queue_id_seq OWNER TO skytools;

--
-- Name: queue_queue_id_seq; Type: SEQUENCE OWNED BY; Schema: pgq; Owner: skytools
--

ALTER SEQUENCE queue_queue_id_seq OWNED BY queue.queue_id;


--
-- Name: queue_queue_id_seq; Type: SEQUENCE SET; Schema: pgq; Owner: skytools
--

SELECT pg_catalog.setval('queue_queue_id_seq', 25, true);


--
-- Name: retry_queue; Type: TABLE; Schema: pgq; Owner: skytools; Tablespace: 
--

CREATE TABLE retry_queue (
    ev_retry_after timestamp with time zone NOT NULL,
    ev_id bigint NOT NULL,
    ev_time timestamp with time zone NOT NULL,
    ev_txid bigint,
    ev_owner integer NOT NULL,
    ev_retry integer,
    ev_type text,
    ev_data text,
    ev_extra1 text,
    ev_extra2 text,
    ev_extra3 text,
    ev_extra4 text
);


ALTER TABLE pgq.retry_queue OWNER TO skytools;

--
-- Name: subscription; Type: TABLE; Schema: pgq; Owner: skytools; Tablespace: 
--

CREATE TABLE subscription (
    sub_id integer NOT NULL,
    sub_queue integer NOT NULL,
    sub_consumer integer NOT NULL,
    sub_last_tick bigint NOT NULL,
    sub_active timestamp with time zone DEFAULT now() NOT NULL,
    sub_batch bigint,
    sub_next_tick bigint
);


ALTER TABLE pgq.subscription OWNER TO skytools;

--
-- Name: subscription_sub_id_seq; Type: SEQUENCE; Schema: pgq; Owner: skytools
--

CREATE SEQUENCE subscription_sub_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


ALTER TABLE pgq.subscription_sub_id_seq OWNER TO skytools;

--
-- Name: subscription_sub_id_seq; Type: SEQUENCE OWNED BY; Schema: pgq; Owner: skytools
--

ALTER SEQUENCE subscription_sub_id_seq OWNED BY subscription.sub_id;


--
-- Name: subscription_sub_id_seq; Type: SEQUENCE SET; Schema: pgq; Owner: skytools
--

SELECT pg_catalog.setval('subscription_sub_id_seq', 1, false);


--
-- Name: tick; Type: TABLE; Schema: pgq; Owner: skytools; Tablespace: 
--

CREATE TABLE tick (
    tick_queue integer NOT NULL,
    tick_id bigint NOT NULL,
    tick_time timestamp with time zone DEFAULT now() NOT NULL,
    tick_snapshot txid_snapshot DEFAULT txid_current_snapshot() NOT NULL
);


ALTER TABLE pgq.tick OWNER TO skytools;

SET search_path = replication, pg_catalog;

--
-- Name: tables; Type: TABLE; Schema: replication; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE tables (
    id integer NOT NULL,
    set_id smallint NOT NULL,
    full_name text NOT NULL,
    CONSTRAINT full_name_must_be_lower_case CHECK ((full_name = lower(full_name))),
    CONSTRAINT full_name_must_be_schema_qualified CHECK ((full_name ~~ '%.%'::text))
);


ALTER TABLE replication.tables OWNER TO db_global_owner;

--
-- Name: tables_id_seq; Type: SEQUENCE; Schema: replication; Owner: db_global_owner
--

CREATE SEQUENCE tables_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


ALTER TABLE replication.tables_id_seq OWNER TO db_global_owner;

--
-- Name: tables_id_seq; Type: SEQUENCE OWNED BY; Schema: replication; Owner: db_global_owner
--

ALTER SEQUENCE tables_id_seq OWNED BY tables.id;


--
-- Name: tables_id_seq; Type: SEQUENCE SET; Schema: replication; Owner: db_global_owner
--

SELECT pg_catalog.setval('tables_id_seq', 1, false);


SET search_path = tools, pg_catalog;

--
-- Name: activity; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW activity AS
    SELECT (now() - stat_activity.query_start) AS age, stat_activity.datid, stat_activity.datname, stat_activity.procpid, stat_activity.usesysid, stat_activity.usename, stat_activity.current_query, stat_activity.waiting, stat_activity.xact_start, stat_activity.query_start, stat_activity.backend_start, stat_activity.client_addr, stat_activity.client_port FROM stat_activity() stat_activity(datid, datname, procpid, usesysid, usename, current_query, waiting, xact_start, query_start, backend_start, client_addr, client_port) WHERE (stat_activity.current_query <> '<IDLE>'::text) ORDER BY (now() - stat_activity.query_start);


ALTER TABLE tools.activity OWNER TO db_global_owner;

--
-- Name: df; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW df AS
    SELECT a.name, a.bytes, a.pages, pg_size_pretty(a.bytes) AS size FROM (SELECT ((((n.nspname)::character varying)::text || '.'::text) || ((c.relname)::character varying)::text) AS name, pg_total_relation_size(c.oid) AS bytes, c.relpages AS pages FROM (pg_class c JOIN pg_namespace n ON ((c.relnamespace = n.oid))) WHERE (c.relkind = 'r'::"char")) a;


ALTER TABLE tools.df OWNER TO db_global_owner;

--
-- Name: application_columns; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW application_columns AS
    SELECT c.oid AS table_oid, n.nspname AS schema_name, c.relname AS table_name, a.attname AS column_name, (((((n.nspname)::text || '.'::text) || (c.relname)::text) || '.'::text) || (a.attname)::text) AS column_full_name FROM ((pg_namespace n JOIN pg_class c ON ((c.relnamespace = n.oid))) JOIN pg_attribute a ON ((a.attrelid = c.oid))) WHERE ((((n.nspname !~ ALL (SELECT schemas__exclude.schema_name FROM replication.schemas__exclude)) AND (c.relkind = 'r'::"char")) AND (a.attisdropped = false)) AND (a.attnum > 0));


ALTER TABLE tools.application_columns OWNER TO db_global_owner;

--
-- Name: VIEW application_columns; Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON VIEW application_columns IS 'Non-infrastructure tables and columns.';


--
-- Name: application_indexes; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW application_indexes AS
    SELECT c.oid AS table_oid, n.nspname AS table_schema, c.relname AS table_name, i.oid AS index_oid, n.nspname AS index_schema, i.relname AS index_name, x.indisvalid AS index_is_valid, pg_get_indexdef(i.oid) AS index_definition, (((n.nspname)::text || '.'::text) || (c.relname)::text) AS table_full_name, (((n.nspname)::text || '.'::text) || (i.relname)::text) AS index_full_name FROM (((pg_index x JOIN pg_class c ON ((c.oid = x.indrelid))) JOIN pg_class i ON ((i.oid = x.indexrelid))) LEFT JOIN pg_namespace n ON ((n.oid = c.relnamespace))) WHERE (((n.nspname !~ ALL (SELECT schemas__exclude.schema_name FROM replication.schemas__exclude)) AND (c.relkind = 'r'::"char")) AND (i.relkind = 'i'::"char"));


ALTER TABLE tools.application_indexes OWNER TO db_global_owner;

--
-- Name: VIEW application_indexes; Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON VIEW application_indexes IS 'Non-infrastructure tables and indexes.';


--
-- Name: column_info; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW column_info AS
    SELECT n.nspname AS relation_schema, c.relname AS relation_name, c.oid AS relation_oid, a.attname AS column_name, a.attnum AS column_position, format_type(a.atttypid, a.atttypmod) AS data_type FROM ((pg_attribute a JOIN pg_class c ON ((c.oid = a.attrelid))) LEFT JOIN pg_namespace n ON ((n.oid = c.relnamespace))) WHERE ((((a.attnum > 0) AND (NOT a.attisdropped)) AND (c.relkind = ANY (ARRAY['r'::"char", 'v'::"char"]))) AND ((((pg_has_role(c.relowner, 'USAGE'::text) OR has_table_privilege(c.oid, 'SELECT'::text)) OR has_table_privilege(c.oid, 'INSERT'::text)) OR has_table_privilege(c.oid, 'UPDATE'::text)) OR has_table_privilege(c.oid, 'REFERENCES'::text)));


ALTER TABLE tools.column_info OWNER TO db_global_owner;

--
-- Name: df_index; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW df_index AS
    SELECT a.name, a.bytes, a.pages, pg_size_pretty(a.bytes) AS size FROM (SELECT ((((n.nspname)::character varying)::text || '.'::text) || ((c.relname)::character varying)::text) AS name, pg_total_relation_size(c.oid) AS bytes, c.relpages AS pages FROM (pg_class c JOIN pg_namespace n ON ((c.relnamespace = n.oid))) WHERE (c.relkind = 'i'::"char")) a;


ALTER TABLE tools.df_index OWNER TO db_global_owner;

--
-- Name: dump_time; Type: TABLE; Schema: tools; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE dump_time (
    dump_time timestamp with time zone NOT NULL
);


ALTER TABLE tools.dump_time OWNER TO db_global_owner;

--
-- Name: indexes; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW indexes AS
    SELECT n.nspname AS index_schema, c.relname AS table_name, i.relname AS index_name, t.spcname AS tablespace, x.indnatts AS number_of_attributes, x.indisunique AS is_unique, x.indisprimary AS is_primary, x.indisclustered AS is_clustered, x.indisvalid AS is_valid, x.indcheckxmin AS check_xmin, x.indisready AS is_ready, i.oid AS index_oid, c.oid AS table_oid FROM ((((pg_index x JOIN pg_class c ON ((c.oid = x.indrelid))) JOIN pg_class i ON ((i.oid = x.indexrelid))) LEFT JOIN pg_namespace n ON ((n.oid = c.relnamespace))) LEFT JOIN pg_tablespace t ON ((t.oid = i.reltablespace))) WHERE ((c.relkind = 'r'::"char") AND (i.relkind = 'i'::"char"));


ALTER TABLE tools.indexes OWNER TO db_global_owner;

--
-- Name: lock_waits; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW lock_waits AS
    SELECT l1.locktype, a.datname, n.nspname, c.relname, l1.page, l1.tuple, l1.virtualxid, (l1.transactionid)::text AS transactionid, (l1.classid)::regclass AS classid, l1.objid, l1.objsubid, (now() - a.query_start) AS age, l1.mode, l1.pid AS query_pid, CASE WHEN (l1.granted IS TRUE) THEN NULL::integer ELSE l2.pid END AS blocking_pid, a.query_start, l1.granted, a.usename, a.current_query FROM ((((pg_locks l1 JOIN pg_locks l2 ON ((NOT ((((((((((l1.locktype IS DISTINCT FROM l2.locktype) OR (l1.database IS DISTINCT FROM l2.database)) OR (l1.relation IS DISTINCT FROM l2.relation)) OR (l1.page IS DISTINCT FROM l2.page)) OR (l1.tuple IS DISTINCT FROM l2.tuple)) OR (l1.virtualxid IS DISTINCT FROM l2.virtualxid)) OR (l1.transactionid IS DISTINCT FROM l2.transactionid)) OR (l1.classid IS DISTINCT FROM l2.classid)) OR (l1.objid IS DISTINCT FROM l2.objid)) OR (l1.objsubid IS DISTINCT FROM l2.objsubid))))) LEFT JOIN pg_class c ON ((l1.relation = c.oid))) LEFT JOIN pg_namespace n ON ((c.relnamespace = n.oid))) LEFT JOIN pg_stat_activity a ON ((l1.pid = a.procpid))) WHERE ((l1.granted AND (NOT l2.granted)) OR (l2.granted AND (NOT l1.granted))) GROUP BY (now() - a.query_start), l1.granted, CASE WHEN (l1.granted IS TRUE) THEN NULL::integer ELSE l2.pid END, l1.locktype, a.datname, n.nspname, c.relname, l1.page, l1.tuple, l1.virtualxid, (l1.transactionid)::text, (l1.classid)::regclass, l1.objid, l1.objsubid, l1.mode, l1.pid, a.query_start, a.usename, a.current_query ORDER BY (now() - a.query_start) DESC, l1.granted DESC, CASE WHEN (l1.granted IS TRUE) THEN NULL::integer ELSE l2.pid END;


ALTER TABLE tools.lock_waits OWNER TO db_global_owner;

--
-- Name: object_types; Type: TABLE; Schema: tools; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE object_types (
    object_type text NOT NULL,
    catalog_table text NOT NULL,
    cast_name text,
    name_prefix text
);


ALTER TABLE tools.object_types OWNER TO db_global_owner;

--
-- Name: TABLE object_types; Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON TABLE object_types IS 'Information about different types of Postgres objects, such as what catalog table they live in. This is a seed table.';


--
-- Name: ownership; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW ownership AS
    SELECT a.object_type, n.nspname AS schema_name, a.object_name, r.rolname AS owner, a.schema_oid, a.owner_oid FROM (((((((((SELECT 'TYPE'::text AS object_type, t.typnamespace AS schema_oid, (t.typname)::text AS object_name, t.typowner AS owner_oid FROM pg_type t WHERE (((t.typrelid = (0)::oid) OR (SELECT (c.relkind = 'c'::"char") FROM pg_class c WHERE (c.oid = t.typrelid))) AND (t.typname !~ '^_'::text)) UNION ALL SELECT CASE WHEN pg_proc.proisagg THEN 'AGGREGATE'::text ELSE 'FUNCTION'::text END AS "case", pg_proc.pronamespace, ((((pg_proc.proname)::text || '('::text) || oidvectortypes(pg_proc.proargtypes)) || ')'::text), pg_proc.proowner FROM pg_proc) UNION ALL SELECT CASE pg_class.relkind WHEN 'r'::"char" THEN 'TABLE'::text WHEN 'i'::"char" THEN 'INDEX'::text WHEN 'S'::"char" THEN 'SEQUENCE'::text WHEN 'v'::"char" THEN 'VIEW'::text ELSE NULL::text END AS "case", pg_class.relnamespace, pg_class.relname, pg_class.relowner FROM pg_class WHERE (pg_class.relkind = ANY (ARRAY['r'::"char", 'S'::"char", 'v'::"char"]))) UNION ALL SELECT 'OPERATOR', pg_operator.oprnamespace, pg_operator.oprname, pg_operator.oprowner FROM pg_operator) UNION ALL SELECT 'OPERATOR CLASS', pg_opclass.opcnamespace, pg_opclass.opcname, pg_opclass.opcowner FROM pg_opclass) UNION ALL SELECT 'SCHEMA', NULL::unknown AS unknown, pg_namespace.nspname, pg_namespace.nspowner FROM pg_namespace) UNION ALL SELECT 'CONVERSION', pg_conversion.connamespace, pg_conversion.conname, pg_conversion.conowner FROM pg_conversion) UNION ALL SELECT 'TABLESPACE', NULL::unknown AS unknown, pg_tablespace.spcname, pg_tablespace.spcowner FROM pg_tablespace) a JOIN pg_namespace n ON ((n.oid = a.schema_oid))) JOIN pg_roles r ON ((r.oid = a.owner_oid)));


ALTER TABLE tools.ownership OWNER TO db_global_owner;

--
-- Name: ownership_bad; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW ownership_bad AS
    SELECT ownership.object_type, ownership.schema_name, ownership.object_name, ownership.owner, ownership.schema_oid, ownership.owner_oid FROM ownership WHERE ((((ownership.owner <> ALL (ARRAY['cnuadmin'::name, 'skytools'::name])) AND (ownership.owner !~~ 'postgres%'::text)) AND (ownership.schema_name !~~ 'pg_%'::text)) AND (ownership.owner <> ownership.schema_name));


ALTER TABLE tools.ownership_bad OWNER TO db_global_owner;

--
-- Name: role_members; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW role_members AS
    SELECT r.rolname AS role_name, m.rolname AS member_name, g.rolname AS granted_by, a.roleid, a.member, a.grantor, a.admin_option FROM (((pg_auth_members a LEFT JOIN pg_roles r ON ((r.oid = a.roleid))) LEFT JOIN pg_roles m ON ((m.oid = a.member))) LEFT JOIN pg_roles g ON ((g.oid = a.member)));


ALTER TABLE tools.role_members OWNER TO db_global_owner;

--
-- Name: schema_patches; Type: TABLE; Schema: tools; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE schema_patches (
    patch_file text NOT NULL,
    applied_at timestamp with time zone,
    depends_on text[],
    component_id smallint,
    CONSTRAINT a_patch_can_not_depend_on_itself CHECK ((NOT (patch_file = ANY (depends_on))))
);


ALTER TABLE tools.schema_patches OWNER TO db_global_owner;

--
-- Name: TABLE schema_patches; Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON TABLE schema_patches IS 'This is a seed table.';


--
-- Name: schema_patches_v; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW schema_patches_v AS
    SELECT t.patch_file, t.applied_at, t.depends_on, t.component_id, (cabar.components__get(t.component_id)).component AS component FROM schema_patches t;


ALTER TABLE tools.schema_patches_v OWNER TO db_global_owner;

--
-- Name: seed_objects; Type: TABLE; Schema: tools; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE seed_objects (
    schema_name text NOT NULL,
    object_name text NOT NULL,
    dump_order smallint DEFAULT 0 NOT NULL
);


ALTER TABLE tools.seed_objects OWNER TO db_global_owner;

--
-- Name: TABLE seed_objects; Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON TABLE seed_objects IS 'List of objects in the database that are considered to be seed. This is a seed table.';


--
-- Name: seed_objects_v; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW seed_objects_v AS
    SELECT o.schema_name, o.object_name, o.dump_order, c.oid AS relation_oid, relation_type(c.relkind) AS relation_type, ((quote_ident(o.schema_name) || '.'::text) || quote_ident(o.object_name)) AS full_name FROM (seed_objects o LEFT JOIN (pg_namespace n JOIN pg_class c ON ((c.relnamespace = n.oid))) ON ((((n.nspname)::text = o.schema_name) AND ((c.relname)::text = o.object_name))));


ALTER TABLE tools.seed_objects_v OWNER TO db_global_owner;

--
-- Name: seed_objects__dump; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW seed_objects__dump AS
    SELECT seed_objects_v.schema_name, seed_objects_v.object_name, seed_objects_v.dump_order, seed_objects_v.relation_oid, seed_objects_v.relation_type, seed_objects_v.full_name FROM seed_objects_v ORDER BY seed_objects_v.dump_order, seed_objects_v.schema_name, seed_objects_v.object_name;


ALTER TABLE tools.seed_objects__dump OWNER TO db_global_owner;

--
-- Name: stat_activity; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW stat_activity AS
    SELECT stat_activity.datid, stat_activity.datname, stat_activity.procpid, stat_activity.usesysid, stat_activity.usename, stat_activity.current_query, stat_activity.waiting, stat_activity.xact_start, stat_activity.query_start, stat_activity.backend_start, stat_activity.client_addr, stat_activity.client_port FROM stat_activity() stat_activity(datid, datname, procpid, usesysid, usename, current_query, waiting, xact_start, query_start, backend_start, client_addr, client_port);


ALTER TABLE tools.stat_activity OWNER TO db_global_owner;

--
-- Name: stats_small; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW stats_small AS
    SELECT pg_stats.schemaname, pg_stats.tablename, pg_stats.attname, pg_stats.null_frac, pg_stats.avg_width, pg_stats.n_distinct, pg_stats.correlation FROM pg_stats;


ALTER TABLE tools.stats_small OWNER TO db_global_owner;

--
-- Name: sub_asserts; Type: TABLE; Schema: tools; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE sub_asserts (
    assert_name character varying(1000) NOT NULL,
    sub_assert_name character varying(1000) NOT NULL
);


ALTER TABLE tools.sub_asserts OWNER TO db_global_owner;

--
-- Name: table_sequences; Type: VIEW; Schema: tools; Owner: db_global_owner
--

CREATE VIEW table_sequences AS
    SELECT t.oid AS table_oid, ts.nspname AS table_schema, t.relname AS table_name, a.attname AS column_name, s.oid AS sequence_oid, ss.nspname AS sequence_schema, s.relname AS sequence_name, (((ts.nspname)::text || '.'::text) || (t.relname)::text) AS table_full_name, (((((ts.nspname)::text || '.'::text) || (t.relname)::text) || '.'::text) || (a.attname)::text) AS column_full_name, (((ss.nspname)::text || '.'::text) || (s.relname)::text) AS sequence_full_name FROM (((((pg_depend d JOIN pg_class t ON ((t.oid = d.refobjid))) JOIN pg_class s ON ((s.oid = d.objid))) JOIN pg_namespace ts ON ((ts.oid = t.relnamespace))) JOIN pg_namespace ss ON ((ss.oid = s.relnamespace))) JOIN pg_attribute a ON (((a.attrelid = t.oid) AND (a.attnum = d.refobjsubid)))) WHERE ((((((d.refclassid = ('pg_class'::regclass)::oid) AND (d.classid = ('pg_class'::regclass)::oid)) AND (d.deptype = 'a'::"char")) AND (ts.nspname <> 'pg_toast'::name)) AND (t.relkind = 'r'::"char")) AND (s.relkind = 'S'::"char"));


ALTER TABLE tools.table_sequences OWNER TO db_global_owner;

--
-- Name: VIEW table_sequences; Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON VIEW table_sequences IS 'List of serial table columns and their corresponding sequences.';


--
-- Name: tables_id_seq; Type: SEQUENCE; Schema: tools; Owner: db_global_owner
--

CREATE SEQUENCE tables_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


ALTER TABLE tools.tables_id_seq OWNER TO db_global_owner;

--
-- Name: tables_id_seq; Type: SEQUENCE OWNED BY; Schema: tools; Owner: db_global_owner
--

ALTER SEQUENCE tables_id_seq OWNED BY tables.id;


--
-- Name: tables_id_seq; Type: SEQUENCE SET; Schema: tools; Owner: db_global_owner
--

SELECT pg_catalog.setval('tables_id_seq', 37, true);


--
-- Name: tests; Type: TABLE; Schema: tools; Owner: db_global_owner; Tablespace: 
--

CREATE TABLE tests (
    test_name text NOT NULL,
    completed boolean DEFAULT false NOT NULL,
    dependencies text[],
    CONSTRAINT test_cant_depend_on_itself CHECK ((test_name <> ALL (dependencies)))
);


ALTER TABLE tools.tests OWNER TO db_global_owner;

SET search_path = code_internal, pg_catalog;

--
-- Name: id; Type: DEFAULT; Schema: code_internal; Owner: db_global_owner
--

ALTER TABLE object_class ALTER COLUMN id SET DEFAULT nextval('object_class_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: code_internal; Owner: db_global_owner
--

ALTER TABLE object_type ALTER COLUMN id SET DEFAULT nextval('object_type_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: code_internal; Owner: db_global_owner
--

ALTER TABLE templates ALTER COLUMN id SET DEFAULT nextval('templates_id_seq'::regclass);


SET search_path = inheritance, pg_catalog;

--
-- Name: id; Type: DEFAULT; Schema: inheritance; Owner: db_global_owner
--

ALTER TABLE child_objects ALTER COLUMN id SET DEFAULT nextval('child_objects_id_seq'::regclass);


SET search_path = pgq, pg_catalog;

--
-- Name: co_id; Type: DEFAULT; Schema: pgq; Owner: skytools
--

ALTER TABLE consumer ALTER COLUMN co_id SET DEFAULT nextval('consumer_co_id_seq'::regclass);


--
-- Name: queue_id; Type: DEFAULT; Schema: pgq; Owner: skytools
--

ALTER TABLE queue ALTER COLUMN queue_id SET DEFAULT nextval('queue_queue_id_seq'::regclass);


--
-- Name: sub_id; Type: DEFAULT; Schema: pgq; Owner: skytools
--

ALTER TABLE subscription ALTER COLUMN sub_id SET DEFAULT nextval('subscription_sub_id_seq'::regclass);


SET search_path = replication, pg_catalog;

--
-- Name: id; Type: DEFAULT; Schema: replication; Owner: db_global_owner
--

ALTER TABLE tables ALTER COLUMN id SET DEFAULT nextval('tables_id_seq'::regclass);


SET search_path = tools, pg_catalog;

--
-- Name: id; Type: DEFAULT; Schema: tools; Owner: db_global_owner
--

ALTER TABLE tables ALTER COLUMN id SET DEFAULT nextval('tables_id_seq'::regclass);


SET search_path = cabar, pg_catalog;

--
-- Data for Name: component_schemas; Type: TABLE DATA; Schema: cabar; Owner: db_global_owner
--

COPY component_schemas (component_id, schema_name) FROM stdin;
1	test
1	tools
1	code
1	code_internal
1	replication
1	inheritance
1	object_monitor
1	setting
1	cabar
\.


--
-- Data for Name: components; Type: TABLE DATA; Schema: cabar; Owner: db_global_owner
--

COPY components (id, component_search_path, component, description) FROM stdin;
1	\N	db_global	\N
2	\N	cnuapp	\N
\.


--
-- Data for Name: env_settings; Type: TABLE DATA; Schema: cabar; Owner: db_global_owner
--

COPY env_settings (id, env_setting, description) FROM stdin;
1	search_path	\N
2	role	\N
3	client_min_messages	\N
\.


SET search_path = code_internal, pg_catalog;

--
-- Data for Name: object_class; Type: TABLE DATA; Schema: code_internal; Owner: db_global_owner
--

COPY object_class (id, object_type_id, object_class, description) FROM stdin;
1	1	function	\N
2	1	get	\N
3	1	get_text	\N
4	1	get_id	\N
5	2	lookup_table_static	\N
6	1	getsert	\N
7	2	lookup_table_dynamic	\N
8	2	valid_transitions	\N
9	1	inheritance_ri	\N
10	1	inheritance_unique	\N
11	3	id_automatic_update	\N
12	3	tg_match_field	\N
13	2	status_history	\N
14	4	denormalized_view	\N
\.


--
-- Data for Name: object_type; Type: TABLE DATA; Schema: code_internal; Owner: db_global_owner
--

COPY object_type (id, object_type) FROM stdin;
1	Function
2	Table
3	TRIGGER
4	View
\.


--
-- Data for Name: objects; Type: TABLE DATA; Schema: code_internal; Owner: db_global_owner
--

COPY objects (full_name, template_id, parameters, created_on, updated_on, creation_function_version) FROM stdin;
tools.assert__add	1	{{function,tools.assert__add},{params,"    p_assert_name tools.asserts.assert_name%TYPE\n    , p_level int"},{returns,void},{options,"LANGUAGE sql SECURITY DEFINER"},{body,"INSERT INTO tools.asserts( assert_name, level ) VALUES( $1, $2 )"}}	2009-04-09 16:48:07.985869	2009-04-09 16:48:07.985869	1
tools.assert__get	1	{{function,tools.assert__get},{params,"    p_assert_name tools.asserts.assert_name%TYPE"},{returns,tools.asserts},{options,"LANGUAGE sql"},{body,"SELECT * FROM tools.asserts WHERE lower( assert_name ) = lower( $1 );"}}	2009-04-09 16:48:07.985869	2009-04-09 16:48:07.985869	1
tools.assert__enabled_session__set	1	{{function,tools.assert__enabled_session__set},{params,"    p_assert_name tools.asserts.assert_name%TYPE\n    , p_enabled tools.asserts.enabled%TYPE"},{returns,void},{options,"LANGUAGE plpgsql SECURITY DEFINER"},{body,"\nBEGIN\n    IF current_setting('custom_variable_classes') !~ 'disabledasserts' THEN\n        RAISE EXCEPTION 'Unable to use per-session assert settings: \\"disabledasserts\\" is not in custom_variable_classes (see postgresql.conf)';\n    END IF;\n\n    PERFORM set_config( 'disabledasserts.' || p_assert_name, CASE WHEN p_enabled THEN '' ELSE '*' END, false );\nEND;\n"}}	2009-04-09 16:48:07.985869	2009-04-09 16:48:07.985869	1
tools.assert__enabled_session	1	{{function,tools.assert__enabled_session},{params,"    p_assert_name tools.asserts.assert_name%TYPE"},{returns,BOOLEAN},{options,"LANGUAGE plpgsql"},{body,"\nBEGIN\n    IF current_setting('custom_variable_classes') ~ 'disabledasserts' THEN\n        RETURN current_setting('disabledasserts.' || p_assert_name) = '';\n    ELSE\n        RETURN true;\n    END IF;\nEND;"}}	2009-04-09 16:48:07.985869	2009-04-09 16:48:07.985869	1
inheritance.reset_child	1	{{function,inheritance.reset_child},{params,"p_child_table text\n  , p_object_type inheritance.object_types.object_type%TYPE"},{returns,text},{options,"LANGUAGE plpgsql SECURITY DEFINER"},{body,"\nDECLARE\n  fname CONSTANT text := 'inheritance.reset_child';\n  v_child oid := p_child_table::regclass::oid;\n\n  v_child_full text := tools.table_full_name( p_child_table );\n  v_child_schema text;\n  v_child_name text;\n\n  r_object record;\n  v_name_array text[][];\n  sql text;\nBEGIN\n  /* We need to be SU to run the dangerous londiste stuff we're doing. That\n   * means it's up to us to check that the calling user owns the tables\n   * involved. We can't check parents yet, but we can at least check the child\n   * now.\n   */\n  /* Need a way to check to see if user has access to SU\n  IF ( SELECT pg_catalog.pg_get_userbyid( relowner ) FROM pg_class WHERE oid = v_child ) != session_user THEN\n    RAISE EXCEPTION 'You must own the child table %', v_child_full;\n  END IF;\n  */\n\n  IF NOT EXISTS( SELECT * FROM pg_inherits WHERE inhrelid = v_child ) THEN\n    RAISE EXCEPTION '%: % is not an inherited table', fname, p_child_table;\n  END IF;\n\n  /* Having londiste triggers MIA could cause problems, so put them back. Note\n   * that subscriber_pending_fkeys is only used during replication setup, so we\n   * don't want to screw with it, but we should bomb if anything is in there\n   * for our table.\n   * MAKE SURE TO CALL drop_all_triggers at the end!\n   */\n  PERFORM tools.assert(\n    'inheritance__process_child no pending fkeys'\n    , NOT EXISTS( SELECT * FROM tools.londiste__pending_fkeys( v_child_full ) )\n    , 'Unable to process table ' || v_child_full || ': there are pending foreign keys for it in londiste'\n  );\n  PERFORM tools.londiste__restore_all_triggers( v_child_full );\n\n  SELECT INTO v_child_schema, v_child_name * FROM tools.table_schema_and_name( p_child_table );\n  FOR r_object IN\n  SELECT c.*\n    FROM inheritance.child_objects_v c\n      JOIN inheritance.get_parents( v_child ) p ON c.inherited_table_oid = p.parent_oid\n    WHERE NOT EXISTS(\n            SELECT *\n              FROM inheritance.child_objects_exclude_tables e\n                JOIN tools.tables_v t ON t.id = e.exclude_table_id\n              WHERE e.child_object_id = c.id\n                AND t.oid = v_child\n          ) AND c.object_type = p_object_type\n  LOOP\n    /* Need a way to check to see if user has access to SU\n    IF ( SELECT pg_catalog.pg_get_userbyid( relowner ) FROM pg_class WHERE oid = r_object.inherited_table_oid ) != session_user\n      AND NOT pg_has_role(session_user, role, 'USAGE' )\n    THEN\n      RAISE EXCEPTION 'You must own the parent table % or be superuser', r_object.inherited_table_full_name;\n    END IF;\n    */\n\n    PERFORM tools.assert(\n      'inheritance__reset_child no pending fkeys'\n      , NOT EXISTS( SELECT * FROM tools.londiste__pending_fkeys( r_object.inherited_table_full_name  ) )\n      , 'Unable to reset table ' || r_object.inherited_table_full_name || ': there are pending foreign keys for it in londiste'\n    );\n\n    v_name_array := array[\n      [ 'parent_schema', r_object.inherited_table_schema ]\n      , [ 'parent_name', r_object.inherited_table_name ]\n      , [ 'parent_full', r_object.inherited_table_full_name ]\n      , [ 'child_schema', v_child_schema ]\n      , [ 'child_name', v_child_name ]\n      , [ 'child_full', v_child_full ]\n    ];\n    SELECT INTO sql reset_template FROM inheritance.object_types__get( r_object.object_type_id );\n    sql := tools.parameter_replace(\n        tools.parameter_replace(\n          sql\n          , array[\n            [ 'object_name_template', r_object.object_name_template ]\n            , [ 'object_body_template', r_object.object_body_template ]\n          ]\n        )\n        , v_name_array\n    );\n    BEGIN\n      PERFORM tools.exec( sql );\n    EXCEPTION\n      WHEN duplicate_table THEN -- We get this for a duplicate unique constraint\n        IF SQLERRM !~ tools.parameter_replace( r_object.object_name_template, v_name_array ) THEN\n          RAISE EXCEPTION '%: Error %, %', fname, SQLSTATE, SQLERRM;\n        END IF;\n      WHEN duplicate_object THEN\n        IF SQLERRM !~ tools.parameter_replace( r_object.object_name_template, v_name_array ) THEN\n          RAISE EXCEPTION '%: Error %, %', fname, SQLSTATE, SQLERRM;\n        END IF;\n      WHEN invalid_table_definition THEN\n        IF SQLERRM !~ 'multiple primary keys for table' THEN\n          RAISE EXCEPTION '%: Error %, %', fname, SQLSTATE, SQLERRM;\n        END IF;\n      WHEN others THEN\n        RAISE EXCEPTION '%: Error %, %', fname, SQLSTATE, SQLERRM;\n    END;\n  END LOOP;\n\n  -- Remove the triggers again now that we're done\n  PERFORM tools.londiste__drop_all_triggers( v_child_full );\n\n  RETURN v_child::regclass::text;\nEND;\n"}}	2010-07-30 02:41:56.877656	2010-07-30 02:41:56.877656	1
inheritance.object_types	5	{{schema,inheritance},{plural,object_types},{singular,object_type},{post_id,""}}	2009-10-09 18:16:05.819132	2009-10-09 18:16:05.819132	1
inheritance.object_types__get_id	1	{{schema,inheritance},{table,object_types},{lookup,object_type}}	2009-10-09 18:16:05.819132	2009-10-09 18:16:05.819132	1
inheritance.object_types__get	1	{{schema,inheritance},{table,object_types}}	2009-10-09 18:16:05.819132	2009-10-09 18:16:05.819132	1
inheritance.object_types__get_text	1	{{schema,inheritance},{table,object_types},{lookup,object_type}}	2009-10-09 18:16:05.819132	2009-10-09 18:16:05.819132	1
lead_sale.tg_ping_tree_checks	1	{{function,lead_sale.tg_ping_tree_checks},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql"},{body,"\nBEGIN\n  PERFORM tools.trigger_sanity( TG_NAME, TG_WHEN, TG_LEVEL, TG_OP, 'after', 'row', '{update}' );\n\n  IF NEW.ping_tree_status_id IS DISTINCT FROM OLD.ping_tree_status_id THEN\n    PERFORM tools.assert(\n      'lead_sale__branch_position_unique'\n      , NOT EXISTS( SELECT position\n                      FROM lead_sale.branches\n                      WHERE ping_tree_id = NEW.id\n                      GROUP BY position\n                      HAVING count(*) > 1\n                    )\n      , 'Unable to change state of ping tree ( ID = ' || NEW.id || ' ); there are branches with duplicate positions'\n    );\n  END IF;\n\n  RETURN NULL;\nEND;\n"}}	2009-09-24 19:18:22.753319	2009-09-24 19:18:22.753319	1
lead_sale.tg_branch_checks	1	{{function,lead_sale.tg_branch_checks},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql"},{body,"\nBEGIN\n  PERFORM tools.trigger_sanity( TG_NAME, TG_WHEN, TG_LEVEL, TG_OP, 'after', 'row', '{insert, update}' );\n\n  PERFORM tools.assert(\n    'lead_sale__branch_change_check_tree_status'\n    , ( SELECT ping_tree_status_id FROM lead_sale.ping_trees WHERE id = NEW.ping_tree_id )\n      = lead_sale.ping_tree_statuses__get_id( 'created' )\n    , 'Lead sale branches may only be changed if the ping tree is in the \\"created\\" status'\n  );\n\n  RETURN NULL;\nEND;\n"}}	2009-09-24 19:18:22.753319	2009-09-24 19:18:22.753319	1
code.inheritance_ri	1	{{function,code.inheritance_ri},{params,"\np_schema text\n, p_parent_table text\n, p_primary_key text\n, p_type_plural text\n, p_type_singular text\n, p_table_name_field text\n, p_child_name_prefix text\n, p_child_name_suffix text\n"},{returns,text},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n    v_function_name CONSTANT text := p_schema || '.tg_' || p_parent_table || '_ri()';\nBEGIN\n    PERFORM code_internal.create_object(\n            v_function_name\n            , 'FUNCTION'\n            , 'inheritance_ri'\n            , array[    ['schema', p_schema]\n                        , [ 'parent_table', p_parent_table ]\n                        , [ 'primary_key', p_primary_key ]\n                        , [ 'types_plural', p_type_plural ]\n                        , [ 'type_singular', p_type_singular ]\n                        , [ 'table_name_field', coalesce( p_table_name_field, p_type_singular ) ]\n                        , [ 'child_name_prefix', coalesce( p_child_name_prefix, p_schema || '.' ) ]\n                        , [ 'child_name_suffix', coalesce( p_child_name_suffix, 's' ) ]\n                    ]\n        )\n    ;\n\n    RETURN v_function_name;\nEND;\n"}}	2010-01-28 00:36:09.002093	2010-01-28 00:36:09.002093	1
code.inheritance_unique	1	{{function,code.inheritance_unique},{params,"\np_schema text\n, p_parent_table text\n, p_primary_key text\n"},{returns,text},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n    v_function_name CONSTANT text := p_schema || '.tg_' || p_parent_table || '_unique()';\nBEGIN\n    PERFORM code_internal.create_object(\n            v_function_name\n            , 'FUNCTION'\n            , 'inheritance_unique'\n            , array[    ['schema', p_schema]\n                        , [ 'parent_table', p_parent_table ]\n                        , [ 'primary_key', p_primary_key ]\n                    ]\n        )\n    ;\n\n    RETURN v_function_name;\nEND;\n"}}	2010-01-28 00:36:09.002093	2010-01-28 00:36:09.002093	1
code.id_automatic_update	1	{{function,code.id_automatic_update},{params,"\np_schema text\n, p_source_singular text\n, p_sources_plural text\n, p_target text\n"},{returns,text},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n    v_trigger_name CONSTANT text := 'tg_' || p_source_singular || '_update_' || p_target;\n    v_full_trigger_name CONSTANT text := p_schema || '.' || v_trigger_name;\nBEGIN\n    RAISE INFO 'Creating status automatic update trigger %', v_full_trigger_name;\n\n    PERFORM code_internal.create_object(\n            v_full_trigger_name\n            , 'TRIGGER'\n            , 'id_automatic_update'\n            , array[    ['schema', p_schema]\n                        , ['source_singular', p_source_singular]\n                        , ['sources_plural', p_sources_plural]\n                        , ['target', p_target]\n                    ]\n        );\n    \n    RETURN v_full_trigger_name;\nEND;\n"}}	2010-01-28 00:36:09.208784	2010-01-28 00:36:09.208784	1
tools.currval	1	{{function,tools.currval},{params,"p_table text, p_column text"},{returns,bigint},{options,"LANGUAGE sql"},{body,"\nSELECT currval( pg_get_serial_sequence( $1, $2 ) )\n  WHERE tools.assert( 'tools__currval__no_sequence'\n            , pg_get_serial_sequence( $1, $2 ) IS NOT NULL\n            , 'column ' || pg_catalog.quote_ident($2) || ' of table ' || pg_catalog.quote_ident($1) || ' does not have a sequence associated with it'\n          )\n"}}	2010-01-28 00:36:09.534231	2010-01-28 00:36:09.534231	1
physical_payments_us.tg_returns_unique	1	{{function,physical_payments_us.tg_returns_unique},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\nname CONSTANT text := 'physical_payments_us.tg_returns_unique';\nc_full_table_name CONSTANT text := TG_TABLE_SCHEMA || '.' || TG_TABLE_NAME;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'BEFORE', TG_NAME || ' ON ' || c_full_table_name ||' must be an BEFORE trigger' );\n    PERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' ON ' || c_full_table_name ||' must be a row-level trigger' );\n    \n    -- Deleting would break RI, so don't allow it. Granted, this should probably be a separate trigger, but...\n    PERFORM tools.assert( 'physical_payments_us__returns__inherit__no_delete'\n        , TG_OP != 'DELETE'\n        , 'DELETEs are not allowed on ' || c_full_table_name || ' (they would break inheritance RI)'\n    );\n\n    RAISE DEBUG '%:\n        TG_OP = %\n        TG_TABLE_NAME = %\n        NEW.return_id = %'\n        , name\n        , TG_OP\n        , TG_TABLE_NAME\n        , NEW.return_id \n    ;\n\n    -- Changing the PK would break RI, so we shouldn't allow it. Granted, this should probably be a separate trigger, but...\n    IF TG_OP = 'UPDATE' THEN\n        PERFORM tools.assert( 'physical_payments_us__returns__inherit__pk_no_change'\n            , NEW.return_id IS NOT DISTINCT FROM OLD.return_id\n            , 'Changing return_id on ' || c_full_table_name || ' is not allowed (it would break inheritance RI)'\n        );\n    ELSE\n        -- Only check for dupes on insert, otherwise we'll see our own ID\n        PERFORM tools.assert( 'physical_payments_us__returns__inherit__unique'\n            , NOT EXISTS( SELECT * FROM physical_payments_us.returns WHERE return_id = NEW.return_id )\n            , 'duplicate row violation, return_id ' || coalesce( NEW.return_id::text, '<NULL>' ) || ' already exists'\n        );\n    END IF;\n\n    RETURN NEW;\nEND;\n"}}	2010-03-04 23:36:43.200725	2010-03-04 23:36:43.200725	1
inheritance.process_all	1	{{function,inheritance.process_all},{params,"OUT child_table text"},{returns,"SETOF text"},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n  v_parent text;\nBEGIN\n  FOR v_parent IN SELECT DISTINCT inherited_table_full_name FROM inheritance.child_objects_v LOOP\n    FOR child_table IN SELECT * FROM inheritance.process_children( v_parent ) LOOP\n      RETURN NEXT;\n    END LOOP;\n  END LOOP;\n\n  RETURN;\nEND;\n"}}	2010-03-18 18:52:13.23773	2010-03-18 18:52:13.23773	1
tools.attribute_positions	1	{{function,tools.attribute_positions},{params,"p_relation_name text\n  , p_attribute_names text"},{returns,smallint[]},{options,"LANGUAGE sql STABLE"},{body,"SELECT tools.attribute_positions( $1, tools.string_or_array( $2 ) )"}}	2010-01-28 00:36:09.651344	2010-01-28 00:36:09.651344	1
tools.foreign_key__get	1	{{function,tools.foreign_key__get},{params,"p_fk_table text\n  , p_fk_columns text[]\n  , p_key_table text\n  , p_key_columns text[]"},{returns,"SETOF pg_catalog.pg_constraint"},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n  v_fk_length CONSTANT int := tools.array_length( p_fk_columns, 1 );\n  v_key_length CONSTANT int := tools.array_length( p_key_columns, 1 );\n\n  v_fk_positions CONSTANT smallint[] := tools.attribute_positions( p_fk_table, p_fk_columns );\n  v_key_positions CONSTANT smallint[] := tools.attribute_positions( p_key_table, p_key_columns );\n  v_return pg_catalog.pg_constraint;\nBEGIN\n  IF v_fk_length <> v_key_length THEN \n    RAISE EXCEPTION 'Number of FK columns (%) must equal number of key columns (%)', v_fk_length, v_key_length;\n  END IF;\n  FOR v_return IN\n  SELECT *\n    FROM pg_constraint\n    WHERE contype = 'f'\n      AND confrelid = p_fk_table::regclass\n      AND confkey = v_fk_positions\n      AND conrelid = p_key_table::regclass\n      AND conkey = v_key_positions\n  LOOP\n    RETURN NEXT v_return;\n  END LOOP;\n  RETURN;\nEND;\n"}}	2010-01-28 00:36:09.651344	2010-01-28 00:36:09.651344	1
tools.foreign_key__exists	1	{{function,tools.foreign_key__exists},{params,"p_fk_table text\n  , p_fk_columns text[]\n  , p_key_table text\n  , p_key_columns text[]"},{returns,boolean},{options,"LANGUAGE sql"},{body,"\nSELECT count(*) > 0 FROM tools.foreign_key__get( $1, $2, $3, $4 )\n"}}	2010-01-28 00:36:09.651344	2010-01-28 00:36:09.651344	1
test.foreign_key__exists	1	{{function,test.foreign_key__exists},{params,"p_fk_table text\n  , p_fk_columns text[]\n  , p_key_table text\n  , p_key_columns text[]"},{returns,void},{options,"LANGUAGE plpgsql"},{body,"\nBEGIN\n    IF NOT tools.foreign_key__exists( p_fk_table, p_fk_columns, p_key_table, p_key_columns ) THEN\n        RAISE EXCEPTION 'Foreign key to % (%) from % (%) does not exist'\n            , p_fk_table\n            , array_to_string( p_fk_columns, ', ' )\n            , p_key_table\n            , array_to_string( p_key_columns, ', ' )\n        ;\n    END IF;\nEND;\n"}}	2010-01-28 00:36:09.651344	2010-01-28 00:36:09.651344	1
tools.primary_unique_constraint__get	1	{{function,tools.primary_unique_constraint__get},{params,"p_table text\n  , p_columns text[]"},{returns,"SETOF pg_catalog.pg_constraint"},{options,"LANGUAGE sql"},{body,"\nSELECT *\n  FROM pg_constraint\n  WHERE conrelid = $1::regclass\n    AND conkey = tools.attribute_positions( $1, $2 )\n    AND contype IN ( 'p', 'u' )\n"}}	2010-01-28 00:36:09.651344	2010-01-28 00:36:09.651344	1
tools.primary_unique_constraint__exists	1	{{function,tools.primary_unique_constraint__exists},{params,"p_table text\n  , p_columns text[]"},{returns,boolean},{options,"LANGUAGE sql"},{body,"\nSELECT count(*) > 0 FROM tools.primary_unique_constraint__get( $1, $2 )\n"}}	2010-01-28 00:36:09.651344	2010-01-28 00:36:09.651344	1
test.primary_unique_constraint__exists	1	{{function,test.primary_unique_constraint__exists},{params,"p_table text\n  , p_columns text[]"},{returns,boolean},{options,"LANGUAGE sql"},{body,"\nSELECT tools.assert( tools.primary_unique_constraint__exists( $1, $2 )\n  , 'There is no primary key/unique constraint on table ' || quote_ident( $1 ) || ' for columns '\n    || array_to_string( $2, ', ' )\n);\n"}}	2010-01-28 00:36:09.651344	2010-01-28 00:36:09.651344	1
code.tg_match_field	1	{{function,code.tg_match_field},{params,"\np_schema text\n, p_target_table text\n, p_target_pk text\n, p_target_field text\n, p_fk_field text\n, p_match_field text\n, p_select text\n"},{returns,text},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n    v_trigger_name CONSTANT text := 'tg_match_' || p_fk_field || '__' || p_match_field;\n    v_full_trigger_name CONSTANT text := p_schema || '.' || v_trigger_name;\n    v_target_pk CONSTANT text := coalesce( p_target_pk, 'id' );\n    v_select text;\nBEGIN\n    RAISE INFO 'Creating status automatic update trigger %', v_full_trigger_name;\n\n    v_select := coalesce( p_select\n        , p_target_field || ' FROM ' || p_schema || '.' || p_target_table || '\n        WHERE ' || v_target_pk || ' = NEW.' || p_fk_field\n    );\n\n    PERFORM code_internal.create_object(\n            v_full_trigger_name\n            , 'TRIGGER'\n            , 'tg_match_field'\n            , array[    [ 'schema', p_schema ]\n                            , [ 'tg_name', v_trigger_name ]\n                            , [ 'target_table', p_schema || '.' || p_target_table ]\n                            , [ 'target_pk', v_target_pk ]\n                            , [ 'target_field', p_target_field ]\n                            , [ 'fk_field', p_fk_field ]\n                            , [ 'match_field', p_match_field ]\n                            , [ 'select', v_select ]\n                    ]\n        );\n    \n    RETURN v_full_trigger_name;\nEND;\n"}}	2010-01-28 00:36:09.844089	2010-01-28 00:36:09.844089	1
payment_instruments.tg_payment_instruments_ri	1	{{function,payment_instruments.tg_payment_instruments_ri},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n    name CONSTANT text := 'payment_instruments.tg_payment_instruments_ri';\n    c_full_table_name CONSTANT text := TG_TABLE_SCHEMA || '.' || TG_TABLE_NAME;\n    v_payment_instrument_type payment_instruments.payment_instrument_types.payment_instrument_type%TYPE;\n    v_table_name text;\n    v_only text := '';\n    v_result int;\n    sql text;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'AFTER', TG_NAME || ' ON ' || c_full_table_name ||' must be an AFTER trigger' );\n    PERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' ON ' || c_full_table_name ||' must be a row-level trigger' );\n    PERFORM tools.assert( TG_OP IN ( 'INSERT', 'UPDATE' ), TG_NAME || ' ON ' || c_full_table_name ||' must be on INSERT or UPDATE' );\n\n    -- Th generally won't be allowed, but the trigger should still support it\n    IF NEW.payment_instrument_id IS NULL THEN\n        RAISE DEBUG '%: payment_instrument_id is NULL, skipping check', name;\n        RETURN NULL;\n    END IF;\n\n    -- If we're updating and we haven't modified payment_instrument_id, just bail\n    IF TG_OP = 'UPDATE' THEN\n        IF NEW.payment_instrument_id IS NOT DISTINCT FROM OLD.payment_instrument_id THEN\n            RAISE DEBUG '%: payment_instrument_id ( = % ) is unchanged, skipping check', name, NEW.payment_instrument_id;\n            RETURN NULL;\n        END IF;\n    END IF;\n\n    /*\n     * We want to not only check for existence of the desired row, we also want\n     * to share-lock it. Unfortunately, sharelocks aren't implemented for\n     * inherited tables, so we need to find the record in the correct table. We\n     * can do this fairly easily if we can find out what \\"type\\" of record it is.\n     */\n\n    -- Figure out what type of record this is (of course, record might not exist here)\n    SELECT INTO v_payment_instrument_type 'payment_instruments.' || payment_instrument_type || 's'\n        FROM payment_instruments.payment_instrument_types__get( (\n                    SELECT payment_instrument_type_id\n                        FROM payment_instruments.payment_instruments pi\n                        WHERE pi.payment_instrument_id = NEW.payment_instrument_id\n                ) )\n    ;\n    IF NOT FOUND OR v_payment_instrument_type IS NULL THEN\n        -- There wasn't a record at all\n        v_only := 'ONLY '; -- FOR SHARE won't work if we select from both the parent and the children\n        v_table_name := 'payment_instruments.payment_instruments';\n    ELSE\n        -- We figured out what type of record this is, now try and lock the row in the right table\n        v_table_name := v_payment_instrument_type;\n    END IF;\n\n    sql := 'SELECT 1 FROM ' || v_only || v_table_name || '\n        WHERE payment_instrument_id = ' || NEW.payment_instrument_id || '\n        FOR SHARE'\n    ;\n    RAISE DEBUG '%: Executing SQL %', name, sql;\n    -- Note that simply using a PERFORM here might get optimized out.\n    EXECUTE sql INTO v_result;\n\n    /*\n     * Normally we should always find an record, but if it was somehow\n     * put in the wrong table, or if it was deleted after our initial\n     * select then we wouldn't have one.\n     */\n    IF v_result = 1 THEN\n        RAISE DEBUG '%: record for payment_instrument_id = % found in table %', name, NEW.payment_instrument_id, v_table_name;\n        RETURN NULL;\n    END IF;\n\n    RAISE EXCEPTION 'insert on update on table \\"%.%\\" violates foreign key; payment_instrument_id=(%) is not present in table \\"%\\"'\n        , TG_TABLE_SCHEMA\n        , TG_TABLE_NAME\n        , NEW.payment_instrument_id\n        , v_table_name\n    ;\nEND;\n"}}	2010-01-28 00:36:10.982205	2010-01-28 00:36:10.982205	1
payment_instruments.tg_payment_instruments_unique	1	{{function,payment_instruments.tg_payment_instruments_unique},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\nname CONSTANT text := 'payment_instruments.tg_payment_instruments_unique';\nc_full_table_name CONSTANT text := TG_TABLE_SCHEMA || '.' || TG_TABLE_NAME;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'BEFORE', TG_NAME || ' ON ' || c_full_table_name ||' must be an BEFORE trigger' );\n    PERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' ON ' || c_full_table_name ||' must be a row-level trigger' );\n    \n    -- Deleting would break RI, so don't allow it. Granted, this should probably be a separate trigger, but...\n    PERFORM tools.assert( 'payment_instruments__payment_instruments__inherit__no_delete'\n        , TG_OP != 'DELETE'\n        , 'DELETEs are not allowed on ' || c_full_table_name || ' (they would break inheritance RI)'\n    );\n\n    RAISE DEBUG '%:\n        TG_OP = %\n        TG_TABLE_NAME = %\n        NEW.payment_instrument_id = %'\n        , name\n        , TG_OP\n        , TG_TABLE_NAME\n        , NEW.payment_instrument_id \n    ;\n\n    -- Changing the PK would break RI, so we shouldn't allow it. Granted, this should probably be a separate trigger, but...\n    IF TG_OP = 'UPDATE' THEN\n        PERFORM tools.assert( 'payment_instruments__payment_instruments__inherit__pk_no_change'\n            , NEW.payment_instrument_id IS NOT DISTINCT FROM OLD.payment_instrument_id\n            , 'Changing payment_instrument_id on ' || c_full_table_name || ' is not allowed (it would break inheritance RI)'\n        );\n    ELSE\n        -- Only check for dupes on insert, otherwise we'll see our own ID\n        PERFORM tools.assert( 'payment_instruments__payment_instruments__inherit__unique'\n            , NOT EXISTS( SELECT * FROM payment_instruments.payment_instruments WHERE payment_instrument_id = NEW.payment_instrument_id )\n            , 'duplicate row violation, payment_instrument_id ' || coalesce( NEW.payment_instrument_id::text, '<NULL>' ) || ' already exists'\n        );\n    END IF;\n\n    RETURN NEW;\nEND;\n"}}	2010-01-28 00:36:10.982205	2010-01-28 00:36:10.982205	1
payment_instruments.payment_instrument__get	1	{{function,payment_instruments.payment_instrument__get},{params,"    p_payment_instrument_id payment_instruments.payment_instruments.payment_instrument_id%TYPE"},{returns,payment_instruments.payment_instruments},{options,"LANGUAGE sql RETURNS NULL ON NULL INPUT"},{body,"\nSELECT * FROM payment_instruments.payment_instruments WHERE payment_instrument_id = $1\n"}}	2010-01-28 00:36:10.982205	2010-01-28 00:36:10.982205	1
code.status_history	1	{{function,code.status_history},{params,"\np_schema                text\n, p_table_plural        text\n, p_table_singular      text\n, p_primary_key         text\n, p_status_name         text\n"},{returns,void},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n    v_status_table_name CONSTANT text := p_status_name || '_status_history';\n    v_status_table_name_full CONSTANT text := p_schema || '.' || v_status_table_name;\n\n    v_table_ri text := '';\n    v_track_table_name_full CONSTANT text := p_schema || '.' || p_table_plural;\nBEGIN\n    IF p_status_name ~* 'status' THEN\n        RAISE WARNING $$p_status_name (\\"%\\") contains the string 'status', resulting in a table name of %; are you sure this is what you want?$$\n            , p_status_name\n            , v_status_table_name\n        ;\n    ELSE\n        RAISE INFO 'Creating status history table %', v_status_table_name_full;\n    END IF;\n\n    -- See if the table being tracked has a primary key. If it does, add RI to it\n    IF EXISTS( SELECT *\n                    FROM pg_index\n                    WHERE indrelid = v_track_table_name_full::regclass\n                        AND indisprimary\n    ) THEN\n        v_table_ri := 'REFERENCES ' || v_track_table_name_full || ' ON DELETE CASCADE ON UPDATE CASCADE';\n    ELSE\n        RAISE WARNING 'Table being tracked(%) does not have a primary key, no foreign key will be created!', v_track_table_name_full;\n    END IF;\n\n    PERFORM code_internal.create_object(\n            v_status_table_name_full\n            , 'TABLE'\n            , 'status_history'\n            , array[    ['schema', p_schema]\n                        , ['table_plural', p_table_plural]\n                        , ['primary_key', p_primary_key]\n                        , ['table_pk_reference', CASE WHEN p_primary_key = 'id' THEN p_table_singular || '_id' ELSE p_primary_key END ]\n                        , ['status_name', p_status_name]\n                        , ['track_table_RI', v_table_ri]\n                    ]\n        );\nEND;\n"}}	2010-01-28 00:36:42.973221	2010-01-28 00:36:42.973221	1
cnu.employee_id_from_by	1	{{function,cnu.employee_id_from_by},{params,text},{returns,employees.id%TYPE},{options,"LANGUAGE sql"},{body,"\n  SELECT id\n    FROM employees\n    WHERE login = substring( $1 from 'E:([^ ]*)' )\n"}}	2010-03-04 23:40:35.054806	2010-03-04 23:40:35.054806	1
cnu.tg_set_created_by_employee_id	1	{{function,cnu.tg_set_created_by_employee_id},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql"},{body,"\nBEGIN\n  PERFORM tools.trigger_sanity( TG_NAME, TG_WHEN, TG_LEVEL, TG_OP, 'before', 'row', '{insert, update}' );\n\n  NEW.created_by_employee_id := cnu.employee_id_from_by( NEW.created_by );\n\tRETURN NEW;\nEND;\n"}}	2010-03-04 23:40:35.054806	2010-03-04 23:40:35.054806	1
cnu.tg_set_updated_by_employee_id	1	{{function,cnu.tg_set_updated_by_employee_id},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql"},{body,"\nBEGIN\n  PERFORM tools.trigger_sanity( TG_NAME, TG_WHEN, TG_LEVEL, TG_OP, 'before', 'row', '{insert, update}' );\n\n  NEW.updated_by_employee_id := cnu.employee_id_from_by( NEW.updated_by );\n\tRETURN NEW;\nEND;\n"}}	2010-03-04 23:40:35.054806	2010-03-04 23:40:35.054806	1
payment_instruments.tg_payment_instrument_status_history	1	{{function,payment_instruments.tg_payment_instrument_status_history},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql SECURITY DEFINER"},{body,"\nDECLARE\n    v_track_changed_by boolean := true;\n    v_changed_by varchar(30) := NULL;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'AFTER', TG_NAME || ' must be an AFTER trigger' );\n\tPERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' must be a row-level trigger' );\n\tPERFORM tools.assert( TG_OP != 'DELETE', 'DELETE is not supported for trigger ' || TG_NAME );\n\n    IF TG_OP = 'UPDATE' THEN\n        IF NEW.payment_instrument_status_id IS NOT DISTINCT FROM OLD.payment_instrument_status_id THEN\n            RETURN NULL;\n        END IF;\n    END IF;\n    \n    IF TG_NARGS > 0 THEN\n        v_track_changed_by := lower( TG_ARGV[0] ) = 'true';\n    END IF;\n    IF v_track_changed_by THEN\n        v_changed_by := NEW.updated_by;\n    END IF;\n\n    INSERT INTO payment_instruments.payment_instrument_status_history( payment_instrument_id, payment_instrument_status_id, changed_by )\n        VALUES( NEW.payment_instrument_id, NEW.payment_instrument_status_id, v_changed_by )\n    ;\n    RETURN NULL;\nEND;\n"}}	2010-01-28 00:36:43.135333	2010-01-28 00:36:43.135333	1
tables__get_id	1	{{function,tables__get_id},{params,"p_table text"},{returns,tools.tables.id%TYPE},{options,"LANGUAGE plpgsql RETURNS NULL ON NULL INPUT"},{body,"\nDECLARE\n  v_return tools.tables.id%TYPE;\n\n  v_table_schema text;\n  v_table_name text;\nBEGIN\n  SELECT INTO v_table_schema, v_table_name * FROM tools.table_schema_and_name( p_table );\n\n  SELECT INTO v_return *\n    FROM tools.tables\n    WHERE table_schema = v_table_schema\n      AND table_name = v_table_name\n  ;\n\n  RETURN v_return;\nEND;\n"}}	2010-01-28 00:37:29.068715	2010-01-28 00:37:29.068715	1
cnu.tg_set_changed_by_employee_id	1	{{function,cnu.tg_set_changed_by_employee_id},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql"},{body,"\nBEGIN\n  PERFORM tools.trigger_sanity( TG_NAME, TG_WHEN, TG_LEVEL, TG_OP, 'before', 'row', '{insert, update}' );\n\n  NEW.changed_by_employee_id := cnu.employee_id_from_by( NEW.changed_by );\n\tRETURN NEW;\nEND;\n"}}	2010-03-04 23:40:35.054806	2010-03-04 23:40:35.054806	1
tables__getsert	1	{{function,tables__getsert},{params,"p_table text"},{returns,tools.tables.id%TYPE},{options,"LANGUAGE plpgsql SECURITY DEFINER RETURNS NULL ON NULL INPUT"},{body,"\nDECLARE\n  v_id tools.tables.id%TYPE;\n\n  v_table_schema text;\n  v_table_name text;\nBEGIN\n  SELECT INTO v_table_schema, v_table_name * FROM tools.table_schema_and_name( p_table );\n\n  FOR i IN 1 .. 10 LOOP\n    SELECT INTO v_id id\n      FROM tools.tables\n      WHERE table_schema = v_table_schema\n        AND table_name = v_table_name\n    ;\n\n    IF FOUND THEN\n      RETURN v_id;\n    END IF;\n\n    BEGIN\n      INSERT INTO tools.tables( table_schema, table_name ) VALUES( v_table_schema, v_table_name );\n      RETURN tools.currval( 'tools.tables' );\n    EXCEPTION WHEN unique_violation THEN\n      -- Do nothing\n    END;\n  END LOOP;\n  PERFORM tools.assert( 'inheritance__tables__getsert loop overflow', FALSE, 'KABOOM!' );\nEND;\n"}}	2010-01-28 00:37:29.068715	2010-01-28 00:37:29.068715	1
tools.assert__enabled_global__set	1	{{function,tools.assert__enabled_global__set},{params,"    p_assert_name tools.asserts.assert_name%TYPE\n    , p_enabled tools.asserts.enabled%TYPE"},{returns,void},{options,"LANGUAGE sql SECURITY DEFINER"},{body,"UPDATE tools.asserts SET enabled = $2 WHERE lower( assert_name ) = lower( $1 );"}}	2009-04-09 16:48:07.985869	2009-04-09 16:48:07.985869	1
replication.sets	5	{{schema,replication},{plural,sets},{singular,set},{post_id,", inclusive boolean NOT NULL, exclude_set text NULL"}}	2010-07-16 00:45:47.66161	2010-07-16 00:45:47.66161	1
physical_payments_us.tg_bundles_unique	1	{{function,physical_payments_us.tg_bundles_unique},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\nname CONSTANT text := 'physical_payments_us.tg_bundles_unique';\nc_full_table_name CONSTANT text := TG_TABLE_SCHEMA || '.' || TG_TABLE_NAME;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'BEFORE', TG_NAME || ' ON ' || c_full_table_name ||' must be an BEFORE trigger' );\n    PERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' ON ' || c_full_table_name ||' must be a row-level trigger' );\n    \n    -- Deleting would break RI, so don't allow it. Granted, this should probably be a separate trigger, but...\n    PERFORM tools.assert( 'physical_payments_us__bundles__inherit__no_delete'\n        , TG_OP != 'DELETE'\n        , 'DELETEs are not allowed on ' || c_full_table_name || ' (they would break inheritance RI)'\n    );\n\n    RAISE DEBUG '%:\n        TG_OP = %\n        TG_TABLE_NAME = %\n        NEW.bundle_id = %'\n        , name\n        , TG_OP\n        , TG_TABLE_NAME\n        , NEW.bundle_id \n    ;\n\n    -- Changing the PK would break RI, so we shouldn't allow it. Granted, this should probably be a separate trigger, but...\n    IF TG_OP = 'UPDATE' THEN\n        PERFORM tools.assert( 'physical_payments_us__bundles__inherit__pk_no_change'\n            , NEW.bundle_id IS NOT DISTINCT FROM OLD.bundle_id\n            , 'Changing bundle_id on ' || c_full_table_name || ' is not allowed (it would break inheritance RI)'\n        );\n    ELSE\n        -- Only check for dupes on insert, otherwise we'll see our own ID\n        PERFORM tools.assert( 'physical_payments_us__bundles__inherit__unique'\n            , NOT EXISTS( SELECT * FROM physical_payments_us.bundles WHERE bundle_id = NEW.bundle_id )\n            , 'duplicate row violation, bundle_id ' || coalesce( NEW.bundle_id::text, '<NULL>' ) || ' already exists'\n        );\n    END IF;\n\n    RETURN NEW;\nEND;\n"}}	2010-03-04 23:36:43.200725	2010-03-04 23:36:43.200725	1
inheritance.child_object__add	1	{{function,inheritance.child_object__add},{params,"p_inherited_table text\n, p_object_type inheritance.object_types.object_type%TYPE\n, p_object_name_template inheritance.child_objects.object_name_template%TYPE\n, p_object_body_template inheritance.child_objects.object_body_template%TYPE\n, p_exclude_tables text[]"},{returns,child_objects.id%TYPE},{options,"LANGUAGE plpgsql SECURITY DEFINER"},{body,"\nDECLARE\n  v_id inheritance.child_objects.id%TYPE;\n  v_exclude_table_ids int[];\nBEGIN\n  INSERT INTO inheritance.child_objects( inherited_table_id, object_type_id, object_name_template, object_body_template )\n    SELECT tools.tables__getsert( p_inherited_table ), inheritance.object_types__get_id( p_object_type )\n        , p_object_name_template, p_object_body_template\n  ;\n  v_id = tools.currval( 'inheritance.child_objects', 'id' );\n\n  IF p_exclude_tables IS NOT NULL AND tools.array_length( p_exclude_tables, 1 ) > 0 THEN\n    v_exclude_table_ids := tools.tables__getsert( p_exclude_tables );\n    INSERT INTO inheritance.child_objects_exclude_tables( child_object_id, exclude_table_id )\n      SELECT v_id, e_id\n        FROM tools.explode_array( tools.tables__getsert( p_exclude_tables ) ) e_id\n        WHERE e_id IS NOT NULL\n    ;\n  END IF;\n\n  RETURN v_id;\nEND;\n"}}	2010-01-28 00:37:29.068715	2010-01-28 00:37:29.068715	1
inheritance.child_object__get	1	{{function,inheritance.child_object__get},{params,"\np_inherited_table text\n, p_object_type inheritance.object_types.object_type%TYPE\n, p_object_name_template inheritance.child_objects.object_name_template%TYPE\n"},{returns,inheritance.child_objects},{options,"LANGUAGE sql SECURITY DEFINER"},{body,"\nSELECT *\n  FROM inheritance.child_objects\n  WHERE inherited_table_id = tools.tables__get_id( $1 )\n    AND object_type_id = inheritance.object_types__get_id( $2 )\n    AND object_name_template = $3\n"}}	2010-01-28 00:37:29.068715	2010-01-28 00:37:29.068715	1
inheritance.child_object__get_id	1	{{function,inheritance.child_object__get_id},{params,"\np_inherited_table text\n, p_object_type inheritance.object_types.object_type%TYPE\n, p_object_name_template inheritance.child_objects.object_name_template%TYPE\n"},{returns,inheritance.child_objects.id%TYPE},{options,"LANGUAGE sql"},{body,"SELECT id FROM inheritance.child_object__get( $1, $2, $3 )"}}	2010-01-28 00:37:29.068715	2010-01-28 00:37:29.068715	1
physical_payments_us.tg_acknowledgment_files_unique	1	{{function,physical_payments_us.tg_acknowledgment_files_unique},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\nname CONSTANT text := 'physical_payments_us.tg_acknowledgment_files_unique';\nc_full_table_name CONSTANT text := TG_TABLE_SCHEMA || '.' || TG_TABLE_NAME;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'BEFORE', TG_NAME || ' ON ' || c_full_table_name ||' must be an BEFORE trigger' );\n    PERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' ON ' || c_full_table_name ||' must be a row-level trigger' );\n    \n    -- Deleting would break RI, so don't allow it. Granted, this should probably be a separate trigger, but...\n    PERFORM tools.assert( 'physical_payments_us__acknowledgment_files__inherit__no_delete'\n        , TG_OP != 'DELETE'\n        , 'DELETEs are not allowed on ' || c_full_table_name || ' (they would break inheritance RI)'\n    );\n\n    RAISE DEBUG '%:\n        TG_OP = %\n        TG_TABLE_NAME = %\n        NEW.acknowledgment_file_id = %'\n        , name\n        , TG_OP\n        , TG_TABLE_NAME\n        , NEW.acknowledgment_file_id \n    ;\n\n    -- Changing the PK would break RI, so we shouldn't allow it. Granted, this should probably be a separate trigger, but...\n    IF TG_OP = 'UPDATE' THEN\n        PERFORM tools.assert( 'physical_payments_us__acknowledgment_files__inherit__pk_no_change'\n            , NEW.acknowledgment_file_id IS NOT DISTINCT FROM OLD.acknowledgment_file_id\n            , 'Changing acknowledgment_file_id on ' || c_full_table_name || ' is not allowed (it would break inheritance RI)'\n        );\n    ELSE\n        -- Only check for dupes on insert, otherwise we'll see our own ID\n        PERFORM tools.assert( 'physical_payments_us__acknowledgment_files__inherit__unique'\n            , NOT EXISTS( SELECT * FROM physical_payments_us.acknowledgment_files WHERE acknowledgment_file_id = NEW.acknowledgment_file_id )\n            , 'duplicate row violation, acknowledgment_file_id ' || coalesce( NEW.acknowledgment_file_id::text, '<NULL>' ) || ' already exists'\n        );\n    END IF;\n\n    RETURN NEW;\nEND;\n"}}	2010-03-04 23:36:43.200725	2010-03-04 23:36:43.200725	1
inheritance.child_object__add_excluded_tables	1	{{function,inheritance.child_object__add_excluded_tables},{params,"p_child_object_id inheritance.child_objects.id%TYPE\n, p_exclude_tables text[]"},{returns,void},{options,"LANGUAGE sql SECURITY DEFINER"},{body,"\nINSERT INTO inheritance.child_objects_exclude_tables( child_object_id, exclude_table_id )\n  SELECT $1, e_id FROM tools.explode_array( tools.tables__getsert( $2 ) ) e_id\n  EXCEPT\n  SELECT child_object_id, exclude_table_id FROM inheritance.child_objects_exclude_tables\n;\n"}}	2010-01-28 00:37:29.068715	2010-01-28 00:37:29.068715	1
inheritance.process_child	1	{{function,inheritance.process_child},{params,"p_child_table text"},{returns,text},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n  fname CONSTANT text := 'inheritance.process_child';\n  v_child oid := p_child_table::regclass::oid;\n\n  v_child_full text := tools.table_full_name( p_child_table );\n  v_child_schema text;\n  v_child_name text;\n\n  r_object record;\n  v_name_array text[][];\n  sql text;\nBEGIN\n  IF NOT EXISTS( SELECT * FROM pg_inherits WHERE inhrelid = v_child ) THEN\n    RAISE EXCEPTION '%: % is not an inherited table', fname, p_child_table;\n  END IF;\n\n  SELECT INTO v_child_schema, v_child_name * FROM tools.table_schema_and_name( p_child_table );\n  FOR r_object IN\n  SELECT c.*\n    FROM inheritance.child_objects_v c\n      JOIN inheritance.get_parents( v_child ) p ON c.inherited_table_oid = p.parent_oid\n    WHERE NOT EXISTS(\n            SELECT *\n              FROM inheritance.child_objects_exclude_tables e\n                JOIN tools.tables_v t ON t.id = e.exclude_table_id\n              WHERE e.child_object_id = c.id\n                AND t.oid = v_child\n          )\n  LOOP\n    v_name_array := array[\n      [ 'parent_schema', r_object.inherited_table_schema ]\n      , [ 'parent_name', r_object.inherited_table_name ]\n      , [ 'parent_full', r_object.inherited_table_full_name ]\n      , [ 'child_schema', v_child_schema ]\n      , [ 'child_name', v_child_name ]\n      , [ 'child_full', v_child_full ]\n    ];\n    SELECT INTO sql template FROM inheritance.object_types__get( r_object.object_type_id );\n    sql := tools.parameter_replace(\n        tools.parameter_replace(\n          sql\n          , array[\n            [ 'object_name_template', r_object.object_name_template ]\n            , [ 'object_body_template', r_object.object_body_template ]\n          ]\n        )\n        , v_name_array\n    );\n    BEGIN\n      PERFORM tools.exec( sql );\n    EXCEPTION\n      WHEN duplicate_table THEN -- We get this for a duplicate unique constraint\n        IF SQLERRM !~ tools.parameter_replace( r_object.object_name_template, v_name_array ) THEN\n          RAISE EXCEPTION '%: Error %, %', fname, SQLSTATE, SQLERRM;\n        END IF;\n      WHEN duplicate_object THEN\n        IF SQLERRM !~ tools.parameter_replace( r_object.object_name_template, v_name_array ) THEN\n          RAISE EXCEPTION '%: Error %, %', fname, SQLSTATE, SQLERRM;\n        END IF;\n      WHEN invalid_table_definition THEN\n        IF SQLERRM !~ 'multiple primary keys for table' THEN\n          RAISE EXCEPTION '%: Error %, %', fname, SQLSTATE, SQLERRM;\n        END IF;\n      WHEN others THEN\n        RAISE EXCEPTION '%: Error %, %', fname, SQLSTATE, SQLERRM;\n    END;\n  END LOOP;\n\n  RETURN v_child::regclass::text;\nEND;\n"}}	2010-01-28 00:37:29.068715	2010-01-28 00:37:29.068715	1
inheritance.child_object__remove_excluded_tables	1	{{function,inheritance.child_object__remove_excluded_tables},{params,"p_child_object_id inheritance.child_objects.id%TYPE\n, p_exclude_tables text[]"},{returns,void},{options,"LANGUAGE sql SECURITY DEFINER"},{body,"\nDELETE FROM inheritance.child_objects_exclude_tables\n  WHERE child_object_id = $1 AND exclude_table_id = ANY( tools.tables__get_id( $2 ) ) \n;\n"}}	2010-01-28 00:37:29.068715	2010-01-28 00:37:29.068715	1
inheritance.get_parents	1	{{function,inheritance.get_parents},{params,"p_child_table_oid oid\n  , OUT parent_oid oid"},{returns,"SETOF oid"},{options,"LANGUAGE plpgsql STABLE"},{body,"\nDECLARE\n  v_oid oid;\nBEGIN\n  FOR v_oid IN SELECT inhparent FROM pg_catalog.pg_inherits WHERE inhrelid = $1::regclass LOOP\n    parent_oid := v_oid;\n    RETURN NEXT;\n    -- For each one of our parents, we also want to return all of their parents\n    FOR parent_oid IN SELECT * FROM inheritance.get_parents( v_oid ) LOOP\n      RETURN NEXT;\n    END LOOP;\n  END LOOP;\nEND"}}	2010-01-28 00:37:29.068715	2010-01-28 00:37:29.068715	1
inheritance.process_children	1	{{function,inheritance.process_children},{params,"p_parent_table text\n  , OUT child_table text"},{returns,"SETOF text"},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n  v_child text;\nBEGIN\n  FOR v_child IN SELECT inhrelid::regclass::text FROM pg_inherits WHERE inhparent = p_parent_table::regclass LOOP\n    child_table := inheritance.process_child( v_child ); -- process_child just returns the table name\n    RETURN NEXT;\n\n    FOR child_table IN SELECT * FROM inheritance.process_children( v_child ) LOOP\n      RETURN NEXT;\n    END LOOP;\n  END LOOP;\n\n  RETURN;\nEND;\n"}}	2010-01-28 00:37:29.068715	2010-01-28 00:37:29.068715	1
physical_payments_us.tg_acknowledgments_unique	1	{{function,physical_payments_us.tg_acknowledgments_unique},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\nname CONSTANT text := 'physical_payments_us.tg_acknowledgments_unique';\nc_full_table_name CONSTANT text := TG_TABLE_SCHEMA || '.' || TG_TABLE_NAME;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'BEFORE', TG_NAME || ' ON ' || c_full_table_name ||' must be an BEFORE trigger' );\n    PERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' ON ' || c_full_table_name ||' must be a row-level trigger' );\n    \n    -- Deleting would break RI, so don't allow it. Granted, this should probably be a separate trigger, but...\n    PERFORM tools.assert( 'physical_payments_us__acknowledgments__inherit__no_delete'\n        , TG_OP != 'DELETE'\n        , 'DELETEs are not allowed on ' || c_full_table_name || ' (they would break inheritance RI)'\n    );\n\n    RAISE DEBUG '%:\n        TG_OP = %\n        TG_TABLE_NAME = %\n        NEW.acknowledgment_id = %'\n        , name\n        , TG_OP\n        , TG_TABLE_NAME\n        , NEW.acknowledgment_id \n    ;\n\n    -- Changing the PK would break RI, so we shouldn't allow it. Granted, this should probably be a separate trigger, but...\n    IF TG_OP = 'UPDATE' THEN\n        PERFORM tools.assert( 'physical_payments_us__acknowledgments__inherit__pk_no_change'\n            , NEW.acknowledgment_id IS NOT DISTINCT FROM OLD.acknowledgment_id\n            , 'Changing acknowledgment_id on ' || c_full_table_name || ' is not allowed (it would break inheritance RI)'\n        );\n    ELSE\n        -- Only check for dupes on insert, otherwise we'll see our own ID\n        PERFORM tools.assert( 'physical_payments_us__acknowledgments__inherit__unique'\n            , NOT EXISTS( SELECT * FROM physical_payments_us.acknowledgments WHERE acknowledgment_id = NEW.acknowledgment_id )\n            , 'duplicate row violation, acknowledgment_id ' || coalesce( NEW.acknowledgment_id::text, '<NULL>' ) || ' already exists'\n        );\n    END IF;\n\n    RETURN NEW;\nEND;\n"}}	2010-03-04 23:36:43.200725	2010-03-04 23:36:43.200725	1
physical_payments.tg_physical_payments_unique	1	{{function,physical_payments.tg_physical_payments_unique},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\nname CONSTANT text := 'physical_payments.tg_physical_payments_unique';\nc_full_table_name CONSTANT text := TG_TABLE_SCHEMA || '.' || TG_TABLE_NAME;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'BEFORE', TG_NAME || ' ON ' || c_full_table_name ||' must be an BEFORE trigger' );\n    PERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' ON ' || c_full_table_name ||' must be a row-level trigger' );\n    \n    -- Deleting would break RI, so don't allow it. Granted, this should probably be a separate trigger, but...\n    PERFORM tools.assert( 'physical_payments__physical_payments__inherit__no_delete'\n        , TG_OP != 'DELETE'\n        , 'DELETEs are not allowed on ' || c_full_table_name || ' (they would break inheritance RI)'\n    );\n\n    RAISE DEBUG '%:\n        TG_OP = %\n        TG_TABLE_NAME = %\n        NEW.physical_payment_id = %'\n        , name\n        , TG_OP\n        , TG_TABLE_NAME\n        , NEW.physical_payment_id \n    ;\n\n    -- Changing the PK would break RI, so we shouldn't allow it. Granted, this should probably be a separate trigger, but...\n    IF TG_OP = 'UPDATE' THEN\n        PERFORM tools.assert( 'physical_payments__physical_payments__inherit__pk_no_change'\n            , NEW.physical_payment_id IS NOT DISTINCT FROM OLD.physical_payment_id\n            , 'Changing physical_payment_id on ' || c_full_table_name || ' is not allowed (it would break inheritance RI)'\n        );\n    ELSE\n        -- Only check for dupes on insert, otherwise we'll see our own ID\n        PERFORM tools.assert( 'physical_payments__physical_payments__inherit__unique'\n            , NOT EXISTS( SELECT * FROM physical_payments.physical_payments WHERE physical_payment_id = NEW.physical_payment_id )\n            , 'duplicate row violation, physical_payment_id ' || coalesce( NEW.physical_payment_id::text, '<NULL>' ) || ' already exists'\n        );\n    END IF;\n\n    RETURN NEW;\nEND;\n"}}	2010-01-28 00:37:29.206866	2010-01-28 00:37:29.206866	1
physical_payments.tg_physical_payments_ri	1	{{function,physical_payments.tg_physical_payments_ri},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n    name CONSTANT text := 'physical_payments.tg_physical_payments_ri';\n    c_full_table_name CONSTANT text := TG_TABLE_SCHEMA || '.' || TG_TABLE_NAME;\n    v_transfer_method physical_payments.transfer_methods.transfer_method%TYPE;\n    v_table_name text;\n    v_only text := '';\n    v_result int;\n    sql text;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'AFTER', TG_NAME || ' ON ' || c_full_table_name ||' must be an AFTER trigger' );\n    PERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' ON ' || c_full_table_name ||' must be a row-level trigger' );\n    PERFORM tools.assert( TG_OP IN ( 'INSERT', 'UPDATE' ), TG_NAME || ' ON ' || c_full_table_name ||' must be on INSERT or UPDATE' );\n\n    -- Th generally won't be allowed, but the trigger should still support it\n    IF NEW.physical_payment_id IS NULL THEN\n        RAISE DEBUG '%: physical_payment_id is NULL, skipping check', name;\n        RETURN NULL;\n    END IF;\n\n    -- If we're updating and we haven't modified physical_payment_id, just bail\n    IF TG_OP = 'UPDATE' THEN\n        IF NEW.physical_payment_id IS NOT DISTINCT FROM OLD.physical_payment_id THEN\n            RAISE DEBUG '%: physical_payment_id ( = % ) is unchanged, skipping check', name, NEW.physical_payment_id;\n            RETURN NULL;\n        END IF;\n    END IF;\n\n    /*\n     * We want to not only check for existence of the desired row, we also want\n     * to share-lock it. Unfortunately, sharelocks aren't implemented for\n     * inherited tables, so we need to find the record in the correct table. We\n     * can do this fairly easily if we can find out what \\"type\\" of record it is.\n     */\n\n    -- Figure out what type of record this is (of course, record might not exist here)\n    SELECT INTO v_transfer_method '' || child_table || ''\n        FROM physical_payments.transfer_methods__get( (\n                    SELECT transfer_method_id\n                        FROM physical_payments.physical_payments pi\n                        WHERE pi.physical_payment_id = NEW.physical_payment_id\n                ) )\n    ;\n    IF NOT FOUND OR v_transfer_method IS NULL THEN\n        -- There wasn't a record at all\n        v_only := 'ONLY '; -- FOR SHARE won't work if we select from both the parent and the children\n        v_table_name := 'physical_payments.physical_payments';\n    ELSE\n        -- We figured out what type of record this is, now try and lock the row in the right table\n        v_table_name := v_transfer_method;\n    END IF;\n\n    sql := 'SELECT 1 FROM ' || v_only || v_table_name || '\n        WHERE physical_payment_id = ' || NEW.physical_payment_id || '\n        FOR SHARE'\n    ;\n    RAISE DEBUG '%: Executing SQL %', name, sql;\n    -- Note that simply using a PERFORM here might get optimized out.\n    EXECUTE sql INTO v_result;\n\n    /*\n     * Normally we should always find an record, but if it was somehow\n     * put in the wrong table, or if it was deleted after our initial\n     * select then we wouldn't have one.\n     */\n    IF v_result = 1 THEN\n        RAISE DEBUG '%: record for physical_payment_id = % found in table %', name, NEW.physical_payment_id, v_table_name;\n        RETURN NULL;\n    END IF;\n\n    RAISE EXCEPTION 'insert on update on table \\"%.%\\" violates foreign key; physical_payment_id=(%) is not present in table \\"%\\"'\n        , TG_TABLE_SCHEMA\n        , TG_TABLE_NAME\n        , NEW.physical_payment_id\n        , v_table_name\n    ;\nEND;\n"}}	2010-01-28 00:37:29.206866	2010-01-28 00:37:29.206866	1
tools.londiste__restore_all_triggers	1	{{function,tools.londiste__restore_all_triggers},{params,"p_table text"},{returns,void},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n  v_table CONSTANT text := tools.full_name_table( p_table );\nBEGIN\n  IF tools.londiste_subscriber( v_table ) THEN\n    PERFORM londiste.subscriber_restore_all_table_triggers( tools.full_name_table( $1 ) );\n  END IF;\nEND;\n"}}	2010-01-28 00:37:44.330245	2010-01-28 00:37:44.330245	1
tools.londiste__pending_fkeys	1	{{function,tools.londiste__pending_fkeys},{params,"p_child text"},{returns,"setof londiste.subscriber_pending_fkeys"},{options,"LANGUAGE sql SECURITY DEFINER"},{body,"\n    SELECT * FROM londiste.subscriber_get_table_pending_fkeys( $1 ) \n"}}	2010-01-28 00:37:44.330245	2010-01-28 00:37:44.330245	1
tools.londiste__drop_all_triggers	1	{{function,tools.londiste__drop_all_triggers},{params,"p_table text"},{returns,void},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n  v_table CONSTANT text := tools.full_name_table( p_table );\nBEGIN\n  IF tools.londiste_subscriber( v_table ) THEN\n    PERFORM londiste.subscriber_drop_all_table_triggers( v_table );\n\n    -- We may have just yanked a londiste_no_dml trigger. If we did, put it back in place.\n    PERFORM londiste.subscriber_restore_table_trigger(table_name, trigger_name)\n      FROM londiste.subscriber_pending_triggers\n      WHERE trigger_def ~ 'EXECUTE PROCEDURE tools.tg_londiste_no_dml()'\n    ;\n  END IF;\nEND;\n"}}	2010-01-28 00:37:44.330245	2010-01-28 00:37:44.330245	1
physical_payments_us.tg_return_files_unique	1	{{function,physical_payments_us.tg_return_files_unique},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\nname CONSTANT text := 'physical_payments_us.tg_return_files_unique';\nc_full_table_name CONSTANT text := TG_TABLE_SCHEMA || '.' || TG_TABLE_NAME;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'BEFORE', TG_NAME || ' ON ' || c_full_table_name ||' must be an BEFORE trigger' );\n    PERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' ON ' || c_full_table_name ||' must be a row-level trigger' );\n    \n    -- Deleting would break RI, so don't allow it. Granted, this should probably be a separate trigger, but...\n    PERFORM tools.assert( 'physical_payments_us__return_files__inherit__no_delete'\n        , TG_OP != 'DELETE'\n        , 'DELETEs are not allowed on ' || c_full_table_name || ' (they would break inheritance RI)'\n    );\n\n    RAISE DEBUG '%:\n        TG_OP = %\n        TG_TABLE_NAME = %\n        NEW.return_file_id = %'\n        , name\n        , TG_OP\n        , TG_TABLE_NAME\n        , NEW.return_file_id \n    ;\n\n    -- Changing the PK would break RI, so we shouldn't allow it. Granted, this should probably be a separate trigger, but...\n    IF TG_OP = 'UPDATE' THEN\n        PERFORM tools.assert( 'physical_payments_us__return_files__inherit__pk_no_change'\n            , NEW.return_file_id IS NOT DISTINCT FROM OLD.return_file_id\n            , 'Changing return_file_id on ' || c_full_table_name || ' is not allowed (it would break inheritance RI)'\n        );\n    ELSE\n        -- Only check for dupes on insert, otherwise we'll see our own ID\n        PERFORM tools.assert( 'physical_payments_us__return_files__inherit__unique'\n            , NOT EXISTS( SELECT * FROM physical_payments_us.return_files WHERE return_file_id = NEW.return_file_id )\n            , 'duplicate row violation, return_file_id ' || coalesce( NEW.return_file_id::text, '<NULL>' ) || ' already exists'\n        );\n    END IF;\n\n    RETURN NEW;\nEND;\n"}}	2010-03-04 23:36:43.200725	2010-03-04 23:36:43.200725	1
replication.sets__get_id	1	{{schema,replication},{table,sets},{lookup,set}}	2010-07-16 00:45:47.66161	2010-07-16 00:45:47.66161	1
replication.sets__get	1	{{schema,replication},{table,sets}}	2010-07-16 00:45:47.66161	2010-07-16 00:45:47.66161	1
tools.seed_object__remove	1	{{function,tools.seed_object__remove},{params,"p_schema_name text\n    , p_relation_name text\n    , p_force boolean"},{returns,void},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n  v_force boolean := COALESCE(p_force,false);\nBEGIN\n  DELETE FROM tools.seed_objects\n    WHERE schema_name = p_schema_name\n      AND object_name = p_relation_name\n  ;\n  IF NOT v_force THEN\n    PERFORM tools.assert(\n      'seed_object_entry__exists'\n      , FOUND\n      , 'Table ' || p_schema_name || '.' || p_relation_name || ' not found in tools.seed_objects!'\n    );\n  END IF;\nEND;\n"}}	2010-10-28 18:16:46.072429	2010-10-28 18:16:46.072429	1
replication.sets__get_text	1	{{schema,replication},{table,sets},{lookup,set}}	2010-07-16 00:45:47.66161	2010-07-16 00:45:47.66161	1
replication.schemas__exclude	5	{{schema,replication},{plural,schemas__exclude},{singular,schema_name},{post_id,""}}	2010-07-16 00:45:47.66161	2010-07-16 00:45:47.66161	1
replication.schemas__exclude__get_id	1	{{schema,replication},{table,schemas__exclude},{lookup,schema_name}}	2010-07-16 00:45:47.66161	2010-07-16 00:45:47.66161	1
replication.schemas__exclude__get	1	{{schema,replication},{table,schemas__exclude}}	2010-07-16 00:45:47.66161	2010-07-16 00:45:47.66161	1
replication.schemas__exclude__get_text	1	{{schema,replication},{table,schemas__exclude},{lookup,schema_name}}	2010-07-16 00:45:47.66161	2010-07-16 00:45:47.66161	1
replication.tables__exclude	5	{{schema,replication},{plural,tables__exclude},{singular,full_name},{post_id,""}}	2010-07-16 00:45:47.66161	2010-07-16 00:45:47.66161	1
replication.tables__exclude__get_id	1	{{schema,replication},{table,tables__exclude},{lookup,full_name}}	2010-07-16 00:45:47.66161	2010-07-16 00:45:47.66161	1
replication.tables__exclude__get	1	{{schema,replication},{table,tables__exclude}}	2010-07-16 00:45:47.66161	2010-07-16 00:45:47.66161	1
replication.tables__exclude__get_text	1	{{schema,replication},{table,tables__exclude},{lookup,full_name}}	2010-07-16 00:45:47.66161	2010-07-16 00:45:47.66161	1
cnu.account_balance	1	{{function,cnu.account_balance},{params,"\n    p_amount payment_transactions.amount%TYPE\n    , p_debit payment_transactions.debit_account_cd%TYPE\n    , p_credit payment_transactions.credit_account_cd%TYPE\n    , p_account_list text[]\n    "},{returns,payment_transactions.amount%TYPE},{options,"LANGUAGE sql CALLED ON NULL INPUT"},{body,"\n    SELECT\n        coalesce( CASE WHEN $2 = ANY( $4 ) THEN $1 END, 0 )\n        - coalesce( CASE WHEN $3 = ANY( $4 ) THEN $1 END, 0 )\n    /*\n    SELECT amount FROM (SELECT\n        coalesce( CASE WHEN $2 = ANY( $4 ) THEN $1 END, 0 )\n                - coalesce( CASE WHEN $3 = ANY( $4 ) THEN $1 END, 0 )\n            AS amount\n            , tools.raise( 'debug', 'amount = ' || coalesce( $1::text, '<NULL>' )\n                || ', debit = '|| coalesce( $2, '<NULL>' )\n                ||', credit = '|| coalesce( $3, '<NULL>' )\n                ||', list = {' || coalesce( array_to_string( $4, ', '), '<NULL>' ) \n                || '}' )\n            ) a\n    */\n    "}}	2010-09-30 23:01:03.767786	2010-09-30 23:01:03.767786	1
inheritance.reset_children	1	{{function,inheritance.reset_children},{params,"p_parent_table text\n  , p_object_type inheritance.object_types.object_type%TYPE\n  , OUT child_table text"},{returns,"SETOF text"},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n  v_child text;\nBEGIN\n  FOR v_child IN SELECT inhrelid::regclass::text FROM pg_inherits WHERE inhparent = p_parent_table::regclass LOOP\n    child_table := inheritance.reset_child( v_child, p_object_type ); -- reset_child just returns the table name\n    RETURN NEXT;\n\n    FOR child_table IN SELECT * FROM inheritance.reset_children( v_child, p_object_type ) LOOP\n      RETURN NEXT;\n    END LOOP;\n  END LOOP;\n\n  RETURN;\nEND;\n"}}	2010-07-30 02:41:56.877656	2010-07-30 02:41:56.877656	1
tools.textcoalesce	1	{{function,tools.textcoalesce},{params,bigint},{returns,text},{options,"LANGUAGE sql"},{body,"SELECT coalesce( $1::text, '<NULL>' )"}}	2009-09-11 02:53:48.840675	2009-09-11 02:53:48.840675	1
cnu.account_balance__add	1	{{function,cnu.account_balance__add},{params,"\n    p_amount_1 payment_transactions.amount%TYPE\n    , p_amount_2 payment_transactions.amount%TYPE\n    , p_debit payment_transactions.debit_account_cd%TYPE\n    , p_credit payment_transactions.credit_account_cd%TYPE\n    , p_account_list text[]\n    "},{returns,payment_transactions.amount%TYPE},{options,"LANGUAGE sql CALLED ON NULL INPUT"},{body,"\n    SELECT coalesce( $1, 0 )\n        + cnu.account_balance( $2, $3, $4, $5 )\n    /*\n    SELECT a FROM (SELECT a, tools.raise( 'debug', 'final amount = ' || coalesce( a::text, '<NULL>' ) ) FROM (\n            SELECT coalesce( $1, 0 )\n        + cnu.account_balance( $2, $3, $4, $5 ) AS a\n    ) b) b\n    */\n    "}}	2010-09-30 23:01:03.767786	2010-09-30 23:01:03.767786	1
tools.next_stats_date	1	{{function,tools.next_stats_date},{params,"p_stats_table text\n  , p_stats_view text\n  , p_offset int\n  "},{returns,date},{options,"LANGUAGE plpgsql STABLE"},{body,"\nDECLARE\n  name CONSTANT text := 'tools.next_stats_date';\n  v_next_date DATE := NULL;\n  c_buffer CONSTANT interval := '10 minutes';\n  c_max_allowed_date CONSTANT date := ( tools.proxynow() - c_buffer )::date - 1;\n\n  v_text text;\n  sql text;\nBEGIN\n  /*\n   * Sanity-check our inputs. If p_stats_* isn't a valid table/view the cast to\n   * regclass will blow up, protecting us from SQL injection (among other\n   * things).\n   */\n  v_text := p_stats_table::regclass;\n  v_text := p_stats_view::regclass;\n\n  PERFORM tools.assert(\n    'next_stats_date__p_offset_must_be_positive'\n    , p_offset >= 0\n    , 'p_offset ( = ' || tools.textcoalesce( p_offset ) || ' ) must be >= 0'\n  );\n  sql = $$\n    SELECT\n    -- If there's no data in the stats table we need to get the date from the view\n    CASE WHEN max IS NULL THEN ( SELECT min(stats_date) FROM $$ || p_stats_view || $$ )\n      /*\n       * If the max date in the stats table is \\"today\\" then we need to return\n       * NULL instead of max + 1. We do this so that we will never actually\n       * return today's date. This is critical because we don't support\n       * updating a row that's already in the stats table.\n       *\n       * However, because node times can be off by a number of minutes, we build a buffer into this calculation.\n       */\n         WHEN max < '$$ || c_max_allowed_date || $$'::date THEN max + 1 + $$ || p_offset || $$\n    END\n    FROM ( SELECT max(stats_date) FROM $$ || p_stats_table || $$ ) AS sum\n  $$;\n  RAISE DEBUG '%: executing sql %', name, sql;\n  EXECUTE sql INTO v_next_date;\n  PERFORM tools.assert(\n    'next_stats_date_must_not_be_today'\n    , v_next_date IS NULL OR v_next_date < c_max_allowed_date\n    , 'ERROR: tools.next_stats_date tried to return a value ( = ' || tools.textcoalesce( v_next_date ) || \n        ' ) that was not less than c_max_allowed_date ( = ' || tools.textcoalesce( c_buffer ) || ' )'\n  );\n  RETURN v_next_date;\nEND;\n"}}	2010-07-28 00:28:01.779584	2010-07-28 00:28:01.779584	1
email_stats__get_another_day	1	{{function,email_stats__get_another_day},{params,""},{returns,"SETOF email_stats_raw_v"},{options,"SECURITY DEFINER LANGUAGE plpgsql"},{body,"\n-- This function is intentionally written in plpgsql. If we don't provide an actual date to the query, the optimizer is lost.\nDECLARE\n  v_next_date date;\n  v_rowcount int := 0;\n  rec record;\nBEGIN\n  FOR v_offset IN 0 .. 100 LOOP\n    v_next_date := email_stats__next_date(v_offset);\n    IF v_next_date IS NULL THEN\n      RETURN;\n    END IF;\n\n    RAISE DEBUG 'Returning ip stats for %', c_next_date;\n    FOR rec IN SELECT * FROM email_stats_raw_v WHERE stats_date = v_next_date\n    LOOP\n      v_rowcount := v_rowcount + 1;\n      RETURN NEXT rec;\n    END LOOP;\n    EXIT WHEN v_rowcount > 0;\n  END LOOP;\n  IF v_rowcount = 0 THEN\n    RAISE EXCEPTION 'Could not find more days to process!';\n  END IF;\n\n  RETURN;\nEND;\n"}}	2010-07-28 00:28:01.779584	2010-07-28 00:28:01.779584	1
ip_stats__get_another_day	1	{{function,ip_stats__get_another_day},{params,""},{returns,"SETOF ip_stats_raw_v"},{options,"SECURITY DEFINER LANGUAGE plpgsql"},{body,"\n-- This function is intentionally written in plpgsql. If we don't provide an actual date to the query, the optimizer is lost.\nDECLARE\n  v_next_date date;\n  v_rowcount int := 0;\n  rec record;\nBEGIN\n  FOR v_offset IN 0 .. 100 LOOP\n    v_next_date := ip_stats__next_date(v_offset);\n    IF v_next_date IS NULL THEN\n      RETURN;\n    END IF;\n\n    RAISE DEBUG 'Returning ip stats for %', c_next_date;\n    FOR rec IN SELECT * FROM ip_stats_raw_v WHERE stats_date = v_next_date\n    LOOP\n      v_rowcount := v_rowcount + 1;\n      RETURN NEXT rec;\n    END LOOP;\n    EXIT WHEN v_rowcount > 0;\n  END LOOP;\n  IF v_rowcount = 0 THEN\n    RAISE EXCEPTION 'Could not find more days to process!';\n  END IF;\n\n  RETURN;\nEND;\n"}}	2010-07-28 00:28:01.779584	2010-07-28 00:28:01.779584	1
lead_stats__get_another_day	1	{{function,lead_stats__get_another_day},{params,""},{returns,"SETOF lead_stats_raw_v"},{options,"SECURITY DEFINER LANGUAGE plpgsql"},{body,"\n-- This function is intentionally written in plpgsql. If we don't provide an actual date to the query, the optimizer is lost.\nDECLARE\n  v_next_date date;\n  v_rowcount int := 0;\n  rec record;\nBEGIN\n  FOR v_offset IN 0 .. 100 LOOP\n    v_next_date := lead_stats__next_date(v_offset);\n    IF v_next_date IS NULL THEN\n      RETURN;\n    END IF;\n\n    RAISE DEBUG 'Returning ip stats for %', c_next_date;\n    FOR rec IN SELECT * FROM lead_stats_raw_v WHERE stats_date = v_next_date\n    LOOP\n      v_rowcount := v_rowcount + 1;\n      RETURN NEXT rec;\n    END LOOP;\n    EXIT WHEN v_rowcount > 0;\n  END LOOP;\n  IF v_rowcount = 0 THEN\n    RAISE EXCEPTION 'Could not find more days to process!';\n  END IF;\n\n  RETURN;\nEND;\n"}}	2010-07-28 00:28:01.779584	2010-07-28 00:28:01.779584	1
backfill.watermark__get	1	{{function,backfill.watermark__get},{params,"\n    p_backfill_id integer\n    "},{returns,"SETOF backfill.watermarks"},{options,"LANGUAGE SQL"},{body,"\n    SELECT * FROM backfill.watermarks WHERE backfill_id=$1;\n    "}}	2010-10-14 20:26:31.691194	2010-10-14 20:26:31.691194	1
backfill.watermark__upsert	1	{{function,backfill.watermark__upsert},{params,"\n    p_backfill_id integer\n    , p_field text\n    , p_value anyelement\n    "},{returns,void},{options,"LANGUAGE plpgsql"},{body,"\n\tDECLARE\n      v_update_sql text := 'UPDATE backfill.watermarks\n\t    SET ' || p_field || ' = ' || p_value || '\n\t    WHERE backfill_id = ' || p_backfill_id || ';';\n      v_insert_sql text := 'INSERT INTO backfill.watermarks ( backfill_id, ' || p_field || ' )\n        VALUES ( '|| p_backfill_id || ', ' || p_value || ' );';\n      v_row_count integer := 0;\n\tBEGIN\n\t  LOOP\n\t\tEXECUTE v_update_sql;\n\t\tGET DIAGNOSTICS v_row_count = ROW_COUNT;\n\t\tIF (v_row_count > 0) THEN\n\t\t\tRETURN;\n\t\tEND IF;\n\t\n\t\tBEGIN\n\t\t  RAISE DEBUG 'Inserting \\"%\\" into backfill.watermarks(%)', p_value, p_field;\n\t\t  EXECUTE v_insert_sql;\n\t\tEXCEPTION WHEN unique_violation THEN\n\t\t  -- do nothing\n\t\tEND;\n\t  END LOOP;\n\tEND;\n    "}}	2010-10-14 20:26:31.691194	2010-10-14 20:26:31.691194	1
backfill.clock__start	1	{{function,backfill.clock__start},{params,""},{returns,void},{options,"LANGUAGE SQL"},{body,"\n    DROP TABLE IF EXISTS backfill__begin;\n    CREATE TEMP TABLE backfill__begin AS SELECT clock_timestamp() AS tstz;\n    "}}	2010-10-14 20:26:31.691194	2010-10-14 20:26:31.691194	1
backfill.sleep_txn	1	{{function,backfill.sleep_txn},{params,"p_txntime timestamp with time zone"},{returns,void},{options,"LANGUAGE SQL"},{body,"\n    SELECT pg_sleep( extract( epoch from clock_timestamp() - $1 ) );\n    "}}	2010-10-14 20:26:31.691194	2010-10-14 20:26:31.691194	1
payment_instruments.default__set	1	{{function,payment_instruments.default__set},{params,"\n    p_payment_instrument_id payment_instruments.default_instrument_history.payment_instrument_id%TYPE\n    , p_created_by payment_instruments.default_instrument_history.created_by%TYPE\n"},{returns,void},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n\tname CONSTANT text := 'payment_instruments.default__set';\n    v_payment_instrument payment_instruments.payment_instruments;\nBEGIN\n\tRAISE DEBUG '%: p_payment_instrument_id = %, p_created_by = %', name, p_payment_instrument_id, p_created_by;\n    v_payment_instrument := payment_instruments.payment_instrument__get( p_payment_instrument_id );\n    IF v_payment_instrument IS NULL THEN\n        RAISE EXCEPTION 'Unknown payment_instrument %', p_payment_instrument_id;\n    END IF;\n\n    -- There's no reason to mark the same instrument as default twice in a row, so don't\n    IF EXISTS(\n        SELECT *\n            FROM payment_instruments.defaults\n            WHERE payment_instrument_id = p_payment_instrument_id\n    ) THEN\n        RAISE WARNING 'payment instrument % is already marked as default!', p_payment_instrument_id;\n    ELSE\n        INSERT INTO payment_instruments.default_instrument_history(\n            customer_id, payment_instrument_type_id, payment_instrument_id, created_by\n        ) VALUES (\n                v_payment_instrument.customer_id, v_payment_instrument.payment_instrument_type_id\n                , p_payment_instrument_id, p_created_by\n        );\n    END IF;\nEND;\n"}}	2010-01-28 00:37:44.417659	2010-01-28 00:37:44.417659	1
payment_instruments.tg_check_customer_and_payment_instrument_type	1	{{function,payment_instruments.tg_check_customer_and_payment_instrument_type},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n\tv_payment_instrument payment_instruments.payment_instruments;\nBEGIN\n\tPERFORM tools.assert( TG_WHEN = 'AFTER', TG_NAME || ' must be an AFTER trigger' );\n\tPERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' must be a row-level trigger' );\n\tPERFORM tools.assert( TG_OP != 'DELETE', 'DELETE is not supported for trigger ' || TG_NAME );\n\n\t-- Note that we shoud allow for payment_instrument_id to be NULL, in which case the other fields better be null too\n\tv_payment_instrument := payment_instruments.payment_instrument__get( NEW.payment_instrument_id );\n\n\tIF v_payment_instrument.customer_id IS DISTINCT FROM NEW.customer_id THEN\n\t\tRAISE EXCEPTION 'customer_id ( = % ) in table % for payment_instrument_id ( = % ) does not match customer_id ( = % ) in payment_instruments table'\n\t\t\t, NEW.customer_id, TG_RELID::regclass::text, NEW.payment_instrument_id, v_payment_instrument.customer_id\n\t\t;\n\tEND IF;\n\n\tIF v_payment_instrument.payment_instrument_type_id IS DISTINCT FROM NEW.payment_instrument_type_id THEN\n\t\tRAISE EXCEPTION 'payment_instrument_type_id ( = % ) in table % for payment_instrument_id ( = % ) does not match payment_instrument_type_id ( = % ) in payment_instruments table'\n\t\t\t, NEW.payment_instrument_type_id, TG_RELID::regclass::text, NEW.payment_instrument_id, v_payment_instrument.payment_instrument_type_id\n\t\t;\n\tEND IF;\n\n\tRETURN NULL;\nEND;\n"}}	2010-01-28 00:36:43.135333	2010-01-28 00:36:43.135333	1
backfill.batch__execute	1	{{function,backfill.batch__execute},{params,"\n    p_backfill_name text\n    "},{returns,void},{options,"LANGUAGE SQL"},{body,"\n    SELECT backfill.batch__execute_id( id ) FROM backfill.backfills WHERE backfill_name = $1;\n    "}}	2010-10-14 20:26:31.691194	2010-10-14 20:26:31.691194	1
backfill.watermark__get_i	1	{{function,backfill.watermark__get_i},{params,"\n\t\tp_backfill_id integer\n\t\t"},{returns,integer},{options,"LANGUAGE SQL"},{body,"\n\t\tSELECT i FROM backfill.watermark__get($1) AS id;\n\t\t"}}	2010-10-14 20:26:31.691194	2010-10-14 20:26:31.691194	1
backfill.watermark__get_d	1	{{function,backfill.watermark__get_d},{params,"\n\t\tp_backfill_id integer\n\t\t"},{returns,date},{options,"LANGUAGE SQL"},{body,"\n\t\tSELECT d FROM backfill.watermark__get($1) AS id;\n\t\t"}}	2010-10-14 20:26:31.691194	2010-10-14 20:26:31.691194	1
backfill.watermark__set	1	{{function,backfill.watermark__set},{params,"\n        p_backfill_id integer\n        , p_value integer\n        "},{returns,void},{options,"LANGUAGE SQL"},{body,"\n        SELECT backfill.watermark__upsert($1, 'i', $2);\n        "}}	2010-10-14 20:26:31.691194	2010-10-14 20:26:31.691194	1
backfill.watermark__get_tstz	1	{{function,backfill.watermark__get_tstz},{params,"\n\t\tp_backfill_id integer\n\t\t"},{returns,timestamptz},{options,"LANGUAGE SQL"},{body,"\n\t\tSELECT tstz FROM backfill.watermark__get($1) AS id;\n\t\t"}}	2010-10-14 20:26:31.691194	2010-10-14 20:26:31.691194	1
backfill.backfill__create	1	{{function,backfill.backfill__create},{params,"\n    p_select_work_queries text[]\n    , p_update_watermark_queries text[]\n    , p_verify_work_exists_queries text[]\n    , p_batch_queries text[]\n    , p_cleanup_queries text[]\n    , p_backfill_name text\n    , p_acunote_task_number integer\n    "},{returns,integer},{options,"LANGUAGE SQL"},{body,"\n    INSERT INTO backfill.backfills (\n    select_work_queries\n    , update_watermark_queries\n    , verify_work_exists_queries\n    , batch_queries\n    , cleanup_queries\n    , backfill_name\n    , acunote_task_number\n    ) VALUES (\n    $1\n    , $2\n    , $3\n    , $4\n    , $5\n    , $6\n    , $7\n    );\n    SELECT currval('backfill.backfills_id_seq')::integer; -- currval returns bigint but this index is on a 'serial' column\n    "}}	2010-10-14 20:26:31.691194	2010-10-14 20:26:31.691194	1
backfill.batch__execute_id	1	{{function,backfill.batch__execute_id},{params,"\n    p_backfill_id integer\n    "},{returns,void},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n  v_sql text[] := NULL;\n  v_backfill backfill.backfills;\n  v_template_vars text[][] := NULL;\nBEGIN\n  -- Get the config for this backfill\n  SELECT INTO v_backfill * FROM backfill.backfills WHERE id=p_backfill_id;\n  \n  -- if not found then raise debug 'yuck';\n  \n  -- Expose some values to parameter_replace\n  v_template_vars := array[\n    [ 'backfill_id', v_backfill.id::text ]\n    , [ 'batch_size', v_backfill.batch_size::text ]\n    , [ 'backfill_name', v_backfill.backfill_name ]\n    , [ 'acunote_task_number', v_backfill.acunote_task_number::text ]\n  ];\n  \n  -- execute select work queries\n  PERFORM tools.exec ( tools.parameter_replace ( cfg_sql, v_template_vars ) ) FROM tools.explode_array( v_backfill.select_work_queries ) AS cfg_sql;\n\n  -- execute verify work exists queries\n  PERFORM tools.exec ( tools.parameter_replace ( cfg_sql, v_template_vars ) ) FROM tools.explode_array( v_backfill.verify_work_exists_queries ) AS cfg_sql;\n  \n  -- execute update watermark queries\n  PERFORM tools.exec ( tools.parameter_replace ( cfg_sql, v_template_vars ) ) FROM tools.explode_array( v_backfill.update_watermark_queries ) AS cfg_sql;\n\n  -- execute batch query\n  PERFORM tools.exec ( tools.parameter_replace ( cfg_sql, v_template_vars ) ) FROM tools.explode_array( v_backfill.batch_queries ) AS cfg_sql;\nEND;\n    "}}	2010-10-14 20:26:31.691194	2010-10-14 20:26:31.691194	1
tools.assert__delete	1	{{function,tools.assert__delete},{params,"    p_assert_name tools.asserts.assert_name%TYPE\n    , p_force boolean\n"},{returns,void},{options,"LANGUAGE plpgsql SECURITY DEFINER SET search_path = 'pg_catalog'"},{body,"\nBEGIN\n  DELETE FROM tools.asserts WHERE lower( assert_name ) = lower( p_assert_name );\n\n  IF NOT FOUND AND NOT p_force THEN\n    RAISE EXCEPTION 'Assert \\"%\\" not found', p_assert_name;\n  END IF;\nEND;\n"}}	2010-11-19 17:08:15.40816	2010-11-19 17:08:15.40816	1
pay_stubs.tg_pay_stubs_unique	1	{{function,pay_stubs.tg_pay_stubs_unique},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\nname CONSTANT text := 'pay_stubs.tg_pay_stubs_unique';\nc_full_table_name CONSTANT text := TG_TABLE_SCHEMA || '.' || TG_TABLE_NAME;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'BEFORE', TG_NAME || ' ON ' || c_full_table_name ||' must be an BEFORE trigger' );\n    PERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' ON ' || c_full_table_name ||' must be a row-level trigger' );\n    \n    -- Deleting would break RI, so don't allow it. Granted, this should probably be a separate trigger, but...\n    PERFORM tools.assert( 'pay_stubs__pay_stubs__inherit__no_delete'\n        , TG_OP != 'DELETE'\n        , 'DELETEs are not allowed on ' || c_full_table_name || ' (they would break inheritance RI)'\n    );\n\n    RAISE DEBUG '%:\n        TG_OP = %\n        TG_TABLE_NAME = %\n        NEW.pay_stub_id = %'\n        , name\n        , TG_OP\n        , TG_TABLE_NAME\n        , NEW.pay_stub_id \n    ;\n\n    -- Changing the PK would break RI, so we shouldn't allow it. Granted, this should probably be a separate trigger, but...\n    IF TG_OP = 'UPDATE' THEN\n        PERFORM tools.assert( 'pay_stubs__pay_stubs__inherit__pk_no_change'\n            , NEW.pay_stub_id IS NOT DISTINCT FROM OLD.pay_stub_id\n            , 'Changing pay_stub_id on ' || c_full_table_name || ' is not allowed (it would break inheritance RI)'\n        );\n    ELSE\n        -- Only check for dupes on insert, otherwise we'll see our own ID\n        PERFORM tools.assert( 'pay_stubs__pay_stubs__inherit__unique'\n            , NOT EXISTS( SELECT * FROM pay_stubs.pay_stubs WHERE pay_stub_id = NEW.pay_stub_id )\n            , 'duplicate row violation, pay_stub_id ' || coalesce( NEW.pay_stub_id::text, '<NULL>' ) || ' already exists'\n        );\n    END IF;\n\n    RETURN NEW;\nEND;\n"}}	2010-12-16 23:16:49.460002	2010-12-16 23:16:49.460002	1
code_internal.create_object	1	{{function,code_internal.create_object},{params,"\n  p_object_name code_internal.objects.full_name%TYPE\n  , p_object_type code_internal.object_type.object_type%TYPE\n  , p_object_class code_internal.object_class.object_class%TYPE\n  , p_parameters code_internal.objects.parameters%TYPE\n  , p_creation_function_version code_internal.objects.creation_function_version%TYPE\n"},{returns,void},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\n  v_saved_min_messages CONSTANT text := current_setting('client_min_messages');\n\n  v_template code_internal.templates;\n  v_sql text;\nBEGIN\n  v_template := code_internal.template__get_latest( p_object_type, p_object_class );\n\n  PERFORM code_internal.object__upsert( p_object_name, v_template.id, p_parameters, p_creation_function_version );\n  FOR i IN array_lower( v_template.template, 1 ) .. array_upper( v_template.template, 1 ) LOOP\n    v_sql := tools.parameter_replace( v_template.template[i], p_parameters );\n\n    -- Lots of objects spit out annoying NOTICEs on creation, so squelch that\n    RAISE DEBUG 'Creating object using dynamic SQL: %', v_sql;\n    PERFORM set_config( 'client_min_messages', 'WARNING', true );\n    EXECUTE v_sql; -- tools.exec is pointless with client_min_messages forced to warning, and using it could result in multiple log messages\n    PERFORM set_config( 'client_min_messages', v_saved_min_messages, true );\n  END LOOP;\nEND;\n"}}	2011-03-18 13:20:59.537165	2011-03-18 13:20:59.537165	1
cnu.bank_info__fps_eligible	1	{{function,cnu.bank_info__fps_eligible},{params,"\n    p_fps_code char\n    "},{returns,boolean},{options,"LANGUAGE SQL"},{body,"\n    -- created by patch: bug_275335__bank_info__fps_eligible\n    SELECT $1 IN ( 'A', 'M' )\n    "}}	2011-03-18 13:20:59.991857	2011-03-18 13:20:59.991857	1
tools.string_or_array	1	{{function,tools.string_or_array},{params,"p_string text"},{returns,text[]},{options,"LANGUAGE plpgsql IMMUTABLE"},{body,"\nBEGIN\n  -- If the string looks like it might be an array or just has commas, fall-back to the array version\n  IF p_string LIKE '{%}' OR p_string ~ ','\n  THEN\n    IF substring(version() from E'PostgreSQL \\\\([^ ]+\\\\)') >= '8.3' THEN\n      RETURN p_string::text[];\n    ELSE\n      RETURN string_to_array( regexp_replace( p_string, '[{} ]',  '', 'g' ), ',' );\n    END IF;\n  ELSE\n    RETURN array[ p_string ];\n  END IF;\nEND;\n"}}	2010-01-28 00:36:09.651344	2010-01-28 00:36:09.651344	1
cabar.components	5	{{schema,cabar},{plural,components},{singular,component},{post_id,", component_search_path text"}}	2011-03-18 13:21:01.665551	2011-03-18 13:21:01.665551	1
cabar.component_schema__add	1	{{function,cabar.component_schema__add},{params,"\n  p_component cabar.components.component%TYPE\n  , p_schema_name cabar.component_schemas.schema_name%type\n  , p_create boolean\n"},{returns,void},{options,"LANGUAGE plpgsql"},{body,"\nDECLARE\nfname CONSTANT text := 'cabar.component_schema__add';\nv_component_id cabar.component_schemas.component_id%TYPE := cabar.components__get_id( p_component );\nBEGIN\n  -- The NOT NULL will blow up if this fails anyway, so no sense in creating an assert for it\n  IF v_component_id IS NULL THEN\n    RAISE EXCEPTION '%: component \\"%\\" does not exist (cabar.components__get_id returned null)', fname, p_component;\n  END IF;\n\n  IF p_create THEN\n    PERFORM code.schema( p_schema_name, p_component || '_owner' );\n  END IF;\n  PERFORM tools.assert(\n    'cabar__component_schema__add__schema_exists'\n    , EXISTS( SELECT * FROM pg_namespace WHERE nspname = p_schema_name )\n    , 'schema ' || p_schema_name || ' does not exist'\n  );\n\n  INSERT INTO cabar.component_schemas( component_id, schema_name ) VALUES( v_component_id, p_schema_name );\nEXCEPTION WHEN unique_violation THEN\n  RAISE DEBUG '%: component/schema combination %/% already exists', fname, p_component, p_schema_name;\nEND;\n"}}	2011-03-18 13:21:02.487407	2011-03-18 13:21:02.487407	1
cabar.components__get_id	1	{{schema,cabar},{table,components},{lookup,component}}	2011-03-18 13:21:01.665551	2011-03-18 13:21:01.665551	1
cabar.components__get	1	{{schema,cabar},{table,components}}	2011-03-18 13:21:01.665551	2011-03-18 13:21:01.665551	1
tools.schema_patches_v	14	{{schema,tools},{table,schema_patches},{selectlist,"\n\t\t, (cabar.components__get(t.component_id)).component"}}	2011-03-18 13:21:02.487407	2011-03-18 13:21:02.487407	1
cabar.components__get_text	1	{{schema,cabar},{table,components},{lookup,component}}	2011-03-18 13:21:01.665551	2011-03-18 13:21:01.665551	1
test.assert	1	{{function,test.assert},{params,"p_condition text\n, p_result text"},{returns,boolean},{options,"LANGUAGE sql"},{body,"\nSELECT tools.exec(\n  'SELECT test.assert( $condition_text$' || $1 || '$condition_text$, ' || $1 || ', ' || $2 || ')'\n) IS NOT NULL\n"}}	2010-01-28 00:36:09.651344	2010-01-28 00:36:09.651344	1
tools.trim_whitespace	1	{{function,tools.trim_whitespace},{params,"t text"},{returns,text},{options,"LANGUAGE sql RETURNS NULL ON NULL INPUT IMMUTABLE"},{body,"-- Created by bug_192474__schema_as\nSELECT regexp_replace( $1, E'***:^\\\\\\\\s+|\\\\\\\\s+$', '', 'g' )"}}	2011-03-18 13:21:01.340467	2011-03-18 13:21:01.340467	1
tools.is_empty_or_null	1	{{function,tools.is_empty_or_null},{params,"t text"},{returns,boolean},{options,"LANGUAGE sql IMMUTABLE"},{body,"-- Created by bug_192474__schema_as\nSELECT coalesce( tools.trim_whitespace( $1 ), '' ) = ''"}}	2011-03-18 13:21:01.340467	2011-03-18 13:21:01.340467	1
cabar.component_schemas_v	14	{{schema,cabar},{table,component_schemas},{selectlist,"\n\t\t, (cabar.components__get(t.component_id)).component"}}	2011-03-18 13:21:01.665551	2011-03-18 13:21:01.665551	1
cabar.env_settings	5	{{schema,cabar},{plural,env_settings},{singular,env_setting},{post_id,""}}	2011-03-18 13:21:02.487407	2011-03-18 13:21:02.487407	1
cnu.tg_installment_status_history	1	{{function,cnu.tg_installment_status_history},{params,""},{returns,trigger},{options,"LANGUAGE plpgsql SECURITY DEFINER"},{body,"\nDECLARE\n    v_track_changed_by boolean := true;\n    v_changed_by varchar(30) := NULL;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'AFTER', TG_NAME || ' must be an AFTER trigger' );\n\tPERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' must be a row-level trigger' );\n\tPERFORM tools.assert( TG_OP != 'DELETE', 'DELETE is not supported for trigger ' || TG_NAME );\n\n    IF TG_OP = 'UPDATE' THEN\n        IF NEW.installment_status_id IS NOT DISTINCT FROM OLD.installment_status_id THEN\n            RETURN NULL;\n        END IF;\n    END IF;\n    \n    IF TG_NARGS > 0 THEN\n        v_track_changed_by := lower( TG_ARGV[0] ) = 'true';\n    END IF;\n    IF v_track_changed_by THEN\n        v_changed_by := NEW.updated_by;\n    END IF;\n\n    INSERT INTO cnu.installment_status_history( installments_static_id, installment_status_id, changed_by )\n        VALUES( NEW.id, NEW.installment_status_id, v_changed_by )\n    ;\n    RETURN NULL;\nEND;\n"}}	2011-03-18 13:41:12.829081	2011-03-18 13:41:12.829081	1
cabar.env_settings__get_id	1	{{schema,cabar},{table,env_settings},{lookup,env_setting}}	2011-03-18 13:21:02.487407	2011-03-18 13:21:02.487407	1
cabar.env_settings__get	1	{{schema,cabar},{table,env_settings}}	2011-03-18 13:21:02.487407	2011-03-18 13:21:02.487407	1
cabar.env_settings__get_text	1	{{schema,cabar},{table,env_settings},{lookup,env_setting}}	2011-03-18 13:21:02.487407	2011-03-18 13:21:02.487407	1
stats__daily.next_date	1	{{function,stats__daily.next_date},{params,"p_stats_table text\n  , p_stats_view text\n  "},{returns,date},{options,"LANGUAGE plpgsql STABLE"},{body,"\n/*\n * Created in: bug_243812_v5_stats\n */\nDECLARE\n  name CONSTANT text := 'stats__daily.next_date';\n\n  -- Sanitize\n  c_stats_table CONSTANT text := p_stats_table::regclass::text;\n  c_stats_view CONSTANT text := p_stats_view::regclass::text;\n\n  v_next_date DATE := NULL;\n  v_exists boolean;\n\n  /*\n   * If the max date in the stats table is \\"today\\" then we need to return\n   * NULL instead of max + 1. We do this so that we will never actually\n   * return today's date. This is critical because we don't support\n   * updating a row that's already in the stats table.\n   *\n   * However, because node times can be off by a number of minutes, we build a buffer into this calculation.\n   */\n  c_buffer CONSTANT interval := setting.setting_iv( 'stats__daily buffer' );\n  c_max_allowed_date CONSTANT date := ( tools.proxynow() - c_buffer )::date - 1;\n\n  sql text;\nBEGIN\n  RAISE DEBUG '%: c_buffer = %, tools.proxynow() = %, c_max_allowed_date = %, c_stats_table = %, c_stats_view = %'\n    , name, c_buffer, tools.proxynow(), c_max_allowed_date, c_stats_table, c_stats_view\n  ;\n  -- If the stats table is empty we need to just return min(stats_date) from the view\n  sql := $$SELECT EXISTS( SELECT * FROM $$ || c_stats_table || $$ )$$;\n  RAISE DEBUG '%: executing sql %', name, sql;\n  EXECUTE sql INTO v_exists;\n\n  IF NOT v_exists THEN\n    sql := $$SELECT min(stats_date) FROM $$ || p_stats_view || $$ WHERE stats_date <= '$$ || c_max_allowed_date || $$'::date$$;\n    RAISE DEBUG '%: executing sql %', name, sql;\n    EXECUTE sql INTO v_next_date;\n  ELSE\n    FOR v_offset IN 1 .. 100 LOOP\n      sql := $$SELECT max(stats_date) + $$ || v_offset || $$ AS max FROM $$ || p_stats_table;\n      RAISE DEBUG '%: executing sql %', name, sql;\n      EXECUTE sql INTO v_next_date;\n\n      IF v_next_date > c_max_allowed_date THEN\n        v_next_date := NULL;\n        v_exists := true;\n      ELSE\n        sql := $$SELECT EXISTS( SELECT * FROM $$ || p_stats_view || $$ WHERE stats_date = '$$ || v_next_date || $$'::date )$$;\n        RAISE DEBUG '%: executing sql %', name, sql;\n        EXECUTE sql INTO v_exists;\n      END IF;\n\n      EXIT WHEN v_exists;\n    END LOOP;\n    IF NOT v_exists THEN\n      RAISE EXCEPTION '% was unable to find a stats date that contained data', name;\n    END IF;\n  END IF;\n  RAISE DEBUG '%: v_exists = %, v_next_date = %', name, v_exists, v_next_date;\n\n  /*\n   * I know this assert looks pointless and redundant since it's essentially\n   * parroting what we've already done, but it's already saved our ass once,\n   * so leave it in.\n   */\n  PERFORM tools.assert(\n    'next_stats_date_must_not_be_today'\n    , v_next_date IS NULL OR v_next_date <= c_max_allowed_date\n    , 'ERROR: ' || name || ' tried to return a value ( = ' || tools.textcoalesce( v_next_date ) || \n        ' ) that was not less than c_max_allowed_date ( = ' || tools.textcoalesce( c_max_allowed_date ) || ' )'\n  );\n  PERFORM tools.assert(\n    'next_stats_date_must_not_be_today'\n    , v_next_date IS NULL OR v_next_date <= current_date\n    , 'ERROR: ' || name || ' tried to return a value ( = ' || tools.textcoalesce( v_next_date ) || \n        ' ) that was not less than current_date ( = ' || tools.textcoalesce( current_date ) || ' )'\n  );\n  RETURN v_next_date;\nEND;\n"}}	2010-11-19 17:14:09.56488	2010-11-19 17:14:09.56488	1
stats__daily.update	1	{{function,stats__daily.update},{params,"p_stats_table text\n  , p_stats_view text\n  "},{returns,void},{options,"LANGUAGE plpgsql"},{body,"\n/*\n * Created in: bug_243812_v5_stats\n */\nDECLARE\n  name CONSTANT text := 'stats__daily.update';\n\n  -- Sanitize\n  c_stats_table CONSTANT text := p_stats_table::regclass::text;\n  c_stats_view CONSTANT text := p_stats_view::regclass::text;\n\n  v_next_date DATE := stats__daily.next_date( c_stats_table, c_stats_view );\n\nBEGIN\n\n  -- Replace tags in template with real values and execute\n  PERFORM tools.exec( tools.parameter_replace(\n      -- Query template\n      $$INSERT INTO %stats_table%\n        SELECT *\n          FROM %stats_view%\n          WHERE stats_date = %next_date%\n      $$\n      , array[\n          array[ 'stats_table', c_stats_table ]\n          , array[ 'stats_view', c_stats_view ]\n          -- Note that we want this to be hard-coded in the query for performance reasons\n          , array[ 'next_date', coalesce( $$'$$ || v_next_date || $$'::date$$, 'NULL' ) ]\n        ]\n  ) );\nEND;\n"}}	2010-11-19 17:14:09.56488	2010-11-19 17:14:09.56488	1
\.


--
-- Data for Name: templates; Type: TABLE DATA; Schema: code_internal; Owner: db_global_owner
--

COPY templates (id, object_class_id, version, template, description) FROM stdin;
2	2	1	{"SELECT code.function( '%schema%.%table%__get'\n    , 'p_id %schema%.%table%.id%TYPE'\n    , '%schema%.%table%'\n    , 'LANGUAGE sql STABLE RETURNS NULL ON NULL INPUT'\n    , $$SELECT * FROM %schema%.%table% WHERE id = $1$$\n    , $$Returns row from %schema%.%table% based on id. This is generated code.$$\n    , 'developer_role'\n);"}	Function to return a row from a table based on an ID value.
3	3	1	{"SELECT code.function( '%schema%.%table%__get_text'\n    , 'p_id %schema%.%table%.id%TYPE'\n    , '%schema%.%table%.%lookup%%TYPE'\n    , 'LANGUAGE sql STABLE RETURNS NULL ON NULL INPUT'\n    , $$SELECT %lookup% FROM %schema%.%table% WHERE id = $1$$\n    , $$Returns %lookup% value for %schema%.%table% based on id. This is generated code.$$\n    , 'developer_role'\n);"}	Function to return an ID based on a text lookup.
4	4	1	{"SELECT code.function( '%schema%.%table%__get_id'\n    , 'p_%lookup% %schema%.%table%.%lookup%%TYPE'\n    , '%schema%.%table%.id%TYPE'\n    , 'LANGUAGE sql STABLE RETURNS NULL ON NULL INPUT'\n    , $$SELECT id FROM %schema%.%table% WHERE lower(%lookup%) = lower($1)$$\n    , $$Returns ID value for %schema%.%table% based on %lookup%. This is generated code.$$\n    , 'developer_role'\n);"}	Function to return an ID based on a text lookup.
6	6	1	{"SELECT code.function( '%schema%.%table%__getsert'\n    , 'p_%lookup% %schema%.%table%.%lookup%%TYPE'\n    , '%schema%.%table%.id%TYPE'\n    , 'LANGUAGE plpgsql SECURITY DEFINER'\n    , $%table%__getsert$\nDECLARE\n    v_id      %schema%.%table%.id%TYPE;\nBEGIN\n    LOOP\n        SELECT INTO v_id\n                id\n            FROM %schema%.%table%\n                WHERE %lookup% = p_%lookup%\n        ;\n        IF found THEN\n            RAISE DEBUG '%schema%.%table% \\"%\\" has id %', p_%lookup%, v_id;\n            RETURN v_id;\n        END IF;\n\n        BEGIN\n            RAISE DEBUG 'Inserting \\"%\\" into %schema%.%table%', p_%lookup%;\n            INSERT INTO %schema%.%table%( %lookup% ) VALUES( p_%lookup% );\n            RETURN tools.currval( '%schema%.%table%', 'id' );\n        EXCEPTION WHEN unique_violation THEN\n            -- do nothing\n        END;\n    END LOOP;\nEND;\n$%table%__getsert$\n    , $$Returns ID value for %schema%.%table% based on %lookup%. If the lookup value does not exist, a record will be created for it. This is generated code.$$\n    , 'cnuapp_role'\n);"}	Function to return an ID based on a text lookup, or if it does not exist create a record for it.
7	7	1	{"\nCREATE TABLE %schema%.%plural%(\n    id              serial          CONSTRAINT %plural%__pk_id PRIMARY KEY\n    , %singular%    text            NOT NULL\n);\n                ","\nCOMMENT ON TABLE %schema%.%plural% IS $$Dynamic lookup table for %singular%. This table will very possibly have values added to it automatically via a __getsert function. This is generated code.$$;\nCREATE UNIQUE INDEX %plural%__u_%singular%_lower ON %schema%.%plural%( lower( %singular% ) );\nGRANT SELECT ON %schema%.%plural% TO cnubase_role;\nSELECT code.getsert( '%schema%.%plural%', '%singular%' );\nSELECT code.get( '%schema%.%plural%' );\n                "}	Lookup table with a serial as the ID
5	5	1	{"\nCREATE TABLE %schema%.%plural%(\n    id              smallint        NOT NULL CONSTRAINT %plural%__pk_id PRIMARY KEY\n    %post_id%\n    , %singular%    text            NOT NULL\n    , description   text\n);\n                ","\nCOMMENT ON TABLE %schema%.%plural% IS $$Lookup table for %singular%. The data in this table is considered to be static, meaning that it should only be changed via a patch. This is generated code.$$;\nCREATE UNIQUE INDEX %plural%__u_%singular%_lower ON %schema%.%plural%( lower( %singular% ) );\nSELECT tools.seed_object__add( '%schema%.%plural%', -50 );\nGRANT SELECT ON %schema%.%plural% TO cnubase_role;\nSELECT code.get_id( '%schema%.%plural%', '%singular%' );\nSELECT code.get( '%schema%.%plural%' );\nSELECT code.get_text( '%schema%.%plural%', '%singular%' );\n                "}	Lookup table with a smallint as the ID
1	1	1	{"CREATE OR REPLACE FUNCTION %function%(\n%params%\n) RETURNS %returns% %options% AS $function_body$\n%body%\n$function_body$;"}	General-purpose function template
8	8	1	{"\nCREATE TABLE %schema%.%status_name%_status_valid_transitions( -- If you change the name, change the check below as well\n    id                            serial        PRIMARY KEY\n    , current_status_id            smallint    %current_not_null% CONSTRAINT %status_name%_status__current REFERENCES %schema%.%status_name%_statuses( id )\n    , allowed_status_id            smallint    %allowed_not_null% CONSTRAINT %status_name%_status__allowed REFERENCES %schema%.%status_name%_statuses( id )\n    , event_name                text\n    , description                text\n    , CONSTRAINT %status_name%_status_useless_transition\n        CHECK( tools.check( current_status_id IS DISTINCT FROM allowed_status_id\n                            , 'Setting current_status_id and allowed_status_id in %schema%.%status_name%_status_valid_transitions is pointless'\n            ) )\n);\n                ","\nCREATE UNIQUE INDEX %status_name%_status_valid_trans__u_curr__allow -- Update the creation function if you change this name\n    ON %schema%.%status_name%_status_valid_transitions( current_status_id, allowed_status_id )\n    WHERE( current_status_id IS NOT NULL AND allowed_status_id IS NOT NULL )\n;\nCREATE UNIQUE INDEX %status_name%_status_valid_trans__u_current_is_null\n    ON %schema%.%status_name%_status_valid_transitions( allowed_status_id )\n    WHERE( current_status_id IS NULL AND allowed_status_id IS NOT NULL )\n;\nCREATE UNIQUE INDEX %status_name%_status_valid_trans__u_allowed_is_null\n    ON %schema%.%status_name%_status_valid_transitions( current_status_id )\n    WHERE( current_status_id IS NOT NULL AND allowed_status_id IS NULL )\n;\nCOMMENT ON TABLE %schema%.%status_name%_status_valid_transitions IS $$Valid status transitions for %status_name%s. NULL indicates lack of a record. This is generated code.$$;\nCOMMENT ON COLUMN %schema%.%status_name%_status_valid_transitions.current_status_id IS $$Current %status_name% status.$$; \nCOMMENT ON COLUMN %schema%.%status_name%_status_valid_transitions.allowed_status_id IS $$A status that the %status_name% is allowed to transition to.$$;\n\nSELECT tools.seed_table__add( '%schema%.%status_name%_status_valid_transitions' );\nGRANT SELECT ON TABLE %schema%.%status_name%_status_valid_transitions TO cnubase_role;\nCREATE OR REPLACE FUNCTION %schema%.tg_%status_name%_status_transitions() RETURNS trigger LANGUAGE plpgsql AS $tg_state_transitions$\nDECLARE\n    v_new_id    int := NULL;\n    v_new_name  text;\n    v_old_id    int := NULL;\n    v_old_name  text;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'AFTER', TG_NAME || ' must be an after trigger');\n    PERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' must be a per-row trigger');\n\n    -- Bail if checks have been disabled\n    IF coalesce( tools.setting_b( 'Checks disabled' ), false ) THEN\n        RETURN NULL;\n    END IF;\n\n    IF TG_OP <> 'DELETE' THEN\n        v_new_id := NEW.%status_name%_status_id;\n        v_new_name := %status_name%_status FROM %schema%.%status_name%_statuses WHERE id = v_new_id;\n    END IF;\n    IF TG_OP <> 'INSERT' THEN\n        v_old_id := OLD.%status_name%_status_id;\n        v_old_name := %status_name%_status FROM %schema%.%status_name%_statuses WHERE id = v_old_id;\n    END IF;\n\n    RAISE DEBUG $$v_old_id=%, v_new_id=%$$, v_old_id, v_new_id;\n\n    IF v_new_id IS DISTINCT FROM v_old_id THEN\n        IF NOT EXISTS ( SELECT *\n                            FROM %schema%.%status_name%_status_valid_transitions\n                            -- We use NOT IS DISTINCT FROM because we have to deal with %status_name%_status_id being NULL\n                            WHERE NOT current_status_id IS DISTINCT FROM v_old_id\n                                AND NOT allowed_status_id IS DISTINCT FROM v_new_id\n                        )\n        THEN\n            RAISE EXCEPTION $$% (%) -> % (%) is not a valid state transition for %.installment_status_id$$\n                    , coalesce( v_old_id::text, '<NULL>' )\n                    , v_old_name\n                    , coalesce( v_new_id::text, '<NULL>' )\n                    , v_new_name\n                    , tg_table_name\n            ;\n        END IF;\n    END IF;\n    RETURN NULL;\nEND;\n$tg_state_transitions$;\n                "}	Valid status transitions for a table
9	9	1	{"\nSELECT code.function( '%schema%.tg_%parent_table%_ri'\n    , ''\n    , 'trigger'\n    , 'LANGUAGE plpgsql'\n    , $tg_ri$\nDECLARE\n    name CONSTANT text := '%schema%.tg_%parent_table%_ri';\n    c_full_table_name CONSTANT text := TG_TABLE_SCHEMA || '.' || TG_TABLE_NAME;\n    v_%type_singular% %schema%.%types_plural%.%type_singular%%TYPE;\n    v_table_name text;\n    v_only text := '';\n    v_result int;\n    sql text;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'AFTER', TG_NAME || ' ON ' || c_full_table_name ||' must be an AFTER trigger' );\n    PERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' ON ' || c_full_table_name ||' must be a row-level trigger' );\n    PERFORM tools.assert( TG_OP IN ( 'INSERT', 'UPDATE' ), TG_NAME || ' ON ' || c_full_table_name ||' must be on INSERT or UPDATE' );\n\n    -- Th generally won't be allowed, but the trigger should still support it\n    IF NEW.%primary_key% IS NULL THEN\n        RAISE DEBUG '%: %primary_key% is NULL, skipping check', name;\n        RETURN NULL;\n    END IF;\n\n    -- If we're updating and we haven't modified %primary_key%, just bail\n    IF TG_OP = 'UPDATE' THEN\n        IF NEW.%primary_key% IS NOT DISTINCT FROM OLD.%primary_key% THEN\n            RAISE DEBUG '%: %primary_key% ( = % ) is unchanged, skipping check', name, NEW.%primary_key%;\n            RETURN NULL;\n        END IF;\n    END IF;\n\n    /*\n     * We want to not only check for existence of the desired row, we also want\n     * to share-lock it. Unfortunately, sharelocks aren't implemented for\n     * inherited tables, so we need to find the record in the correct table. We\n     * can do this fairly easily if we can find out what \\"type\\" of record it is.\n     */\n\n    -- Figure out what type of record this is (of course, record might not exist here)\n    SELECT INTO v_%type_singular% '%child_name_prefix%' || %table_name_field% || '%child_name_suffix%'\n        FROM %schema%.%types_plural%__get( (\n                    SELECT %type_singular%_id\n                        FROM %schema%.%parent_table% pi\n                        WHERE pi.%primary_key% = NEW.%primary_key%\n                ) )\n    ;\n    IF NOT FOUND OR v_%type_singular% IS NULL THEN\n        -- There wasn't a record at all\n        v_only := 'ONLY '; -- FOR SHARE won't work if we select from both the parent and the children\n        v_table_name := '%schema%.%parent_table%';\n    ELSE\n        -- We figured out what type of record this is, now try and lock the row in the right table\n        v_table_name := v_%type_singular%;\n    END IF;\n\n    sql := 'SELECT 1 FROM ' || v_only || v_table_name || '\n        WHERE %primary_key% = ' || NEW.%primary_key% || '\n        FOR SHARE'\n    ;\n    RAISE DEBUG '%: Executing SQL %', name, sql;\n    -- Note that simply using a PERFORM here might get optimized out.\n    EXECUTE sql INTO v_result;\n\n    /*\n     * Normally we should always find an record, but if it was somehow\n     * put in the wrong table, or if it was deleted after our initial\n     * select then we wouldn't have one.\n     */\n    IF v_result = 1 THEN\n        RAISE DEBUG '%: record for %primary_key% = % found in table %', name, NEW.%primary_key%, v_table_name;\n        RETURN NULL;\n    END IF;\n\n    RAISE EXCEPTION 'insert on update on table \\"%.%\\" violates foreign key; %primary_key%=(%) is not present in table \\"%\\"'\n        , TG_TABLE_SCHEMA\n        , TG_TABLE_NAME\n        , NEW.%primary_key%\n        , v_table_name\n    ;\nEND;\n$tg_ri$\n    , $$Enables Refferential Integrity at the %parent_table% level (normal RI on a table that is an inheritance parent does not really work)$$\n    , NULL\n);\n            "}	Trigger function to provide RI on inherited (parent) tables.
10	10	1	{"\nSELECT code.function( '%schema%.tg_%parent_table%_unique'\n    , ''\n    , 'trigger'\n    , 'LANGUAGE plpgsql'\n    , $tg_unique$\nDECLARE\nname CONSTANT text := '%schema%.tg_%parent_table%_unique';\nc_full_table_name CONSTANT text := TG_TABLE_SCHEMA || '.' || TG_TABLE_NAME;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'BEFORE', TG_NAME || ' ON ' || c_full_table_name ||' must be an BEFORE trigger' );\n    PERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' ON ' || c_full_table_name ||' must be a row-level trigger' );\n    \n    -- Deleting would break RI, so don't allow it. Granted, this should probably be a separate trigger, but...\n    PERFORM tools.assert( '%schema%__%parent_table%__inherit__no_delete'\n        , TG_OP != 'DELETE'\n        , 'DELETEs are not allowed on ' || c_full_table_name || ' (they would break inheritance RI)'\n    );\n\n    RAISE DEBUG '%:\n        TG_OP = %\n        TG_TABLE_NAME = %\n        NEW.%primary_key% = %'\n        , name\n        , TG_OP\n        , TG_TABLE_NAME\n        , NEW.%primary_key% \n    ;\n\n    -- Changing the PK would break RI, so we shouldn't allow it. Granted, this should probably be a separate trigger, but...\n    IF TG_OP = 'UPDATE' THEN\n        PERFORM tools.assert( '%schema%__%parent_table%__inherit__pk_no_change'\n            , NEW.%primary_key% IS NOT DISTINCT FROM OLD.%primary_key%\n            , 'Changing %primary_key% on ' || c_full_table_name || ' is not allowed (it would break inheritance RI)'\n        );\n    ELSE\n        -- Only check for dupes on insert, otherwise we'll see our own ID\n        PERFORM tools.assert( '%schema%__%parent_table%__inherit__unique'\n            , NOT EXISTS( SELECT * FROM %schema%.%parent_table% WHERE %primary_key% = NEW.%primary_key% )\n            , 'duplicate row violation, %primary_key% ' || coalesce( NEW.%primary_key%::text, '<NULL>' ) || ' already exists'\n        );\n    END IF;\n\n    RETURN NEW;\nEND;\n$tg_unique$\n    , $$Trigger to try and prevent duplicated %primary_key%s. This trigger is in no way perfect and has a huge race condition, but generally these IDs should be getting assigned by a sequence, so we should not normally have an issue with duped IDs anyway.$$\n    , NULL\n);\nSELECT tools.assert__add( '%schema%__%parent_table%__inherit__no_delete', 200 );\nSELECT tools.assert__add( '%schema%__%parent_table%__inherit__unique', 200 );\nSELECT tools.assert__add( '%schema%__%parent_table%__inherit__pk_no_change', 200 );\n            "}	Trigger function to provide RI on inherited (parent) tables.
11	11	1	{"\nSELECT code.function( '%schema%.tg_%source_singular%_update_%target%'\n    , ''\n    , 'trigger'\n    , 'LANGUAGE plpgsql'\n    , $status_update$\nDECLARE\n    v_status_record %schema%.%sources_plural%;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'BEFORE', TG_NAME || ' must be an AFTER trigger' );\n\tPERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' must be a row-level trigger' );\n\tPERFORM tools.assert( TG_OP != 'DELETE', 'DELETE is not supported for trigger ' || TG_NAME );\n\n    IF TG_OP = 'UPDATE' THEN\n        IF NEW.%source_singular%_id IS NOT DISTINCT FROM OLD.%source_singular%_id THEN\n            RETURN NEW;\n        END IF;\n    END IF;\n    \n    v_status_record := %schema%.%sources_plural%__get( NEW.%source_singular%_id );\n\n    -- If the status doesn't exist, or the mapping field is null, we shouldn't change anything\n    IF v_status_record.%target%_id IS NOT NULL THEN\n        RAISE DEBUG 'Automatically updating %target%_id to %, due to %source_singular%_id being set to %'\n            , NEW.%source_singular%_id\n            , v_status_record.%target%_id\n        ;\n        NEW.%target%_id := v_status_record.%target%_id;\n    END IF;\n\n    RETURN NEW;\nEND;\n$status_update$\n    , $$Trigger to automatically change %target%_id based on changes to %source_singular%_id.$$\n    , NULL\n);\n            "}	Table and trigger to track changes to a status field.
12	12	1	{"\nSELECT code.function( '%schema%.%tg_name%'\n    , ''\n    , 'trigger'\n    , 'LANGUAGE plpgsql'\n    , $tg_match$\nDECLARE\n    v_field %target_table%.%target_field%%TYPE;\n    v_found boolean;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'AFTER', TG_NAME || ' must be an AFTER trigger' );\n\tPERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' must be a row-level trigger' );\n\tPERFORM tools.assert( TG_OP != 'DELETE', 'DELETE is not supported for trigger ' || TG_NAME );\n\n    -- If this is and update and nothing has changed, bounce out. Must check TG_OP before referencing OLD\n    IF TG_OP = 'UPDATE' THEN\n        IF NEW.%fk_field% IS NOT DISTINCT FROM OLD.%fk_field%\n            AND NEW.%match_field% IS NOT DISTINCT FROM OLD.%match_field%\n        THEN\n            RETURN NEW;\n        END IF;\n    END IF;\n    \n    SELECT INTO v_field %select%;\n    \n    -- Make sure we got a record back (or that the FK field is null)\n    PERFORM tools.assert(\n        '%schema%__%tg_name%__no_record'\n        , FOUND OR NEW.%fk_field% IS NULL\n        , 'No record found in %target_table% where %target_pk% = ' || tools.textcoalesce( NEW.%fk_field% )\n    );\n\n    PERFORM tools.assert(\n        '%schema%__%tg_name%__match'\n        , NEW.%match_field% IS NOT DISTINCT FROM v_field\n        , TG_TABLE_SCHEMA || '.' || TG_TABLE_NAME || '.%match_field% ( = ' || tools.textcoalesce( NEW.%match_field% )\n            || ' ) must match %target_table%.%target_field% ( = ' || tools.textcoalesce( v_field )\n            || ' ) where %target_table%.%target_pk% = ' || tools.textcoalesce( NEW.%fk_field% )\n    );\n\n    RETURN NEW;\nEND;\n$tg_match$\n    , $$Trigger to ensure that %match_field% in a table matches %target_field% in %target_table%.$$\n    , NULL\n);\nSELECT tools.assert__add( '%schema%__%tg_name%__no_record', 100 );\nSELECT tools.assert__add( '%schema%__%tg_name%__match', 100 );\n            "}	Table and trigger to track changes to a status field.
13	13	1	{"\nCREATE TABLE %schema%.%status_name%_status_history(\n    id                          serial          PRIMARY KEY\n    , %table_pk_reference%      int             NOT NULL %track_table_RI%\n    , changed_on                timestamptz     NOT NULL DEFAULT now()\n    , %status_name%_status_id   smallint        NOT NULL REFERENCES %schema%.%status_name%_statuses\n    , changed_by                varchar(30)\n);\n            ","\nCREATE INDEX %status_name%_status_history__%table_pk_reference%__status\n    ON %schema%.%status_name%_status_history( %table_pk_reference%, %status_name%_status_id )\n;\nGRANT SELECT ON %schema%.%status_name%_status_history TO cnubase_role;\nCOMMENT ON TABLE %schema%.%status_name%_status_history IS $$Tracks history of %status_name%_status for %table_plural%.$$;\nCOMMENT ON COLUMN %schema%.%status_name%_status_history.%status_name%_status_id IS $$Status that %table_pk_reference% was changed *TO*.$$;\n\n-- Create trigger function\nSELECT code.function( '%schema%.tg_%status_name%_status_history'\n    , ''\n    , 'trigger'\n    , 'LANGUAGE plpgsql SECURITY DEFINER'\n    , $status_history$\nDECLARE\n    v_track_changed_by boolean := true;\n    v_changed_by varchar(30) := NULL;\nBEGIN\n    PERFORM tools.assert( TG_WHEN = 'AFTER', TG_NAME || ' must be an AFTER trigger' );\n\tPERFORM tools.assert( TG_LEVEL = 'ROW', TG_NAME || ' must be a row-level trigger' );\n\tPERFORM tools.assert( TG_OP != 'DELETE', 'DELETE is not supported for trigger ' || TG_NAME );\n\n    IF TG_OP = 'UPDATE' THEN\n        IF NEW.%status_name%_status_id IS NOT DISTINCT FROM OLD.%status_name%_status_id THEN\n            RETURN NULL;\n        END IF;\n    END IF;\n    \n    IF TG_NARGS > 0 THEN\n        v_track_changed_by := lower( TG_ARGV[0] ) = 'true';\n    END IF;\n    IF v_track_changed_by THEN\n        v_changed_by := NEW.updated_by;\n    END IF;\n\n    INSERT INTO %schema%.%status_name%_status_history( %table_pk_reference%, %status_name%_status_id, changed_by )\n        VALUES( NEW.%primary_key%, NEW.%status_name%_status_id, v_changed_by )\n    ;\n    RETURN NULL;\nEND;\n$status_history$\n    , $$Trigger to track changes to %status_name%_status_id in table\n%schema%.%status_name%_history. It accepts one (optional) argument that\nspecifies if the table has an updated_on field (default is 'true').$$\n    , NULL\n);\n        "}	Table and trigger to track changes to a status field.
14	14	1	{"CREATE OR REPLACE VIEW %schema%.%table%_v AS \n    SELECT t.* \n        %selectlist%\n    FROM %schema%.%table% t\n;"}	A view that denormalizes lookup tables
\.


SET search_path = inheritance, pg_catalog;

--
-- Data for Name: child_objects; Type: TABLE DATA; Schema: inheritance; Owner: db_global_owner
--

COPY child_objects (id, inherited_table_id, object_type_id, object_name_template, object_body_template) FROM stdin;
\.


--
-- Data for Name: child_objects_exclude_tables; Type: TABLE DATA; Schema: inheritance; Owner: db_global_owner
--

COPY child_objects_exclude_tables (child_object_id, exclude_table_id) FROM stdin;
\.


--
-- Data for Name: object_types; Type: TABLE DATA; Schema: inheritance; Owner: db_global_owner
--

COPY object_types (id, object_type, description, template, reset_template) FROM stdin;
1	trigger	\N	CREATE TRIGGER %object_name_template% %object_body_template%	DROP TRIGGER %object_name_template% ON %child_full%
2	constraint	\N	ALTER TABLE %child_full% ADD CONSTRAINT %object_name_template% %object_body_template%	ALTER TABLE %child_full% DROP CONSTRAINT %object_name_template%
3	column set	\N	ALTER TABLE %child_full% ALTER %object_name_template% SET %object_body_template%	ALTER TABLE %child_full% ALTER %object_name_template% DROP %object_body_template%
\.


SET search_path = object_monitor, pg_catalog;

--
-- Data for Name: monitors; Type: TABLE DATA; Schema: object_monitor; Owner: db_global_owner
--

COPY monitors (monitor_name) FROM stdin;
\.


--
-- Data for Name: monitors_consumers; Type: TABLE DATA; Schema: object_monitor; Owner: db_global_owner
--

COPY monitors_consumers (monitor_name, consumer_name) FROM stdin;
\.


--
-- Data for Name: monitors_tables; Type: TABLE DATA; Schema: object_monitor; Owner: db_global_owner
--

COPY monitors_tables (monitor_name, table_id) FROM stdin;
\.


SET search_path = pgq, pg_catalog;

--
-- Data for Name: consumer; Type: TABLE DATA; Schema: pgq; Owner: skytools
--

COPY consumer (co_id, co_name) FROM stdin;
\.


--
-- Data for Name: event_template; Type: TABLE DATA; Schema: pgq; Owner: skytools
--

COPY event_template (ev_id, ev_time, ev_txid, ev_owner, ev_retry, ev_type, ev_data, ev_extra1, ev_extra2, ev_extra3, ev_extra4) FROM stdin;
\.


--
-- Data for Name: failed_queue; Type: TABLE DATA; Schema: pgq; Owner: skytools
--

COPY failed_queue (ev_failed_reason, ev_failed_time, ev_id, ev_time, ev_txid, ev_owner, ev_retry, ev_type, ev_data, ev_extra1, ev_extra2, ev_extra3, ev_extra4) FROM stdin;
\.


--
-- Data for Name: queue; Type: TABLE DATA; Schema: pgq; Owner: skytools
--

COPY queue (queue_id, queue_name, queue_ntables, queue_cur_table, queue_rotation_period, queue_switch_step1, queue_switch_step2, queue_switch_time, queue_external_ticker, queue_ticker_max_count, queue_ticker_max_lag, queue_ticker_idle_period, queue_data_pfx, queue_event_seq, queue_tick_seq) FROM stdin;
\.


--
-- Data for Name: retry_queue; Type: TABLE DATA; Schema: pgq; Owner: skytools
--

COPY retry_queue (ev_retry_after, ev_id, ev_time, ev_txid, ev_owner, ev_retry, ev_type, ev_data, ev_extra1, ev_extra2, ev_extra3, ev_extra4) FROM stdin;
\.


--
-- Data for Name: subscription; Type: TABLE DATA; Schema: pgq; Owner: skytools
--

COPY subscription (sub_id, sub_queue, sub_consumer, sub_last_tick, sub_active, sub_batch, sub_next_tick) FROM stdin;
\.


--
-- Data for Name: tick; Type: TABLE DATA; Schema: pgq; Owner: skytools
--

COPY tick (tick_queue, tick_id, tick_time, tick_snapshot) FROM stdin;
\.


SET search_path = replication, pg_catalog;

--
-- Data for Name: schemas__exclude; Type: TABLE DATA; Schema: replication; Owner: db_global_owner
--

COPY schemas__exclude (id, schema_name, description) FROM stdin;
1	^information_schema$	\N
2	^londiste$	\N
3	^pgq$	\N
4	^replication$	\N
5	^pgq_ext$	\N
6	^garbage$	\N
7	^txid$	\N
8	^dbcheck$	\N
9	^constraint_check$	\N
10	^tools$	\N
11	^code$	\N
12	^code_internal$	\N
13	^object_monitor$	\N
14	delete	\N
15	^pg_	\N
16	^inheritance$	\N
\.


--
-- Data for Name: sets; Type: TABLE DATA; Schema: replication; Owner: db_global_owner
--

COPY sets (id, inclusive, exclude_set, set, description) FROM stdin;
1	t	\N	loc-cnu	Replication of tables from LOC to CNU
2	t	\N	cnu-loc	Replication of tables from CNU to LOC
3	f	\N	cnu-slave	Replication of tables from CNU to a CNU slave
4	f	loc-cnu	cnu-reporting	Replication of tables from CNU to a PI reporting slave
5	f	cnu-loc	loc-reporting	Replication of tables from LOC to a PI reporting slave
\.


--
-- Data for Name: tables; Type: TABLE DATA; Schema: replication; Owner: db_global_owner
--

COPY tables (id, set_id, full_name) FROM stdin;
\.


--
-- Data for Name: tables__exclude; Type: TABLE DATA; Schema: replication; Owner: db_global_owner
--

COPY tables__exclude (id, full_name, description) FROM stdin;
1	cnu_stats.page_hits_queue	\N
\.


SET search_path = tools, pg_catalog;

--
-- Data for Name: asserts; Type: TABLE DATA; Schema: tools; Owner: db_global_owner
--

COPY asserts (assert_name, enabled, level) FROM stdin;
assert_exists	t	100
tools_reject_update NARGS	t	50
md5_size	t	100
loan_contracts__no_update_of_state_sequence_once_set	t	50
tg_loans__checks tg_when	t	100
tg_loans__checks tg_level	t	100
tg_loans__checks tg_op	t	100
loans bank_authorization_revoked_on	t	5
tools__object__get_oid__relkind_check	t	100
tools__seed_object__add__no_system_schema	t	100
md5_characters	t	100
test assert	t	1
tools__trigger_sanity__good_when	t	75
tools__trigger_sanity__good_level	t	75
tools__trigger_sanity__good_op	t	75
tools__trigger_sanity__when	t	75
tools__trigger_sanity__level	t	75
tools__trigger_sanity__op	t	75
blacklisted_banks__criteria_not_null_for_automatic_blacklist	t	0
tools__seed_object__add__dump_order_out_of_range	t	50
lead_sale__ping_trees__max_seconds_must_be_positive	t	0
lead_sale__ping_trees__percentage_between_0_and_100	t	0
lead_sale__ping_tree__change_only_when_status_created	t	10
lead_sale__branches__max_seconds_must_be_positive	t	0
lead_sale__branch_position_unique	t	10
lead_sale__branch_change_check_tree_status	t	10
language_cd_check	t	0
country_cd_check	t	0
tools__currval__no_sequence	t	100
payment_instruments__payment_instruments__inherit__no_delete	t	200
payment_instruments__payment_instruments__inherit__unique	t	200
payment_instruments__payment_instruments__inherit__pk_no_change	t	200
physical_payments__eff_date	t	50
payment_instruments__tg_match_payment_instrument_id__customer_id__no_record	t	100
payment_instruments__tg_match_payment_instrument_id__customer_id__match	t	100
payment_instruments__tg_match_payment_instrument_id__payment_instrument_type_id__no_record	t	100
payment_instruments__tg_match_payment_instrument_id__payment_instrument_type_id__match	t	100
payment_instruments__tg_default_instrument_history_previous_default_id__tg_op	t	100
payment_instruments__tg_defaults__loop_overflow	t	20
payment_instruments__tg_defaults__table_check	t	100
payment_instruments__tg_defaults__no_delete	t	100
inheritance__tables__getsert loop overflow	t	100
inheritance__child_object__not_found	t	100
physical_payment__amount	t	0
physical_payments__physical_payments__inherit__no_delete	t	200
physical_payments__physical_payments__inherit__unique	t	200
physical_payments__physical_payments__inherit__pk_no_change	t	200
inheritance__process_child no pending fkeys	t	100
physical_payment_amount_positive	t	10
loans__deny_updates	t	100
installments_static__deny_updates	t	100
payment_instruments__payment_instruments__deny_updates	t	100
payment_instruments__bank_accounts__deny_updates	t	100
payment_instruments__debit_cards__deny_updates	t	100
physical_payments_us__debit_card_lines__deny_updates	t	100
physical_payments_us__direct_entry_lines__deny_updates	t	100
physical_payments_us__direct_entry_files__deny_updates	t	100
physical_payments_us__bundles__deny_updates	t	100
physical_payments_us__debit_card_lines__no_update_on_final_status	t	100
physical_payments_us__direct_entry_lines__no_update_on_final_status	t	100
collections__tg_last_collection_results__loop_overflow	t	20
collections__tg_last_collection_results__table_check	t	100
collections__tg_last_collection_results__no_delete	t	100
business_entity_id is null XOR customer_id is null	t	50
payment_instruments__bank_accounts__routing_number not null for customer accounts	t	90
payment_instruments__bank_accounts__account_number not null for customer accounts	t	90
payment_instruments__bank_accounts__deny_updates_to_bank_fields	t	100
bank_accounts__getsert loop overflow	t	200
customer_bank_acct_hack__insert__id_not_set	t	100
customer_bank_acct_hack__insert__lock_version_not_set	t	100
customer_bank_acct_hack__update__id_no_change	t	100
customer_bank_acct_hack__update__customer_id_no_change	t	100
customer_bank_acct_hack__update__account_type_cd_no_change	t	100
customer_bank_acct_hack__update__eff_start_date_no_change	t	50
customer_bank_acct_hack__update__eff_end_date_no_change	t	50
customer_bank_acct_hack__update__country_cd_no_change	t	100
not found	t	100
physical_payments_us__tg_no_customer_bank_accounts_for_origin customer payment instrument	t	50
bank_account not found	t	100
physical_payments__bank_accounts amount for business accounts	t	0
physical_payments_us__bundles__line_count	t	0
physical_payments_us__bundles__credit_amount	t	0
physical_payments_us__bundles__debit_amount	t	0
physical_payments_us__bundles__inherit__no_delete	t	200
physical_payments_us__bundles__inherit__unique	t	200
physical_payments_us__bundles__inherit__pk_no_change	t	200
physical_payments_us__direct_entry_files__can_not_replace_yourself	t	90
physical_payments_us__tg_match_direct_entry_file_id__bundle_id__no_record	t	100
physical_payments_us__tg_match_direct_entry_file_id__bundle_id__match	t	100
physical_payments_us__direct_entry_lines__transaction_code	t	50
physical_payments_us__direct_entry_lines payment_instrument_ids must be different	t	60
physical_payments_us__acknowledgement_files__bundle_not_null	t	10
physical_payments_us__acknowledgment_files__inherit__no_delete	t	200
physical_payments_us__acknowledgment_files__inherit__unique	t	200
physical_payments_us__acknowledgment_files__inherit__pk_no_change	t	200
physical_payments_us__direct_entry_acknowledgment_files__unknown_acknowledgment_code_interlock	t	0
physical_payments_us__direct_entry_acknowledgment_files__direct_entry_files_id_not_null_for_non_failed	t	0
physical_payments_us__acknowledgments__inherit__no_delete	t	200
physical_payments_us__acknowledgments__inherit__unique	t	200
physical_payments_us__acknowledgments__inherit__pk_no_change	t	200
physical_payments_us__direct_entry_acknowledgments__unknown_acknowledgment_code_interlock	t	0
physical_payments_us__return_files__failed_processing_description_null_check	t	0
physical_payments_us__return_files__inherit__no_delete	t	200
physical_payments_us__return_files__inherit__unique	t	200
physical_payments_us__return_files__inherit__pk_no_change	t	200
physical_payments_us__returns__inherit__no_delete	t	200
physical_payments_us__returns__inherit__unique	t	200
physical_payments_us__returns__inherit__pk_no_change	t	200
physical_payments_us__direct_entry_returns__unknown_return_code_interlock	t	0
pads__ordinal_range	t	10
attachments__couch_url_change_disallowed	t	50
employees_no_spaces_in_login	t	100
payment_instruments__tg_check_customer_and_payment_instrument_type customer_id mismatch	t	100
payment_instruments__tg_check_customer_and_payment_instrument_type payment_instrument_type_id mismatch	t	100
mef__deny_updates	t	99
parameter_tracking__requests__customer__xor__session_key	t	10
tg_datetime_constraint	t	100
current_environment__valid	t	100
loan_approval__upsert__loop_overflow	t	20
mef_inputs__deny_updates_on_not_null	t	100
credit_model_results__track_last_credit_model_result_per_customer__customer_id_changed	t	100
credit_model_results__track_last_credit_model_result_per_customer__loop_overflow	t	20
loans__external_identifier_nullability	t	10
loans__arrangement_version_id_nullability	t	10
loans__paid_off_nullability	t	10
customers__external_identifier_nullability	t	10
tg_loans__valid_customer enroll_date	t	100
tg_loans__valid_customer status_cd	t	100
account_bins__account_bin_must_be_6_digits	t	10
physical_payments_us__ppc_lines payment_instrument_ids must be different	t	60
physical_payments_them__paycard_errors__line_number_ge_0	t	0
enrollment_lines__amount_positive	t	0
ppc__enrollment_lines__line_number_ge_0	t	0
ppc__enrollment_errors__line_number_ge_0	t	0
replacement_lines__amount_positive	t	0
ppc__replacement_lines__line_number_ge_0	t	0
ppc__replacement_errors__line_number_ge_0	t	0
participation__breakdowns__percentage_gt_0	t	10
credit_bureau__credit_bureau_lines__past_due_amount_must_be_positive	t	10
credit_bureau__credit_bureau_lines__charge_off_amount_must_be_positive	t	10
credit_bureau__credit_bureau_lines__monthly_payment_amount_must_be_positive	t	10
credit_bureau__credit_bureau_lines__actual_payment_amount_must_be_positive	t	10
credit_bureau__credit_bureau_lines__last_payment_date_le_account_date	t	10
credit_bureau__credit_bureau_lines__account_date_must_be_last_day_of_month	t	0
credit_bureau__credit_bureau_lines__account_date_must_be_in_past	t	10
credit_bureau__credit_bureau_lines__payment_history_letters	t	0
deposit_lines__amount_positive	t	0
ppc__deposit_lines__line_number_ge_0	t	0
deposit_pending_lines__amount_positive	t	0
ppc__deposit_pending_lines__line_number_ge_0	t	0
ppc__deposit_errors__line_number_ge_0	t	0
credit_statistics__credit_limit_positive	t	0
noaa_requests__tnc_signed_date_null_check	t	1
noaa_requests__notification_date_null_check	t	2
noaa_requests__final_status_date_null_check_with_status_id	t	3
physical_payments_us__ppc_lines__no_update_on_final_status	t	100
physical_payments_them__paycard_lines__no_update_on_final_status	t	100
payment_instruments__paycards__deny_updates	t	100
credit_bureau_files__deny_updates	t	100
participation__breakdown_percentage	t	0
pdf_statements__deny_updates	t	100
physical_payments_them__paycard_lines__deny_updates	t	100
physical_payments_us__ppc_lines__deny_updates	t	100
next_stats_date_must_not_be_today	t	50
pay_stubs__pay_stubs__inherit__no_delete	t	200
inheritance__reset_child no pending fkeys	t	100
physical_payments__tg_check_customer_id__mismatch	t	100
business_entity_id is null XOR person_id is null	t	100
customers__person_id_must_not_change	t	100
disallow_multiple_emails_per_person	t	99
oec_statements__credit_limit_positive	t	90
oec_statements__available_credit_positive	t	90
oec_statements__minimum_payment_positive	t	90
oec_statements__statement_balance_positive	t	90
oec_statements__total_interest_due_positive	t	90
oec_statements__principal_due_positive	t	90
oec_statements__fees_due_positive	t	90
oec_statements__previous_balance_positive	t	90
oec_statements__past_due_positive	t	90
oec__statement_transactions__amount_positive	t	90
future_oec_draws__amount_positive	t	90
not null	t	100
semicolon not allowed	t	100
customers_email_btrim	t	90
seed_object_entry__exists	t	99
shift__shift_definition__lead_sellers__brand_id_match	t	80
shift__shift_values_null_check	t	80
shift__values_load_no_delete	t	90
pay_stubs__pay_stubs__inherit__unique	t	200
pay_stubs__pay_stubs__inherit__pk_no_change	t	200
pay_stubs__loan_id_must_not_change	t	100
code__role__single_role	t	100
code__role__no_semicolon_in_p_in_roles	t	100
cabar__component_schema__add__schema_exists	t	100
loans__valid_loan_length	t	99
next_stats_date_offset_must_be_negative	t	50
\.


--
-- Data for Name: dump_time; Type: TABLE DATA; Schema: tools; Owner: db_global_owner
--

COPY dump_time (dump_time) FROM stdin;
\.


--
-- Data for Name: object_types; Type: TABLE DATA; Schema: tools; Owner: db_global_owner
--

COPY object_types (object_type, catalog_table, cast_name, name_prefix) FROM stdin;
table	pg_class	regclass	\N
conversion	pg_conversion	\N	\N
database	pg_database	\N	\N
domain	pg_type	\N	\N
index	pg_class	regclass	\N
role	pg_roles	\N	\N
schema	pg_namespace	\N	nsp
sequence	pg_class	regclass	\N
tablespace	pg_tablespace	\N	spc
type	pg_type	regtype	\N
view	pg_class	regclass	\N
\.


--
-- Data for Name: schema_patches; Type: TABLE DATA; Schema: tools; Owner: db_global_owner
--

COPY schema_patches (patch_file, applied_at, depends_on, component_id) FROM stdin;
bug_8967_seed_table_information	2008-10-30 19:48:08.357615-05	{bug_8967__functions}	\N
bug_10209_fix_objects_alexg	2008-11-07 20:35:17.695388-06	{bug_10209_ownership_views}	\N
bug_45823_fix_installments_view_accounting	2009-03-26 16:31:28.122222-05	{bug_40782__installment_principal_interest_coalesce_with_zero}	\N
bug_9667_iovation_reports	2008-09-30 20:05:56.62688-05	{bug_9667__tables}	\N
bug_8007__londiste_no_dml_trigger	2008-07-10 19:40:33.095546-05	\N	\N
bug_10024_fix_views	2008-10-03 19:07:11.063041-05	{bug_8338_early,bug_8338_late}	\N
bug_8101__add_callback_number	2008-07-18 18:11:58.721299-05	\N	\N
bug_8640__dialer_bp_60_90_view	2008-07-18 18:12:50.31889-05	\N	\N
bug_9667__tables	2008-09-30 20:05:30.170322-05	\N	\N
bug_4036_contract_values_function	2008-08-23 01:31:45.721502-05	\N	\N
bug_5221__add_lead_buyers	2008-07-24 21:47:22.422739-05	\N	\N
bug_9589__dialer_function	2008-08-29 14:58:19.10209-05	\N	\N
bug_9369__deny_trigger	2008-09-18 18:32:49.383537-05	\N	\N
bug_4988__normalize_page_stats	2008-07-24 19:32:11.484271-05	\N	\N
bug_8686__fix_tg_schema_patches	2008-09-25 22:55:04.262411-05	\N	\N
bug_7571_add_pending_reason_column_to_veritec_transaction_log	2008-07-29 23:28:39.590794-05	\N	\N
bug_3914_share_report_retract	2008-08-07 19:30:38.510752-05	\N	\N
bug_9657__update_lead_buyers_timeout	2008-09-04 19:51:23.60775-05	\N	\N
bug_8338_early	2008-09-16 23:40:34.07386-05	\N	\N
bug_7349__grace_period_status	2008-07-29 23:32:12.609197-05	{bug_7349__functions}	\N
bug_9562_db_config	2008-11-14 01:34:35.277388-06	\N	\N
bug_9658__prevent_gov_law_state_cd_changes	2008-09-18 18:45:54.667-05	{bug_9369__deny_trigger}	\N
bug_9563__checks_and_asserts	2008-11-14 01:34:35.522431-06	{bug_9562_db_config}	\N
bug_9689__disallow_triggers	2008-11-14 01:34:35.701772-06	\N	\N
bug_8703__add_bank_wizard_absolute_reports_tables	2008-09-25 22:55:04.34336-05	\N	\N
bug_7349__functions	2008-07-29 23:28:54.982964-05	\N	\N
bug_9839__customer_uuid	2008-09-30 20:12:49.005569-05	\N	\N
bug_9064__londiste_drop_slave_trigger	2008-08-19 20:24:31.521574-05	\N	\N
bug_8281__datax_tables	2008-09-25 22:55:53.195111-05	\N	\N
bug_8338_late	2008-09-16 23:40:34.282333-05	\N	\N
bug_8974__add_app_and_host_name_to_page_hits	2008-08-19 20:24:42.904319-05	{bug_4988__normalize_page_stats}	\N
bug_7332_uniqueness_constraint_on_loan_tags	2008-05-08 00:00:00-05	\N	\N
bug_7332_contract_values_function	2008-05-08 00:00:00-05	\N	\N
bug_7332_payment_transactions	2008-05-08 00:00:00-05	\N	\N
bug_7332_tags	2008-05-08 00:00:00-05	\N	\N
bug_10117_add_uuid_type	2008-10-23 19:33:51.16399-05	\N	\N
bug_2198__alter_loan_tasks_payment_transactions_phase_1	2008-11-15 22:44:11.88059-06	{bug_2198__partial_rollback}	\N
bug_10761__seed_sequences	2008-11-10 14:08:52.453975-06	{bug_8967__functions}	\N
bug_10122_add_columns_to_clv_reports	2008-11-20 21:04:34.13763-06	\N	\N
bug_2198__functions	2008-11-14 01:34:36.007782-06	{bug_9563__checks_and_asserts,bug_10421_convert_check_and_assert_to_plpgsql}	\N
bug_44360__add_activity_codes	2009-02-19 19:09:19.62601-06	\N	\N
bug_10631_revamp_asserts	2009-04-09 16:48:07.985869-05	{bug_10421_convert_check_and_assert_to_plpgsql,bug_11879_code_tools}	\N
bug_139035_add_ast_activity_code	2010-03-10 18:24:35.922959-06	\N	\N
bug_123060_object_monitor_authority_dw_role	2010-02-04 19:15:11.677164-06	{bug_77655_pgq_object_monitor}	\N
bug_155792_add_inheritance	2010-03-18 18:52:13.23773-05	{bug_126341__drop_trigger_payment_instrument_status_history,bug_71767_functions,bug_73403_au_mergeback_phase_2,bug_73541_allow_override_on_tg_deny_updates,bug_74461_bank_accounts_refactor,bug_89116_switch_debit_card_to_inheritance,bug_89116_switch_to_inheritance}	\N
bug_88704_collection_model	2010-02-18 20:27:29.925678-06	{bug_71117_mef,bug_120611_fixing_variable_constraint,bug_126479_fix_bug_120611_mef_variables,bug_92573_ast_override_model}	\N
bug_10122_update_clverify_reports_1	2008-11-20 21:04:34.276383-06	{bug_10122_add_columns_to_clv_reports}	\N
bug_10761__seed_sequences_information	2008-11-10 14:08:52.560919-06	{bug_10761__seed_sequences}	\N
bug_10158_have_deny_update_trigger_obey_check_enabled_setting	2008-11-14 01:34:35.79438-06	{bug_9563__checks_and_asserts,bug_9369__deny_trigger,bug_9689__disallow_triggers}	\N
bug_10421_convert_check_and_assert_to_plpgsql	2008-11-14 01:34:35.921741-06	{bug_10158_have_deny_update_trigger_obey_check_enabled_setting}	\N
bug_9998__functions	2010-03-04 23:36:43.09968-06	{bug_52488__textcoalesce,bug_50338_master_physical_payments_refactor_and_inheritance,bug_45122_payment_instruments.bank_accounts}	\N
bug_9998__anz_tables	2010-03-04 23:36:43.200725-06	{bug_9369__deny_trigger,bug_9998__functions}	\N
bug_8729_add_email_log_type_opt_out	2008-10-10 17:42:03.047672-05	\N	\N
nobug_6__test_assert	2008-10-16 18:50:48.539209-05	\N	\N
bug_10309_array_length	2008-10-16 18:50:48.68078-05	\N	\N
bug_7433__create_new_customer_permission_for_executive_role	2008-11-07 20:35:28.239055-06	\N	\N
bug_115648__add_new_activity_codes	2009-12-14 19:15:11.840631-06	\N	\N
bug_93090__fix_kill_own	2010-02-04 19:15:11.947839-06	\N	\N
bug_88704__dialer_view_refactorings	2010-02-18 20:27:33.606738-06	{bug_88704_collection_model}	\N
bug_89116_switch_to_inheritance	2010-03-04 23:36:57.408441-06	{bug_89116_switch_debit_card_to_inheritance,bug_9998__anz_tables}	\N
bug_59744_veritec_payment_plan_email_population	2010-03-10 18:24:36.260755-06	\N	\N
bug_138354_canada_equifax_eidcompare	2010-03-18 18:52:13.663118-05	\N	\N
bug_103639__ca_loan_finished_processing_email_template	2010-03-05 03:16:38.31451-06	\N	\N
bug_9718_update_dialer_views	2008-12-09 21:11:01.672291-06	\N	\N
bug_8967__functions	2008-10-30 19:48:08.072037-05	\N	\N
bug_10209_ownership_views	2008-11-07 20:35:17.474091-06	\N	\N
bug_6347_lead_providers_tables	2008-11-07 20:35:28.356057-06	\N	\N
bug_11324_change_ach_revoke_constraint	2008-12-05 00:14:55.81557-06	{bug_2198_installments_master}	\N
bug_9339__email_templates_population	2009-01-28 00:17:57.336094-06	{bug_9339__tables}	\N
bug_85662__add_new_note_types	2009-11-05 20:32:44.330829-06	\N	\N
bug_10604_allow_inserting_applied_patches_into_schema_patches	2008-10-30 19:48:08.197861-05	{bug_8686__fix_tg_schema_patches}	\N
bug_11759__employment_verifications	2009-03-05 19:12:09.637444-06	{bug_11879_code_tools}	\N
bug_10476_fix_uri_trim_position	2008-11-07 20:35:17.589205-06	{bug_4988__normalize_page_stats}	\N
bug_10253_fix_page_hits_fields	2008-10-23 19:35:25.810858-05	\N	\N
bug_10786_add_email_log_type	2008-12-16 17:13:01.614913-06	\N	\N
bug_150602__modify_lead_sale_buyers_table	2010-05-20 19:13:29.724282-05	{bug_74752__create_ping_tree_and_tracking_objects}	\N
bug_10517_track_approvals	2008-11-21 21:20:08.42561-06	{bug_10158_have_deny_update_trigger_obey_check_enabled_setting,bug_9369__deny_trigger}	\N
bug_9339__insert_faq_content	2009-01-28 00:17:57.510173-06	{bug_9339__tables}	\N
bug_88950_dialer_view_update	2009-10-22 18:43:58.135163-05	\N	\N
bug_2198__partial_rollback	2008-11-14 10:03:26.251788-06	{bug_2198__installment_loans}	\N
bug_9858_create_developer_group	2009-03-05 19:12:08.574284-06	\N	\N
bug_100052_add_pk_to_tools_tables	2009-11-05 20:34:05.614047-06	{bug_9562_db_config,bug_61726_new_seed_framework}	\N
bug_104614_8.3_session_assert_workaround	2009-11-05 20:34:16.679607-06	{bug_10631_revamp_asserts}	\N
bug_79197_change_loans_bank_authorization_revoked_is_only_for_installments_constraint_for_AU	2009-11-05 20:34:28.562798-06	{bug_2198__tables}	\N
bug_2198__update_loan_tasks_payment_transactions	2008-11-16 23:26:03.942919-06	{bug_2198__alter_loan_tasks_payment_transactions_phase_1}	\N
bug_2198__valuelist_additions	2008-11-14 01:35:14.711058-06	\N	\N
bug_2198__10pc_ach_discount	2008-11-14 01:35:14.904577-06	\N	\N
bug_10894__seo_columns	2009-01-28 00:17:57.634893-06	{bug_9339_jv_master,bug_11483_is_table_londiste_subscriber}	\N
bug_12076__db_doc_milestone_4	2009-02-26 21:08:41.887453-06	\N	\N
bug_47727_fix_patch_functions_again	2009-03-26 16:31:43.067706-05	{bug_11484_patch_preamble,bug_11879__framework,bug_11957_patch_preamble_using_wrong_search_path}	\N
bug_85857__renaming_referral_template_to_referral_notify	2009-10-22 18:43:58.346768-05	\N	\N
bug_10125__adding_ip_stats_table	2008-11-21 22:54:47.596362-06	\N	\N
bug_10126__adding_email_stats	2008-11-21 22:54:47.756875-06	\N	\N
bug_10122_add_columns_to_approvals	2008-11-21 22:54:47.87259-06	\N	\N
bug_10122_update_clverify_reports_2	2008-11-22 01:12:36.863341-06	{bug_10122_update_clverify_reports_1}	\N
bug_10122_update_clverify_reports_3	2008-11-22 01:12:38.734963-06	{bug_10122_update_clverify_reports_2}	\N
bug_10122_credit_model_v4_master	2008-11-22 01:12:39.045182-06	{bug_10122_add_columns_to_approvals,bug_10125__adding_ip_stats_table,bug_10126__adding_email_stats,bug_10517_track_approvals_and_sources_per_customer,bug_10122_update_clverify_reports_3}	\N
bug_11331__remove_sum_accounts_from_installments_view	2008-12-02 23:20:02.835382-06	{bug_2198_installments_master}	\N
bug_11209__sync_jv_to_us	2009-01-28 00:21:19.836577-06	{bug_9339__1_add_columns_and_objects,bug_9339__2_updates,bug_9339__3_not_null,bug_9339__email_templates_population,bug_9339__insert_faq_content,bug_9339__jv_roles,bug_9339_post_deploy,bug_9339__tables,bug_10122_credit_model_v4_master,bug_10421_convert_check_and_assert_to_plpgsql,bug_10517_fix_customer_sources_trigger_on_slaves,bug_11324_change_ach_revoke_constraint,bug_11331__fix_view_schema_issue,bug_11331__remove_sum_accounts_from_installments_view,bug_2198_installments_master,bug_9718_update_dialer_views,bug_10786_add_email_log_type,bug_6773_birthday_greetings_email_templates,bug_10424__sms_preferences_data,bug_11607__sms_reporting_access,bug_11372__re_add_collected_by_and_entered_by_checks,bug_10296__add_itemtype_call_times_to_seed_tables}	\N
bug_45322_fix_tg_schema_patches_for_83	2009-02-26 21:08:42.030762-06	{bug_10604_allow_inserting_applied_patches_into_schema_patches}	\N
bug_67520_valuelists_delete_au_twice_monthly_income_freq	2009-08-07 18:26:10.044011-05	\N	\N
bug_70791_email_templates_sms_message_codes_join_table	2009-09-11 00:47:33.330223-05	\N	\N
bug_95012_alter_blacklisted_banks_constraint	2009-10-22 18:43:58.472667-05	{bug_53087__add_new_columns_blacklisted_bank}	\N
bug_2198__tables	2008-11-14 01:35:30.304224-06	{bug_2198__functions,bug_9689__disallow_triggers}	\N
bug_2198__installment_loans	2008-11-14 01:51:02.800308-06	{bug_2198__tables,bug_10309_array_length,bug_2198__valuelist_additions,bug_2198__10pc_ach_discount}	\N
bug_11331__fix_view_schema_issue	2008-12-05 10:37:32.45627-06	{bug_11331__remove_sum_accounts_from_installments_view}	\N
bug_9064__londiste_drop_slave_trigger_perms_fix	2008-11-21 22:44:05.084526-06	{bug_9064__londiste_drop_slave_trigger}	\N
bug_7306__add_valuelist_data	2009-02-26 21:08:42.272361-06	{bug_11483_is_table_londiste_subscriber}	\N
bug_60198__fix_blacklisted_routing_number_check	2009-08-07 18:26:10.143832-05	{bug_44745__add_blacklisted_banks_table,bug_68477_fix_asserts_in_au}	\N
bug_60761__add_created_on_and_updated_on_to_sms_preferences	2009-09-11 00:47:33.708967-05	\N	\N
bug_11535_add_paid_off__in_default_installment_transition	2009-01-15 19:33:07.926238-06	{bug_11483_is_table_londiste_subscriber,bug_2198_10941__add_missing_transition_to_installments}	\N
bug_2198_10941__add_missing_transition_to_installments	2008-11-16 23:56:47.856251-06	{bug_2198__tables}	\N
bug_2198__fix_view_types	2008-11-16 23:56:47.973514-06	{bug_2198__alter_loan_tasks,bug_2198__alter_payment_transactions}	\N
bug_2198__valuelist_additions_part2	2008-11-16 23:56:50.383746-06	\N	\N
bug_103253_dialer_view_explicit_date_cast	2009-10-29 19:34:14.994728-05	\N	\N
bug_10517_fix_customer_sources_trigger_on_slaves	2008-11-22 02:13:19.391815-06	{bug_10517_track_approvals_and_sources_per_customer}	\N
bug_73388__change_au_twice_monthly_paydates	2009-08-07 18:26:10.280419-05	{bug_67520_valuelists_delete_au_twice_monthly_income_freq}	\N
bug_2198__londiiste-ify_triggers	2008-11-21 22:44:05.243569-06	{bug_2198__tables,bug_9064__londiste_drop_slave_trigger_perms_fix}	\N
bug_2198__seed_tables	2008-11-21 22:44:05.334331-06	{bug_2198__tables,bug_8967__functions,bug_10761__seed_sequences}	\N
bug_2198__redefine_installments_view	2008-11-21 22:44:05.428506-06	{bug_2198__installment_loans}	\N
bug_2198__drop_installment_unique_pt_index	2008-11-21 22:44:05.521152-06	{bug_2198__tables}	\N
bug_2198_installments_master	2008-11-21 22:44:05.703808-06	{bug_2198_10941__add_missing_transition_to_installments,bug_2198__seed_tables,bug_2198__valuelist_additions_part2,bug_2198__fix_view_types,bug_2198__drop_installment_unique_pt_index,bug_2198__londiiste-ify_triggers,bug_2198__redefine_installments_view}	\N
bug_10517_update_customer_sources_2	2008-11-21 22:45:07.397794-06	{bug_10517_update_customer_sources,bug_10517_update_customer_sources}	\N
bug_10517_update_approvals_2	2008-11-21 22:53:18.588708-06	{bug_10517_update_approvals}	\N
bug_149804_create_activity_codes	2010-03-10 18:24:36.381954-06	\N	\N
bug_73558_dialer_views_and_cuts	2009-08-07 18:26:10.35976-05	{bug_63461_dialer_views}	\N
bug_79121_add_ast_activity_code	2009-09-11 00:47:56.013801-05	\N	\N
bug_2198__alter_loan_tasks	2008-11-16 23:34:55.187139-06	{bug_2198__update_loan_tasks_payment_transactions}	\N
bug_2198__alter_payment_transactions	2008-11-16 23:35:07.319397-06	{bug_2198__update_loan_tasks_payment_transactions}	\N
bug_11484_patch_preamble	2008-12-18 20:40:37.627976-06	\N	\N
bug_10517_track_approvals_and_sources_per_customer	2008-11-21 22:53:21.598578-06	{bug_10517_update_approvals_2,bug_10517_update_customer_sources_2}	\N
bug_10424__sms_opt_in_tables	2008-12-18 20:40:37.745773-06	{bug_10158_have_deny_update_trigger_obey_check_enabled_setting}	\N
bug_48549_fix_hosed_view	2009-03-26 17:50:48.510684-05	{bug_47727_fix_patch_functions_again}	\N
bug_56906__valuelist_passport_country_cd_change	2009-08-07 18:26:10.699173-05	\N	\N
bug_11868__make_legal_changes_to_faq_page	2009-09-11 00:47:56.189079-05	\N	\N
bug_9980_patches_readable	2009-02-12 19:02:07.98542-06	\N	\N
bug_11483_is_table_londiste_subscriber	2008-12-18 20:40:38.064319-06	{bug_11484_patch_preamble}	\N
bug_52754__cancel_instl_task_permissions	2009-08-07 18:26:10.901344-05	\N	\N
bug_66828__implement_loan_rule_persistence	2009-09-11 00:47:56.275938-05	\N	\N
bug_10517_track_customer_sources	2008-11-21 21:20:09.675023-06	{bug_10158_have_deny_update_trigger_obey_check_enabled_setting,bug_9369__deny_trigger}	\N
bug_10517_update_approvals	2008-11-21 21:20:12.764557-06	{bug_10517_track_approvals}	\N
bug_10517_update_customer_sources	2008-11-21 21:20:13.819447-06	{bug_10517_track_customer_sources}	\N
bug_6773_birthday_greetings_email_templates	2008-12-18 20:40:38.148487-06	{bug_11483_is_table_londiste_subscriber}	\N
bug_9339__tables	2009-01-26 20:52:57.76859-06	{bug_8967__functions}	\N
bug_45353_pmt_methods_permissions	2009-03-05 20:50:07.082739-06	\N	\N
bug_45802_dialer_coll_90_view	2009-03-05 20:50:29.744244-06	\N	\N
bug_34140_drop_duplicate_leads_ssn_contraint	2009-08-07 18:26:33.010678-05	\N	\N
bug_84939_add_PK_to_email_templates_sms_message_codes	2009-09-11 00:47:56.603786-05	{bug_70791_email_templates_sms_message_codes_join_table}	\N
bug_11791__widen_seo_contents_index	2009-01-28 00:21:19.60514-06	{bug_10894__seo_columns}	\N
bug_11860_merge_jv_stable_to_i18n	2009-01-28 00:21:19.913365-06	{bug_11209__sync_jv_to_us,bug_11371__all_seo_modifications,bug_10894__seo_columns,bug_9657__update_lead_buyers_timeout,bug_11791__widen_seo_contents_index}	\N
bug_10424__sms_preferences_data	2008-12-18 20:40:38.260519-06	{bug_10424__sms_opt_in_tables}	\N
bug_9339__1_add_columns_and_objects	2009-01-26 20:52:58.216824-06	{bug_9339__tables,bug_9657__update_lead_buyers_timeout}	\N
bug_63505_birthday_greetings_au_email_log_type	2009-08-07 18:26:38.851078-05	\N	\N
bug_82793_update_dialer_views_wi_timezone	2009-09-11 00:47:56.702809-05	{bug_73558_dialer_views_and_cuts}	\N
bug_126666_callml_report_backfill	2010-03-18 18:52:13.973546-05	{bug_126666_uk_fraud_check_automation}	\N
bug_99374_add_sms_leads_email_log_type	2009-10-29 19:34:01.876191-05	\N	\N
bug_48063__add_new_activity_codes	2009-04-09 16:48:08.551816-05	\N	\N
bug_86162_opsqa_permissions	2009-11-05 20:32:50.838308-06	{bug_84950_grant_select_on_loan_rules_tables_to_ops_qa}	\N
bug_58105_add_columns_to_clv_reports	2009-08-07 18:26:38.960351-05	\N	\N
bug_11587__db_doc_milestone_1_customer_tables	2009-02-12 19:02:52.386689-06	\N	\N
bug_44745__add_blacklisted_banks_table	2009-04-09 16:48:08.411843-05	{bug_10631_revamp_asserts}	\N
bug_66894__replace_installment_payment_reminder_template	2009-08-07 18:26:39.144542-05	\N	\N
bug_44855__delete_qa_valuelists	2009-03-12 19:17:21.817525-05	\N	\N
bug_58423_valuelists_additional_collections_statuses	2009-08-07 18:26:39.289528-05	\N	\N
bug_10296__add_itemtype_call_times_to_seed_tables	2009-01-08 18:10:33.5138-06	{bug_8967__functions,bug_10761__seed_sequences}	\N
bug_40782__installment_principal_interest_coalesce_with_zero	2009-03-12 19:17:21.914055-05	{bug_2198_installments_master}	\N
bug_11879__framework	2009-03-05 19:12:08.676541-06	{bug_9858_create_developer_group}	\N
bug_11879__fix_seed_sequences	2009-03-05 19:12:09.361602-06	{bug_10761__seed_sequences}	\N
bug_11879_code_tools	2009-03-05 19:12:09.499913-06	{bug_11879__framework,bug_11879__fix_seed_sequences}	\N
bug_45052__add_two_new_activity_codes	2009-03-12 19:17:22.249626-05	\N	\N
bug_11957_patch_preamble_using_wrong_search_path	2009-01-27 23:37:05.984705-06	{bug_11484_patch_preamble}	\N
bug_71767_tables	2009-10-09 18:16:05.819132-05	\N	\N
bug_11607__sms_reporting_access	2009-01-08 18:13:13.738817-06	{bug_10424__sms_opt_in_tables,bug_11484_patch_preamble,bug_8967_seed_table_information}	\N
bug_45405_dialer_bp_60_view	2009-03-05 20:50:19.183619-06	\N	\N
bug_11372__re_add_collected_by_and_entered_by_checks	2009-01-08 19:12:01.913049-06	{bug_2198__fix_view_types,bug_11484_patch_preamble}	\N
bug_11697_fix_faqs_country_cd_index_name	2009-01-27 23:37:06.135689-06	{bug_11957_patch_preamble_using_wrong_search_path}	\N
bug_11371__all_seo_modifications	2009-01-28 00:09:40.009764-06	{bug_9339_jv_master,bug_11483_is_table_londiste_subscriber}	\N
bug_43653_state_contract_numbering	2009-04-14 15:51:38.276863-05	{bug_9562_db_config,bug_10631_revamp_asserts}	\N
bug_72219_tg_allow_on_condition	2009-09-24 19:08:17.626795-05	\N	\N
bug_83090_regclass_cast	2009-09-11 00:48:11.895783-05	\N	\N
bug_71073_fix_seed_objects	2009-09-11 00:48:12.170775-05	{bug_61726_new_seed_framework,bug_52712__remove_bad_tables_from_private,bug_83090_regclass_cast}	\N
bug_9339__2_updates	2009-01-27 23:37:17.468849-06	{bug_9339__1_add_columns_and_objects}	\N
bug_12069__db_doc_milestone_3	2009-02-26 21:08:42.153692-06	\N	\N
bug_10158__boolean_test_assert	2009-09-11 00:48:11.766169-05	{nobug_6__test_assert}	\N
bug_78560_clean_public_schema	2009-09-11 00:48:12.943509-05	{bug_71073_fix_seed_objects,bug_11697_add_lead_buyers_to_uk,bug_9799__au_dml,bug_38252__sms_message_codes_phase_2,bug_64193_add_test.assert}	\N
bug_74878_expand_iovation.reports.tracking_number	2009-08-11 18:29:42.770657-05	{bug_9667_iovation_reports}	\N
bug_78560_fix_public_func_refs	2009-09-11 00:48:34.646683-05	{bug_78560_clean_public_schema}	\N
bug_88519_mark_employees_roles_as_seed	2009-11-05 20:33:44.224424-06	{bug_71073_fix_seed_objects}	\N
bug_91156_fix_lead_sale_seed_fuckup	2009-11-05 20:33:56.414394-06	{bug_74752__master}	\N
bug_10460__employment_verification_work_items__roles__permissions	2009-03-12 19:17:22.022917-05	\N	\N
bug_9339__3_not_null	2009-01-27 23:59:14.109791-06	{bug_9339__2_updates,bug_11697_fix_faqs_country_cd_index_name}	\N
bug_9339__jv_roles	2009-01-27 23:59:18.13845-06	\N	\N
bug_9339_post_deploy	2009-01-27 23:59:18.211977-06	{bug_9339__3_not_null}	\N
bug_9339_jv_master	2009-01-27 23:59:18.501027-06	{bug_9339__jv_roles,bug_9339_post_deploy}	\N
bug_69338__couchdb_attachments_patch	2010-03-18 18:52:14.055211-05	\N	\N
bug_38051_dialer_views	2009-05-07 18:52:57.073428-05	\N	\N
bug_84950_grant_select_on_loan_rules_tables_to_ops_qa	2009-09-11 02:53:48.953408-05	{bug_66828__implement_loan_rule_persistence,bug_52488__textcoalesce}	\N
bug_10432_credit_reports_dnb	2009-06-24 03:58:04.162523-05	{bug_47727_fix_patch_functions_again}	\N
bug_10313_veda_credit_reports_and_vericheck_aml_reports	2009-06-24 03:58:04.981683-05	{bug_47727_fix_patch_functions_again}	\N
bug_50860__add_lead_stats	2009-05-07 19:10:27.589557-05	\N	\N
bug_11417__add_email_templates	2009-06-24 04:06:25.393917-05	{bug_9799__AU_brand}	\N
bug_10346__add_call_validate_reports_tables	2009-05-07 19:10:32.904613-05	\N	\N
bug_48700__lead_seller_versions_additional_columns	2009-05-18 14:50:40.072396-05	\N	\N
bug_107725__add_au_reports_to_mef_results	2009-12-03 16:19:32.947087-06	{bug_71117_mef}	\N
bug_53000__add_new_ast_activity_code	2009-05-21 20:29:08.977597-05	\N	\N
bug_10437_grant_public_access_to_test.assert	2009-09-11 00:48:11.998312-05	{bug_10158__boolean_test_assert}	\N
bug_71117_mef	2009-10-29 19:35:52.333252-05	{bug_10346__add_call_validate_reports_tables}	\N
bug_46636_create_new_activity_codes	2009-05-21 20:29:21.001784-05	\N	\N
bug_46636_delete_unusable_items_add_new_items	2009-05-21 20:29:21.186553-05	{bug_46636_create_new_activity_codes}	\N
bug_46636_add_activity_codes	2009-05-21 20:29:21.328387-05	{bug_46636_delete_unusable_items_add_new_items}	\N
bug_65776__add_loan_and_installment_id_to_email_log	2009-08-20 19:16:07.086997-05	\N	\N
bug_73433__change_biweekly_disp_to_fornightly	2009-08-20 19:16:08.837292-05	\N	\N
bug_46636_add_activity_codes_UK	2009-05-21 20:29:40.156961-05	{bug_46636_add_activity_codes}	\N
bug_74752__create_ping_tree_and_tracking_objects	2009-09-24 19:18:21.530142-05	{bug_72219_tg_allow_on_condition,bug_52488__textcoalesce}	\N
bug_74752__seed_data	2009-09-24 19:18:22.606361-05	{bug_74752__create_ping_tree_and_tracking_objects}	\N
bug_74752__master	2009-09-24 19:18:22.753319-05	{bug_74752__create_ping_tree_and_tracking_objects,bug_64193_trigger_sanity_check}	\N
bug_65776__update_email_log_view_definition	2009-08-20 19:16:08.990659-05	{bug_65776__add_loan_and_installment_id_to_email_log}	\N
bug_86513_change_compliant_to_complaint_activity_code	2009-09-24 19:18:22.41193-05	\N	\N
bug_65776__add_email_template_and_email_log_type	2009-08-20 19:16:09.947855-05	\N	\N
bug_75602__fix_AU_privacy_flags	2009-08-20 19:16:10.059976-05	\N	\N
bug_83189_add_bad_loans_to_clv	2009-09-24 19:18:22.856588-05	\N	\N
bug_74077_manager_workitems	2009-08-20 19:16:10.132895-05	\N	\N
bug_87061_clean_up_email_templat_sms_message_codes_indexes	2009-09-24 19:18:22.960828-05	{bug_84939_add_PK_to_email_templates_sms_message_codes,bug_71073_fix_seed_objects}	\N
bug_38252__sms_message_codes_phase_2	2009-08-20 19:16:10.336856-05	{bug_10424__sms_opt_in_tables}	\N
bug_74803_new_columns_for_veda_credit_client_reports_and_approvals	2009-09-24 19:18:23.183885-05	\N	\N
bug_66891_update_welcome_email_template	2009-08-20 19:16:10.425396-05	\N	\N
bug_60587__add_experian_reports_and_fico_score	2009-09-24 19:18:23.422719-05	\N	\N
bug_103854_dialer_view_fix_last_called_on	2009-10-30 11:45:30.640356-05	\N	\N
bug_56703__callcredit_additional_columns	2009-11-05 20:32:16.61903-06	\N	\N
bug_50860__redeux_function	2009-06-04 21:05:43.392507-05	{bug_50860__add_lead_stats}	\N
bug_53087__add_new_columns_blacklisted_bank	2009-10-01 16:31:34.254415-05	{bug_44745__add_blacklisted_banks_table}	\N
bug_91131_add_customer_id_to_page_hits	2009-11-05 20:33:33.659475-06	{bug_4988__normalize_page_stats,bug_10253_fix_page_hits_fields}	\N
bug_80453_rearranging_dialer_collections_ordering	2009-08-27 20:38:00.264161-05	\N	\N
bug_55377_contract_values_function	2009-06-04 21:05:49.597252-05	\N	\N
bug_74077_fix_gb_au_issue_types	2009-09-11 00:47:56.533774-05	{bug_74077_manager_workitems}	\N
bug_62473_add_new_activity_code	2009-08-27 20:38:02.393218-05	\N	\N
bug_11395_bank_authorization_revoked_on_checks	2009-06-04 21:05:57.462502-05	{bug_11324_change_ach_revoke_constraint,bug_10631_revamp_asserts}	\N
bug_155792_fix_excluded_tables	2010-03-18 18:52:14.174473-05	{bug_155792_add_inheritance}	\N
bug_66006__add_list_management_accept_flg_to_people_view	2009-11-05 20:32:22.343084-06	{bug_63461_dialer_views,bug_56830__add_list_management_accept_flg_to_people}	\N
bug_50323_add_email_templates	2009-06-04 21:07:28.518894-05	\N	\N
bug_50323_add_email_log_types	2009-06-04 21:07:28.668993-05	{bug_50323_add_email_templates}	\N
bug_11697_add_lead_buyers_to_uk	2009-07-23 19:57:01.058351-05	{bug_5221__add_lead_buyers,bug_11957_patch_preamble_using_wrong_search_path}	\N
bug_79235__fix_installments_update	2009-10-13 20:35:40.084143-05	\N	\N
bug_50860__backfill_lead_stats	2009-06-04 22:20:05.812867-05	{bug_50860__add_lead_stats}	\N
bug_52712__remove_bad_tables_from_private	2009-06-24 03:58:04.464891-05	\N	\N
bug_8101__add_callback_number_uk	1970-01-01 00:00:00-06	\N	\N
bug_9799__AU_brand	2009-06-24 03:58:05.113079-05	\N	\N
bug_91597__add_registration_email_templates	2009-10-19 22:07:13.536493-05	\N	\N
bug_9799__au_ddl	2009-06-24 03:58:06.802146-05	{bug_10313_veda_credit_reports_and_vericheck_aml_reports,bug_10432_credit_reports_dnb,bug_11879_code_tools,bug_52712__remove_bad_tables_from_private,bug_9799__AU_brand}	\N
bug_9799__country_data	2009-06-24 04:06:25.01349-05	{bug_9799__au_ddl}	\N
bug_9799__au_zips	2009-06-24 04:06:25.107327-05	{bug_9799__au_ddl}	\N
bug_9779__insert_faq_content	2009-06-24 04:06:25.277797-05	\N	\N
bug_9799__au_bank_info	2009-06-24 04:06:25.595219-05	{bug_9799__au_ddl}	\N
bug_9799__valuelist_additions	2009-06-24 04:06:25.769693-05	\N	\N
bug_186789__hyperbolic_trig_functions	2010-07-16 00:40:26.449859-05	\N	\N
bug_9799__au_dml	2009-06-24 04:06:26.281649-05	{bug_9799__au_ddl,bug_9799__AU_brand,bug_9799__valuelist_additions,bug_9799__country_data,bug_9799__AU_brand,bug_9779__insert_faq_content,bug_11417__add_email_templates,bug_9799__au_bank_info,bug_9799__au_zips}	\N
bug_39755_veritec_resign_contract_email_population	2009-06-24 04:30:09.10587-05	\N	\N
bug_49827_add_columns_to_customer_paydates	2009-06-24 04:30:09.187059-05	\N	\N
bug_92598_fix_lookup_table_seed_status	2009-10-09 18:16:06.340451-05	{bug_71073_fix_seed_objects,bug_71767_tables}	\N
bug_80219__sol_reporting_queue	2009-11-05 20:32:36.281638-06	\N	\N
bug_52488__textcoalesce	2009-09-11 02:53:48.840675-05	{bug_11879_code_tools}	\N
bug_64193_trigger_sanity_check	2009-09-24 19:08:17.822893-05	\N	\N
bug_64193_add_test.assert	2009-09-11 00:48:12.083574-05	{bug_10437_grant_public_access_to_test.assert}	\N
bug_58335__add_columns_to_veritec_transaction_log	2009-06-24 04:06:48.09985-05	\N	\N
bug_48309__add_loans_paid_off_date_column	2009-06-24 04:06:48.180884-05	{bug_10421_convert_check_and_assert_to_plpgsql}	\N
bug_56830__add_list_management_accept_flg_to_people	2009-06-24 04:06:48.299319-05	\N	\N
bug_91597__create_customer_registration_path_table	2009-10-19 22:07:33.356873-05	\N	\N
bug_63461_dialer_views	2009-06-25 19:34:02.961623-05	{bug_38051_dialer_views}	\N
bug_67731_switch_page_hits_queue_from_timeofday	2009-07-23 19:57:01.422101-05	{bug_4988__normalize_page_stats}	\N
bug_77931_add_sms_pending_email	2009-11-05 20:34:22.409716-06	\N	\N
bug_61076__fix_due_date_adjusted_issue	2009-07-09 23:59:11.145959-05	{bug_11331__fix_view_schema_issue}	\N
bug_47939__add_email_log_type__ach_daily_usbank_for_ncp_oh	2009-07-09 23:59:11.293004-05	\N	\N
bug_62555__add_debit_card_auth_logs_customer_id_ix	2009-07-09 23:59:11.390375-05	\N	\N
bug_50323_add_email_templates_with_clear	2009-07-09 23:59:11.517593-05	{bug_50323_add_email_templates}	\N
bug_93181_mark_seed_objects_as_seed	2009-10-20 12:20:25.65847-05	{bug_71073_fix_seed_objects}	\N
bug_60948__update_faqs_table	2009-07-23 19:57:01.229756-05	\N	\N
bug_59330_change_itemtype_call_times	2009-07-23 19:57:01.345615-05	\N	\N
bug_129753_remove_londiste_dependancy	2010-01-28 00:37:51.964863-06	{bug_89116_fix_inheritance_on_londiste}	\N
bug_122288__swishm_to_mef	2010-03-18 18:52:14.627408-05	{bug_71117_mef}	\N
bug_120611_fixing_variable_constraint	2010-02-18 20:27:28.60377-06	{bug_71117_mef}	\N
bug_11518__installments_collections_work_items__roles__permissions	2009-07-23 19:57:01.629055-05	\N	\N
bug_61726_new_seed_framework	2009-07-23 19:57:01.918284-05	{bug_11879__fix_seed_sequences}	\N
bug_68477_fix_asserts_in_au	2009-07-23 19:57:02.494345-05	\N	\N
bug_66424__installments_collections_issues_work_items__roles__permissions	2009-07-23 19:57:02.577488-05	\N	\N
bug_69890_fix_jacked_up_8101_patch_records	2009-07-23 19:57:02.677354-05	\N	\N
bug_68488_fix_cnuapp_search_path	2009-07-23 19:57:02.861053-05	{bug_9858_create_developer_group}	\N
bug_126341_au_mergeback_phase_3	2010-03-05 06:45:09.782605-06	{bug_133667_au_phase_3_early_fixes,bug_49812_migrate_bank_account_info,bug_126341_veritrac_permissions,bug_126341_valuelists_bank_account_type,bug_126341__drop_trigger_payment_instrument_status_history,bug_126341_verifications}	\N
bug_126479_fix_bug_120611_mef_variables	2010-02-18 20:27:29.818198-06	{bug_120611_fixing_variable_constraint}	\N
bug_139007_add_collection_pmt_set_to_permissions	2010-03-18 18:52:14.398321-05	\N	\N
bug_126898_seed_mef_variables_equations	2010-03-18 18:52:14.728238-05	{bug_71073_fix_seed_objects,bug_71117_mef}	\N
bug_138354_eidcompare_add_pk	2010-03-18 18:52:14.830105-05	{bug_138354_canada_equifax_eidcompare}	\N
bug_155075_mark_inheritance_as_seed	2010-03-12 18:56:35.816793-06	{bug_92598_fix_lookup_table_seed_status,bug_8967_seed_table_information,bug_2198_installments_master,bug_90845_mark_pgq_londiste_as_seed}	\N
bug_152386_indexes_for_bank_accounts	2010-03-05 08:05:15.98558-06	{bug_45122_payment_instruments.bank_accounts}	\N
bug_143265_add_late_bank_file_flg	2010-03-18 18:52:15.011765-05	\N	\N
bug_196522__generalize_override_lookup	2010-07-16 00:40:26.660816-05	\N	\N
bug_73541_allow_override_on_tg_deny_updates	2010-01-28 00:37:52.045373-06	{bug_10158_have_deny_update_trigger_obey_check_enabled_setting,bug_89116_switch_debit_card_to_inheritance,bug_9658__prevent_gov_law_state_cd_changes,bug_129753_remove_londiste_dependancy}	\N
bug_73403__valuelists_card_type_cd	2010-01-28 00:37:52.160967-06	\N	\N
bug_73403_au_mergeback_phase_2	2010-01-28 00:38:50.727387-06	{bug_73541_allow_override_on_tg_deny_updates,bug_62484_au_phase_2}	\N
bug_71767_functions	2010-01-28 00:37:29.068715-06	{bug_71767_tables,bug_76048_tools.currval,bug_79609_revamp_test_functions,bug_11483_is_table_londiste_subscriber}	\N
bug_68325_fix_size_of_physical_payments_amount	2010-03-04 23:36:45.62971-06	{bug_45122_payment_instruments.bank_accounts}	\N
bug_62485_au_phase_3	2010-03-04 23:36:57.83583-06	{bug_9799__master,bug_57513__unable_to_update_customer_bank_acct_hack,bug_68325_fix_size_of_physical_payments_amount,bug_89116_switch_to_inheritance}	\N
bug_71393_canada	2010-03-04 23:36:57.910513-06	{bug_78560_fix_public_func_refs,bug_62485_au_phase_3}	\N
bug_71393_canada_bank_data	2010-03-05 02:59:38.14613-06	{bug_71393_canada}	\N
bug_129898_active_customers	2010-03-24 23:22:32.729555-05	{bug_135948_import_production_schema}	\N
bug_135948_import_production_schema	2010-03-23 16:17:43.952296-05	{bug_125333_bi_production_schema}	\N
bug_76048_tools.currval	2010-01-28 00:36:09.534231-06	\N	\N
bug_89116_fix_inheritance_on_londiste	2010-01-28 00:37:44.330245-06	{bug_71767_functions}	\N
bug_93698__au_credit_model_v2_mef	2009-12-03 16:19:32.492359-06	{bug_71117_mef}	\N
bug_94266_add_new_note_activities_to_valuelists	2009-12-03 16:19:32.773551-06	\N	\N
bug_93698__rename_vericheck_columns	2009-12-03 16:19:33.132536-06	{bug_93698__au_credit_model_v2_mef}	\N
bug_74461_bank_accounts_refactor	2010-03-04 23:40:35.054806-06	{bug_9799__master,bug_57513__unable_to_update_customer_bank_acct_hack,bug_48262_new_payment_instruments_functionality,bug_71393_canada,bug_89116_switch_to_inheritance,bug_89116_fix_inheritance_on_londiste}	\N
bug_92901__insert_ca_collection_statuses_with_admeris	2010-03-05 03:16:38.504524-06	\N	\N
bug_91126_fix_canada_people_sin	2010-03-05 03:20:29.48877-06	{bug_71393_canada}	\N
bug_8928__add_new_column_to_loan_contracts	2010-03-18 18:52:14.534378-05	\N	\N
bug_78291_micr_check_index	2009-12-03 16:19:32.683313-06	\N	\N
bug_119588_allow_installment_paid_off_to_issued	2010-02-04 19:15:11.140883-06	{bug_2198__tables}	\N
bug_93698__au_credit_model_backfill_setup	2010-03-05 02:59:21.274727-06	{bug_93698__au_credit_model_v2_mef}	\N
bug_99730_extra_logger_data	2010-03-05 02:59:21.407491-06	\N	\N
bug_93698__au_credit_model_backfill	2010-03-05 02:59:47.790745-06	{bug_93698__au_credit_model_backfill_setup}	\N
bug_114978__new_collections_model	2010-02-04 19:15:11.232057-06	\N	\N
bug_125924_add_email_log_types	2010-02-04 19:15:12.031454-06	\N	\N
bug_52109__seed_data	2010-02-04 19:15:11.761935-06	{bug_91156_fix_lead_sale_seed_fuckup}	\N
bug_79573_valuelists_income_freq_reorder_gb	2010-02-09 17:38:09.313117-06	\N	\N
bug_146552_collection_model_removing_constraints	2010-02-25 19:09:47.976022-06	{bug_88704_collection_model,bug_88704__dialer_view_refactorings}	\N
bug_74461_valuelists_institution_number	2010-03-05 02:59:21.171469-06	\N	\N
bug_31978__move_faq_categories_out_of_valuelist	2009-11-19 22:53:17.718176-06	{bug_78560_fix_public_func_refs}	\N
bug_106596_veda_vericheck_aml_add_minimal_requirements_indicator_column	2010-03-10 18:24:36.104223-06	\N	\N
bug_125333_bi_production_schema	2010-03-23 16:17:18.32562-05	\N	\N
bug_116327_increasing_income_freq_type_cd_field	2010-03-05 02:59:21.509179-06	{bug_71117_mef}	\N
bug_107592_add_collection_fields_to_transunion_and_equifax_tables	2010-03-05 03:16:38.167233-06	\N	\N
bug_110557_add_status_fields_to_payment_plan	2010-02-18 20:27:32.166351-06	\N	\N
bug_126666_uk_fraud_check_automation	2010-03-18 18:52:13.080546-05	\N	\N
bug_104436_handle_empty_asserts_table	2009-11-19 22:53:39.549075-06	{bug_10631_revamp_asserts}	\N
bug_77655_pgq_object_monitor	2010-02-04 19:15:10.890672-06	{bug_78560_clean_public_schema,bug_52488__textcoalesce,bug_71767_tables,bug_71767_functions,bug_93180_fix_tools.tables_v,bug_80993_session_hash_tools,bug_90845_mark_pgq_londiste_as_seed}	\N
bug_199153__mef_project_harmony	2010-07-16 00:40:27.759347-05	{bug_71117_mef}	\N
bug_126341_veritrac_permissions	2010-03-05 06:06:03.635599-06	\N	\N
bug_126341_valuelists_bank_account_type	2010-03-05 06:06:03.722643-06	\N	\N
bug_126341_verifications	2010-03-05 06:06:03.822446-06	\N	\N
bug_10539_re-add_debit_card_triggers	2010-01-28 00:36:08.893855-06	\N	\N
bug_8761__functions	2010-01-28 00:36:09.327144-06	\N	\N
bug_79609_revamp_test_functions	2010-01-28 00:36:09.651344-06	{bug_47640_test_functions,bug_11879_code_tools}	\N
bug_8761__physical_payment_constraint_trigger	2010-01-28 00:36:09.752183-06	{bug_8761__functions}	\N
bug_71393_canada_email_templates	2010-03-05 02:59:37.723174-06	{bug_71393_canada}	\N
bug_123086_drop_old_sha_type	2010-03-05 03:02:25.946082-06	\N	\N
bug_121906__drop_trigger_bank_accounts__payment_instrument_status_history	2010-03-05 06:04:48.868051-06	{bug_89116_switch_debit_card_to_inheritance,bug_74461_bank_accounts_refactor}	\N
bug_126341__drop_trigger_payment_instrument_status_history	2010-03-05 06:04:48.955243-06	{bug_121906__drop_trigger_bank_accounts__payment_instrument_status_history,bug_133667_au_phase_3_early_fixes}	\N
bug_133667_au_phase_3_early_fixes	2010-03-05 03:16:11.982212-06	{bug_62485_au_phase_3,bug_73403_au_mergeback_phase_2,bug_74461_bank_accounts_refactor,bug_74461_valuelists_institution_number,bug_73541_allow_override_on_tg_deny_updates}	\N
bug_77655_londiste_drop_all_triggers	2010-02-04 19:15:11.383182-06	{bug_89116_fix_inheritance_on_londiste,bug_77655_pgq_object_monitor}	\N
bug_83411__restrict_access_to_drop_table	2010-02-04 19:15:11.597536-06	\N	\N
bug_40530__callcredit_additional_columns	2010-03-05 02:59:15.229581-06	\N	\N
bug_72282_faqs_for_canada	2010-03-05 02:59:37.987162-06	{bug_31978__londiste_post_patch}	\N
bug_32267_normalize_payment_instruments	2010-01-28 00:36:10.982205-06	{bug_11879_code_tools,bug_47727_fix_patch_functions_again,bug_10539_re-add_debit_card_triggers,bug_50365_inheritance_trigger_code,bug_10631_revamp_asserts,bug_78560_fix_public_func_refs,bug_64193_add_test.assert}	\N
bug_31978__londiste_post_patch	2009-11-19 22:59:07.955479-06	{bug_31978__move_faq_categories_out_of_valuelist}	\N
bug_45122_payment_instruments.bank_accounts	2010-03-04 23:36:42.320203-06	{bug_48262__master,bug_49159_code.id_automatic_update,bug_78560_fix_public_func_refs,bug_89116_switch_debit_card_to_inheritance}	\N
bug_40530__callcredit_backfill_setup	2010-03-05 02:59:21.028287-06	{bug_40530__callcredit_additional_columns}	\N
bug_49812_pre	2010-03-05 01:30:06.068714-06	{bug_74461_bank_accounts_refactor}	\N
bug_49812_migrate_bank_account_info	2010-03-05 06:04:49.1116-06	{bug_49812_pre}	\N
bug_40530__callcredit_backfill	2010-03-05 02:59:48.61308-06	{bug_40530__callcredit_backfill_setup}	\N
bug_71393_canada_roles_employees	2010-03-05 02:59:48.696045-06	\N	\N
bug_92573_ast_override_model	2010-01-14 18:33:07.609792-06	{bug_71117_mef,bug_93698__rename_vericheck_columns}	\N
bug_49159_code.id_automatic_update	2010-01-28 00:36:09.208784-06	{bug_11879_code_tools}	\N
bug_47640_test_functions	2010-01-28 00:36:09.438641-06	{bug_10309_array_length}	\N
bug_49158_status_history	2010-01-28 00:36:42.973221-06	{bug_11879_code_tools}	\N
bug_48262_new_payment_instruments_functionality	2010-01-28 00:36:43.135333-06	{bug_32267_normalize_payment_instruments,bug_10631_revamp_asserts,bug_52136__tg_match_field,bug_49158_status_history,bug_83090_regclass_cast}	\N
bug_48262__master	2010-01-28 00:37:44.417659-06	{bug_48262_new_payment_instruments_functionality}	\N
bug_57544_fix_verifications_field_sizes	2010-01-28 00:38:40.548744-06	\N	\N
bug_62484_au_phase_2	2010-01-28 00:38:50.651988-06	{bug_89116_switch_debit_card_to_inheritance,bug_57544_fix_verifications_field_sizes,bug_73403__valuelists_card_type_cd,bug_73541_allow_override_on_tg_deny_updates}	\N
bug_57513__unable_to_update_customer_bank_acct_hack	2010-03-04 23:36:45.549365-06	{bug_45122_payment_instruments.bank_accounts}	\N
bug_59904__au_bank_info_with_bank_name	2010-03-05 04:19:39.212759-06	{bug_9799__au_ddl}	\N
bug_110427_create_activity_codes	2009-11-19 22:54:22.696771-06	\N	\N
bug_59170__add_pk_to_countries	2010-03-05 02:59:15.32462-06	{bug_9799__au_ddl}	\N
bug_87077__update_no_news_flg	2010-03-05 02:59:47.701064-06	\N	\N
bug_72936__canada__zips	2010-03-05 02:59:48.783804-06	{bug_71393_canada}	\N
bug_59170__add_primary_keys	2010-03-05 03:16:38.677585-06	{bug_59170__add_pk_to_countries,bug_32267_normalize_payment_instruments}	\N
bug_50365_inheritance_trigger_code	2010-01-28 00:36:09.002093-06	{bug_11879_code_tools}	\N
bug_52136__tg_match_field	2010-01-28 00:36:09.844089-06	{bug_11879_code_tools,bug_52488__textcoalesce,bug_49159_code.id_automatic_update}	\N
bug_50338__functions	2010-01-28 00:36:43.060081-06	{bug_32267_normalize_payment_instruments,bug_10631_revamp_asserts,bug_8761__physical_payment_constraint_trigger}	\N
bug_50338_master_physical_payments_refactor_and_inheritance	2010-01-28 00:37:29.206866-06	{bug_50338__functions,bug_78560_fix_public_func_refs}	\N
bug_89116_switch_debit_card_to_inheritance	2010-01-28 00:37:44.549152-06	{bug_89116_fix_inheritance_on_londiste,bug_50338_master_physical_payments_refactor_and_inheritance,bug_48262__master}	\N
bug_90845_mark_pgq_londiste_as_seed	2010-02-04 19:15:10.148702-06	{bug_71073_fix_seed_objects,bug_92598_fix_lookup_table_seed_status}	\N
bug_110892__new_ping_tree	2009-12-10 20:04:51.920903-06	{bug_74752__seed_data,bug_91156_fix_lead_sale_seed_fuckup}	\N
bug_80993_session_hash_tools	2010-02-04 19:15:10.446665-06	\N	\N
bug_189618_step1-alter_table_pdf_statements	2010-07-16 00:45:58.304308-05	{bug_46317__master}	\N
bug_189618_step2-update_table_pdf_statements	2010-07-16 00:45:58.662295-05	{bug_189618_step1-alter_table_pdf_statements}	\N
bug_159863__remove_amex_from_cc_types_in_valuelist	2010-07-16 00:45:59.231507-05	{bug_7306__add_valuelist_data}	\N
bug_189618_step3-add-indexes-pdf_statements	2010-07-16 00:45:59.425062-05	{bug_189618_step2-update_table_pdf_statements}	\N
bug_175087_ppc_valuelist_fix	2010-07-16 00:46:00.13985-05	{bug_46317_ppc_cnu}	\N
bug_180271_ut_replication_tables	2010-07-16 00:46:00.529132-05	{bug_46317_replication_table}	\N
bug_93180_fix_tools.tables_v	2010-02-04 19:15:10.326967-06	{bug_71767_tables}	\N
bug_119012__add_columns_to_ip_stats	2010-02-04 19:15:11.500104-06	{bug_10125__adding_ip_stats_table}	\N
bug_9799__master	2010-03-04 23:36:57.759435-06	{bug_9799__au_dml,bug_9998__anz_tables}	\N
bug_104494_pass_through	2009-11-19 22:54:07.058989-06	{bug_71117_mef,bug_6347_lead_providers_tables,bug_64193_add_test.assert}	\N
bug_201635_drop_diagnostic_view	2010-07-16 00:45:47.574453-05	\N	\N
bug_46317_replication_table	2010-07-16 00:45:47.66161-05	\N	\N
bug_46317_replication	2010-07-16 00:45:48.946258-05	{bug_46317_replication_table}	\N
bug_46317__prereq	2010-07-16 00:45:49.066547-05	{bug_74461_bank_accounts_refactor,bug_126341_au_mergeback_phase_3,bug_48262_new_payment_instruments_functionality,bug_50338_master_physical_payments_refactor_and_inheritance,bug_61726_new_seed_framework,bug_99010_current_environment_refactored,bug_120940_fix_last_day_of_month,bug_45122_payment_instruments.bank_accounts,bug_63276_datetime_constraint,bug_9998__functions,bug_68325_fix_size_of_physical_payments_amount,bug_201635_drop_diagnostic_view}	\N
bug_46317_ppc_cnu	2010-07-16 00:45:49.147074-05	{bug_46317__prereq,bug_46317_replication}	\N
bug_46317__insert_faq_content	2010-07-16 00:45:55.741624-05	{bug_46317_ppc_cnu}	\N
bug_46317_credit_bureau_changes	2010-07-16 00:45:56.114079-05	{bug_46317_ppc_cnu}	\N
bug_46317_credit_bureau_file_statuses	2010-07-16 00:45:56.495213-05	{bug_46317_ppc_cnu}	\N
bug_46317_loc_tables	2010-07-16 00:45:56.96403-05	{bug_46317_ppc_cnu}	\N
bug_46317_non_financial_files_table	2010-07-16 00:45:57.550814-05	{bug_46317_ppc_cnu}	\N
bug_46317_paycards_null_customer_id	2010-07-16 00:45:57.665852-05	{bug_46317_ppc_cnu}	\N
bug_46317_tmp_data	2010-07-16 00:45:57.954693-05	{bug_46317_ppc_cnu}	\N
bug_46317_triggers	2010-07-16 00:45:58.044599-05	{bug_46317_ppc_cnu}	\N
bug_46317_valuelists_for_activity_list	2010-07-16 00:45:58.132469-05	{bug_46317_ppc_cnu}	\N
bug_46317__master	2010-07-16 00:45:58.224232-05	{bug_46317_ppc_cnu,bug_46317__insert_faq_content,bug_46317_paycards_null_customer_id,bug_46317_loc_tables,bug_46317_tmp_data,bug_46317_triggers,bug_46317_valuelists_for_activity_list,bug_143991_index_ppc_deposit_pending_lines_account_token,bug_46317_non_financial_files_table,bug_46317_credit_bureau_changes,bug_46317_credit_bureau_file_statuses}	\N
bug_77655_object_monitor_permissions	2010-07-16 00:45:58.467882-05	{bug_77655_pgq_object_monitor}	\N
bug_78060__insert_faq_content	2010-07-16 00:45:58.556169-05	{bug_46317_ppc_cnu}	\N
bug_88752_personal_data_tracker_view	2010-07-16 00:45:58.755565-05	{bug_77655_pgq_object_monitor,bug_46317_ppc_cnu,bug_93180_fix_tools.tables_v,bug_77655_object_monitor_permissions}	\N
bug_191434_paycard_lines_fixes	2010-07-16 00:45:59.528039-05	{bug_46317_ppc_cnu}	\N
bug_192630_ppc_lines_fixes	2010-07-16 00:45:59.616215-05	{bug_46317_ppc_cnu}	\N
bug_193474_card_act_changes	2010-07-16 00:45:59.715783-05	{bug_46317_ppc_cnu}	\N
bug_201721_enroll_date_pi_only	2010-07-16 00:45:59.836215-05	{bug_46317_ppc_cnu}	\N
bug_34300__valuelist_additions	2010-07-16 00:45:59.9329-05	{bug_11483_is_table_londiste_subscriber}	\N
bug_91687_payment_transactions_committed	2010-07-16 00:46:00.237348-05	\N	\N
bug_46317_remove_test_data	2010-07-16 00:46:00.624756-05	{bug_46317__master,bug_78060__insert_faq_content}	\N
bug_90542_valuelists_for_transaction_account	2010-07-16 00:46:00.946852-05	{bug_11483_is_table_londiste_subscriber}	\N
bug_120940_fix_last_day_of_month	2010-07-16 00:45:47.41054-05	\N	\N
bug_151192__pre_v5_database_changes	2010-06-15 19:34:08.092443-05	\N	\N
bug_131838_add_cash_doctors_lead_buyer_AU	2010-07-30 02:33:04.894285-05	\N	\N
bug_160533_valuelist_income_type_change	2010-06-15 19:27:12.645341-05	\N	\N
bug_150602__add_second_lead_flash_account_to_ping_tree	2010-05-20 19:13:34.974386-05	{bug_176151__new_ping_tree,bug_150602__modify_lead_sale_buyers_table}	\N
bug_172666__gbi_right_to_cure	2010-10-14 20:27:24.569563-05	\N	\N
bug_143991_index_ppc_deposit_pending_lines_account_token	2010-07-16 00:45:56.01504-05	{bug_46317_ppc_cnu}	\N
bug_266947__add_clarity_reports_table	2010-11-29 12:27:28.678615-06	\N	\N
bug_176463_service_user_number_2_backpopulate	2010-06-15 20:26:02.642391-05	{bug_176463_service_user_number_1_add_column}	\N
bug_176463_service_user_number_3_set_not_null	2010-06-15 20:26:02.74591-05	{bug_176463_service_user_number_2_backpopulate}	\N
bug_196652_update_ddis_londiste_trigger	2010-06-15 19:26:39.733809-05	{bug_176463_service_user_number_1_add_column}	\N
bug_181514_master	2010-06-15 19:26:39.884967-05	{bug_181514__backpopulate}	\N
bug_188675_loan_amount_on_mef_changes	2010-06-15 19:26:39.967897-05	{bug_174836_adding_selectors_to_mef}	\N
bug_166476__GBI_faqs	2010-06-15 19:27:12.466292-05	{bug_31978__londiste_post_patch}	\N
bug_166872__email_log__brand_id	2010-06-15 19:27:12.836621-05	{bug_181514_gbi_brand_awareness}	\N
bug_186275_personal_data_tracker_changes	2010-07-16 00:46:00.42017-05	{bug_77655_pgq_object_monitor,bug_46317_ppc_cnu,bug_93180_fix_tools.tables_v,bug_77655_object_monitor_permissions,bug_88752_personal_data_tracker_view}	\N
bug_217487_inheritance_londiste_trigger_refresh	2010-07-30 02:41:57.111094-05	{bug_208300_inheritance_reset_children}	\N
bug_181495_gbi_loan_amount_calc_mef	2010-09-30 22:57:31.096717-05	{bug_174836_adding_selectors_to_mef}	\N
bug_192387__tools__next_stats_date	2010-07-16 23:31:44.888446-05	{bug_151192__pre_v5_database_changes}	\N
bug_192387__ip_stats_raw_v	2010-07-16 23:31:44.989032-05	{bug_151192__pre_v5_database_changes,bug_192387__tools__next_stats_date}	\N
bug_192387__lead_stats_raw_v	2010-07-16 23:31:45.110539-05	{bug_151192__pre_v5_database_changes,bug_192387__tools__next_stats_date}	\N
bug_192387__email_stats_raw_v	2010-07-26 15:38:24.769851-05	{bug_151192__pre_v5_database_changes,bug_192387__tools__next_stats_date}	\N
bug_192387__clverify_backfill_cleanup_p3	2010-07-19 15:17:21.074122-05	{bug_192387__clverify_parse_p1}	\N
bug_209913_sparse__add_pk_constraints	2010-07-16 23:33:43.606886-05	\N	\N
bug_192387__datax_backfill_cleanup_p3	2010-07-19 15:17:35.025327-05	{bug_192387__datax_parse_p1}	\N
bug_192387__clverify_parse_p1	2010-07-16 23:53:06.904429-05	{bug_151192__pre_v5_database_changes}	\N
bug_192387__datax_parse_p1	2010-07-17 03:41:29.214055-05	{bug_151192__pre_v5_database_changes}	\N
bug_99018_add_rule_number_col_to_customer_paydates	2010-05-10 23:48:57.420648-05	\N	\N
bug_265574__unkill_people_customers_pre_gbi	2010-10-31 23:23:28.623392-05	{bug_190110_swap_customers_people_fields_step_2}	\N
bug_135270__lead_conversion_results	2010-07-30 02:33:06.521042-05	\N	\N
bug_137309_add_default_to_credit_score	2010-05-10 23:20:56.332871-05	{bug_137309_finalize}	\N
bug_226053_remove_forgotten_GBI_trigger_funcs	2010-08-18 02:10:34.257323-05	{bug_199977__customers_people__cleanup_p3,bug_190110_swap_customers_people_fields_step_1}	\N
bug_83411__restrict_access_to_drop_table_round_2	2010-07-30 03:08:34.632877-05	\N	\N
bug_226356_disallow_multiple_emails_per_person	2010-08-18 02:49:31.532191-05	{bug_190110_swap_customers_people_fields_step_2}	\N
bug_235606_notice_of_arrears_email_type	2010-09-30 23:01:03.058493-05	\N	\N
bug_245468_deploy_some_patches_to_sync_databases	2010-09-30 23:01:05.555471-05	{bug_132450__physical_payments_paid_status,bug_125346__insert_ca_collection_status_corporate_creditor,bug_82913_add_activity_code,bug_196231_fix_tools.ownership_types,bug_44677__sum_coalesce_case,bug_93698__au_credit_model_backfill_cleanup,release_2010-09-01,bug_40530__callcredit_backfill_cleanup,bug_11535_add_paid_off__in_default_installment_transition}	\N
bug_203204_add_customer_id_to_physical_payments_step_1	2010-07-30 02:59:02.253064-05	{bug_182165_add_person_id_to_payment_instruments_step_1,bug_217487_inheritance_londiste_trigger_refresh}	\N
bug_223674_fix_brand_id_on_page_hits	2010-08-18 02:49:31.632171-05	{bug_189992_add_brand_id_to_page_hits}	\N
bug_208300_inheritance_reset_children	2010-07-30 02:41:56.877656-05	{bug_89116_fix_inheritance_on_londiste}	\N
bug_246516_speed_up_tools.assert	2010-09-30 22:57:30.459569-05	{bug_9562_db_config,bug_90886_assert_exists_fast,bug_11879__framework}	\N
bug_99010_current_environment	2010-06-15 19:18:07.986527-05	{bug_90886_assert_exists_fast}	\N
bug_99010_current_environment_refactored	2010-06-15 19:18:08.129353-05	{bug_99010_current_environment}	\N
bug_117199_flag_installments_with_loan_task_generated	2010-04-08 21:21:31.831911-05	{bug_2198__tables,bug_48549_fix_hosed_view,bug_79235__fix_installments_update,bug_11331__fix_view_schema_issue}	\N
bug_160599_artiva_production_schema	2010-04-09 17:33:16.987276-05	\N	\N
bug_189310_brand__get_functions	2010-06-15 19:18:08.222321-05	{bug_9339__tables}	\N
bug_62249__add_loan_approvals	2010-06-15 19:27:12.935352-05	{bug_11879_code_tools}	\N
bug_182440_replace_can_call_now	2010-07-30 02:55:31.239534-05	\N	\N
bug_217447_fix_tools.next_stats_date	2010-07-27 22:00:39.665144-05	{bug_192387__tools__next_stats_date,bug_52488__textcoalesce}	\N
bug_181514_gbi_brand_awareness	2010-06-15 19:18:08.42645-05	{bug_163809__GBI_brand,bug_99010_current_environment_refactored,bug_189310_brand__get_functions}	\N
bug_131757__add_user_agent_and_referrer_to_page_hits	2010-04-21 23:37:43.118465-05	{bug_91131_add_customer_id_to_page_hits}	\N
bug_181514__fix_bug_on_incoming_brand_id	2010-06-15 19:18:09.129559-05	{bug_181514_gbi_brand_awareness}	\N
bug_201944_rename_typo_column_oec_statement	2010-09-09 21:10:45.126517-05	{bug_200308__add_oec_statements_table}	\N
bug_137309_finalize	2010-05-03 20:10:36.639011-05	{bug_137309__us_credit_model_v4_mef}	\N
bug_113856_add_apply_fee_credit_email_log_type	2010-04-21 23:37:48.195699-05	\N	\N
bug_217447_improve_ip_stats	2010-07-27 22:00:39.802381-05	{bug_192387__ip_stats_raw_v,bug_217447_fix_tools.next_stats_date}	\N
bug_137309__us_credit_model_v4_mef	2010-04-21 23:37:31.676018-05	{bug_71117_mef}	\N
bug_169670__v5_credit_model	2010-07-30 02:33:07.757834-05	{bug_71117_mef,bug_199153__mef_project_harmony}	\N
bug_132171_add_two_columns_to_people_table_and_update_people_table_flags	2010-04-21 23:37:37.496983-05	\N	\N
bug_169891_artiva_transaction_lines_data_columns	2010-04-21 23:37:53.427505-05	{bug_160599_artiva_production_schema}	\N
bug_176463_service_user_number_1_add_column	2010-06-15 19:18:09.21711-05	\N	\N
bug_135270__mef_inputs_additional_columns	2010-07-30 02:33:08.582162-05	{bug_93698__au_credit_model_v2_mef,bug_169670__v5_credit_model}	\N
bug_201944__add_future_oec_draws_table	2010-09-14 18:18:41.044157-05	{bug_200308__add_oec_statements_table}	\N
bug_163809__GBI_brand	2010-05-20 19:13:39.856958-05	\N	\N
bug_181514__backpopulate	2010-06-15 19:18:09.42074-05	{bug_181514__fix_bug_on_incoming_brand_id}	\N
release_gbimerge_2_master_first	2010-09-30 23:01:05.153266-05	{release_2010-09-08_2_master_first,bug_246516_speed_up_tools.assert}	\N
release_2010-09-08	2010-09-30 23:01:05.467939-05	{release_2010-09-08_2_master_first}	\N
release_2010-08-12	2010-08-18 02:49:32.57897-05	{bug_166870_employees_brands_join_table,bug_214868_speed_up_patch_get_patches,bug_220828_GBI_ONE_PATCH_TO_BIND_THEM}	\N
release_gbilaunch	2010-09-30 23:01:05.639772-05	{bug_181495_gbi_loan_amount_calc_mef,bug_229025_fix_customer_gbi_constraints,bug_235606_notice_of_arrears_email_type}	\N
release_gbimerge_1_slave_first	2010-09-30 23:01:05.728899-05	{release_gbimerge_2_master_first,release_2010-09-08_1_slave_first}	\N
release_gbimerge	2010-09-30 23:01:05.806085-05	{release_gbimerge_1_slave_first,release_2010-09-08,bug_137078_change_type_for_credit_reports_collection_balance_fields,bug_169670__v5_emergency_variable_change,bug_174099_remove_customers_RI_from_page_hits,bug_226053_remove_forgotten_GBI_trigger_funcs,release_gbilaunch,bug_236443_drop_ri_for_microoutages,bug_245468_deploy_some_patches_to_sync_databases}	\N
release_2010-09-08_1_slave_first	2010-09-30 22:57:30.371016-05	{bug_201944_MD_LOC_master,bug_201944__add_future_oec_draws_table,bug_201944__add_future_oec_draws_table2}	\N
release_2010-09-08_2_master_first	2010-09-30 22:57:30.916124-05	{bug_201944_rename_typo_column_oec_statement,release_2010-09-08_1_slave_first}	\N
release_2010-09-01	2010-09-30 22:57:31.007812-05	{bug_169670__v5_emergency_variable_change}	\N
bug_163907_normalized_logs_006__enable	2010-07-30 02:33:09.246646-05	{bug_163907_normalized_logs_005__indexes}	\N
bug_189994_fix_broken_londiste_trigger_on_brand_id_tables	2010-08-09 11:46:53.310489-05	{bug_189994__multi_brand_id_setup_p1_1,bug_181514__backpopulate}	\N
bug_229025_fix_customer_gbi_constraints	2010-09-30 22:57:31.791078-05	{bug_226356_disallow_multiple_emails_per_person,bug_220828_GBI_ONE_PATCH_TO_BIND_THEM,bug_246516_speed_up_tools.assert}	\N
bug_173160_handle_empty_user_agent	2010-04-22 20:33:00.06465-05	{bug_131757__add_user_agent_and_referrer_to_page_hits}	\N
bug_182165_add_person_id_to_payment_instruments_indexes	2010-08-18 02:48:08.418856-05	{bug_182165_add_person_id_to_payment_instruments_step_1}	\N
bug_189994__brand_id_backfill_cleanup_p3	2010-08-18 01:16:43.647773-05	{bug_189994__multi_brand_id_setup_p1_1,bug_189994_fix_broken_londiste_trigger_on_brand_id_tables,bug_189994_email_brand_forward_backfill_p1}	\N
bug_135270__customer_sources_additional_columns	2010-07-30 03:10:11.394332-05	{bug_10517_track_customer_sources,bug_192387__lead_stats_raw_v}	\N
bug_169247_responsible_lending	2010-06-29 20:45:53.813467-05	\N	\N
bug_150608__add_payment_gateway_reports_table	2010-04-08 21:21:44.368223-05	\N	\N
bug_133159_lookup_view	2010-05-10 23:49:16.581133-05	\N	\N
bug_170828_contract_values_function	2010-04-21 23:37:24.26215-05	{bug_55377_contract_values_function}	\N
bug_201944__add_future_oec_draws_table2	2010-09-14 22:42:12.365224-05	{bug_201944__add_future_oec_draws_table}	\N
bug_190110_swap_customers_people_fields_indexes	2010-08-18 01:36:04.613374-05	{bug_190110_swap_customers_people_fields_step_1}	\N
bug_163907_normalized_logs_000__create_logs_schema	2010-07-06 16:25:11.701328-05	\N	\N
bug_190100__customers_new_fields__backfill_setup_p1	2010-07-30 04:05:11.983903-05	{bug_190110_swap_customers_people_fields_step_1}	\N
bug_195643_add_missing_trigger_to_loans	2010-07-30 02:59:00.122435-05	{bug_7349__grace_period_status}	\N
bug_190108__payment_instruments_person_id__backfill_setup_p1	2010-07-30 04:05:31.577106-05	{bug_182165_add_person_id_to_payment_instruments_step_1}	\N
bug_182165_add_person_id_to_payment_instruments_step_1	2010-07-30 02:41:57.200464-05	{bug_126341__drop_trigger_payment_instrument_status_history,bug_217487_inheritance_londiste_trigger_refresh}	\N
bug_199977__customers_people__cleanup_p3	2010-08-18 01:02:31.255117-05	{bug_221417_swap_customers_people_fields_step_1_1,bug_181514_master}	\N
bug_190108__payment_instruments_person_id__backfill_cleanup_p3	2010-08-18 01:20:44.154928-05	{bug_190108__payment_instruments_person_id_setup_p1_1}	\N
bug_199977__customers_people__bfsetup_p1	2010-07-30 04:06:03.623826-05	{bug_190110_swap_customers_people_fields_step_1}	\N
bug_189994__brand_id_backfill_setup_p1	2010-07-30 03:17:19.673211-05	{bug_189994__gbi_brand_discounts}	\N
bug_189994__gbi_brand_discounts	2010-07-30 02:55:31.78932-05	{bug_181514_gbi_brand_awareness}	\N
bug_190110_swap_customers_people_fields_step_1	2010-07-30 02:55:32.499684-05	{bug_181514_gbi_brand_awareness}	\N
bug_177128__customers__unique_person_id_brand_id	2010-07-30 02:33:17.485349-05	{bug_163809__GBI_brand}	\N
bug_221975__backfill_physical_payments_customer_id_cleanup_p3	2010-08-18 01:16:45.228996-05	{bug_221975__backfill_physical_payments_customer_id_setup_p1,bug_222815_allow_disabling_old_checks_and_asserts}	\N
bug_182655__fill_ccr_proxy	2010-06-29 20:46:05.201534-05	{bug_135270__callcredit_additional_columns}	\N
bug_182655__update_ccr	2010-06-29 20:46:10.482997-05	{bug_182655__fill_ccr_proxy}	\N
bug_186633_mef_arbitrary_rules	2010-06-29 20:46:15.883924-05	{bug_71117_mef}	\N
bug_173468__loan_payment_email_template_update_for_FL	2010-06-29 20:45:59.938149-05	\N	\N
bug_217447_improve_email_stats	2010-07-27 22:20:37.373095-05	{bug_192387__email_stats_raw_v,bug_217447_fix_tools.next_stats_date}	\N
bug_217447_improve_lead_stats	2010-07-27 22:20:37.484352-05	{bug_192387__lead_stats_raw_v,bug_217447_fix_tools.next_stats_date}	\N
bug_178663__add_new_equation_category	2010-05-04 19:23:07.902303-05	{bug_71117_mef}	\N
bug_154277_uk_cust_serv_activity_codes	2010-04-08 21:21:49.25227-05	\N	\N
bug_174836_adding_selectors_to_mef	2010-05-20 19:13:44.454076-05	{bug_71117_mef,bug_178663__add_new_equation_category}	\N
bug_221975__backfill_physical_payments_customer_id_setup_p1	2010-08-10 18:03:54.416013-05	{bug_203204_add_customer_id_to_physical_payments_step_1}	\N
bug_169670__v5_emergency_variable_change	2010-08-31 14:01:54.918201-05	{bug_71117_mef,bug_199153__mef_project_harmony}	\N
bug_229706_staging_tables_bi_object_monitor	2010-10-03 23:31:01.634921-05	{bug_77655_pgq_object_monitor,bug_77655_londiste_drop_all_triggers}	\N
bug_245913_add_new_oec_workitems	2010-10-07 01:25:18.330367-05	\N	\N
bug_164190_paydate_improvements_inert_pre_patch	2010-10-12 17:23:58.396339-05	\N	\N
bug_163907_normalized_logs_008__fix_schema	2010-10-14 20:27:24.694575-05	\N	\N
bug_222815_allow_disabling_old_checks_and_asserts	2010-08-18 01:02:31.090881-05	{bug_10158_have_deny_update_trigger_obey_check_enabled_setting}	\N
bug_177122_ip_address_moving_average_tables	2010-07-27 17:00:35.680812-05	\N	\N
bug_169670__stats_permissions	2010-07-26 15:41:28.251709-05	{bug_151192__pre_v5_database_changes}	\N
bug_166529__change_loan_payment_and_loan_extenstion_email_templates	2010-10-14 20:26:31.956793-05	\N	\N
bug_164190_paydate_improvements_hot_patch	2010-10-14 22:28:20.559429-05	{bug_164190_paydate_improvements_inert_pre_patch}	\N
bug_217447_deal_with_days_with_no_data	2010-07-28 00:28:01.779584-05	{bug_217447_improve_email_stats,bug_217447_improve_ip_stats,bug_217447_improve_lead_stats}	\N
bug_166795_iovation_device_alias_index	2010-07-30 02:33:15.917028-05	{bug_9667__tables}	\N
bug_166799_customers_registration_ip_index	2010-07-30 02:33:16.480574-05	\N	\N
bug_172863__au_add_column_credit_model_results	2010-10-14 20:26:32.071891-05	\N	\N
bug_172863__inc_column_size_of_override_lookup	2010-10-14 20:26:32.277679-05	{bug_137309__us_credit_model_v4_mef}	\N
bug_181590_24x5_funding	2010-10-14 20:26:32.853506-05	\N	\N
bug_192387__clverify_parse_p1_1	2010-07-26 15:41:28.498592-05	{bug_151192__pre_v5_database_changes,bug_169670__stats_permissions}	\N
bug_192387__clverify_backfill_cleanup_p3_1	2010-07-26 23:09:11.452764-05	{bug_192387__clverify_parse_p1}	\N
bug_90886_session_hash_tools	2010-05-20 19:13:09.736018-05	{bug_80993_session_hash_tools}	\N
bug_90886_assert_exists_fast	2010-05-20 19:13:09.985457-05	{bug_80993_session_hash_tools,bug_10631_revamp_asserts,bug_90886_session_hash_tools}	\N
bug_122843__sol_eligibility_request	2010-05-26 12:08:48.659797-05	\N	\N
bug_192715__add_columns_mef_inputs	2010-10-14 20:26:32.958735-05	\N	\N
bug_192715__au_arbitrary_add_column_vv_aml	2010-10-14 20:26:33.063755-05	\N	\N
bug_11322_add_email_log_type	2010-06-15 19:26:39.62712-05	\N	\N
bug_48841_address_standardization	2010-05-27 15:01:26.039539-05	\N	\N
bug_220327__conversion_model_mef	2010-10-14 20:26:33.174079-05	{bug_71117_mef,bug_169670__v5_credit_model,bug_169670__v5_emergency_variable_change,bug_135270__mef_inputs_additional_columns,bug_135270__lead_conversion_results}	\N
bug_194744_fix_tools_activity	2010-09-03 17:51:23.927301-05	\N	\N
bug_220327__drop_duplicate_columns_mef_inputs	2010-10-14 20:22:57.865002-05	{bug_71117_mef,bug_169670__v5_credit_model,bug_135270__mef_inputs_additional_columns}	\N
bug_224777__backfill_framework	2010-10-14 20:27:22.994491-05	{bug_224777__backfill_generic,bug_224777__backfill_generic_functions}	\N
bug_232911_create_loans.payday_view	2010-10-14 20:27:23.081782-05	\N	\N
bug_236403__add_new_column_installments_static	2010-10-14 20:27:23.179793-05	{bug_2198__tables,bug_48549_fix_hosed_view,bug_79235__fix_installments_update,bug_11331__fix_view_schema_issue,bug_117199_flag_installments_with_loan_task_generated}	\N
bug_191999_gb_in_default_emails	2010-12-16 19:49:45.641136-06	{bug_172666__gbi_right_to_cure}	\N
bug_252483_fix_asserts_for_dump	2010-10-14 20:27:24.325748-05	{bug_10631_revamp_asserts,bug_104436_handle_empty_asserts_table,bug_246516_speed_up_tools.assert,bug_90886_session_hash_tools}	\N
bug_66855__fix_patch_get_patches_error_message	2010-08-06 23:15:20.711164-05	\N	\N
bug_257158__blinding_backfill_setup	2010-10-14 22:11:39.020733-05	{bug_224777__backfill_framework}	\N
bug_224777__backfill_generic	2010-10-14 20:26:31.28448-05	\N	\N
bug_224777__backfill_generic_functions	2010-10-14 20:26:31.691194-05	{bug_224777__backfill_generic}	\N
bug_222065_long_term_on_mef	2010-10-14 20:26:33.565179-05	{bug_174836_adding_selectors_to_mef}	\N
bug_223654_fix_iovation.reports.tracking_number	2010-10-14 20:26:43.153165-05	{bug_74878_expand_iovation.reports.tracking_number}	\N
bug_253361_no_dml_triggers	2010-10-14 20:27:24.473564-05	\N	\N
bug_217447_fix_views	2010-07-29 14:15:29.976854-05	{bug_217447_deal_with_days_with_no_data}	\N
bug_137480_create_credit_report_payday_collection_tables	2010-06-29 20:45:11.477658-05	\N	\N
bug_189994_email_brand_forward_backfill_p1	2010-08-11 23:09:42.399213-05	{bug_189994__multi_brand_id_setup_p1_1,bug_155075_mark_inheritance_as_seed}	\N
bug_192387__lead_stats_raw_v_v2	2010-07-26 15:38:38.362151-05	{bug_151192__pre_v5_database_changes,bug_192387__tools__next_stats_date}	\N
bug_201944__valuelist_additions	2010-09-08 23:15:38.638226-05	\N	\N
bug_201944_email_template_update	2010-09-08 23:15:38.844034-05	\N	\N
bug_195465__au_existing_model_mef_inputs	2010-10-28 18:16:45.872319-05	{bug_71117_mef,bug_244550__veda_vericheck_new_columns}	\N
release_2010-10-14_2_pre-code	2010-10-14 20:27:24.869785-05	{bug_172666__gbi_right_to_cure,bug_166529__change_loan_payment_and_loan_extenstion_email_templates,bug_236403__add_new_column_installments_static,bug_181590_24x5_funding,bug_253361_no_dml_triggers,bug_164190_paydate_improvements_inert_pre_patch,bug_172863__au_add_column_credit_model_results,bug_172863__inc_column_size_of_override_lookup,bug_192715__add_columns_mef_inputs,bug_192715__au_arbitrary_add_column_vv_aml,bug_223654_fix_iovation.reports.tracking_number,bug_232911_create_loans.payday_view,bug_220327__conversion_model_mef,bug_222065_long_term_on_mef,bug_224777__backfill_framework,bug_252483_fix_asserts_for_dump,release_2010-10-14_1_master_first,bug_163907_normalized_logs_008__fix_schema}	\N
bug_236443_drop_ri_for_microoutages	2010-09-08 23:20:52.200855-05	{bug_91131_add_customer_id_to_page_hits,bug_174099_remove_customers_RI_from_page_hits,bug_74752__create_ping_tree_and_tracking_objects}	\N
release_2010-10-14_3_post-code	2010-10-14 22:28:22.410295-05	{release_2010-10-14_2_pre-code,bug_164190_paydate_improvements_hot_patch}	\N
release_2010-10-14	2010-10-14 22:28:37.472313-05	{release_2010-10-14_3_post-code}	\N
release_2010-10-14_1_master_first	2010-10-14 20:22:58.069247-05	{bug_220327__drop_duplicate_columns_mef_inputs}	\N
bug_236091__seed_object__remove	2010-10-28 18:16:46.072429-05	{bug_74461_bank_accounts_refactor}	\N
bug_237329_rtc_for_mo	2010-10-28 18:16:46.351872-05	\N	\N
bug_223603__mef_inputs_au_loan_amount	2010-10-28 18:16:46.592733-05	{bug_71117_mef,bug_195465__au_existing_model_mef_inputs}	\N
bug_256312__veda_vericheck_aml_reports_backfill_cleanup	2010-10-28 18:16:46.809374-05	{bug_256312__veda_vericheck_aml_reports_backfill_setup}	\N
bug_257158__blinding_backfill_cleanup	2010-10-28 18:16:46.905646-05	{bug_257158__blinding_backfill_setup}	\N
bug_244550__veda_vericheck_new_columns	2010-10-27 17:22:17.890739-05	\N	\N
bug_262805_session_hash_tools	2010-10-28 18:16:46.453651-05	{bug_90886_session_hash_tools}	\N
bug_258975__au_existing_customers_aml_backfill_cleanup	2010-10-28 18:16:46.697444-05	{bug_258975__au_existing_customers_aml_backfill_setup}	\N
bug_258975__au_existing_customers_client_backfill_cleanup	2010-10-28 18:17:03.894284-05	{bug_258975__au_existing_customers_client_backfill_setup}	\N
bug_137480_master	2010-06-29 20:45:30.327572-05	{bug_137480_create_credit_report_payday_collection_tables}	\N
bug_201944_add_oec_workitems	2010-09-08 23:15:39.138048-05	\N	\N
bug_200308__add_oec_statements_table	2010-09-08 23:15:39.362833-05	\N	\N
bug_201944_modify_dialer_view	2010-09-08 23:16:04.310252-05	\N	\N
bug_201944_add_email_confirmed_on	2010-09-08 23:16:04.442093-05	{bug_169247_responsible_lending}	\N
release_2010-10-28	2010-10-28 18:17:04.211662-05	{release_2010-10-28_3_post_ruby_code}	\N
release_2010-10-28_2_backfill_cleanup	\N	{bug_256312__veda_vericheck_aml_reports_backfill_cleanup,bug_258975__au_existing_customers_aml_backfill_cleanup,bug_258975__au_existing_customers_client_backfill_cleanup,release_2010-10-28_1_backfill_setup,bug_257158__blinding_backfill_cleanup}	\N
release_2010-10-28_1_backfill_setup	2010-10-27 17:22:18.688236-05	{bug_256312__veda_vericheck_aml_reports_backfill_setup,bug_258975__au_existing_customers_aml_backfill_setup,bug_258975__au_existing_customers_client_backfill_setup}	\N
bug_196906_add_channel_to_bi_projections_1_truncate	2010-07-09 15:39:51.708338-05	{bug_135948_import_production_schema}	\N
bug_201944_MD_LOC_master	2010-09-08 23:16:04.574924-05	{bug_200308__add_oec_statements_table,bug_201944__valuelist_additions,bug_201944_add_email_confirmed_on,bug_201944_add_oec_workitems,bug_201944_email_template_update,bug_201944_modify_dialer_view}	\N
bug_196906_add_channel_to_bi_projections_2_structure	2010-07-09 16:08:38.869443-05	{bug_135948_import_production_schema,bug_196906_add_channel_to_bi_projections_1_truncate}	\N
bug_256312__veda_vericheck_aml_reports_backfill_setup	2010-10-27 17:22:18.271993-05	{bug_224777__backfill_framework,bug_192715__au_arbitrary_add_column_vv_aml}	\N
bug_258975__au_existing_customers_aml_backfill_setup	2010-10-27 17:22:18.449076-05	{bug_224777__backfill_framework,bug_192715__au_arbitrary_add_column_vv_aml,bug_244550__veda_vericheck_new_columns}	\N
bug_258975__au_existing_customers_client_backfill_setup	2010-10-27 17:22:18.553329-05	{bug_224777__backfill_framework,bug_192715__au_arbitrary_add_column_vv_aml,bug_244550__veda_vericheck_new_columns}	\N
bug_152460_adding_sampling_to_mef	2010-05-10 23:49:25.89442-05	{bug_71117_mef}	\N
bug_151831__add_new_columns_to_debit_card_lines_table	2010-06-29 20:45:38.624124-05	{bug_73403_au_mergeback_phase_2}	\N
bug_150602__backfill_lead_sale_buyers_table	2010-06-15 19:25:59.311194-05	{bug_176151__new_ping_tree,bug_150602__modify_lead_sale_buyers_table}	\N
bug_150602__add_constraints_to_lead_sale_buyers_table	2010-06-15 19:26:01.611215-05	{bug_176151__new_ping_tree,bug_150602__backfill_lead_sale_buyers_table}	\N
bug_135270__callcredit_additional_columns	2010-05-20 19:13:25.031557-05	\N	\N
bug_189992_add_brand_id_to_page_hits	2010-06-29 20:46:21.089407-05	{bug_131757__add_user_agent_and_referrer_to_page_hits,bug_181514_gbi_brand_awareness}	\N
bug_155316_create_request_parameter_tracking	2010-05-10 23:49:31.009557-05	{bug_9339__tables}	\N
bug_163907_normalized_logs_003__data_interval_functions	2010-07-06 16:25:42.595032-05	{bug_163907_normalized_logs_002__log_wh}	\N
bug_163907_normalized_logs_004__pure_sql_insert	2010-07-06 16:25:51.749541-05	{bug_163907_normalized_logs_003__data_interval_functions}	\N
bug_163907_normalized_logs_001__normalized_hash	2010-07-06 16:25:21.567727-05	{bug_163907_normalized_logs_000__create_logs_schema}	\N
bug_163907_normalized_logs_005__indexes	2010-07-06 16:26:02.032519-05	{bug_163907_normalized_logs_004__pure_sql_insert}	\N
bug_163907_normalized_logs_002__log_wh	2010-07-06 16:25:31.460594-05	{bug_163907_normalized_logs_001__normalized_hash}	\N
bug_174099_remove_customers_RI_from_page_hits	2010-09-08 23:18:10.660347-05	{bug_91131_add_customer_id_to_page_hits}	\N
bug_176151__new_ping_tree	2010-05-11 00:12:31.472095-05	{bug_74752__seed_data,bug_91156_fix_lead_sale_seed_fuckup,bug_110892__new_ping_tree}	\N
bug_172567_acquisition_cost_manual_load_table	2010-05-10 23:49:41.842968-05	{bug_135948_import_production_schema}	\N
bug_152460_removing_intermediate_results_from_mef	2010-05-11 00:16:48.787745-05	{bug_152460_adding_sampling_to_mef}	\N
bug_173171_calendar_manual_load_table	2010-05-10 23:49:54.380447-05	{bug_135948_import_production_schema}	\N
bug_169228__add_new_contract_partials	2010-06-29 20:45:43.679415-05	\N	\N
bug_179819_artiva_drop_constraint_on_ach_and_payment_lines	2010-05-10 23:51:38.307436-05	\N	\N
bug_63276_datetime_constraint	2010-05-20 19:13:10.44428-05	{bug_90886_assert_exists_fast}	\N
bug_125346__insert_ca_collection_status_corporate_creditor	2010-09-30 23:01:05.240706-05	\N	\N
bug_132450__physical_payments_paid_status	2010-09-30 23:01:05.337266-05	{bug_73403_au_mergeback_phase_2}	\N
bug_189992_gbi_view_brand_awareness	2010-06-29 20:46:26.323961-05	{bug_104494_pass_through,bug_181514_gbi_brand_awareness}	\N
bug_137078_change_type_for_credit_reports_collection_balance_fields	2010-09-15 10:39:28.506306-05	{bug_107592_add_collection_fields_to_transunion_and_equifax_tables}	\N
bug_163907_normalized_logs__create_logs_schema	\N	\N	\N
bug_144006_lock_waits_view	2010-11-19 17:08:16.387396-06	\N	\N
release_2010-10-28_2_main	2010-10-28 18:17:03.993311-05	{release_2010-10-28_1_backfill_setup,bug_195465__au_existing_model_mef_inputs,bug_223603__mef_inputs_au_loan_amount,bug_236091__seed_object__remove,bug_237329_rtc_for_mo,bug_244550__veda_vericheck_new_columns,bug_262805_session_hash_tools,bug_126341_verifications,bug_146552_collection_model_removing_constraints,bug_150602__add_constraints_to_lead_sale_buyers_table,bug_172666__gbi_right_to_cure,bug_173171_calendar_manual_load_table,bug_196652_update_ddis_londiste_trigger,bug_52109__seed_data,bug_59170__add_pk_to_countries,bug_59170__add_primary_keys,bug_8007__londiste_no_dml_trigger}	\N
release_2010-10-28_3_post_ruby_code	2010-10-28 18:17:04.081894-05	{release_2010-10-28_2_main,bug_256312__veda_vericheck_aml_reports_backfill_cleanup,bug_258975__au_existing_customers_aml_backfill_cleanup,bug_258975__au_existing_customers_client_backfill_cleanup,bug_257158__blinding_backfill_cleanup}	\N
bug_155349_veda_vericheck_aml_reports_permissions	2010-05-10 23:49:37.052022-05	{bug_10313_veda_credit_reports_and_vericheck_aml_reports}	\N
bug_202222_add_brand_id_to_page_hits_fix	2010-06-30 20:40:02.777821-05	{bug_189992_add_brand_id_to_page_hits,bug_173160_handle_empty_user_agent}	\N
bug_196231_fix_tools.ownership_types	2010-09-30 23:01:03.177002-05	{bug_10209_ownership_views}	\N
bug_169670__id_seq_permissions	2010-08-03 14:10:21.003524-05	{bug_151192__pre_v5_database_changes}	\N
bug_169228__valuelist_additions	2010-06-29 20:45:48.841454-05	\N	\N
bug_176212_mark_payment_instruments_business_entities_seed	2010-05-20 19:13:50.179659-05	{bug_71073_fix_seed_objects,bug_45122_payment_instruments.bank_accounts}	\N
bug_40530__callcredit_backfill_cleanup	2010-09-30 23:01:03.292027-05	{bug_40530__callcredit_backfill}	\N
bug_44677__sum_coalesce_case	2010-09-30 23:01:03.767786-05	{bug_11879_code_tools}	\N
bug_166870_employees_brands_join_table	2010-08-18 02:44:07.816402-05	{bug_189992_gbi_view_brand_awareness,bug_166870__fix_tg_maintain_brand_id}	\N
bug_181514_gbi_brand_awareness_step2	2010-08-18 02:49:31.758761-05	{bug_189994__brand_id_backfill_cleanup_p3,bug_166870_employees_brands_join_table,bug_189995_set_brand_not_null}	\N
bug_56703__callcredit_backfill_cleanup	\N	{bug_56703__callcredit_backfill}	\N
bug_56703__callcredit_backfill_setup	\N	{bug_56703__callcredit_additional_columns}	\N
bug_56703__callcredit_backfill	\N	{bug_56703__callcredit_backfill_setup}	\N
bug_166870__fix_tg_maintain_brand_id	2010-08-18 01:36:04.510608-05	{bug_181514_gbi_brand_awareness}	\N
bug_182165_add_person_id_to_payment_instruments_step_2	2010-08-18 01:37:54.900561-05	{bug_182165_add_person_id_to_payment_instruments_step_1,bug_48262__master,bug_74461_bank_accounts_refactor,bug_203204_add_customer_id_to_physical_payments_step_2,bug_9998__functions,bug_208300_inheritance_reset_children,bug_190108__payment_instruments_person_id__backfill_cleanup_p3,bug_186275_personal_data_tracker_changes}	\N
bug_190110_swap_customers_people_fields_step_2	2010-08-18 02:10:42.153352-05	{bug_190110_swap_customers_people_fields_step_1,bug_88704__dialer_view_refactorings,bug_151192__pre_v5_database_changes,bug_182165_add_person_id_to_payment_instruments_step_2,bug_192387__email_stats_raw_v,bug_199977__customers_people__cleanup_p3,bug_186275_personal_data_tracker_changes,bug_190110_swap_customers_people_fields_indexes,bug_226053_remove_forgotten_GBI_trigger_funcs}	\N
bug_189995_set_brand_not_null	2010-08-18 02:44:10.572765-05	{bug_189994__brand_id_backfill_cleanup_p3}	\N
bug_196633_drop_brand_customer_sources	2010-08-18 02:49:31.208972-05	{bug_189992_gbi_view_brand_awareness,bug_192387__lead_stats_raw_v}	\N
bug_214868_speed_up_patch_get_patches	2010-08-06 23:15:25.710785-05	{bug_66855__fix_patch_get_patches_error_message}	\N
bug_220559_gbi_backfill_master_p1	2010-08-06 23:15:25.83021-05	{bug_189994__multi_brand_id_setup_p1_1,bug_190108__payment_instruments_person_id_setup_p1_1,bug_199977__customers_people_setup_p1_1,bug_218229__customer_sources_incoming_brand_id_setup_p1,bug_219969__customer_identities_brand_id_setup_p1,bug_214868_speed_up_patch_get_patches,bug_221417_swap_customers_people_fields_step_1_1}	\N
bug_203204_add_customer_id_to_physical_payments_step_2	2010-08-18 01:36:03.71793-05	{bug_203204_add_customer_id_to_physical_payments_step_1,bug_221975__backfill_physical_payments_customer_id_cleanup_p3}	\N
bug_203204_add_customer_id_to_physical_payments_indexes	2010-08-18 02:49:31.852857-05	{bug_203204_add_customer_id_to_physical_payments_step_1}	\N
bug_220828_GBI_ONE_PATCH_TO_BIND_THEM	2010-08-18 02:49:32.440952-05	{bug_181514_gbi_brand_awareness_step2,bug_182165_add_person_id_to_payment_instruments_step_2,bug_189995_set_brand_not_null,bug_220559_gbi_backfill_master_p1,bug_203204_add_customer_id_to_physical_payments_step_2,bug_203204_add_customer_id_to_physical_payments_indexes,bug_166870_employees_brands_join_table,bug_196633_drop_brand_customer_sources,bug_166870__fix_tg_maintain_brand_id,bug_190110_swap_customers_people_fields_step_2,bug_220559_gbi_backfill_master_p3,bug_178663__add_new_equation_category,bug_182165_add_person_id_to_payment_instruments_indexes,bug_186633_mef_arbitrary_rules,bug_196522__generalize_override_lookup,bug_174836_adding_selectors_to_mef,bug_190110_swap_customers_people_fields_indexes,bug_223674_fix_brand_id_on_page_hits,bug_226356_disallow_multiple_emails_per_person}	\N
bug_82913_add_activity_code	2010-09-30 23:01:03.988345-05	{bug_46636_delete_unusable_items_add_new_items}	\N
bug_219969__customer_identities_brand_id_cleanup_p3	2010-08-18 01:16:44.964498-05	{bug_219969__customer_identities_brand_id_setup_p1}	\N
bug_218229__customer_sources_incoming_brand_id_cleanup_p3	2010-08-18 01:20:44.676778-05	{bug_218229__customer_sources_incoming_brand_id_setup_p1,bug_189994_fix_broken_londiste_trigger_on_brand_id_tables}	\N
bug_220559_gbi_backfill_master_p3	2010-08-18 01:20:44.889972-05	{bug_220559_gbi_backfill_master_p1,bug_190108__payment_instruments_person_id__backfill_cleanup_p3,bug_189994__brand_id_backfill_cleanup_p3,bug_199977__customers_people__cleanup_p3,bug_218229__customer_sources_incoming_brand_id_cleanup_p3,bug_219969__customer_identities_brand_id_cleanup_p3,bug_221975__backfill_physical_payments_customer_id_cleanup_p3}	\N
bug_93698__au_credit_model_backfill_cleanup	2010-09-30 23:01:04.237567-05	{bug_93698__au_credit_model_backfill}	\N
release_2010-09-09	2010-09-09 19:50:38.211705-05	{bug_166868_bi_monitor,bug_186133_outages_manual_load_table,bug_194744_fix_tools_activity,bug_197843_skytools_functions,bug_214868_speed_up_patch_get_patches}	\N
bug_197843_skytools_functions	2010-09-09 19:50:26.04659-05	\N	\N
bug_186133_outages_manual_load_table	2010-09-09 19:50:26.151602-05	{bug_135948_import_production_schema}	\N
bug_166868_bi_monitor	2010-09-09 19:50:36.44516-05	{bug_77655_pgq_object_monitor}	\N
bug_199977__customers_people_setup_p1_1	2010-08-06 23:15:20.831067-05	{bug_190110_swap_customers_people_fields_step_1,bug_181514__fix_bug_on_incoming_brand_id}	\N
bug_218698__add_brand_id_to_customer_identities	2010-08-06 23:15:21.09857-05	{bug_181514_gbi_brand_awareness}	\N
bug_218229__customer_sources_incoming_brand_id_setup_p1	2010-08-06 23:15:21.293979-05	{bug_181514_master,bug_181514__fix_bug_on_incoming_brand_id}	\N
bug_189994__multi_brand_id_setup_p1_1	2010-08-06 23:15:21.405411-05	{bug_189994__gbi_brand_discounts,bug_181514__fix_bug_on_incoming_brand_id}	\N
bug_219969__customer_identities_brand_id_setup_p1	2010-08-06 23:15:21.893493-05	{bug_218698__add_brand_id_to_customer_identities,bug_181514__fix_bug_on_incoming_brand_id}	\N
bug_221417_swap_customers_people_fields_step_1_1	2010-08-06 23:15:22.012775-05	{bug_199977__customers_people_setup_p1_1}	\N
bug_190108__payment_instruments_person_id_setup_p1_1	2010-08-06 23:15:22.297427-05	{bug_182165_add_person_id_to_payment_instruments_step_1}	\N
bug_264208__dollarsdirect_credit_guide_email	2010-12-16 19:49:46.449046-06	\N	\N
bug_264208__loan_use_responsible_lending	2010-12-16 19:49:51.199217-06	{bug_169247_responsible_lending}	\N
bug_264208__south_pacific_lenders_credit_guide_email	2010-12-16 19:49:59.652198-06	\N	\N
bug_281706_add_lock_version_to_installments_static	2010-12-16 19:57:10.172101-06	{bug_236403__add_new_column_installments_static}	\N
bug_284868__improve_uk_dialer_views	2010-12-16 19:57:10.643953-06	{bug_232881__ukdialer}	\N
bug_169725__modify_note_types	2010-11-19 17:08:18.314011-06	\N	\N
bug_172504__add_columns_mef_inputs	2010-11-19 17:08:18.496532-06	\N	\N
bug_172504__add_columns_mef_results	2010-11-19 17:08:18.594994-06	\N	\N
bug_163907_normalized_logs_007__views	2010-12-16 19:49:13.896104-06	\N	\N
bug_163907_normalized_logs_009__add_cnuapp_perms	2010-12-16 19:49:44.20287-06	{bug_163907_normalized_logs_000__create_logs_schema,bug_163907_normalized_logs_001__normalized_hash,bug_163907_normalized_logs_002__log_wh,bug_163907_normalized_logs_003__data_interval_functions,bug_163907_normalized_logs_004__pure_sql_insert,bug_163907_normalized_logs_005__indexes,bug_163907_normalized_logs_006__enable,bug_163907_normalized_logs_007__views,bug_163907_normalized_logs_008__fix_schema}	\N
bug_264208__ncp_lenders_credit_guide_email	2010-12-16 19:49:51.63175-06	\N	\N
bug_172504__add_columns_equifax_credit_report	2010-11-19 01:01:28.596956-06	\N	\N
bug_214289_create_ukppc_schema	2010-11-19 17:08:18.693043-06	\N	\N
bug_217447_master	2010-11-19 13:29:50.069692-06	{bug_217447_fix_views}	\N
bug_218550_ip_velocity_defaults_column_drops	2010-11-19 17:08:13.413432-06	{bug_177122_ip_address_moving_average_tables}	\N
bug_218550_ip_velocity_defaults_column_updates	2010-11-19 17:08:19.009358-06	{bug_177122_ip_address_moving_average_tables}	\N
bug_241970_shifts_for_mef	2010-11-19 17:08:13.844841-06	{bug_174836_adding_selectors_to_mef,bug_222065_long_term_on_mef,bug_199153__mef_project_harmony}	\N
bug_252306__add_ach_return_file_tables	2010-11-19 17:08:15.512265-06	\N	\N
bug_256445__update_welcome_email_template	2010-11-19 17:08:15.691751-06	\N	\N
bug_223301_refresh_londiste_on_lead_sale.buyers	2010-11-19 17:08:19.498799-06	{bug_150602__modify_lead_sale_buyers_table}	\N
bug_230258_USB_brand	2010-11-19 17:08:19.572441-06	\N	\N
bug_231147__add_new_column_parameter_tracking_request_parameters	2010-11-19 17:08:19.684595-06	{bug_155316_create_request_parameter_tracking}	\N
bug_232881__ukdialer	2010-11-19 17:08:58.528464-06	\N	\N
bug_233171__add_columns_mef_inputs	2010-11-19 17:08:58.91243-06	\N	\N
bug_233171__bank_name_inc_column_size	2010-11-19 17:08:58.992043-06	{bug_195465__au_existing_model_mef_inputs}	\N
bug_234014_fix_participation.arrangement_versions_id_seq_linkage	2010-11-19 17:14:09.391146-06	{bug_46317_ppc_cnu}	\N
bug_243812_v5_stats	2010-11-19 17:14:09.56488-06	{bug_232911_create_loans.payday_view,bug_265807_assert__delete,bug_246516_speed_up_tools.assert,bug_192387__email_stats_raw_v,bug_192387__ip_stats_raw_v,bug_192387__lead_stats_raw_v,bug_192387__lead_stats_raw_v_v2,bug_192387__tools__next_stats_date,bug_217447_deal_with_days_with_no_data,bug_217447_fix_tools.next_stats_date,bug_217447_improve_email_stats,bug_217447_improve_ip_stats,bug_217447_improve_lead_stats,bug_217447_master,bug_265694__code_index__create,bug_265433__requested_loan_amount_datatype,bug_241970_shifts_for_mef}	\N
bug_243877_bi_outages_reconcile_fix	2010-11-19 17:14:17.772066-06	{bug_186133_outages_manual_load_table}	\N
bug_248544__grant_select_on_credit_reports_payday_collections	2010-11-19 17:14:17.849124-06	{bug_137480_create_credit_report_payday_collection_tables}	\N
bug_266670_lead_data_change_velocity_based_declines	2010-12-16 22:33:34.672374-06	{bug_241970_shifts_for_mef}	\N
bug_233171__ca_credit_reports_add_columns	2010-11-19 01:01:28.801803-06	\N	\N
bug_273932_truncate_referrer_on_insert	2010-12-16 19:57:09.423881-06	\N	\N
bug_286001_remove_loans_lock_from_loan_tasks_pending__wipe_per_loan	2010-12-16 20:49:03.04868-06	{bug_2198__installment_loans}	\N
bug_260755__ca_new_customer_creditmodel_setup	2010-11-19 13:29:49.142311-06	{bug_172504__add_columns_equifax_credit_report,bug_265217__backfill_convenient_accessors,bug_264924__backfill_create_parameters}	\N
bug_265463__ca_existing_customers_efx_creditmodel_setup	2010-11-19 13:29:49.489498-06	{bug_233171__ca_credit_reports_add_columns,bug_265217__backfill_convenient_accessors,bug_264924__backfill_create_parameters}	\N
bug_265463__ca_existing_customers_tru_creditmodel_setup	2010-11-19 13:29:49.737523-06	{bug_233171__ca_credit_reports_add_columns,bug_265217__backfill_convenient_accessors,bug_264924__backfill_create_parameters}	\N
bug_265433__requested_loan_amount_datatype	2010-11-19 13:29:50.14793-06	{bug_196633_drop_brand_customer_sources,bug_220327__conversion_model_mef}	\N
bug_265694__code_index__create	2010-11-19 17:08:15.313045-06	\N	\N
bug_265807_assert__delete	2010-11-19 17:08:15.40816-06	{bug_252483_fix_asserts_for_dump,bug_10631_revamp_asserts}	\N
bug_259237_revoke_delete_in_fraud_ip_tracking_schema	2010-11-19 17:08:15.880162-06	{bug_177122_ip_address_moving_average_tables}	\N
bug_260755__ca_new_customer_creditmodel_cleanup	2010-11-19 17:08:15.953045-06	{bug_260755__ca_new_customer_creditmodel_setup}	\N
bug_265144_call_transfer_note	2010-11-19 17:08:16.471793-06	\N	\N
bug_265463__ca_existing_customers_efx_creditmodel_cleanup	2010-11-19 17:08:16.653688-06	{bug_265463__ca_existing_customers_efx_creditmodel_setup}	\N
bug_265463__ca_existing_customers_tru_creditmodel_cleanup	2010-11-19 17:08:17.078552-06	{bug_265463__ca_existing_customers_tru_creditmodel_setup}	\N
bug_265540__update_email_templates	2010-11-19 17:08:17.496387-06	\N	\N
bug_267220__installment_loan_transition_fix	2010-11-19 17:08:17.680041-06	{bug_236403__add_new_column_installments_static}	\N
bug_271411_reoke_update_from_brands	2010-11-19 17:08:17.808544-06	\N	\N
bug_260755__ca_new_customer_creditmodel_tru_cleanup	2010-11-19 17:08:17.897221-06	{bug_260755__ca_new_customer_creditmodel_tru_setup}	\N
bug_270473__virtual_paystubs_au	2010-12-16 23:16:49.460002-06	{bug_164190_paydate_improvements_inert_pre_patch}	\N
bug_279088_add_equation_type	2010-12-28 13:53:13.459199-06	{bug_241970_shifts_for_mef}	\N
release_2010-11-18_1_backfill_setup	2010-11-19 13:29:49.993027-06	{bug_260755__ca_new_customer_creditmodel_setup,bug_260755__ca_new_customer_creditmodel_tru_setup,bug_265463__ca_existing_customers_efx_creditmodel_setup,bug_265463__ca_existing_customers_tru_creditmodel_setup}	\N
release_2010-11-18_2_master_first	2010-11-19 17:14:17.922289-06	{bug_218550_ip_velocity_defaults_column_drops,release_2010-11-18_1_backfill_setup}	\N
bug_258938_rework_code_function	\N	{bug_111955_master,bug_74603_code_function_versioning}	1
bug_258938_code_internal_drop_object	\N	{bug_258938_rework_code_function,bug_236091__seed_object__remove,bug_11879__framework,bug_133159_lookup_view}	1
bug_214289a_correct_ukppc_schema_permissions	2011-03-18 13:20:59.620597-05	{bug_214289_create_ukppc_schema}	\N
bug_45659__db_doc_milestone_5	\N	\N	2
release_2010-11-18	2010-11-19 17:14:17.993882-06	{bug_169725__modify_note_types,bug_231147__add_new_column_parameter_tracking_request_parameters,bug_223301_refresh_londiste_on_lead_sale.buyers,bug_218550_ip_velocity_defaults_column_updates,bug_172504__add_columns_equifax_credit_report,bug_172504__add_columns_mef_inputs,bug_172504__add_columns_mef_results,bug_172504__add_columns_transunion_credit_report,bug_233171__add_columns_mef_inputs,bug_233171__bank_name_inc_column_size,bug_233171__ca_credit_reports_add_columns,bug_265574__unkill_people_customers_pre_gbi,bug_243812_v5_stats,bug_256445__update_welcome_email_template,bug_265540__update_email_templates,bug_230258_USB_brand,bug_214289_create_ukppc_schema,bug_229706_staging_tables_bi_object_monitor,bug_243877_bi_outages_reconcile_fix,bug_144006_lock_waits_view,bug_248544__grant_select_on_credit_reports_payday_collections,bug_265144_call_transfer_note,bug_259237_revoke_delete_in_fraud_ip_tracking_schema,bug_271411_reoke_update_from_brands,bug_234014_fix_participation.arrangement_versions_id_seq_linkage,bug_241970_shifts_for_mef,bug_232881__ukdialer,bug_265433__requested_loan_amount_datatype,bug_252306__add_ach_return_file_tables,bug_260755__ca_new_customer_creditmodel_cleanup,bug_260755__ca_new_customer_creditmodel_tru_cleanup,bug_265463__ca_existing_customers_efx_creditmodel_cleanup,bug_265463__ca_existing_customers_tru_creditmodel_cleanup,release_2010-11-18_2_master_first,bug_267220__installment_loan_transition_fix}	\N
bug_265217__backfill_convenient_accessors	2010-11-19 01:01:27.555614-06	{bug_224777__backfill_framework}	\N
bug_264924__backfill_create_parameters	2010-11-19 01:01:28.708312-06	{bug_224777__backfill_framework}	\N
bug_172504__add_columns_transunion_credit_report	2010-11-19 01:01:28.879919-06	\N	\N
bug_260755__ca_new_customer_creditmodel_tru_setup	2010-11-19 01:01:28.972911-06	{bug_172504__add_columns_transunion_credit_report,bug_265217__backfill_convenient_accessors,bug_264924__backfill_create_parameters}	\N
bug_270464__change_email_templates_to_not_use_default_WI	2010-12-09 14:54:11.238811-06	\N	\N
bug_270464_rtc_for_wi	2010-12-09 14:54:27.779975-06	\N	\N
bug_270464_add_variable_to_input	2010-12-09 14:54:38.050614-06	{bug_241970_shifts_for_mef}	\N
release_2010-12-28	2010-12-28 13:53:14.036231-06	{bug_279088_add_equation_type}	\N
release_2010-12-16	2010-12-16 23:17:54.563893-06	{bug_270473__virtual_paystubs_au,bug_264208__loan_use_responsible_lending,bug_264208__dollarsdirect_credit_guide_email,bug_264208__ncp_lenders_credit_guide_email,bug_264208__south_pacific_lenders_credit_guide_email,bug_281706_add_lock_version_to_installments_static,bug_191999_gb_in_default_emails,bug_270464_add_variable_to_input,bug_270464__change_email_templates_to_not_use_default_WI,bug_270464_rtc_for_wi,bug_266947__add_clarity_reports_table,bug_273932_truncate_referrer_on_insert,bug_266670_lead_data_change_velocity_based_declines,bug_284868__improve_uk_dialer_views,bug_163907_normalized_logs_009__add_cnuapp_perms}	\N
bug_109669_fixes_for_83	2011-03-18 13:21:01.23092-05	\N	\N
bug_11573__email_templates_update	\N	\N	2
bug_118910_LL_brand	\N	\N	2
bug_74603_code_function_versioning	2011-03-18 13:20:59.537165-05	{bug_11879_code_tools}	\N
bug_229622_convert_check_customer_to_after_trigger	\N	{bug_203204_add_customer_id_to_physical_payments_step_2,bug_220828_GBI_ONE_PATCH_TO_BIND_THEM}	2
bug_229622_post_gbi_rollout_fixes	\N	{bug_182165_add_person_id_to_payment_instruments_step_2,bug_203204_add_customer_id_to_physical_payments_step_2,bug_220828_GBI_ONE_PATCH_TO_BIND_THEM}	2
bug_233171__ca_backfill_queries	\N	\N	2
bug_4615__no_functions	\N	{bug_47727_fix_patch_functions_again}	2
bug_46594_schema_views	\N	\N	2
bug_57178_loan_tasks_commit_add_dating	\N	{bug_2198__installment_loans}	2
bug_7349__functions.dev	\N	\N	2
bug_7349__grace_period_status.dev	\N	{bug_7349__functions}	2
bug_71393_canada_loan_constraint	2011-03-18 13:21:17.218716-05	\N	2
bug_71393_master	2011-03-18 13:21:18.503767-05	{bug_71393_canada,bug_71393_canada_bank_data,bug_72936__canada__zips,bug_91126_fix_canada_people_sin,bug_71393_canada_email_templates,bug_71393_canada_loan_constraint,bug_72282_faqs_for_canada,bug_71393_canada_roles_employees,bug_99730_extra_logger_data}	2
bug_102920_fix_tools.string_or_array	2011-03-18 13:21:00.716986-05	{bug_79609_revamp_test_functions}	\N
bug_111955_test_a2	2011-03-18 13:21:03.176205-05	\N	\N
bug_111955_test_b	2011-03-18 13:21:03.238062-05	{bug_111955_test_a2,bug_111955_test_a}	\N
bug_115755_contract_values_function	2011-03-18 13:21:18.56536-05	{bug_170828_contract_values_function}	2
bug_145867__cnu.calculate_override_approval_amount	2011-03-18 13:21:16.734763-05	\N	2
bug_175025__tools_settings__mark_functions_stable	2011-03-18 13:21:00.83883-05	{bug_9562_db_config,bug_246516_speed_up_tools.assert}	\N
bug_172504__backfill_queries	\N	\N	2
bug_163939_remove_old_tables	2011-03-18 13:21:16.871366-05	\N	2
bug_178389_insert_faqs_jv	2011-03-18 13:21:18.631254-05	\N	2
bug_190110_fix_private.tg_update_new_customer_columns	\N	\N	2
bug_190100__customers_new_fields__cleanup_p3	2011-03-18 13:21:18.708534-05	{bug_190100__customers_new_fields__backfill_setup_p1}	2
bug_216059__proxynow_sanity	2011-03-18 13:21:02.429746-05	{bug_192387__tools__next_stats_date}	\N
bug_214645_missing_pi_changes_slow	\N	{bug_46317__master,bug_190110_swap_customers_people_fields_step_2,bug_243812_v5_stats,bug_111955_modify_patch_system}	2
bug_214868_speed_up_patch_get_patches_2	\N	{bug_214868_speed_up_patch_get_patches,bug_90886_session_hash_tools}	2
bug_217667__customer_email_history_backfill	\N	{bug_265574__unkill_people_customers_pre_gbi,bug_265217__backfill_convenient_accessors,bug_264924__backfill_create_parameters,bug_217667_rename_person_email_history}	2
bug_217667__customer_email_history_cleanup	\N	{bug_217667__customer_email_history_backfill}	2
bug_229681_modify_object_monitor_to_accept_max_number_of_events_to_process	\N	{bug_77655_pgq_object_monitor,bug_154842_om_pgq_defaults,bug_77655_object_monitor_permissions,bug_123060_object_monitor_authority_dw_role}	2
bug_253205__new_columns_credit_reports	2011-03-18 13:21:15.913214-05	{bug_56703__callcredit_additional_columns,bug_126666_uk_fraud_check_automation}	2
bug_253205__callcredit_reports_backfill1_setup	2011-03-18 13:21:16.016718-05	{bug_265217__backfill_convenient_accessors,bug_264924__backfill_create_parameters,bug_253205__new_columns_credit_reports}	2
bug_253205__callml_reports_backfill_setup	2011-03-18 13:21:16.124513-05	{bug_265217__backfill_convenient_accessors,bug_264924__backfill_create_parameters,bug_253205__new_columns_credit_reports}	2
bug_253205__callml_reports_backfill_cleanup	2011-03-18 13:21:16.527843-05	{bug_253205__callml_reports_backfill_setup}	2
bug_232881_ukdialer_perms	2011-03-18 13:21:16.599815-05	\N	2
bug_252301_add_early_debit_complain_flag	2011-03-18 13:21:16.801092-05	{bug_169247_responsible_lending}	2
bug_252444__leads_welcome_email_updates	2011-03-18 13:21:16.949704-05	\N	2
bug_253205__callcredit_reports_backfill1_cleanup	2011-03-18 13:21:17.069801-05	{bug_253205__callcredit_reports_backfill1_setup}	2
bug_270312_drop_unused_indexes_20101108	2011-03-18 13:20:59.796499-05	\N	\N
bug_273651_bi_add_channels_to_traditional_data_part1	2011-03-18 13:20:59.879963-05	{bug_135948_import_production_schema}	\N
bug_275335__bank_info__fps_eligible	2011-03-18 13:20:59.991857-05	{bug_181590_24x5_funding}	\N
bug_277073_us_email_templates_cleanup	2011-03-18 13:21:00.145561-05	{bug_277073_us_email_templates_cleanup_before}	\N
bug_278248_gbi_notice_of_default_sums_email	2011-03-18 13:21:00.214917-05	\N	\N
bug_278498_bi_raw_score_modifier_manual_load_table	2011-03-18 13:21:00.276126-05	{bug_135948_import_production_schema}	\N
bug_258847_create_tools_convenience_views	2011-03-18 13:21:01.161399-05	{bug_46317_replication_table}	\N
bug_261205__add_9global_logical_to_lead_sale	\N	{bug_150602__modify_lead_sale_buyers_table}	2
bug_273651_bi_add_channels_to_traditional_data_part2	\N	{bug_273651_bi_add_channels_to_traditional_data_part1}	2
bug_274050_fix_eff_time_trigger_in_shifts	\N	{bug_241970_shifts_for_mef}	2
bug_260633__data_changes_setup	2011-03-18 13:21:16.379833-05	{bug_265574__unkill_people_customers_pre_gbi,bug_265217__backfill_convenient_accessors,bug_264924__backfill_create_parameters}	2
bug_260633__data_changes_setup_2	2011-03-18 13:21:17.691594-05	{bug_265574__unkill_people_customers_pre_gbi,bug_260633__data_changes_setup,bug_265217__backfill_convenient_accessors,bug_264924__backfill_create_parameters}	2
bug_260633__data_changes_cleanup	2011-03-18 13:21:18.777913-05	{bug_260633__data_changes_setup}	2
bug_260633__data_changes_cleanup_2	2011-03-18 13:21:18.85479-05	{bug_260633__data_changes_setup_2}	2
bug_267180__add_partial_payment_receipt_email_template	2011-03-18 13:21:18.960719-05	\N	2
bug_267197_revoke_ownership_of_cnuapp_from_cnuapp_and_cnuapp_role	2011-03-18 13:21:19.050614-05	{bug_9858_create_developer_group}	2
bug_279179__gb_responsible_lending	2011-03-18 13:21:19.129969-05	{bug_48700__lead_seller_versions_additional_columns,bug_169247_responsible_lending,bug_264208__loan_use_responsible_lending}	2
bug_281081_add_customer_set_indicator	2011-03-18 13:21:19.250457-05	{bug_169247_responsible_lending,bug_71117_mef}	2
bug_282103_gbi_notice_of_debt_sale_email	2011-03-18 13:21:00.417349-05	\N	\N
bug_285887_ach__return_files__ach_return_lines__allow_update	2011-03-18 13:21:00.479729-05	{bug_252306__add_ach_return_file_tables,bug_252306__add_ach_return_file_tables}	\N
bug_287671_grant_select_on_stats_user_agent	2011-03-18 13:21:00.533735-05	\N	\N
bug_287679_virtual_paystubs_au_phase_2	2011-03-18 13:21:00.585489-05	{bug_270473__virtual_paystubs_au}	\N
bug_303673__modify_clarity_constraints	\N	{bug_303673__rename_clarity_reports_table}	2
bug_298809__tracking_paystub_records	2011-03-18 13:21:16.206716-05	{bug_287679_virtual_paystubs_au_phase_2}	2
bug_281986_create_credit_limit_history_table	2011-03-18 13:21:16.452125-05	{bug_232911_create_loans.payday_view}	2
bug_290133_drop_all_objects_in_public_schema_belonging_to_user	2011-03-18 13:21:16.6616-05	\N	2
bug_295582__add_columns_callcredit	2011-03-18 13:21:17.145225-05	\N	2
bug_295582__callcredit_reports_backfill_setup	2011-03-18 13:21:17.28601-05	{bug_295582__add_columns_callcredit,bug_265217__backfill_convenient_accessors,bug_264924__backfill_create_parameters}	2
bug_303673__rename_clarity_reports_table	2011-03-18 13:21:17.353335-05	{bug_266947__add_clarity_reports_table}	2
bug_303120_auto_adjust_due_date_email	2011-03-18 13:21:17.765766-05	\N	2
bug_303316_update_role_item_types_us	2011-03-18 13:21:17.832954-05	\N	2
bug_303673__new_clearbank_variables	2011-03-18 13:21:17.902923-05	{bug_303673__rename_clarity_reports_table}	2
bug_303763__add_columns_mef_inputs	2011-03-18 13:21:17.967982-05	{bug_266947__add_clarity_reports_table}	2
bug_307458__lead_provider_fees	2011-03-18 13:21:18.046835-05	\N	2
bug_307703_person_email_history__add_customer_id	2011-03-18 13:21:18.176256-05	{bug_190110_swap_customers_people_fields_step_2}	2
bug_309048_add_new_activity_code	2011-03-18 13:21:18.24592-05	\N	2
bug_281081_add_variables_to_mef_inputs	2011-03-18 13:21:19.326089-05	{bug_71117_mef}	2
bug_281986_credit_line_decrease_email	2011-03-18 13:21:19.401184-05	\N	2
bug_284792_zoo_tool_permissions_indexes	2011-03-18 13:21:19.468309-05	{bug_258847_create_tools_convenience_views}	2
bug_290847__uk_long_term_model_mef	2011-03-18 13:21:19.6258-05	{bug_270464_add_variable_to_input}	2
bug_291204_fix_bug_in_person_lead_denorm	2011-03-18 13:21:19.701086-05	{bug_266670_lead_data_change_velocity_based_declines}	2
bug_291639_bi_acquisition_cost_update_unique_key	2011-03-18 13:21:19.77762-05	{bug_172567_acquisition_cost_manual_load_table}	2
bug_295582__add_columns_mef_inputs	2011-03-18 13:21:19.846634-05	\N	2
bug_295582__add_new_mef_variable_type	2011-03-18 13:21:19.919091-05	{bug_196522__generalize_override_lookup}	2
bug_295582__callcredit_reports_backfill_cleanup	2011-03-18 13:21:19.986009-05	{bug_295582__callcredit_reports_backfill_setup}	2
bug_297093_fix__gbi_notice_of_default_sums_email	2011-03-18 13:21:20.054194-05	{bug_278248_gbi_notice_of_default_sums_email}	2
bug_298438_allow_custom_date_offset_for_stats_daily	2011-03-18 13:21:20.125289-05	{bug_243812_v5_stats}	2
bug_298619_ivr_requests	2011-03-18 13:21:20.20702-05	\N	2
bug_303105_add_columns_to__collections__last_collection_results	2011-03-18 13:21:20.312849-05	{bug_88704_collection_model}	2
bug_78560_prereq	\N	{bug_74077_fix_gb_au_issue_types,bug_70791_email_templates_sms_message_codes_join_table,bug_60761__add_created_on_and_updated_on_to_sms_preferences,bug_66828__implement_loan_rule_persistence,bug_11868__make_legal_changes_to_faq_page,bug_73433__change_biweekly_disp_to_fornightly,bug_75602__fix_AU_privacy_flags,bug_73558_dialer_views_and_cuts,bug_82793_update_dialer_views_wi_timezone,bug_79121_add_ast_activity_code,bug_50860__backfill_lead_stats,bug_78560_prereq2}	2
bug_78560_prereq2	\N	{bug_66894__replace_installment_payment_reminder_template,bug_53000__add_new_ast_activity_code,bug_46636_add_activity_codes,bug_67520_valuelists_delete_au_twice_monthly_income_freq,bug_65776__add_email_template_and_email_log_type,bug_80453_rearranging_dialer_collections_ordering,bug_63505_birthday_greetings_au_email_log_type,bug_10346__add_call_validate_reports_tables,bug_62473_add_new_activity_code,bug_58335__add_columns_to_veritec_transaction_log,bug_66891_update_welcome_email_template,bug_69890_fix_jacked_up_8101_patch_records,bug_11322_add_email_log_type,bug_78560_prereq3}	2
bug_78560_prereq3	\N	{bug_39755_veritec_resign_contract_email_population,bug_58105_add_columns_to_clv_reports,bug_47939__add_email_log_type__ach_daily_usbank_for_ncp_oh,bug_60948__update_faqs_table,bug_46636_add_activity_codes_UK}	2
master_to_slave	\N	\N	2
release_2011-01-13	2011-03-18 13:21:01.056884-05	{release_2010-12-16,release_2010-12-28,bug_214289a_correct_ukppc_schema_permissions,bug_203820_sitemap_typo,bug_278248_gbi_notice_of_default_sums_email,bug_282103_gbi_notice_of_debt_sale_email,bug_278498_bi_raw_score_modifier_manual_load_table,bug_270312_drop_unused_indexes_20101108,bug_273651_bi_add_channels_to_traditional_data_part1,bug_189235__unique_brand_code,bug_275335__bank_info__fps_eligible,bug_277073_us_email_templates_cleanup,bug_277073_us_email_templates_cleanup_before,bug_282045__add_new_equation_type,bug_287679_virtual_paystubs_au_phase_2,bug_287671_grant_select_on_stats_user_agent,bug_285887_ach__return_files__ach_return_lines__allow_update}	\N
release_2011-01-13_01_pre_code	\N	{release_2010-12-16,release_2010-12-28,bug_214289a_correct_ukppc_schema_permissions,bug_203820_sitemap_typo,bug_278248_gbi_notice_of_default_sums_email,bug_282103_gbi_notice_of_debt_sale_email,bug_278498_bi_raw_score_modifier_manual_load_table,bug_270312_drop_unused_indexes_20101108,bug_273651_bi_add_channels_to_traditional_data_part1,bug_189235__unique_brand_code,bug_275335__bank_info__fps_eligible,bug_277073_us_email_templates_cleanup_before,bug_282045__add_new_equation_type,bug_287679_virtual_paystubs_au_phase_2,bug_287671_grant_select_on_stats_user_agent,bug_285887_ach__return_files__ach_return_lines__allow_update}	2
release_2011-02-10_1_pre_cabar	2011-03-18 13:21:17.559866-05	{bug_253205__callcredit_reports_backfill1_cleanup,bug_253205__callml_reports_backfill_cleanup,bug_252444__leads_welcome_email_updates,bug_163939_remove_old_tables,bug_290133_drop_all_objects_in_public_schema_belonging_to_user,bug_298809__tracking_paystub_records,bug_145867__cnu.calculate_override_approval_amount,bug_252301_add_early_debit_complain_flag,bug_232881_ukdialer_perms}	2
release_2011-02-10_2_cabar	2011-03-18 13:21:18.31547-05	{release_2011-02-10_1_pre_cabar,bug_111955_master}	2
release_2011-03-08_1_master_first	2011-03-18 13:21:18.377087-05	{bug_303673__rename_clarity_reports_table}	2
release_2011-03-08	2011-03-18 13:41:13.870849-05	{bug_303673__new_clearbank_variables,bug_303763__add_columns_mef_inputs,release_2011-03-08_1_master_first}	2
release_2011-02-24	2011-03-18 13:41:13.931641-05	{bug_281986_create_credit_limit_history_table,bug_281986_credit_line_decrease_email,bug_297093_fix__gbi_notice_of_default_sums_email,bug_303316_update_role_item_types_us,bug_295582__add_columns_callcredit,bug_295582__callcredit_reports_backfill_setup,bug_290847__uk_long_term_model_mef,bug_307458__lead_provider_fees,bug_298619_ivr_requests,bug_265574__unkill_people_customers_pre_gbi,bug_260633__data_changes_setup,bug_260633__data_changes_setup_2,bug_315903_remove_errant_triggers}	2
release_2011-02-10	2011-03-18 13:41:14.030468-05	{release_2011-02-10_2_cabar}	2
release_2011-01-27	2011-03-18 13:41:14.327147-05	{bug_214289a_correct_ukppc_schema_permissions,bug_279179__gb_responsible_lending,bug_115755_contract_values_function,bug_123060_object_monitor_authority_dw_role,bug_132862_object_monitor_seed,bug_155053_restrict_updates_on_all_physical_payments,bug_166868_bi_monitor,bug_175025__tools_settings__mark_functions_stable,bug_236091__remove_payment_instruments_bank_account_statuses_from_seed,bug_253205__callcredit_reports_backfill1_setup,bug_253205__callml_reports_backfill_setup,bug_258847_create_tools_convenience_views,bug_267197_revoke_ownership_of_cnuapp_from_cnuapp_and_cnuapp_role,bug_284792_zoo_tool_permissions_indexes,bug_291639_bi_acquisition_cost_update_unique_key,bug_298438_allow_custom_date_offset_for_stats_daily}	2
bug_192459__exec_as	2011-03-18 13:21:00.965194-05	{bug_11879__fix_seed_sequences}	1
bug_192474__schema_as	2011-03-18 13:21:01.340467-05	{bug_192459__exec_as,bug_155075_mark_inheritance_as_seed,bug_48262_new_payment_instruments_functionality,bug_79609_revamp_test_functions}	1
bug_192852_change_code.lookup_view_to_single_parameter	2011-03-18 13:21:01.490527-05	{bug_133159_lookup_view,bug_89116_fix_inheritance_on_londiste}	1
bug_192860_code.role	2011-03-18 13:21:01.558087-05	{bug_192459__exec_as}	1
bug_111955__tables	2011-03-18 13:21:01.665551-05	{bug_111955__deps,bug_192860_code.role,bug_192852_change_code.lookup_view_to_single_parameter,bug_192474__schema_as,bug_90886_assert_exists_fast,bug_109669_fixes_for_83}	1
bug_111955_modify_patch_system	2011-03-18 13:21:02.487407-05	{bug_216059__proxynow_sanity,bug_111955__tables}	1
bug_111955_test_a	2011-03-18 13:21:03.106542-05	{bug_111955_modify_patch_system}	1
bug_277073_us_email_templates_cleanup_before	2011-03-18 13:20:59.470643-05	\N	\N
bug_189235__unique_brand_code	2011-03-18 13:20:59.674904-05	{bug_9339__tables}	\N
bug_203820_sitemap_typo	2011-03-18 13:20:59.736407-05	{bug_10894__seo_columns}	\N
bug_282045__add_new_equation_type	2011-03-18 13:21:00.352746-05	{bug_71117_mef}	\N
bug_74600_code.valid_transitions_deal_with_existing_statuses_table	2011-03-18 13:21:00.776341-05	{bug_74603_code_function_versioning}	\N
bug_236091__remove_payment_instruments_bank_account_statuses_from_seed	2011-03-18 13:21:00.894144-05	{bug_236091__seed_object__remove,bug_74461_bank_accounts_refactor}	\N
bug_328676_remove_old_roles_from_db_global	2011-03-18 13:41:14.390212-05	{bug_111955_master}	1
bug_111955_test_c	2011-03-18 13:21:03.298347-05	{bug_111955_test_b}	1
bug_111955_master	2011-03-18 13:21:03.358058-05	{bug_111955_test_c}	1
bug_286344_test_patch	2011-03-18 13:41:12.140341-05	\N	1
slave_to_master	\N	\N	2
bug_307686__backfill_upgrades	\N	{bug_264924__backfill_create_parameters,bug_265217__backfill_convenient_accessors}	1
release_2011-03-10_1_master_first	2011-03-18 13:21:18.440276-05	{bug_91602__drop_unused_columns_from_approvals}	2
bug_315784_mark_cabar_tables_seed	2011-03-18 13:41:12.230975-05	{bug_111955_master}	1
release_2011-03-10	2011-03-18 13:41:14.187105-05	{release_2011-03-10_1_master_first,bug_295582__callcredit_reports_backfill_cleanup,bug_260633__data_changes_cleanup,bug_260633__data_changes_cleanup_2,bug_286344_test_patch,bug_295582__add_columns_mef_inputs,bug_295582__add_new_mef_variable_type,bug_315903_remove_errant_triggers,bug_307703_person_email_history__add_customer_id,bug_315753_fix_pay_stubs_au_replication,bug_315784_mark_cabar_tables_seed,bug_307728_remove_bad_seo_url,bug_214289a_correct_ukppc_schema_permissions,bug_291204_fix_bug_in_person_lead_denorm,bug_178389_insert_faqs_jv,bug_281081_add_customer_set_indicator,bug_281081_add_variables_to_mef_inputs,bug_267180__add_partial_payment_receipt_email_template,bug_303120_auto_adjust_due_date_email,bug_303105_add_columns_to__collections__last_collection_results,bug_309048_add_new_activity_code,bug_48313_create_installment_status_history}	2
bug_111955__deps	2011-03-18 13:21:01.613052-05	{bug_78560_clean_public_schema,bug_258847_create_tools_convenience_views,bug_175025__tools_settings__mark_functions_stable,bug_236091__remove_payment_instruments_bank_account_statuses_from_seed,bug_77655_pgq_object_monitor,bug_196231_fix_tools.ownership_types,bug_102920_fix_tools.string_or_array,bug_74603_code_function_versioning,bug_74600_code.valid_transitions_deal_with_existing_statuses_table,release_2010-08-12,release_2010-09-01,release_2010-09-08,release_2010-09-08_1_slave_first,release_2010-09-08_2_master_first,release_2010-09-09,release_2010-10-14,release_2010-10-14_1_master_first,release_2010-10-14_2_pre-code,release_2010-10-14_3_post-code,release_2010-10-28,release_2010-10-28_1_backfill_setup,release_2010-10-28_2_main,release_2010-10-28_3_post_ruby_code,release_2010-11-18,release_2010-11-18_1_backfill_setup,release_2010-11-18_2_master_first,release_2010-12-16,release_gbilaunch,release_gbimerge,release_gbimerge_1_slave_first,release_gbimerge_2_master_first,release_2011-01-13}	1
bug_154842_om_pgq_defaults	\N	{bug_77655_pgq_object_monitor,bug_246516_speed_up_tools.assert,bug_243812_v5_stats}	2
bug_155053_restrict_updates_on_all_physical_payments	2011-03-18 13:41:12.397039-05	{bug_73541_allow_override_on_tg_deny_updates}	2
bug_217667_rename_person_email_history	\N	\N	2
bug_217811_drop_country_from_customers	\N	{bug_190110_swap_customers_people_fields_step_2}	2
bug_220253__iovation_to_mef	\N	{bug_71117_mef}	2
bug_220847_post_gbi	\N	{bug_217811_drop_country_from_customers,bug_214645_missing_pi_changes_slow}	2
bug_315088__add_dow_to_dialer_cut	\N	{bug_232881__ukdialer}	2
bug_315899_fix_pay_stub_history_triggers	2011-03-18 13:21:17.492209-05	{bug_287679_virtual_paystubs_au_phase_2,bug_111955_master,bug_315753_fix_pay_stubs_au_replication}	2
bug_315829_fix_loan_amount_history_trigger	2011-03-18 13:21:17.622778-05	{bug_281986_create_credit_limit_history_table,bug_111955_master}	2
bug_307728_remove_bad_seo_url	2011-03-18 13:41:12.61155-05	{bug_10894__seo_columns}	2
bug_315903_remove_errant_triggers	2011-03-18 13:41:12.685424-05	{bug_315829_fix_loan_amount_history_trigger,bug_315899_fix_pay_stub_history_triggers}	2
bug_315408__add_new_lead_buyer_milarco	2011-03-18 13:41:13.228721-05	{bug_74752__seed_data}	2
bug_52182_fix_documentation_comment__customer_sources___souce_type_cd	\N	{bug_12069__db_doc_milestone_3}	2
bug_57543__delete_faq_how_much_can_i_receive	\N	\N	2
bug_87419_insert_workitem_types_for_CA	\N	\N	2
bug_91602__drop_unused_columns_from_approvals	2011-03-18 13:21:17.417927-05	\N	2
bug_9540__dialer_bp_60_90_view	\N	\N	2
old_patches_050_075	2011-03-18 13:41:14.920804-05	{bug_50323_add_email_templates_with_clear,bug_50338_master_physical_payments_refactor_and_inheritance,bug_50860__backfill_lead_stats,bug_50860__redeux_function,bug_52109__seed_data,bug_52754__cancel_instl_task_permissions,bug_53000__add_new_ast_activity_code,bug_53087__add_new_columns_blacklisted_bank,bug_55377_contract_values_function,bug_56703__callcredit_additional_columns,bug_56830__add_list_management_accept_flg_to_people,bug_57513__unable_to_update_customer_bank_acct_hack,bug_57544_fix_verifications_field_sizes,bug_58105_add_columns_to_clv_reports,bug_58335__add_columns_to_veritec_transaction_log,bug_59170__add_primary_keys,bug_59330_change_itemtype_call_times,bug_59744_veritec_payment_plan_email_population,bug_59904__au_bank_info_with_bank_name,bug_60587__add_experian_reports_and_fico_score,bug_60761__add_created_on_and_updated_on_to_sms_preferences,bug_60948__update_faqs_table,bug_61076__fix_due_date_adjusted_issue,bug_61726_new_seed_framework,bug_62473_add_new_activity_code,bug_62484_au_phase_2,bug_62485_au_phase_3,bug_63276_datetime_constraint,bug_63461_dialer_views,bug_63505_birthday_greetings_au_email_log_type,bug_65776__add_email_template_and_email_log_type,bug_65776__update_email_log_view_definition,bug_66006__add_list_management_accept_flg_to_people_view,bug_66424__installments_collections_issues_work_items__roles__permissions,bug_66828__implement_loan_rule_persistence,bug_66855__fix_patch_get_patches_error_message,bug_66891_update_welcome_email_template,bug_66894__replace_installment_payment_reminder_template,bug_67731_switch_page_hits_queue_from_timeofday,bug_68325_fix_size_of_physical_payments_amount,bug_68477_fix_asserts_in_au,bug_68488_fix_cnuapp_search_path,bug_69338__couchdb_attachments_patch,bug_69890_fix_jacked_up_8101_patch_records,bug_70791_email_templates_sms_message_codes_join_table,bug_71073_fix_seed_objects,bug_71117_mef,bug_71393_canada,bug_71393_canada_bank_data,bug_71393_canada_email_templates,bug_71393_canada_roles_employees,bug_71393_master,bug_72282_faqs_for_canada,bug_72936__canada__zips,bug_73388__change_au_twice_monthly_paydates,bug_73403__valuelists_card_type_cd,bug_73403_au_mergeback_phase_2,bug_73433__change_biweekly_disp_to_fornightly,bug_73558_dialer_views_and_cuts,bug_74077_fix_gb_au_issue_types,bug_74077_manager_workitems,bug_74461_bank_accounts_refactor,bug_74461_valuelists_institution_number,bug_74752__create_ping_tree_and_tracking_objects,bug_74752__seed_data,bug_74803_new_columns_for_veda_credit_client_reports_and_approvals,bug_74878_expand_iovation.reports.tracking_number}	2
old_patches_075_100	2011-03-18 13:41:15.002537-05	{bug_75602__fix_AU_privacy_flags,bug_77931_add_sms_pending_email,bug_78060__insert_faq_content,bug_78291_micr_check_index,bug_78560_fix_public_func_refs,bug_79121_add_ast_activity_code,bug_79197_change_loans_bank_authorization_revoked_is_only_for_installments_constraint_for_AU,bug_79235__fix_installments_update,bug_79573_valuelists_income_freq_reorder_gb,bug_80219__sol_reporting_queue,bug_80453_rearranging_dialer_collections_ordering,bug_82793_update_dialer_views_wi_timezone,bug_82913_add_activity_code,bug_83189_add_bad_loans_to_clv,bug_83411__restrict_access_to_drop_table,bug_83411__restrict_access_to_drop_table_round_2,bug_84939_add_PK_to_email_templates_sms_message_codes,bug_84950_grant_select_on_loan_rules_tables_to_ops_qa,bug_85662__add_new_note_types,bug_85857__renaming_referral_template_to_referral_notify,bug_86162_opsqa_permissions,bug_86513_change_compliant_to_complaint_activity_code,bug_87077__update_no_news_flg,bug_88519_mark_employees_roles_as_seed,bug_88704__dialer_view_refactorings,bug_88704_collection_model,bug_88950_dialer_view_update,bug_89116_switch_to_inheritance,bug_90542_valuelists_for_transaction_account,bug_90845_mark_pgq_londiste_as_seed,bug_90886_assert_exists_fast,bug_91126_fix_canada_people_sin,bug_91131_add_customer_id_to_page_hits,bug_91156_fix_lead_sale_seed_fuckup,bug_91597__add_registration_email_templates,bug_91597__create_customer_registration_path_table,bug_91687_payment_transactions_committed,bug_92573_ast_override_model,bug_92598_fix_lookup_table_seed_status,bug_92901__insert_ca_collection_statuses_with_admeris,bug_93090__fix_kill_own,bug_93181_mark_seed_objects_as_seed,bug_93698__au_credit_model_backfill_cleanup,bug_93698__au_credit_model_v2_mef,bug_93698__rename_vericheck_columns,bug_94266_add_new_note_activities_to_valuelists,bug_95012_alter_blacklisted_banks_constraint,bug_99010_current_environment_refactored,bug_99018_add_rule_number_col_to_customer_paydates,bug_99374_add_sms_leads_email_log_type,bug_99730_extra_logger_data}	2
old_patches_150_	2011-03-18 13:41:14.262154-05	{bug_150602__add_constraints_to_lead_sale_buyers_table,bug_150602__add_second_lead_flash_account_to_ping_tree,bug_150602__backfill_lead_sale_buyers_table,bug_150602__modify_lead_sale_buyers_table,bug_150608__add_payment_gateway_reports_table,bug_152460_removing_intermediate_results_from_mef,bug_154277_uk_cust_serv_activity_codes,bug_155075_mark_inheritance_as_seed,bug_155316_create_request_parameter_tracking,bug_155349_veda_vericheck_aml_reports_permissions,bug_155792_add_inheritance,bug_159863__remove_amex_from_cc_types_in_valuelist,bug_160533_valuelist_income_type_change,bug_160599_artiva_production_schema,bug_163809__GBI_brand,bug_163907_normalized_logs_005__indexes,bug_163907_normalized_logs_006__enable,bug_166476__GBI_faqs,bug_166795_iovation_device_alias_index,bug_166799_customers_registration_ip_index,bug_166872__email_log__brand_id,bug_169228__add_new_contract_partials,bug_169228__valuelist_additions,bug_169247_responsible_lending,bug_169670__id_seq_permissions,bug_169670__stats_permissions,bug_169670__v5_credit_model,bug_169891_artiva_transaction_lines_data_columns,bug_170828_contract_values_function,bug_172567_acquisition_cost_manual_load_table,bug_173160_handle_empty_user_agent,bug_173171_calendar_manual_load_table,bug_175087_ppc_valuelist_fix,bug_176151__new_ping_tree,bug_176212_mark_payment_instruments_business_entities_seed,bug_176463_service_user_number_3_set_not_null,bug_177122_ip_address_moving_average_tables,bug_177128__customers__unique_person_id_brand_id,bug_179819_artiva_drop_constraint_on_ach_and_payment_lines,bug_180271_ut_replication_tables,bug_181514_gbi_brand_awareness,bug_181514_master,bug_182165_add_person_id_to_payment_instruments_step_1,bug_182440_replace_can_call_now,bug_186633_mef_arbitrary_rules,bug_186789__hyperbolic_trig_functions,bug_188675_loan_amount_on_mef_changes,bug_189618_step1-alter_table_pdf_statements,bug_189618_step2-update_table_pdf_statements,bug_189618_step3-add-indexes-pdf_statements,bug_189992_add_brand_id_to_page_hits,bug_189992_gbi_view_brand_awareness,bug_189994__brand_id_backfill_cleanup_p3,bug_189994__brand_id_backfill_setup_p1,bug_189994__gbi_brand_discounts,bug_190100__customers_new_fields__backfill_setup_p1,bug_190100__customers_new_fields__cleanup_p3,bug_190108__payment_instruments_person_id__backfill_cleanup_p3,bug_190108__payment_instruments_person_id__backfill_setup_p1,bug_190110_swap_customers_people_fields_step_1,bug_192387__clverify_backfill_cleanup_p3,bug_192387__clverify_backfill_cleanup_p3_1,bug_192387__datax_backfill_cleanup_p3,bug_192387__email_stats_raw_v,bug_192387__ip_stats_raw_v,bug_192387__lead_stats_raw_v,bug_192387__lead_stats_raw_v_v2,bug_193474_card_act_changes,bug_195643_add_missing_trigger_to_loans,bug_196231_fix_tools.ownership_types,bug_196652_update_ddis_londiste_trigger,bug_196906_add_channel_to_bi_projections_2_structure,bug_199153__mef_project_harmony,bug_199977__customers_people__bfsetup_p1,bug_199977__customers_people__cleanup_p3,bug_201635_drop_diagnostic_view,bug_201721_enroll_date_pi_only,bug_202222_add_brand_id_to_page_hits_fix,bug_203204_add_customer_id_to_physical_payments_step_1,bug_209913_sparse__add_pk_constraints}	2
bug_315753_fix_pay_stubs_au_replication	2011-03-18 13:21:16.306502-05	{bug_298809__tracking_paystub_records}	2
bug_132862_object_monitor_seed	2011-03-18 13:41:12.311159-05	{bug_77655_pgq_object_monitor}	2
bug_48313_create_installment_status_history	2011-03-18 13:41:12.829081-05	{bug_281706_add_lock_version_to_installments_static}	2
old_patches_100_150	2011-03-18 13:41:14.121379-05	{bug_100052_add_pk_to_tools_tables,bug_103253_dialer_view_explicit_date_cast,bug_103639__ca_loan_finished_processing_email_template,bug_103854_dialer_view_fix_last_called_on,bug_104436_handle_empty_asserts_table,bug_104494_pass_through,bug_104614_8.3_session_assert_workaround,bug_106596_veda_vericheck_aml_add_minimal_requirements_indicator_column,bug_107592_add_collection_fields_to_transunion_and_equifax_tables,bug_107725__add_au_reports_to_mef_results,bug_110427_create_activity_codes,bug_110557_add_status_fields_to_payment_plan,bug_110892__new_ping_tree,bug_113856_add_apply_fee_credit_email_log_type,bug_114978__new_collections_model,bug_115648__add_new_activity_codes,bug_116327_increasing_income_freq_type_cd_field,bug_117199_flag_installments_with_loan_task_generated,bug_119012__add_columns_to_ip_stats,bug_119588_allow_installment_paid_off_to_issued,bug_120611_fixing_variable_constraint,bug_121906__drop_trigger_bank_accounts__payment_instrument_status_history,bug_122288__swishm_to_mef,bug_123086_drop_old_sha_type,bug_125346__insert_ca_collection_status_corporate_creditor,bug_125924_add_email_log_types,bug_126341_au_mergeback_phase_3,bug_126479_fix_bug_120611_mef_variables,bug_126666_callml_report_backfill,bug_126666_uk_fraud_check_automation,bug_126898_seed_mef_variables_equations,bug_131757__add_user_agent_and_referrer_to_page_hits,bug_131838_add_cash_doctors_lead_buyer_AU,bug_132171_add_two_columns_to_people_table_and_update_people_table_flags,bug_132450__physical_payments_paid_status,bug_133159_lookup_view,bug_133667_au_phase_3_early_fixes,bug_135270__callcredit_additional_columns,bug_135270__customer_sources_additional_columns,bug_135270__lead_conversion_results,bug_135270__mef_inputs_additional_columns,bug_137309__us_credit_model_v4_mef,bug_137309_add_default_to_credit_score,bug_137309_finalize,bug_138354_eidcompare_add_pk,bug_139007_add_collection_pmt_set_to_permissions,bug_139035_add_ast_activity_code,bug_143265_add_late_bank_file_flg,bug_146552_collection_model_removing_constraints,bug_149804_create_activity_codes}	2
old_patches_000_050	2011-03-18 13:41:14.84248-05	{bug_4036_contract_values_function,bug_8007__londiste_no_dml_trigger,bug_8101__add_callback_number,bug_8928__add_new_column_to_loan_contracts,bug_9799__au_dml,bug_9799__master,bug_9980_patches_readable,bug_9998__anz_tables,bug_9998__functions,bug_10346__add_call_validate_reports_tables,bug_10460__employment_verification_work_items__roles__permissions,bug_11209__sync_jv_to_us,bug_11372__re_add_collected_by_and_entered_by_checks,bug_11395_bank_authorization_revoked_on_checks,bug_11518__installments_collections_work_items__roles__permissions,bug_11587__db_doc_milestone_1_customer_tables,bug_11697_add_lead_buyers_to_uk,bug_11697_fix_faqs_country_cd_index_name,bug_11759__employment_verifications,bug_11860_merge_jv_stable_to_i18n,bug_11868__make_legal_changes_to_faq_page,bug_12069__db_doc_milestone_3,bug_12076__db_doc_milestone_4,bug_31978__londiste_post_patch,bug_31978__move_faq_categories_out_of_valuelist,bug_34140_drop_duplicate_leads_ssn_contraint,bug_34300__valuelist_additions,bug_38051_dialer_views,bug_38252__sms_message_codes_phase_2,bug_39755_veritec_resign_contract_email_population,bug_40530__callcredit_additional_columns,bug_40530__callcredit_backfill,bug_40530__callcredit_backfill_setup,bug_40782__installment_principal_interest_coalesce_with_zero,bug_43653_state_contract_numbering,bug_44677__sum_coalesce_case,bug_44855__delete_qa_valuelists,bug_45052__add_two_new_activity_codes,bug_45122_payment_instruments.bank_accounts,bug_45353_pmt_methods_permissions,bug_45823_fix_installments_view_accounting,bug_46317__master,bug_46636_add_activity_codes_UK,bug_47640_test_functions,bug_47727_fix_patch_functions_again,bug_47939__add_email_log_type__ach_daily_usbank_for_ncp_oh,bug_48063__add_new_activity_codes,bug_48309__add_loans_paid_off_date_column,bug_48700__lead_seller_versions_additional_columns,bug_49827_add_columns_to_customer_paydates}	2
\.


--
-- Data for Name: seed_objects; Type: TABLE DATA; Schema: tools; Owner: db_global_owner
--

COPY seed_objects (schema_name, object_name, dump_order) FROM stdin;
tools	object_types	0
code_internal	object_class	0
code_internal	objects	0
code_internal	object_type	0
code_internal	templates	0
tools	seed_objects	0
cnu	blacklisted_bank_criteria	0
tools	tables	0
employment	commute_distances	-50
lead_sale	ping_tree_status_valid_transitions	0
tools	schema_patches	0
tools	asserts	-100
tools	settings	-90
cnu	email_templates_sms_message_codes	0
employment	employer_contact_types	-50
employment	phone_number_info	-50
employment	verification_responses	-50
employment	verification_urls	-50
cnu	blacklisted_banks_add_methods	-50
cnu	blacklisted_banks_actions	-50
inheritance	object_types	-50
lead_sale	ping_tree_statuses	-50
lead_sale	attempt_statuses	-50
cnu	veda_bankruptcy_discharge_statuses	-50
cnu	contract_partial_types	-50
cnu	contract_variance_reasons	-50
cnu	lead_seller_versions	0
cnu	lead_seller_tiers	0
cnu	sms_message_codes	0
cnu	installment_statuses	0
cnu	installment_valid_transitions	0
cnu	blacklisted_banks	0
cnu	roles	0
cnu	roles_permissions	0
cnu	lead_sellers	0
cnu	lead_seller_version_valid_transitions	0
cnu	email_log_type	0
cnu	building_societies	0
cnu	affiliate_links	0
cnu	bank_info	0
cnu	coupons	0
cnu	culture_first_names	0
cnu	culture_last_names	0
cnu	denorm_next_business_dates	0
cnu	discounts	0
cnu	email_templates	0
cnu	faqs	0
cnu	grace_period_valid_transitions	0
cnu	grace_period_status	0
cnu	images	0
cnu	itemtype_call_times	0
cnu	itemtypes	0
cnu	jobs	0
cnu	mw_document_types	0
cnu	mw_documents	0
cnu	mw_folders	0
cnu	mw_document_types_mappings	0
cnu	mw_mappings	0
cnu	mw_metas	0
cnu	mw_roles	0
cnu	mw_thumbnails	0
cnu	payment_plan__plan_types	0
cnu	permissions	0
cnu	pr_news	0
cnu	seo_contents	0
cnu	tables_valuelists	0
cnu	testimonials	0
cnu	valuelists	0
cnu	zips	0
cnu	brands	0
cnu	lead_seller_version_statuses	0
mef	equation_categories	-50
mef	equation_statuses	-50
mef	equation_status_valid_transitions	0
mef	equation_types	-50
mef	variable_types	-50
cnu	employees_roles	0
mef	variable_tags	0
cnu	faq_categories	0
cnu	pass_through_result_decision_types	-50
payment_instruments	payment_instrument_types	-50
payment_instruments	payment_instrument_statuses	-50
physical_payments	transfer_methods	-50
physical_payments	physical_payment_statuses	-50
pgq	queue	-100
londiste	provider_seq	0
londiste	subscriber_seq	0
londiste	subscriber_pending_fkeys	0
londiste	subscriber_pending_triggers	0
londiste	provider_table	0
londiste	subscriber_table	0
cnu	phone_types	-50
collections	collection_types	-50
payment_instruments	banking_regions	-50
payment_instruments	bank_account_types	-50
physical_payments_us	bundle_statuses	-50
physical_payments_us	direct_entry_acknowledgment_file_codes	-50
physical_payments_us	acknowledgment_codes	-50
physical_payments_us	direct_entry_acknowledgment_codes	-50
physical_payments_us	return_codes	-50
physical_payments_us	direct_entry_return_codes	-50
inheritance	child_objects	0
inheritance	child_objects_exclude_tables	0
cnu	eidcompare_reason_codes	-50
mef	variables	0
mef	equations	0
mef	equation_variables	0
mef	equation_variables_override_lookup	0
mef	equation_samples	0
mef	sample_rules	0
mef	equation_sample_variables	0
parameter_tracking	tracked_parameter_keys	-50
parameter_tracking	tracked_uris	-50
parameter_tracking	http_methods	-50
mef	selector_rules	0
payment_instruments	business_entities	0
cnu	service_user_numbers	-50
cnu	loan_events	-50
cnu	loan_proceed_uses	-50
cnu	customer_dependents	-50
replication	sets	-50
replication	schemas__exclude	-50
replication	tables__exclude	-50
cnu	loan_sources	-50
cnu	programs	-50
physical_payments_us	ppc_lines_transaction_types	-50
physical_payments_us	ppc_lines_business_errors	-50
ppc	bank_names	-50
ppc	paycard_line_statuses	-50
oec	future_oec_draw_statuses	-50
paydates	income_frequencies	-50
paydates	paydate_schedule_types	-50
ppc	enrollment_line_statuses	-50
ppc	replacement_line_statuses	-50
physical_payments_them	paycard_error_codes	-50
participation	participants	-50
participation	arrangement_versions	0
participation	breakdowns	0
ppc	deposit_types	-50
ppc	deposit_ignore_reasons	-50
ppc	deposit_pending_lines_statuses	-50
ppc	deposit_error_codes	-50
ppc	outgoing_file_types	-50
ppc	outgoing_file_statuses	-50
ppc	noaa_request_sources	-50
ppc	noaa_request_statuses	-50
credit_bureau	file_statuses	-50
loc	customer_account_types	-50
loc	account_types	-50
loc	customer_account_statuses	-50
loc	business_account_types	-50
loc	business_account_owners	-50
loc	business_account_statuses	-50
cnu	pdf_statement_sent_types	-50
fraud_ip_tracking	ip_segments	-50
fraud_ip_tracking	breach_types	-50
oec	oec_statement_statuses	-50
oec	statement_transaction_categories	-50
oec	statement_transaction_descriptions	-50
paydates	paydate_schedules	0
paydates	schedule_dates	0
logs	log_levels	0
shift	shift_definitions	0
shift	shift_values_history	0
mef	shift_category_mappings	0
logs	interval_dimensions	0
pay_stubs	pay_stub_submission_methods	-50
pay_stubs	pay_stub_verification_methods	-50
pay_stubs	pay_stub_environments	-50
cnu	customer_source_link_types	-50
cabar	components	0
cabar	component_schemas	0
cabar	env_settings	0
object_monitor	monitors	0
object_monitor	monitors_consumers	0
object_monitor	monitors_tables	0
\.


--
-- Data for Name: settings; Type: TABLE DATA; Schema: tools; Owner: db_global_owner
--

COPY settings (setting_name, b, f, i, t, iv) FROM stdin;
Asserts disabled	f	\N	\N	\N	\N
Minimum assert level	\N	\N	0	\N	\N
State Contract Numbering: Maximum Contracts Per Run	\N	\N	2000	\N	\N
disable assert exists check for true conditions	t	\N	\N	\N	\N
attachments.body_couchdb_url populate	\N	\N	11624508	\N	\N
normalize_logs	t	\N	\N	\N	\N
approvals backpopulate	\N	\N	31007150	\N	\N
Checks disabled	f	\N	\N	\N	\N
stats__daily buffer	\N	\N	\N	\N	00:10:00
\.


--
-- Data for Name: sub_asserts; Type: TABLE DATA; Schema: tools; Owner: db_global_owner
--

COPY sub_asserts (assert_name, sub_assert_name) FROM stdin;
\.


--
-- Data for Name: tables; Type: TABLE DATA; Schema: tools; Owner: db_global_owner
--

COPY tables (id, table_schema, table_name) FROM stdin;
\.


--
-- Data for Name: tests; Type: TABLE DATA; Schema: tools; Owner: db_global_owner
--

COPY tests (test_name, completed, dependencies) FROM stdin;
\.


SET search_path = cabar, pg_catalog;

--
-- Name: component_schemas__component__schema_name; Type: CONSTRAINT; Schema: cabar; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY component_schemas
    ADD CONSTRAINT component_schemas__component__schema_name UNIQUE (component_id, schema_name);


--
-- Name: components__pk_id; Type: CONSTRAINT; Schema: cabar; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY components
    ADD CONSTRAINT components__pk_id PRIMARY KEY (id);


--
-- Name: env_settings__pk_id; Type: CONSTRAINT; Schema: cabar; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY env_settings
    ADD CONSTRAINT env_settings__pk_id PRIMARY KEY (id);


SET search_path = code_internal, pg_catalog;

--
-- Name: object_class_pkey; Type: CONSTRAINT; Schema: code_internal; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY object_class
    ADD CONSTRAINT object_class_pkey PRIMARY KEY (id);


--
-- Name: object_type_pkey; Type: CONSTRAINT; Schema: code_internal; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY object_type
    ADD CONSTRAINT object_type_pkey PRIMARY KEY (id);


--
-- Name: objects__pk_full_name; Type: CONSTRAINT; Schema: code_internal; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY objects
    ADD CONSTRAINT objects__pk_full_name PRIMARY KEY (full_name);


--
-- Name: templates__u_object_class__version; Type: CONSTRAINT; Schema: code_internal; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY templates
    ADD CONSTRAINT templates__u_object_class__version UNIQUE (version, object_class_id);


--
-- Name: templates_pkey; Type: CONSTRAINT; Schema: code_internal; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY templates
    ADD CONSTRAINT templates_pkey PRIMARY KEY (id);


SET search_path = inheritance, pg_catalog;

--
-- Name: child_objects__u_table__object_type__object_name_template; Type: CONSTRAINT; Schema: inheritance; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY child_objects
    ADD CONSTRAINT child_objects__u_table__object_type__object_name_template UNIQUE (inherited_table_id, object_type_id, object_name_template);


--
-- Name: child_objects_exclude_tables__pk_child_object__exclude_table; Type: CONSTRAINT; Schema: inheritance; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY child_objects_exclude_tables
    ADD CONSTRAINT child_objects_exclude_tables__pk_child_object__exclude_table PRIMARY KEY (child_object_id, exclude_table_id);


--
-- Name: child_objects_pkey; Type: CONSTRAINT; Schema: inheritance; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY child_objects
    ADD CONSTRAINT child_objects_pkey PRIMARY KEY (id);


--
-- Name: object_types__pk_id; Type: CONSTRAINT; Schema: inheritance; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY object_types
    ADD CONSTRAINT object_types__pk_id PRIMARY KEY (id);


SET search_path = object_monitor, pg_catalog;

--
-- Name: monitors_consumers_pkey; Type: CONSTRAINT; Schema: object_monitor; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY monitors_consumers
    ADD CONSTRAINT monitors_consumers_pkey PRIMARY KEY (monitor_name, consumer_name);


--
-- Name: monitors_pkey; Type: CONSTRAINT; Schema: object_monitor; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY monitors
    ADD CONSTRAINT monitors_pkey PRIMARY KEY (monitor_name);


--
-- Name: monitors_tables_pkey; Type: CONSTRAINT; Schema: object_monitor; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY monitors_tables
    ADD CONSTRAINT monitors_tables_pkey PRIMARY KEY (monitor_name, table_id);


SET search_path = pgq, pg_catalog;

--
-- Name: consumer_name_uq; Type: CONSTRAINT; Schema: pgq; Owner: skytools; Tablespace: 
--

ALTER TABLE ONLY consumer
    ADD CONSTRAINT consumer_name_uq UNIQUE (co_name);


--
-- Name: consumer_pkey; Type: CONSTRAINT; Schema: pgq; Owner: skytools; Tablespace: 
--

ALTER TABLE ONLY consumer
    ADD CONSTRAINT consumer_pkey PRIMARY KEY (co_id);


--
-- Name: fq_pkey; Type: CONSTRAINT; Schema: pgq; Owner: skytools; Tablespace: 
--

ALTER TABLE ONLY failed_queue
    ADD CONSTRAINT fq_pkey PRIMARY KEY (ev_owner, ev_id);


--
-- Name: queue_name_uq; Type: CONSTRAINT; Schema: pgq; Owner: skytools; Tablespace: 
--

ALTER TABLE ONLY queue
    ADD CONSTRAINT queue_name_uq UNIQUE (queue_name);


--
-- Name: queue_pkey; Type: CONSTRAINT; Schema: pgq; Owner: skytools; Tablespace: 
--

ALTER TABLE ONLY queue
    ADD CONSTRAINT queue_pkey PRIMARY KEY (queue_id);


--
-- Name: rq_pkey; Type: CONSTRAINT; Schema: pgq; Owner: skytools; Tablespace: 
--

ALTER TABLE ONLY retry_queue
    ADD CONSTRAINT rq_pkey PRIMARY KEY (ev_owner, ev_id);


--
-- Name: subscription_pkey; Type: CONSTRAINT; Schema: pgq; Owner: skytools; Tablespace: 
--

ALTER TABLE ONLY subscription
    ADD CONSTRAINT subscription_pkey PRIMARY KEY (sub_id);


--
-- Name: subscription_ukey; Type: CONSTRAINT; Schema: pgq; Owner: skytools; Tablespace: 
--

ALTER TABLE ONLY subscription
    ADD CONSTRAINT subscription_ukey UNIQUE (sub_queue, sub_consumer);


--
-- Name: tick_pkey; Type: CONSTRAINT; Schema: pgq; Owner: skytools; Tablespace: 
--

ALTER TABLE ONLY tick
    ADD CONSTRAINT tick_pkey PRIMARY KEY (tick_queue, tick_id);


SET search_path = replication, pg_catalog;

--
-- Name: schemas__exclude__pk_id; Type: CONSTRAINT; Schema: replication; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY schemas__exclude
    ADD CONSTRAINT schemas__exclude__pk_id PRIMARY KEY (id);


--
-- Name: sets__pk_id; Type: CONSTRAINT; Schema: replication; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY sets
    ADD CONSTRAINT sets__pk_id PRIMARY KEY (id);


--
-- Name: tables__exclude__pk_id; Type: CONSTRAINT; Schema: replication; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY tables__exclude
    ADD CONSTRAINT tables__exclude__pk_id PRIMARY KEY (id);


--
-- Name: tables_full_name_key; Type: CONSTRAINT; Schema: replication; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY tables
    ADD CONSTRAINT tables_full_name_key UNIQUE (full_name);


--
-- Name: tables_pkey; Type: CONSTRAINT; Schema: replication; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY tables
    ADD CONSTRAINT tables_pkey PRIMARY KEY (id);


SET search_path = tools, pg_catalog;

--
-- Name: asserts_pkey; Type: CONSTRAINT; Schema: tools; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY asserts
    ADD CONSTRAINT asserts_pkey PRIMARY KEY (assert_name);


--
-- Name: object_types__pk_object_type; Type: CONSTRAINT; Schema: tools; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY object_types
    ADD CONSTRAINT object_types__pk_object_type PRIMARY KEY (object_type);


--
-- Name: CONSTRAINT object_types__pk_object_type ON object_types; Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON CONSTRAINT object_types__pk_object_type ON object_types IS 'This PK is superfluous given the unique index, but londiste bitches without it.';


--
-- Name: schema_patches_pkey; Type: CONSTRAINT; Schema: tools; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY schema_patches
    ADD CONSTRAINT schema_patches_pkey PRIMARY KEY (patch_file);


--
-- Name: seed_objects__pk_schema__object; Type: CONSTRAINT; Schema: tools; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY seed_objects
    ADD CONSTRAINT seed_objects__pk_schema__object PRIMARY KEY (schema_name, object_name);


--
-- Name: settings__pk_setting_name; Type: CONSTRAINT; Schema: tools; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY settings
    ADD CONSTRAINT settings__pk_setting_name PRIMARY KEY (setting_name);


--
-- Name: CONSTRAINT settings__pk_setting_name ON settings; Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON CONSTRAINT settings__pk_setting_name ON settings IS 'This PK is superfluous given the unique index, but londiste bitches without it.';


--
-- Name: sub_asserts_pkey; Type: CONSTRAINT; Schema: tools; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY sub_asserts
    ADD CONSTRAINT sub_asserts_pkey PRIMARY KEY (assert_name, sub_assert_name);


--
-- Name: tables__u_schema__table; Type: CONSTRAINT; Schema: tools; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY tables
    ADD CONSTRAINT tables__u_schema__table UNIQUE (table_schema, table_name);


--
-- Name: tables_pkey; Type: CONSTRAINT; Schema: tools; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY tables
    ADD CONSTRAINT tables_pkey PRIMARY KEY (id);


--
-- Name: tests_pkey; Type: CONSTRAINT; Schema: tools; Owner: db_global_owner; Tablespace: 
--

ALTER TABLE ONLY tests
    ADD CONSTRAINT tests_pkey PRIMARY KEY (test_name);


SET search_path = cabar, pg_catalog;

--
-- Name: components__u_component_lower; Type: INDEX; Schema: cabar; Owner: db_global_owner; Tablespace: 
--

CREATE UNIQUE INDEX components__u_component_lower ON components USING btree (lower(component));


--
-- Name: env_settings__u_env_setting_lower; Type: INDEX; Schema: cabar; Owner: db_global_owner; Tablespace: 
--

CREATE UNIQUE INDEX env_settings__u_env_setting_lower ON env_settings USING btree (lower(env_setting));


SET search_path = code_internal, pg_catalog;

--
-- Name: object_class__u_type__class_lower; Type: INDEX; Schema: code_internal; Owner: db_global_owner; Tablespace: 
--

CREATE UNIQUE INDEX object_class__u_type__class_lower ON object_class USING btree (lower(object_class), object_type_id);


--
-- Name: object_type__u_type_lower; Type: INDEX; Schema: code_internal; Owner: db_global_owner; Tablespace: 
--

CREATE UNIQUE INDEX object_type__u_type_lower ON object_type USING btree (lower(object_type));


SET search_path = inheritance, pg_catalog;

--
-- Name: object_types__u_object_type_lower; Type: INDEX; Schema: inheritance; Owner: db_global_owner; Tablespace: 
--

CREATE UNIQUE INDEX object_types__u_object_type_lower ON object_types USING btree (lower(object_type));


SET search_path = pgq, pg_catalog;

--
-- Name: rq_retry_idx; Type: INDEX; Schema: pgq; Owner: skytools; Tablespace: 
--

CREATE INDEX rq_retry_idx ON retry_queue USING btree (ev_retry_after);


--
-- Name: rq_retry_owner_idx; Type: INDEX; Schema: pgq; Owner: skytools; Tablespace: 
--

CREATE INDEX rq_retry_owner_idx ON retry_queue USING btree (ev_owner, ev_id);


SET search_path = replication, pg_catalog;

--
-- Name: schemas__exclude__u_schema_name_lower; Type: INDEX; Schema: replication; Owner: db_global_owner; Tablespace: 
--

CREATE UNIQUE INDEX schemas__exclude__u_schema_name_lower ON schemas__exclude USING btree (lower(schema_name));


--
-- Name: sets__u_set_lower; Type: INDEX; Schema: replication; Owner: db_global_owner; Tablespace: 
--

CREATE UNIQUE INDEX sets__u_set_lower ON sets USING btree (lower(set));


--
-- Name: tables__exclude__u_full_name_lower; Type: INDEX; Schema: replication; Owner: db_global_owner; Tablespace: 
--

CREATE UNIQUE INDEX tables__exclude__u_full_name_lower ON tables__exclude USING btree (lower(full_name));


SET search_path = tools, pg_catalog;

--
-- Name: asserts__u_assert_name_lower; Type: INDEX; Schema: tools; Owner: db_global_owner; Tablespace: 
--

CREATE UNIQUE INDEX asserts__u_assert_name_lower ON asserts USING btree (lower((assert_name)::text));


--
-- Name: object_types__u_lower_object_type; Type: INDEX; Schema: tools; Owner: db_global_owner; Tablespace: 
--

CREATE UNIQUE INDEX object_types__u_lower_object_type ON object_types USING btree (lower(object_type));


--
-- Name: settings__setting_name; Type: INDEX; Schema: tools; Owner: db_global_owner; Tablespace: 
--

CREATE UNIQUE INDEX settings__setting_name ON settings USING btree (lower(setting_name));


--
-- Name: sub_asserts__u_sub_assert_name_lower; Type: INDEX; Schema: tools; Owner: db_global_owner; Tablespace: 
--

CREATE UNIQUE INDEX sub_asserts__u_sub_assert_name_lower ON sub_asserts USING btree (lower((assert_name)::text), lower((sub_assert_name)::text));


--
-- Name: not_on_prod; Type: TRIGGER; Schema: tools; Owner: db_global_owner
--

CREATE TRIGGER not_on_prod
    AFTER INSERT OR DELETE OR UPDATE ON tests
    FOR EACH STATEMENT
    EXECUTE PROCEDURE tg_tests_not_on_prod();


--
-- Name: TRIGGER not_on_prod ON tests; Type: COMMENT; Schema: tools; Owner: db_global_owner
--

COMMENT ON TRIGGER not_on_prod ON tests IS 'Disallow tests on databases with ''prod'' or ''slave'' in the name by disallowing DML on the tools.tests table';


--
-- Name: schema_patches_sanity; Type: TRIGGER; Schema: tools; Owner: db_global_owner
--

CREATE TRIGGER schema_patches_sanity
    AFTER INSERT OR DELETE OR UPDATE ON schema_patches
    FOR EACH ROW
    EXECUTE PROCEDURE tg_schema_patches();


SET search_path = cabar, pg_catalog;

--
-- Name: component_schemas_component_id_fkey; Type: FK CONSTRAINT; Schema: cabar; Owner: db_global_owner
--

ALTER TABLE ONLY component_schemas
    ADD CONSTRAINT component_schemas_component_id_fkey FOREIGN KEY (component_id) REFERENCES components(id);


SET search_path = code_internal, pg_catalog;

--
-- Name: objects__ri_template; Type: FK CONSTRAINT; Schema: code_internal; Owner: db_global_owner
--

ALTER TABLE ONLY objects
    ADD CONSTRAINT objects__ri_template FOREIGN KEY (template_id) REFERENCES templates(id);


--
-- Name: templates__ri_object_class; Type: FK CONSTRAINT; Schema: code_internal; Owner: db_global_owner
--

ALTER TABLE ONLY templates
    ADD CONSTRAINT templates__ri_object_class FOREIGN KEY (object_class_id) REFERENCES object_class(id);


--
-- Name: templates__ri_object_type; Type: FK CONSTRAINT; Schema: code_internal; Owner: db_global_owner
--

ALTER TABLE ONLY object_class
    ADD CONSTRAINT templates__ri_object_type FOREIGN KEY (object_type_id) REFERENCES object_type(id);


SET search_path = inheritance, pg_catalog;

--
-- Name: child_objects_exclude_tables_child_object_id_fkey; Type: FK CONSTRAINT; Schema: inheritance; Owner: db_global_owner
--

ALTER TABLE ONLY child_objects_exclude_tables
    ADD CONSTRAINT child_objects_exclude_tables_child_object_id_fkey FOREIGN KEY (child_object_id) REFERENCES child_objects(id);


--
-- Name: child_objects_exclude_tables_exclude_table_id_fkey; Type: FK CONSTRAINT; Schema: inheritance; Owner: db_global_owner
--

ALTER TABLE ONLY child_objects_exclude_tables
    ADD CONSTRAINT child_objects_exclude_tables_exclude_table_id_fkey FOREIGN KEY (exclude_table_id) REFERENCES tools.tables(id);


--
-- Name: child_objects_inherited_table_id_fkey; Type: FK CONSTRAINT; Schema: inheritance; Owner: db_global_owner
--

ALTER TABLE ONLY child_objects
    ADD CONSTRAINT child_objects_inherited_table_id_fkey FOREIGN KEY (inherited_table_id) REFERENCES tools.tables(id);


--
-- Name: child_objects_object_type_id_fkey; Type: FK CONSTRAINT; Schema: inheritance; Owner: db_global_owner
--

ALTER TABLE ONLY child_objects
    ADD CONSTRAINT child_objects_object_type_id_fkey FOREIGN KEY (object_type_id) REFERENCES object_types(id);


SET search_path = object_monitor, pg_catalog;

--
-- Name: monitors_consumers_monitor_name_fkey; Type: FK CONSTRAINT; Schema: object_monitor; Owner: db_global_owner
--

ALTER TABLE ONLY monitors_consumers
    ADD CONSTRAINT monitors_consumers_monitor_name_fkey FOREIGN KEY (monitor_name) REFERENCES monitors(monitor_name);


--
-- Name: monitors_tables_monitor_name_fkey; Type: FK CONSTRAINT; Schema: object_monitor; Owner: db_global_owner
--

ALTER TABLE ONLY monitors_tables
    ADD CONSTRAINT monitors_tables_monitor_name_fkey FOREIGN KEY (monitor_name) REFERENCES monitors(monitor_name);


--
-- Name: monitors_tables_table_id_fkey; Type: FK CONSTRAINT; Schema: object_monitor; Owner: db_global_owner
--

ALTER TABLE ONLY monitors_tables
    ADD CONSTRAINT monitors_tables_table_id_fkey FOREIGN KEY (table_id) REFERENCES tools.tables(id);


SET search_path = pgq, pg_catalog;

--
-- Name: fq_owner_fkey; Type: FK CONSTRAINT; Schema: pgq; Owner: skytools
--

ALTER TABLE ONLY failed_queue
    ADD CONSTRAINT fq_owner_fkey FOREIGN KEY (ev_owner) REFERENCES subscription(sub_id);


--
-- Name: rq_owner_fkey; Type: FK CONSTRAINT; Schema: pgq; Owner: skytools
--

ALTER TABLE ONLY retry_queue
    ADD CONSTRAINT rq_owner_fkey FOREIGN KEY (ev_owner) REFERENCES subscription(sub_id);


--
-- Name: sub_consumer_fkey; Type: FK CONSTRAINT; Schema: pgq; Owner: skytools
--

ALTER TABLE ONLY subscription
    ADD CONSTRAINT sub_consumer_fkey FOREIGN KEY (sub_consumer) REFERENCES consumer(co_id);


--
-- Name: sub_queue_fkey; Type: FK CONSTRAINT; Schema: pgq; Owner: skytools
--

ALTER TABLE ONLY subscription
    ADD CONSTRAINT sub_queue_fkey FOREIGN KEY (sub_queue) REFERENCES queue(queue_id);


--
-- Name: tick_queue_fkey; Type: FK CONSTRAINT; Schema: pgq; Owner: skytools
--

ALTER TABLE ONLY tick
    ADD CONSTRAINT tick_queue_fkey FOREIGN KEY (tick_queue) REFERENCES queue(queue_id);


SET search_path = replication, pg_catalog;

--
-- Name: tables_set_id_fkey; Type: FK CONSTRAINT; Schema: replication; Owner: db_global_owner
--

ALTER TABLE ONLY tables
    ADD CONSTRAINT tables_set_id_fkey FOREIGN KEY (set_id) REFERENCES sets(id);


SET search_path = tools, pg_catalog;

--
-- Name: schema_patches_component_id_fkey; Type: FK CONSTRAINT; Schema: tools; Owner: db_global_owner
--

ALTER TABLE ONLY schema_patches
    ADD CONSTRAINT schema_patches_component_id_fkey FOREIGN KEY (component_id) REFERENCES cabar.components(id);


--
-- Name: sub_asserts_assert_name_fkey; Type: FK CONSTRAINT; Schema: tools; Owner: db_global_owner
--

ALTER TABLE ONLY sub_asserts
    ADD CONSTRAINT sub_asserts_assert_name_fkey FOREIGN KEY (assert_name) REFERENCES asserts(assert_name);


--
-- Name: cabar; Type: ACL; Schema: -; Owner: db_global_owner
--

REVOKE ALL ON SCHEMA cabar FROM PUBLIC;
REVOKE ALL ON SCHEMA cabar FROM db_global_owner;
GRANT ALL ON SCHEMA cabar TO db_global_owner;
GRANT USAGE ON SCHEMA cabar TO db_global__read_role;


--
-- Name: code; Type: ACL; Schema: -; Owner: db_global_owner
--

REVOKE ALL ON SCHEMA code FROM PUBLIC;
REVOKE ALL ON SCHEMA code FROM db_global_owner;
GRANT ALL ON SCHEMA code TO db_global_owner;
GRANT USAGE ON SCHEMA code TO db_global__developer_role;


--
-- Name: code_internal; Type: ACL; Schema: -; Owner: db_global_owner
--

REVOKE ALL ON SCHEMA code_internal FROM PUBLIC;
REVOKE ALL ON SCHEMA code_internal FROM db_global_owner;
GRANT ALL ON SCHEMA code_internal TO db_global_owner;
GRANT USAGE ON SCHEMA code_internal TO db_global__developer_role;


--
-- Name: inheritance; Type: ACL; Schema: -; Owner: db_global_owner
--

REVOKE ALL ON SCHEMA inheritance FROM PUBLIC;
REVOKE ALL ON SCHEMA inheritance FROM db_global_owner;
GRANT ALL ON SCHEMA inheritance TO db_global_owner;
GRANT USAGE ON SCHEMA inheritance TO db_global__developer_role;


--
-- Name: object_monitor; Type: ACL; Schema: -; Owner: db_global_owner
--

REVOKE ALL ON SCHEMA object_monitor FROM PUBLIC;
REVOKE ALL ON SCHEMA object_monitor FROM db_global_owner;
GRANT ALL ON SCHEMA object_monitor TO db_global_owner;
GRANT USAGE ON SCHEMA object_monitor TO db_global__read_role;


--
-- Name: pgq; Type: ACL; Schema: -; Owner: skytools
--

REVOKE ALL ON SCHEMA pgq FROM PUBLIC;
REVOKE ALL ON SCHEMA pgq FROM skytools;
GRANT ALL ON SCHEMA pgq TO skytools;
GRANT USAGE ON SCHEMA pgq TO PUBLIC;


--
-- Name: replication; Type: ACL; Schema: -; Owner: db_global_owner
--

REVOKE ALL ON SCHEMA replication FROM PUBLIC;
REVOKE ALL ON SCHEMA replication FROM db_global_owner;
GRANT ALL ON SCHEMA replication TO db_global_owner;
GRANT USAGE ON SCHEMA replication TO db_global__read_role;


--
-- Name: setting; Type: ACL; Schema: -; Owner: db_global_owner
--

REVOKE ALL ON SCHEMA setting FROM PUBLIC;
REVOKE ALL ON SCHEMA setting FROM db_global_owner;
GRANT ALL ON SCHEMA setting TO db_global_owner;
GRANT USAGE ON SCHEMA setting TO db_global__read_role;


--
-- Name: test; Type: ACL; Schema: -; Owner: db_global_owner
--

REVOKE ALL ON SCHEMA test FROM PUBLIC;
REVOKE ALL ON SCHEMA test FROM db_global_owner;
GRANT ALL ON SCHEMA test TO db_global_owner;
GRANT USAGE ON SCHEMA test TO PUBLIC;
GRANT USAGE ON SCHEMA test TO db_global__read_role;


--
-- Name: tools; Type: ACL; Schema: -; Owner: db_global_owner
--

REVOKE ALL ON SCHEMA tools FROM PUBLIC;
REVOKE ALL ON SCHEMA tools FROM db_global_owner;
GRANT ALL ON SCHEMA tools TO db_global_owner;
GRANT USAGE ON SCHEMA tools TO db_global__app_role;
GRANT USAGE ON SCHEMA tools TO db_global__read_role;


SET search_path = cabar, pg_catalog;

--
-- Name: components; Type: ACL; Schema: cabar; Owner: db_global_owner
--

REVOKE ALL ON TABLE components FROM PUBLIC;
REVOKE ALL ON TABLE components FROM db_global_owner;
GRANT ALL ON TABLE components TO db_global_owner;
GRANT SELECT ON TABLE components TO db_global__read_role;


--
-- Name: components__get(smallint); Type: ACL; Schema: cabar; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION components__get(p_id smallint) FROM PUBLIC;
REVOKE ALL ON FUNCTION components__get(p_id smallint) FROM db_global_owner;
GRANT ALL ON FUNCTION components__get(p_id smallint) TO db_global_owner;
GRANT ALL ON FUNCTION components__get(p_id smallint) TO db_global__developer_role;


--
-- Name: components__get_id(text); Type: ACL; Schema: cabar; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION components__get_id(p_component text) FROM PUBLIC;
REVOKE ALL ON FUNCTION components__get_id(p_component text) FROM db_global_owner;
GRANT ALL ON FUNCTION components__get_id(p_component text) TO db_global_owner;
GRANT ALL ON FUNCTION components__get_id(p_component text) TO db_global__developer_role;


--
-- Name: components__get_text(smallint); Type: ACL; Schema: cabar; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION components__get_text(p_id smallint) FROM PUBLIC;
REVOKE ALL ON FUNCTION components__get_text(p_id smallint) FROM db_global_owner;
GRANT ALL ON FUNCTION components__get_text(p_id smallint) TO db_global_owner;
GRANT ALL ON FUNCTION components__get_text(p_id smallint) TO db_global__developer_role;


--
-- Name: env_settings; Type: ACL; Schema: cabar; Owner: db_global_owner
--

REVOKE ALL ON TABLE env_settings FROM PUBLIC;
REVOKE ALL ON TABLE env_settings FROM db_global_owner;
GRANT ALL ON TABLE env_settings TO db_global_owner;
GRANT SELECT ON TABLE env_settings TO db_global__read_role;


--
-- Name: env_settings__get(smallint); Type: ACL; Schema: cabar; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION env_settings__get(p_id smallint) FROM PUBLIC;
REVOKE ALL ON FUNCTION env_settings__get(p_id smallint) FROM db_global_owner;
GRANT ALL ON FUNCTION env_settings__get(p_id smallint) TO db_global_owner;
GRANT ALL ON FUNCTION env_settings__get(p_id smallint) TO db_global__developer_role;


--
-- Name: env_settings__get_id(text); Type: ACL; Schema: cabar; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION env_settings__get_id(p_env_setting text) FROM PUBLIC;
REVOKE ALL ON FUNCTION env_settings__get_id(p_env_setting text) FROM db_global_owner;
GRANT ALL ON FUNCTION env_settings__get_id(p_env_setting text) TO db_global_owner;
GRANT ALL ON FUNCTION env_settings__get_id(p_env_setting text) TO db_global__developer_role;


--
-- Name: env_settings__get_text(smallint); Type: ACL; Schema: cabar; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION env_settings__get_text(p_id smallint) FROM PUBLIC;
REVOKE ALL ON FUNCTION env_settings__get_text(p_id smallint) FROM db_global_owner;
GRANT ALL ON FUNCTION env_settings__get_text(p_id smallint) TO db_global_owner;
GRANT ALL ON FUNCTION env_settings__get_text(p_id smallint) TO db_global__developer_role;


SET search_path = code_internal, pg_catalog;

--
-- Name: object_class; Type: ACL; Schema: code_internal; Owner: db_global_owner
--

REVOKE ALL ON TABLE object_class FROM PUBLIC;
REVOKE ALL ON TABLE object_class FROM db_global_owner;
GRANT ALL ON TABLE object_class TO db_global_owner;
GRANT SELECT ON TABLE object_class TO db_global__developer_role;


--
-- Name: object_class__get(integer, text); Type: ACL; Schema: code_internal; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION object_class__get(p_object_type_id integer, p_object_class text) FROM PUBLIC;
REVOKE ALL ON FUNCTION object_class__get(p_object_type_id integer, p_object_class text) FROM db_global_owner;
GRANT ALL ON FUNCTION object_class__get(p_object_type_id integer, p_object_class text) TO db_global_owner;
GRANT ALL ON FUNCTION object_class__get(p_object_type_id integer, p_object_class text) TO PUBLIC;
GRANT ALL ON FUNCTION object_class__get(p_object_type_id integer, p_object_class text) TO db_global__developer_role;


--
-- Name: object_class__get(text, text); Type: ACL; Schema: code_internal; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION object_class__get(p_object_type text, p_object_class text) FROM PUBLIC;
REVOKE ALL ON FUNCTION object_class__get(p_object_type text, p_object_class text) FROM db_global_owner;
GRANT ALL ON FUNCTION object_class__get(p_object_type text, p_object_class text) TO db_global_owner;
GRANT ALL ON FUNCTION object_class__get(p_object_type text, p_object_class text) TO PUBLIC;
GRANT ALL ON FUNCTION object_class__get(p_object_type text, p_object_class text) TO db_global__developer_role;


--
-- Name: object_class__get_id(text, text); Type: ACL; Schema: code_internal; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION object_class__get_id(p_object_type text, p_object_class text) FROM PUBLIC;
REVOKE ALL ON FUNCTION object_class__get_id(p_object_type text, p_object_class text) FROM db_global_owner;
GRANT ALL ON FUNCTION object_class__get_id(p_object_type text, p_object_class text) TO db_global_owner;
GRANT ALL ON FUNCTION object_class__get_id(p_object_type text, p_object_class text) TO PUBLIC;
GRANT ALL ON FUNCTION object_class__get_id(p_object_type text, p_object_class text) TO db_global__developer_role;


--
-- Name: object_class__getsert(integer, text); Type: ACL; Schema: code_internal; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION object_class__getsert(p_object_type_id integer, p_object_class text) FROM PUBLIC;
REVOKE ALL ON FUNCTION object_class__getsert(p_object_type_id integer, p_object_class text) FROM db_global_owner;
GRANT ALL ON FUNCTION object_class__getsert(p_object_type_id integer, p_object_class text) TO db_global_owner;
GRANT ALL ON FUNCTION object_class__getsert(p_object_type_id integer, p_object_class text) TO PUBLIC;
GRANT ALL ON FUNCTION object_class__getsert(p_object_type_id integer, p_object_class text) TO db_global__developer_role;


--
-- Name: object_type__get_id(text); Type: ACL; Schema: code_internal; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION object_type__get_id(p_object_type text) FROM PUBLIC;
REVOKE ALL ON FUNCTION object_type__get_id(p_object_type text) FROM db_global_owner;
GRANT ALL ON FUNCTION object_type__get_id(p_object_type text) TO db_global_owner;
GRANT ALL ON FUNCTION object_type__get_id(p_object_type text) TO PUBLIC;
GRANT ALL ON FUNCTION object_type__get_id(p_object_type text) TO db_global__developer_role;


--
-- Name: templates; Type: ACL; Schema: code_internal; Owner: db_global_owner
--

REVOKE ALL ON TABLE templates FROM PUBLIC;
REVOKE ALL ON TABLE templates FROM db_global_owner;
GRANT ALL ON TABLE templates TO db_global_owner;
GRANT SELECT ON TABLE templates TO db_global__developer_role;


--
-- Name: template__get_latest(text, text); Type: ACL; Schema: code_internal; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION template__get_latest(p_object_type text, p_object_class text) FROM PUBLIC;
REVOKE ALL ON FUNCTION template__get_latest(p_object_type text, p_object_class text) FROM db_global_owner;
GRANT ALL ON FUNCTION template__get_latest(p_object_type text, p_object_class text) TO db_global_owner;
GRANT ALL ON FUNCTION template__get_latest(p_object_type text, p_object_class text) TO PUBLIC;
GRANT ALL ON FUNCTION template__get_latest(p_object_type text, p_object_class text) TO db_global__developer_role;


SET search_path = inheritance, pg_catalog;

--
-- Name: child_object__add(text, text, text, text, text[]); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text, p_exclude_tables text[]) FROM PUBLIC;
REVOKE ALL ON FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text, p_exclude_tables text[]) FROM db_global_owner;
GRANT ALL ON FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text, p_exclude_tables text[]) TO db_global_owner;
GRANT ALL ON FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text, p_exclude_tables text[]) TO db_global__app_owner_role;


--
-- Name: child_object__add(text, text, text, text, text); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text, p_exclude_tables text) FROM PUBLIC;
REVOKE ALL ON FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text, p_exclude_tables text) FROM db_global_owner;
GRANT ALL ON FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text, p_exclude_tables text) TO db_global_owner;
GRANT ALL ON FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text, p_exclude_tables text) TO db_global__app_owner_role;


--
-- Name: child_object__add(text, text, text, text); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text) FROM PUBLIC;
REVOKE ALL ON FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text) FROM db_global_owner;
GRANT ALL ON FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text) TO db_global_owner;
GRANT ALL ON FUNCTION child_object__add(p_inherited_table text, p_object_type text, p_object_name_template text, p_object_body_template text) TO db_global__app_owner_role;


--
-- Name: child_object__add_excluded_tables(integer, text[]); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION child_object__add_excluded_tables(p_child_object_id integer, p_exclude_tables text[]) FROM PUBLIC;
REVOKE ALL ON FUNCTION child_object__add_excluded_tables(p_child_object_id integer, p_exclude_tables text[]) FROM db_global_owner;
GRANT ALL ON FUNCTION child_object__add_excluded_tables(p_child_object_id integer, p_exclude_tables text[]) TO db_global_owner;
GRANT ALL ON FUNCTION child_object__add_excluded_tables(p_child_object_id integer, p_exclude_tables text[]) TO db_global__app_owner_role;


--
-- Name: child_object__add_excluded_tables(integer, text); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION child_object__add_excluded_tables(p_child_object_id integer, p_exclude_tables text) FROM PUBLIC;
REVOKE ALL ON FUNCTION child_object__add_excluded_tables(p_child_object_id integer, p_exclude_tables text) FROM db_global_owner;
GRANT ALL ON FUNCTION child_object__add_excluded_tables(p_child_object_id integer, p_exclude_tables text) TO db_global_owner;
GRANT ALL ON FUNCTION child_object__add_excluded_tables(p_child_object_id integer, p_exclude_tables text) TO db_global__app_owner_role;


--
-- Name: child_object__add_excluded_tables(text, text, text, text[]); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION child_object__add_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text[]) FROM PUBLIC;
REVOKE ALL ON FUNCTION child_object__add_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text[]) FROM db_global_owner;
GRANT ALL ON FUNCTION child_object__add_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text[]) TO db_global_owner;
GRANT ALL ON FUNCTION child_object__add_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text[]) TO db_global__app_owner_role;


--
-- Name: child_object__add_excluded_tables(text, text, text, text); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION child_object__add_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text) FROM PUBLIC;
REVOKE ALL ON FUNCTION child_object__add_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text) FROM db_global_owner;
GRANT ALL ON FUNCTION child_object__add_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text) TO db_global_owner;
GRANT ALL ON FUNCTION child_object__add_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text) TO db_global__app_owner_role;


--
-- Name: child_objects; Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON TABLE child_objects FROM PUBLIC;
REVOKE ALL ON TABLE child_objects FROM db_global_owner;
GRANT ALL ON TABLE child_objects TO db_global_owner;
GRANT SELECT ON TABLE child_objects TO db_global__developer_role;


--
-- Name: child_object__get(text, text, text); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION child_object__get(p_inherited_table text, p_object_type text, p_object_name_template text) FROM PUBLIC;
REVOKE ALL ON FUNCTION child_object__get(p_inherited_table text, p_object_type text, p_object_name_template text) FROM db_global_owner;
GRANT ALL ON FUNCTION child_object__get(p_inherited_table text, p_object_type text, p_object_name_template text) TO db_global_owner;
GRANT ALL ON FUNCTION child_object__get(p_inherited_table text, p_object_type text, p_object_name_template text) TO db_global__app_owner_role;


--
-- Name: child_object__get_id(text, text, text); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION child_object__get_id(p_inherited_table text, p_object_type text, p_object_name_template text) FROM PUBLIC;
REVOKE ALL ON FUNCTION child_object__get_id(p_inherited_table text, p_object_type text, p_object_name_template text) FROM db_global_owner;
GRANT ALL ON FUNCTION child_object__get_id(p_inherited_table text, p_object_type text, p_object_name_template text) TO db_global_owner;
GRANT ALL ON FUNCTION child_object__get_id(p_inherited_table text, p_object_type text, p_object_name_template text) TO db_global__app_owner_role;


--
-- Name: child_object__remove_excluded_tables(integer, text[]); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION child_object__remove_excluded_tables(p_child_object_id integer, p_exclude_tables text[]) FROM PUBLIC;
REVOKE ALL ON FUNCTION child_object__remove_excluded_tables(p_child_object_id integer, p_exclude_tables text[]) FROM db_global_owner;
GRANT ALL ON FUNCTION child_object__remove_excluded_tables(p_child_object_id integer, p_exclude_tables text[]) TO db_global_owner;
GRANT ALL ON FUNCTION child_object__remove_excluded_tables(p_child_object_id integer, p_exclude_tables text[]) TO db_global__app_owner_role;


--
-- Name: child_object__remove_excluded_tables(integer, text); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION child_object__remove_excluded_tables(p_child_object_id integer, p_exclude_tables text) FROM PUBLIC;
REVOKE ALL ON FUNCTION child_object__remove_excluded_tables(p_child_object_id integer, p_exclude_tables text) FROM db_global_owner;
GRANT ALL ON FUNCTION child_object__remove_excluded_tables(p_child_object_id integer, p_exclude_tables text) TO db_global_owner;
GRANT ALL ON FUNCTION child_object__remove_excluded_tables(p_child_object_id integer, p_exclude_tables text) TO db_global__app_owner_role;


--
-- Name: child_object__remove_excluded_tables(text, text, text, text[]); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION child_object__remove_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text[]) FROM PUBLIC;
REVOKE ALL ON FUNCTION child_object__remove_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text[]) FROM db_global_owner;
GRANT ALL ON FUNCTION child_object__remove_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text[]) TO db_global_owner;
GRANT ALL ON FUNCTION child_object__remove_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text[]) TO db_global__app_owner_role;


--
-- Name: child_object__remove_excluded_tables(text, text, text, text); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION child_object__remove_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text) FROM PUBLIC;
REVOKE ALL ON FUNCTION child_object__remove_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text) FROM db_global_owner;
GRANT ALL ON FUNCTION child_object__remove_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text) TO db_global_owner;
GRANT ALL ON FUNCTION child_object__remove_excluded_tables(p_inherited_table text, p_object_type text, p_object_name_template text, p_exclude_tables text) TO db_global__app_owner_role;


--
-- Name: get_parents(oid); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION get_parents(p_child_table_oid oid, OUT parent_oid oid) FROM PUBLIC;
REVOKE ALL ON FUNCTION get_parents(p_child_table_oid oid, OUT parent_oid oid) FROM db_global_owner;
GRANT ALL ON FUNCTION get_parents(p_child_table_oid oid, OUT parent_oid oid) TO db_global_owner;
GRANT ALL ON FUNCTION get_parents(p_child_table_oid oid, OUT parent_oid oid) TO db_global__developer_role;
GRANT ALL ON FUNCTION get_parents(p_child_table_oid oid, OUT parent_oid oid) TO db_global__app_owner_role;


--
-- Name: londiste_trigger_refresh(text); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION londiste_trigger_refresh(p_parent_table text, OUT child_table text) FROM PUBLIC;
REVOKE ALL ON FUNCTION londiste_trigger_refresh(p_parent_table text, OUT child_table text) FROM db_global_owner;
GRANT ALL ON FUNCTION londiste_trigger_refresh(p_parent_table text, OUT child_table text) TO db_global_owner;
GRANT ALL ON FUNCTION londiste_trigger_refresh(p_parent_table text, OUT child_table text) TO PUBLIC;


--
-- Name: object_types; Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON TABLE object_types FROM PUBLIC;
REVOKE ALL ON TABLE object_types FROM db_global_owner;
GRANT ALL ON TABLE object_types TO db_global_owner;
GRANT SELECT ON TABLE object_types TO db_global__read_role;


--
-- Name: object_types__get(smallint); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION object_types__get(p_id smallint) FROM PUBLIC;
REVOKE ALL ON FUNCTION object_types__get(p_id smallint) FROM db_global_owner;
GRANT ALL ON FUNCTION object_types__get(p_id smallint) TO db_global_owner;
GRANT ALL ON FUNCTION object_types__get(p_id smallint) TO db_global__developer_role;
GRANT ALL ON FUNCTION object_types__get(p_id smallint) TO db_global__app_owner_role;


--
-- Name: object_types__get_id(text); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION object_types__get_id(p_object_type text) FROM PUBLIC;
REVOKE ALL ON FUNCTION object_types__get_id(p_object_type text) FROM db_global_owner;
GRANT ALL ON FUNCTION object_types__get_id(p_object_type text) TO db_global_owner;
GRANT ALL ON FUNCTION object_types__get_id(p_object_type text) TO db_global__developer_role;
GRANT ALL ON FUNCTION object_types__get_id(p_object_type text) TO db_global__app_owner_role;


--
-- Name: object_types__get_text(smallint); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION object_types__get_text(p_id smallint) FROM PUBLIC;
REVOKE ALL ON FUNCTION object_types__get_text(p_id smallint) FROM db_global_owner;
GRANT ALL ON FUNCTION object_types__get_text(p_id smallint) TO db_global_owner;
GRANT ALL ON FUNCTION object_types__get_text(p_id smallint) TO db_global__developer_role;
GRANT ALL ON FUNCTION object_types__get_text(p_id smallint) TO db_global__app_owner_role;


--
-- Name: process_all(); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION process_all(OUT child_table text) FROM PUBLIC;
REVOKE ALL ON FUNCTION process_all(OUT child_table text) FROM db_global_owner;
GRANT ALL ON FUNCTION process_all(OUT child_table text) TO db_global_owner;
GRANT ALL ON FUNCTION process_all(OUT child_table text) TO db_global__app_owner_role;


--
-- Name: process_child(text); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION process_child(p_child_table text) FROM PUBLIC;
REVOKE ALL ON FUNCTION process_child(p_child_table text) FROM db_global_owner;
GRANT ALL ON FUNCTION process_child(p_child_table text) TO db_global_owner;
GRANT ALL ON FUNCTION process_child(p_child_table text) TO db_global__app_owner_role;


--
-- Name: process_children(text); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION process_children(p_parent_table text, OUT child_table text) FROM PUBLIC;
REVOKE ALL ON FUNCTION process_children(p_parent_table text, OUT child_table text) FROM db_global_owner;
GRANT ALL ON FUNCTION process_children(p_parent_table text, OUT child_table text) TO db_global_owner;
GRANT ALL ON FUNCTION process_children(p_parent_table text, OUT child_table text) TO db_global__app_owner_role;


--
-- Name: reset_child(text, text); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION reset_child(p_child_table text, p_object_type text) FROM PUBLIC;
REVOKE ALL ON FUNCTION reset_child(p_child_table text, p_object_type text) FROM db_global_owner;
GRANT ALL ON FUNCTION reset_child(p_child_table text, p_object_type text) TO db_global_owner;
GRANT ALL ON FUNCTION reset_child(p_child_table text, p_object_type text) TO db_global__app_owner_role;


--
-- Name: reset_children(text, text); Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION reset_children(p_parent_table text, p_object_type text, OUT child_table text) FROM PUBLIC;
REVOKE ALL ON FUNCTION reset_children(p_parent_table text, p_object_type text, OUT child_table text) FROM db_global_owner;
GRANT ALL ON FUNCTION reset_children(p_parent_table text, p_object_type text, OUT child_table text) TO db_global_owner;
GRANT ALL ON FUNCTION reset_children(p_parent_table text, p_object_type text, OUT child_table text) TO db_global__app_owner_role;


SET search_path = object_monitor, pg_catalog;

--
-- Name: create_queues(text); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION create_queues(p_monitor_name text) FROM PUBLIC;
REVOKE ALL ON FUNCTION create_queues(p_monitor_name text) FROM db_global_owner;
GRANT ALL ON FUNCTION create_queues(p_monitor_name text) TO db_global_owner;


--
-- Name: create_trigger(text); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION create_trigger(p_table_name text) FROM PUBLIC;
REVOKE ALL ON FUNCTION create_trigger(p_table_name text) FROM db_global_owner;
GRANT ALL ON FUNCTION create_trigger(p_table_name text) TO db_global_owner;


--
-- Name: drain_queue(text); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION drain_queue(p_table_name text) FROM PUBLIC;
REVOKE ALL ON FUNCTION drain_queue(p_table_name text) FROM db_global_owner;
GRANT ALL ON FUNCTION drain_queue(p_table_name text) TO db_global_owner;
GRANT ALL ON FUNCTION drain_queue(p_table_name text) TO db_global__app_role;


--
-- Name: drain_queue(text, text, bigint); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION drain_queue(p_consumer_name text, p_table_name text, p_batch_count bigint) FROM PUBLIC;
REVOKE ALL ON FUNCTION drain_queue(p_consumer_name text, p_table_name text, p_batch_count bigint) FROM db_global_owner;
GRANT ALL ON FUNCTION drain_queue(p_consumer_name text, p_table_name text, p_batch_count bigint) TO db_global_owner;
GRANT ALL ON FUNCTION drain_queue(p_consumer_name text, p_table_name text, p_batch_count bigint) TO object_monitor__user_role;


--
-- Name: drain_queue_dups(text); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION drain_queue_dups(p_table_name text) FROM PUBLIC;
REVOKE ALL ON FUNCTION drain_queue_dups(p_table_name text) FROM db_global_owner;
GRANT ALL ON FUNCTION drain_queue_dups(p_table_name text) TO db_global_owner;
GRANT ALL ON FUNCTION drain_queue_dups(p_table_name text) TO db_global__app_role;


--
-- Name: drain_queue_dups(text, text, bigint); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION drain_queue_dups(p_consumer_name text, p_table_name text, p_batch_count bigint) FROM PUBLIC;
REVOKE ALL ON FUNCTION drain_queue_dups(p_consumer_name text, p_table_name text, p_batch_count bigint) FROM db_global_owner;
GRANT ALL ON FUNCTION drain_queue_dups(p_consumer_name text, p_table_name text, p_batch_count bigint) TO db_global_owner;
GRANT ALL ON FUNCTION drain_queue_dups(p_consumer_name text, p_table_name text, p_batch_count bigint) TO object_monitor__user_role;


--
-- Name: drop_queues(text); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION drop_queues(p_monitor_name text) FROM PUBLIC;
REVOKE ALL ON FUNCTION drop_queues(p_monitor_name text) FROM db_global_owner;
GRANT ALL ON FUNCTION drop_queues(p_monitor_name text) TO db_global_owner;


--
-- Name: drop_trigger(text); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION drop_trigger(p_table_name text) FROM PUBLIC;
REVOKE ALL ON FUNCTION drop_trigger(p_table_name text) FROM db_global_owner;
GRANT ALL ON FUNCTION drop_trigger(p_table_name text) TO db_global_owner;


--
-- Name: finish_batches(text, text, bigint); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION finish_batches(p_consumer_name text, p_table_name text, p_batch_count bigint) FROM PUBLIC;
REVOKE ALL ON FUNCTION finish_batches(p_consumer_name text, p_table_name text, p_batch_count bigint) FROM db_global_owner;
GRANT ALL ON FUNCTION finish_batches(p_consumer_name text, p_table_name text, p_batch_count bigint) TO db_global_owner;
GRANT ALL ON FUNCTION finish_batches(p_consumer_name text, p_table_name text, p_batch_count bigint) TO object_monitor__user_role;


--
-- Name: get_batch_count(text, text); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION get_batch_count(p_consumer_name text, p_table_name text) FROM PUBLIC;
REVOKE ALL ON FUNCTION get_batch_count(p_consumer_name text, p_table_name text) FROM db_global_owner;
GRANT ALL ON FUNCTION get_batch_count(p_consumer_name text, p_table_name text) TO db_global_owner;
GRANT ALL ON FUNCTION get_batch_count(p_consumer_name text, p_table_name text) TO object_monitor__user_role;


--
-- Name: monitor__add_tables(text, text[]); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION monitor__add_tables(p_monitor_name text, p_table_list text[]) FROM PUBLIC;
REVOKE ALL ON FUNCTION monitor__add_tables(p_monitor_name text, p_table_list text[]) FROM db_global_owner;
GRANT ALL ON FUNCTION monitor__add_tables(p_monitor_name text, p_table_list text[]) TO db_global_owner;


--
-- Name: monitor__create(text, text[]); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION monitor__create(p_monitor_name text, p_table_list text[]) FROM PUBLIC;
REVOKE ALL ON FUNCTION monitor__create(p_monitor_name text, p_table_list text[]) FROM db_global_owner;
GRANT ALL ON FUNCTION monitor__create(p_monitor_name text, p_table_list text[]) TO db_global_owner;


--
-- Name: monitor__drop(text); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION monitor__drop(p_monitor_name text) FROM PUBLIC;
REVOKE ALL ON FUNCTION monitor__drop(p_monitor_name text) FROM db_global_owner;
GRANT ALL ON FUNCTION monitor__drop(p_monitor_name text) TO db_global_owner;


--
-- Name: monitor__drop_tables(text, text[]); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION monitor__drop_tables(p_monitor_name text, p_table_list text[]) FROM PUBLIC;
REVOKE ALL ON FUNCTION monitor__drop_tables(p_monitor_name text, p_table_list text[]) FROM db_global_owner;
GRANT ALL ON FUNCTION monitor__drop_tables(p_monitor_name text, p_table_list text[]) TO db_global_owner;


--
-- Name: process_monitor_create(text, text[]); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION process_monitor_create(p_monitor_name text, p_table_list text[]) FROM PUBLIC;
REVOKE ALL ON FUNCTION process_monitor_create(p_monitor_name text, p_table_list text[]) FROM db_global_owner;
GRANT ALL ON FUNCTION process_monitor_create(p_monitor_name text, p_table_list text[]) TO db_global_owner;


--
-- Name: process_monitor_drop(text); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION process_monitor_drop(p_monitor_name text) FROM PUBLIC;
REVOKE ALL ON FUNCTION process_monitor_drop(p_monitor_name text) FROM db_global_owner;
GRANT ALL ON FUNCTION process_monitor_drop(p_monitor_name text) TO db_global_owner;


--
-- Name: register_consumer(text, text); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION register_consumer(p_monitor_name text, p_consumer_name text) FROM PUBLIC;
REVOKE ALL ON FUNCTION register_consumer(p_monitor_name text, p_consumer_name text) FROM db_global_owner;
GRANT ALL ON FUNCTION register_consumer(p_monitor_name text, p_consumer_name text) TO db_global_owner;
GRANT ALL ON FUNCTION register_consumer(p_monitor_name text, p_consumer_name text) TO object_monitor__user_role;
GRANT ALL ON FUNCTION register_consumer(p_monitor_name text, p_consumer_name text) TO db_global__app_owner_role;


--
-- Name: set_consumer(text); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION set_consumer(p_consumer_name text) FROM PUBLIC;
REVOKE ALL ON FUNCTION set_consumer(p_consumer_name text) FROM db_global_owner;
GRANT ALL ON FUNCTION set_consumer(p_consumer_name text) TO db_global_owner;
GRANT ALL ON FUNCTION set_consumer(p_consumer_name text) TO db_global__app_role;


--
-- Name: subscribe_consumer(text, text); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION subscribe_consumer(p_monitor_name text, p_consumer_name text) FROM PUBLIC;
REVOKE ALL ON FUNCTION subscribe_consumer(p_monitor_name text, p_consumer_name text) FROM db_global_owner;
GRANT ALL ON FUNCTION subscribe_consumer(p_monitor_name text, p_consumer_name text) TO db_global_owner;


--
-- Name: tg_populate_queue(); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION tg_populate_queue() FROM PUBLIC;
REVOKE ALL ON FUNCTION tg_populate_queue() FROM db_global_owner;
GRANT ALL ON FUNCTION tg_populate_queue() TO db_global_owner;
GRANT ALL ON FUNCTION tg_populate_queue() TO PUBLIC;


--
-- Name: unregister_consumer(text, text); Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION unregister_consumer(p_monitor_name text, p_consumer_name text) FROM PUBLIC;
REVOKE ALL ON FUNCTION unregister_consumer(p_monitor_name text, p_consumer_name text) FROM db_global_owner;
GRANT ALL ON FUNCTION unregister_consumer(p_monitor_name text, p_consumer_name text) TO db_global_owner;
GRANT ALL ON FUNCTION unregister_consumer(p_monitor_name text, p_consumer_name text) TO object_monitor__user_role;
GRANT ALL ON FUNCTION unregister_consumer(p_monitor_name text, p_consumer_name text) TO db_global__app_owner_role;


SET search_path = pgq, pg_catalog;

--
-- Name: failed_queue; Type: ACL; Schema: pgq; Owner: skytools
--

REVOKE ALL ON TABLE failed_queue FROM PUBLIC;
REVOKE ALL ON TABLE failed_queue FROM skytools;
GRANT ALL ON TABLE failed_queue TO skytools;
GRANT SELECT ON TABLE failed_queue TO PUBLIC;


SET search_path = replication, pg_catalog;

--
-- Name: schemas__exclude; Type: ACL; Schema: replication; Owner: db_global_owner
--

REVOKE ALL ON TABLE schemas__exclude FROM PUBLIC;
REVOKE ALL ON TABLE schemas__exclude FROM db_global_owner;
GRANT ALL ON TABLE schemas__exclude TO db_global_owner;
GRANT SELECT ON TABLE schemas__exclude TO db_global__read_role;


--
-- Name: schemas__exclude__get(smallint); Type: ACL; Schema: replication; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION schemas__exclude__get(p_id smallint) FROM PUBLIC;
REVOKE ALL ON FUNCTION schemas__exclude__get(p_id smallint) FROM db_global_owner;
GRANT ALL ON FUNCTION schemas__exclude__get(p_id smallint) TO db_global_owner;
GRANT ALL ON FUNCTION schemas__exclude__get(p_id smallint) TO db_global__developer_role;


--
-- Name: schemas__exclude__get_id(text); Type: ACL; Schema: replication; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION schemas__exclude__get_id(p_schema_name text) FROM PUBLIC;
REVOKE ALL ON FUNCTION schemas__exclude__get_id(p_schema_name text) FROM db_global_owner;
GRANT ALL ON FUNCTION schemas__exclude__get_id(p_schema_name text) TO db_global_owner;
GRANT ALL ON FUNCTION schemas__exclude__get_id(p_schema_name text) TO db_global__developer_role;


--
-- Name: schemas__exclude__get_text(smallint); Type: ACL; Schema: replication; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION schemas__exclude__get_text(p_id smallint) FROM PUBLIC;
REVOKE ALL ON FUNCTION schemas__exclude__get_text(p_id smallint) FROM db_global_owner;
GRANT ALL ON FUNCTION schemas__exclude__get_text(p_id smallint) TO db_global_owner;
GRANT ALL ON FUNCTION schemas__exclude__get_text(p_id smallint) TO db_global__developer_role;


--
-- Name: sets; Type: ACL; Schema: replication; Owner: db_global_owner
--

REVOKE ALL ON TABLE sets FROM PUBLIC;
REVOKE ALL ON TABLE sets FROM db_global_owner;
GRANT ALL ON TABLE sets TO db_global_owner;
GRANT SELECT ON TABLE sets TO db_global__read_role;


--
-- Name: sets__get(smallint); Type: ACL; Schema: replication; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION sets__get(p_id smallint) FROM PUBLIC;
REVOKE ALL ON FUNCTION sets__get(p_id smallint) FROM db_global_owner;
GRANT ALL ON FUNCTION sets__get(p_id smallint) TO db_global_owner;
GRANT ALL ON FUNCTION sets__get(p_id smallint) TO db_global__developer_role;


--
-- Name: sets__get_id(text); Type: ACL; Schema: replication; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION sets__get_id(p_set text) FROM PUBLIC;
REVOKE ALL ON FUNCTION sets__get_id(p_set text) FROM db_global_owner;
GRANT ALL ON FUNCTION sets__get_id(p_set text) TO db_global_owner;
GRANT ALL ON FUNCTION sets__get_id(p_set text) TO db_global__developer_role;


--
-- Name: sets__get_text(smallint); Type: ACL; Schema: replication; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION sets__get_text(p_id smallint) FROM PUBLIC;
REVOKE ALL ON FUNCTION sets__get_text(p_id smallint) FROM db_global_owner;
GRANT ALL ON FUNCTION sets__get_text(p_id smallint) TO db_global_owner;
GRANT ALL ON FUNCTION sets__get_text(p_id smallint) TO db_global__developer_role;


--
-- Name: tables__exclude; Type: ACL; Schema: replication; Owner: db_global_owner
--

REVOKE ALL ON TABLE tables__exclude FROM PUBLIC;
REVOKE ALL ON TABLE tables__exclude FROM db_global_owner;
GRANT ALL ON TABLE tables__exclude TO db_global_owner;
GRANT SELECT ON TABLE tables__exclude TO db_global__read_role;


--
-- Name: tables__exclude__get(smallint); Type: ACL; Schema: replication; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION tables__exclude__get(p_id smallint) FROM PUBLIC;
REVOKE ALL ON FUNCTION tables__exclude__get(p_id smallint) FROM db_global_owner;
GRANT ALL ON FUNCTION tables__exclude__get(p_id smallint) TO db_global_owner;
GRANT ALL ON FUNCTION tables__exclude__get(p_id smallint) TO db_global__developer_role;


--
-- Name: tables__exclude__get_id(text); Type: ACL; Schema: replication; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION tables__exclude__get_id(p_full_name text) FROM PUBLIC;
REVOKE ALL ON FUNCTION tables__exclude__get_id(p_full_name text) FROM db_global_owner;
GRANT ALL ON FUNCTION tables__exclude__get_id(p_full_name text) TO db_global_owner;
GRANT ALL ON FUNCTION tables__exclude__get_id(p_full_name text) TO db_global__developer_role;


--
-- Name: tables__exclude__get_text(smallint); Type: ACL; Schema: replication; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION tables__exclude__get_text(p_id smallint) FROM PUBLIC;
REVOKE ALL ON FUNCTION tables__exclude__get_text(p_id smallint) FROM db_global_owner;
GRANT ALL ON FUNCTION tables__exclude__get_text(p_id smallint) TO db_global_owner;
GRANT ALL ON FUNCTION tables__exclude__get_text(p_id smallint) TO db_global__developer_role;


SET search_path = tools, pg_catalog;

--
-- Name: settings; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE settings FROM PUBLIC;
REVOKE ALL ON TABLE settings FROM db_global_owner;
GRANT ALL ON TABLE settings TO db_global_owner;
GRANT INSERT,UPDATE ON TABLE settings TO cabar_base_role;
GRANT SELECT ON TABLE settings TO db_global__developer_role;


SET search_path = test, pg_catalog;

--
-- Name: assert(text, bigint, bigint); Type: ACL; Schema: test; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert(p_expression text, p_result bigint, p_expected bigint) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert(p_expression text, p_result bigint, p_expected bigint) FROM db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result bigint, p_expected bigint) TO db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result bigint, p_expected bigint) TO PUBLIC;


--
-- Name: assert(text, text, text); Type: ACL; Schema: test; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert(p_expression text, p_result text, p_expected text) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert(p_expression text, p_result text, p_expected text) FROM db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result text, p_expected text) TO db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result text, p_expected text) TO PUBLIC;


--
-- Name: assert(text, double precision, double precision); Type: ACL; Schema: test; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert(p_expression text, p_result double precision, p_expected double precision) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert(p_expression text, p_result double precision, p_expected double precision) FROM db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result double precision, p_expected double precision) TO db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result double precision, p_expected double precision) TO PUBLIC;


--
-- Name: assert(text, timestamp with time zone, timestamp with time zone); Type: ACL; Schema: test; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert(p_expression text, p_result timestamp with time zone, p_expected timestamp with time zone) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert(p_expression text, p_result timestamp with time zone, p_expected timestamp with time zone) FROM db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result timestamp with time zone, p_expected timestamp with time zone) TO db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result timestamp with time zone, p_expected timestamp with time zone) TO PUBLIC;


--
-- Name: assert(text, interval, interval); Type: ACL; Schema: test; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert(p_expression text, p_result interval, p_expected interval) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert(p_expression text, p_result interval, p_expected interval) FROM db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result interval, p_expected interval) TO db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result interval, p_expected interval) TO PUBLIC;


--
-- Name: assert(text, numeric, numeric); Type: ACL; Schema: test; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert(p_expression text, p_result numeric, p_expected numeric) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert(p_expression text, p_result numeric, p_expected numeric) FROM db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result numeric, p_expected numeric) TO db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result numeric, p_expected numeric) TO PUBLIC;


--
-- Name: assert(text, boolean, boolean); Type: ACL; Schema: test; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert(p_expression text, p_result boolean, p_expected boolean) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert(p_expression text, p_result boolean, p_expected boolean) FROM db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result boolean, p_expected boolean) TO db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result boolean, p_expected boolean) TO PUBLIC;


--
-- Name: assert(text, bytea, bytea); Type: ACL; Schema: test; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert(p_expression text, p_result bytea, p_expected bytea) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert(p_expression text, p_result bytea, p_expected bytea) FROM db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result bytea, p_expected bytea) TO db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result bytea, p_expected bytea) TO PUBLIC;


--
-- Name: assert(text, oid, oid); Type: ACL; Schema: test; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert(p_expression text, p_result oid, p_expected oid) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert(p_expression text, p_result oid, p_expected oid) FROM db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result oid, p_expected oid) TO db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result oid, p_expected oid) TO PUBLIC;


--
-- Name: assert(text, integer, integer); Type: ACL; Schema: test; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert(p_expression text, p_result integer, p_expected integer) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert(p_expression text, p_result integer, p_expected integer) FROM db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result integer, p_expected integer) TO db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result integer, p_expected integer) TO PUBLIC;


--
-- Name: assert(text, smallint, smallint); Type: ACL; Schema: test; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert(p_expression text, p_result smallint, p_expected smallint) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert(p_expression text, p_result smallint, p_expected smallint) FROM db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result smallint, p_expected smallint) TO db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result smallint, p_expected smallint) TO PUBLIC;


--
-- Name: assert(text, real, real); Type: ACL; Schema: test; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert(p_expression text, p_result real, p_expected real) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert(p_expression text, p_result real, p_expected real) FROM db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result real, p_expected real) TO db_global_owner;
GRANT ALL ON FUNCTION assert(p_expression text, p_result real, p_expected real) TO PUBLIC;


SET search_path = tools, pg_catalog;

--
-- Name: array_length(anyarray, integer); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION array_length(p_array anyarray, p_dimension integer) FROM PUBLIC;
REVOKE ALL ON FUNCTION array_length(p_array anyarray, p_dimension integer) FROM db_global_owner;
GRANT ALL ON FUNCTION array_length(p_array anyarray, p_dimension integer) TO db_global_owner;
GRANT ALL ON FUNCTION array_length(p_array anyarray, p_dimension integer) TO PUBLIC;


--
-- Name: assert(boolean, text); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert(p_condition boolean, p_message text) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert(p_condition boolean, p_message text) FROM db_global_owner;
GRANT ALL ON FUNCTION assert(p_condition boolean, p_message text) TO db_global_owner;
GRANT ALL ON FUNCTION assert(p_condition boolean, p_message text) TO PUBLIC;
GRANT ALL ON FUNCTION assert(p_condition boolean, p_message text) TO db_global__read_role;


--
-- Name: assert__add(character varying, integer); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert__add(p_assert_name character varying, p_level integer) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert__add(p_assert_name character varying, p_level integer) FROM db_global_owner;
GRANT ALL ON FUNCTION assert__add(p_assert_name character varying, p_level integer) TO db_global_owner;
GRANT ALL ON FUNCTION assert__add(p_assert_name character varying, p_level integer) TO cabar_base_role;
GRANT ALL ON FUNCTION assert__add(p_assert_name character varying, p_level integer) TO db_global__app_owner_role;


--
-- Name: assert__delete(character varying, boolean); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert__delete(p_assert_name character varying, p_force boolean) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert__delete(p_assert_name character varying, p_force boolean) FROM db_global_owner;
GRANT ALL ON FUNCTION assert__delete(p_assert_name character varying, p_force boolean) TO db_global_owner;
GRANT ALL ON FUNCTION assert__delete(p_assert_name character varying, p_force boolean) TO db_global__app_owner_role;


--
-- Name: assert__delete(character varying); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert__delete(p_assert_name character varying) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert__delete(p_assert_name character varying) FROM db_global_owner;
GRANT ALL ON FUNCTION assert__delete(p_assert_name character varying) TO db_global_owner;
GRANT ALL ON FUNCTION assert__delete(p_assert_name character varying) TO db_global__app_owner_role;


--
-- Name: assert__enabled_global__set(character varying, boolean); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert__enabled_global__set(p_assert_name character varying, p_enabled boolean) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert__enabled_global__set(p_assert_name character varying, p_enabled boolean) FROM db_global_owner;
GRANT ALL ON FUNCTION assert__enabled_global__set(p_assert_name character varying, p_enabled boolean) TO db_global_owner;
GRANT ALL ON FUNCTION assert__enabled_global__set(p_assert_name character varying, p_enabled boolean) TO db_global__ops_support_role;
GRANT ALL ON FUNCTION assert__enabled_global__set(p_assert_name character varying, p_enabled boolean) TO db_global__app_owner_role;


--
-- Name: assert__enabled_global__set(character varying, character varying, boolean); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert__enabled_global__set(p_assert_name character varying, p_sub_assert_name character varying, p_enabled boolean) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert__enabled_global__set(p_assert_name character varying, p_sub_assert_name character varying, p_enabled boolean) FROM db_global_owner;
GRANT ALL ON FUNCTION assert__enabled_global__set(p_assert_name character varying, p_sub_assert_name character varying, p_enabled boolean) TO db_global_owner;
GRANT ALL ON FUNCTION assert__enabled_global__set(p_assert_name character varying, p_sub_assert_name character varying, p_enabled boolean) TO db_global__app_owner_role;
GRANT ALL ON FUNCTION assert__enabled_global__set(p_assert_name character varying, p_sub_assert_name character varying, p_enabled boolean) TO db_global__ops_support_role;


--
-- Name: assert__enabled_session__set(character varying, boolean); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert__enabled_session__set(p_assert_name character varying, p_enabled boolean) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert__enabled_session__set(p_assert_name character varying, p_enabled boolean) FROM db_global_owner;
GRANT ALL ON FUNCTION assert__enabled_session__set(p_assert_name character varying, p_enabled boolean) TO db_global_owner;
GRANT ALL ON FUNCTION assert__enabled_session__set(p_assert_name character varying, p_enabled boolean) TO db_global__app_owner_role;
GRANT ALL ON FUNCTION assert__enabled_session__set(p_assert_name character varying, p_enabled boolean) TO db_global__ops_support_role;


--
-- Name: assert__enabled_session__set(character varying, character varying, boolean); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION assert__enabled_session__set(p_assert_name character varying, p_sub_assert_name character varying, p_enabled boolean) FROM PUBLIC;
REVOKE ALL ON FUNCTION assert__enabled_session__set(p_assert_name character varying, p_sub_assert_name character varying, p_enabled boolean) FROM db_global_owner;
GRANT ALL ON FUNCTION assert__enabled_session__set(p_assert_name character varying, p_sub_assert_name character varying, p_enabled boolean) TO db_global_owner;
GRANT ALL ON FUNCTION assert__enabled_session__set(p_assert_name character varying, p_sub_assert_name character varying, p_enabled boolean) TO db_global__ops_support_role;


--
-- Name: asserts; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE asserts FROM PUBLIC;
REVOKE ALL ON TABLE asserts FROM db_global_owner;
GRANT ALL ON TABLE asserts TO db_global_owner;
GRANT SELECT ON TABLE asserts TO PUBLIC;
GRANT SELECT ON TABLE asserts TO db_global__developer_role;


--
-- Name: asserts__disabled(); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION asserts__disabled() FROM PUBLIC;
REVOKE ALL ON FUNCTION asserts__disabled() FROM db_global_owner;
GRANT ALL ON FUNCTION asserts__disabled() TO db_global_owner;
GRANT ALL ON FUNCTION asserts__disabled() TO PUBLIC;
GRANT ALL ON FUNCTION asserts__disabled() TO db_global__read_role;


--
-- Name: asserts__enabled(); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION asserts__enabled() FROM PUBLIC;
REVOKE ALL ON FUNCTION asserts__enabled() FROM db_global_owner;
GRANT ALL ON FUNCTION asserts__enabled() TO db_global_owner;
GRANT ALL ON FUNCTION asserts__enabled() TO PUBLIC;
GRANT ALL ON FUNCTION asserts__enabled() TO db_global__read_role;


--
-- Name: checks__disabled(); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION checks__disabled() FROM PUBLIC;
REVOKE ALL ON FUNCTION checks__disabled() FROM db_global_owner;
GRANT ALL ON FUNCTION checks__disabled() TO db_global_owner;
GRANT ALL ON FUNCTION checks__disabled() TO PUBLIC;
GRANT ALL ON FUNCTION checks__disabled() TO db_global__read_role;


--
-- Name: checks__enabled(); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION checks__enabled() FROM PUBLIC;
REVOKE ALL ON FUNCTION checks__enabled() FROM db_global_owner;
GRANT ALL ON FUNCTION checks__enabled() TO db_global_owner;
GRANT ALL ON FUNCTION checks__enabled() TO PUBLIC;
GRANT ALL ON FUNCTION checks__enabled() TO db_global__read_role;


--
-- Name: currval(text, text); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION currval(p_table text, p_column text) FROM PUBLIC;
REVOKE ALL ON FUNCTION currval(p_table text, p_column text) FROM db_global_owner;
GRANT ALL ON FUNCTION currval(p_table text, p_column text) TO db_global_owner;
GRANT ALL ON FUNCTION currval(p_table text, p_column text) TO PUBLIC;
GRANT ALL ON FUNCTION currval(p_table text, p_column text) TO db_global__read_role;


--
-- Name: londiste__drop_all_triggers(text); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION londiste__drop_all_triggers(p_table text) FROM PUBLIC;
REVOKE ALL ON FUNCTION londiste__drop_all_triggers(p_table text) FROM db_global_owner;
GRANT ALL ON FUNCTION londiste__drop_all_triggers(p_table text) TO db_global_owner;
GRANT ALL ON FUNCTION londiste__drop_all_triggers(p_table text) TO cabar_base_role;
GRANT ALL ON FUNCTION londiste__drop_all_triggers(p_table text) TO db_global__app_owner_role;


--
-- Name: londiste__pending_fkeys(text); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION londiste__pending_fkeys(p_child text, OUT from_table text, OUT to_table text, OUT fkey_name text, OUT fkey_def text) FROM PUBLIC;
REVOKE ALL ON FUNCTION londiste__pending_fkeys(p_child text, OUT from_table text, OUT to_table text, OUT fkey_name text, OUT fkey_def text) FROM db_global_owner;
GRANT ALL ON FUNCTION londiste__pending_fkeys(p_child text, OUT from_table text, OUT to_table text, OUT fkey_name text, OUT fkey_def text) TO db_global_owner;
GRANT ALL ON FUNCTION londiste__pending_fkeys(p_child text, OUT from_table text, OUT to_table text, OUT fkey_name text, OUT fkey_def text) TO cabar_base_role;
GRANT ALL ON FUNCTION londiste__pending_fkeys(p_child text, OUT from_table text, OUT to_table text, OUT fkey_name text, OUT fkey_def text) TO db_global__app_owner_role;


--
-- Name: londiste__restore_all_triggers(text); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION londiste__restore_all_triggers(p_table text) FROM PUBLIC;
REVOKE ALL ON FUNCTION londiste__restore_all_triggers(p_table text) FROM db_global_owner;
GRANT ALL ON FUNCTION londiste__restore_all_triggers(p_table text) TO db_global_owner;
GRANT ALL ON FUNCTION londiste__restore_all_triggers(p_table text) TO cabar_base_role;
GRANT ALL ON FUNCTION londiste__restore_all_triggers(p_table text) TO db_global__app_owner_role;


--
-- Name: patch_insert(text, text[]); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION patch_insert(p_patch_file text, p_depends_on text[]) FROM PUBLIC;
REVOKE ALL ON FUNCTION patch_insert(p_patch_file text, p_depends_on text[]) FROM db_global_owner;
GRANT ALL ON FUNCTION patch_insert(p_patch_file text, p_depends_on text[]) TO db_global_owner;
GRANT ALL ON FUNCTION patch_insert(p_patch_file text, p_depends_on text[]) TO PUBLIC;
GRANT ALL ON FUNCTION patch_insert(p_patch_file text, p_depends_on text[]) TO db_global__app_owner_role;


--
-- Name: reject_update(); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION reject_update() FROM PUBLIC;
REVOKE ALL ON FUNCTION reject_update() FROM db_global_owner;
GRANT ALL ON FUNCTION reject_update() TO db_global_owner;
GRANT ALL ON FUNCTION reject_update() TO PUBLIC;
GRANT ALL ON FUNCTION reject_update() TO db_global__read_role;


--
-- Name: seed_object__remove(text, text, boolean); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION seed_object__remove(p_schema_name text, p_relation_name text, p_force boolean) FROM PUBLIC;
REVOKE ALL ON FUNCTION seed_object__remove(p_schema_name text, p_relation_name text, p_force boolean) FROM db_global_owner;
GRANT ALL ON FUNCTION seed_object__remove(p_schema_name text, p_relation_name text, p_force boolean) TO db_global_owner;


--
-- Name: seed_object__remove(text, text); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION seed_object__remove(p_schema_name text, p_relation_name text) FROM PUBLIC;
REVOKE ALL ON FUNCTION seed_object__remove(p_schema_name text, p_relation_name text) FROM db_global_owner;
GRANT ALL ON FUNCTION seed_object__remove(p_schema_name text, p_relation_name text) TO db_global_owner;


--
-- Name: session__clean(text, text); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION session__clean(mode text, source_keys text) FROM PUBLIC;
REVOKE ALL ON FUNCTION session__clean(mode text, source_keys text) FROM db_global_owner;
GRANT ALL ON FUNCTION session__clean(mode text, source_keys text) TO db_global_owner;
GRANT ALL ON FUNCTION session__clean(mode text, source_keys text) TO cabar_base_role;
GRANT ALL ON FUNCTION session__clean(mode text, source_keys text) TO db_global__app_role;


--
-- Name: session__get(text); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION session__get(p_key text) FROM PUBLIC;
REVOKE ALL ON FUNCTION session__get(p_key text) FROM db_global_owner;
GRANT ALL ON FUNCTION session__get(p_key text) TO db_global_owner;
GRANT ALL ON FUNCTION session__get(p_key text) TO db_global__app_role;
GRANT ALL ON FUNCTION session__get(p_key text) TO db_global__read_role;


--
-- Name: session__get(text, text); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION session__get(p_key1 text, p_key2 text) FROM PUBLIC;
REVOKE ALL ON FUNCTION session__get(p_key1 text, p_key2 text) FROM db_global_owner;
GRANT ALL ON FUNCTION session__get(p_key1 text, p_key2 text) TO db_global_owner;
GRANT ALL ON FUNCTION session__get(p_key1 text, p_key2 text) TO db_global__app_role;
GRANT ALL ON FUNCTION session__get(p_key1 text, p_key2 text) TO db_global__read_role;


--
-- Name: session__get(text, text, text); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION session__get(p_key1 text, p_key2 text, p_key3 text) FROM PUBLIC;
REVOKE ALL ON FUNCTION session__get(p_key1 text, p_key2 text, p_key3 text) FROM db_global_owner;
GRANT ALL ON FUNCTION session__get(p_key1 text, p_key2 text, p_key3 text) TO db_global_owner;
GRANT ALL ON FUNCTION session__get(p_key1 text, p_key2 text, p_key3 text) TO db_global__read_role;


--
-- Name: session__list(); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION session__list() FROM PUBLIC;
REVOKE ALL ON FUNCTION session__list() FROM db_global_owner;
GRANT ALL ON FUNCTION session__list() TO db_global_owner;
GRANT ALL ON FUNCTION session__list() TO cabar_base_role;
GRANT ALL ON FUNCTION session__list() TO db_global__app_role;


--
-- Name: session__reset(); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION session__reset() FROM PUBLIC;
REVOKE ALL ON FUNCTION session__reset() FROM db_global_owner;
GRANT ALL ON FUNCTION session__reset() TO db_global_owner;
GRANT ALL ON FUNCTION session__reset() TO cabar_base_role;
GRANT ALL ON FUNCTION session__reset() TO db_global__app_role;


--
-- Name: session__set(text, text); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION session__set(p_key text, value text) FROM PUBLIC;
REVOKE ALL ON FUNCTION session__set(p_key text, value text) FROM db_global_owner;
GRANT ALL ON FUNCTION session__set(p_key text, value text) TO db_global_owner;
GRANT ALL ON FUNCTION session__set(p_key text, value text) TO cabar_base_role;
GRANT ALL ON FUNCTION session__set(p_key text, value text) TO db_global__app_role;


--
-- Name: session__set(text, text, text); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION session__set(p_key1 text, p_key2 text, value text) FROM PUBLIC;
REVOKE ALL ON FUNCTION session__set(p_key1 text, p_key2 text, value text) FROM db_global_owner;
GRANT ALL ON FUNCTION session__set(p_key1 text, p_key2 text, value text) TO db_global_owner;
GRANT ALL ON FUNCTION session__set(p_key1 text, p_key2 text, value text) TO cabar_base_role;
GRANT ALL ON FUNCTION session__set(p_key1 text, p_key2 text, value text) TO db_global__app_role;


--
-- Name: session__set(text, text, text, text); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION session__set(p_key1 text, p_key2 text, p_key3 text, value text) FROM PUBLIC;
REVOKE ALL ON FUNCTION session__set(p_key1 text, p_key2 text, p_key3 text, value text) FROM db_global_owner;
GRANT ALL ON FUNCTION session__set(p_key1 text, p_key2 text, p_key3 text, value text) TO db_global_owner;
GRANT ALL ON FUNCTION session__set(p_key1 text, p_key2 text, p_key3 text, value text) TO cabar_base_role;
GRANT ALL ON FUNCTION session__set(p_key1 text, p_key2 text, p_key3 text, value text) TO db_global__app_role;


--
-- Name: tables__getsert(text); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION tables__getsert(p_table text) FROM PUBLIC;
REVOKE ALL ON FUNCTION tables__getsert(p_table text) FROM db_global_owner;
GRANT ALL ON FUNCTION tables__getsert(p_table text) TO db_global_owner;
GRANT ALL ON FUNCTION tables__getsert(p_table text) TO cabar_base_role;
GRANT ALL ON FUNCTION tables__getsert(p_table text) TO db_global__app_owner_role;


--
-- Name: tables__getsert(text[]); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION tables__getsert(p_tables text[]) FROM PUBLIC;
REVOKE ALL ON FUNCTION tables__getsert(p_tables text[]) FROM db_global_owner;
GRANT ALL ON FUNCTION tables__getsert(p_tables text[]) TO db_global_owner;
GRANT ALL ON FUNCTION tables__getsert(p_tables text[]) TO cabar_base_role;
GRANT ALL ON FUNCTION tables__getsert(p_tables text[]) TO db_global__app_owner_role;


--
-- Name: tg_deny_updates(); Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON FUNCTION tg_deny_updates() FROM PUBLIC;
REVOKE ALL ON FUNCTION tg_deny_updates() FROM db_global_owner;
GRANT ALL ON FUNCTION tg_deny_updates() TO db_global_owner;
GRANT ALL ON FUNCTION tg_deny_updates() TO PUBLIC;
GRANT ALL ON FUNCTION tg_deny_updates() TO db_global__read_role;


SET search_path = cabar, pg_catalog;

--
-- Name: component_schemas; Type: ACL; Schema: cabar; Owner: db_global_owner
--

REVOKE ALL ON TABLE component_schemas FROM PUBLIC;
REVOKE ALL ON TABLE component_schemas FROM db_global_owner;
GRANT ALL ON TABLE component_schemas TO db_global_owner;
GRANT INSERT ON TABLE component_schemas TO cabar_base_role;
GRANT SELECT ON TABLE component_schemas TO db_global__ops_support_role;
GRANT SELECT ON TABLE component_schemas TO db_global__developer_role;


--
-- Name: component_schemas_v; Type: ACL; Schema: cabar; Owner: db_global_owner
--

REVOKE ALL ON TABLE component_schemas_v FROM PUBLIC;
REVOKE ALL ON TABLE component_schemas_v FROM db_global_owner;
GRANT ALL ON TABLE component_schemas_v TO db_global_owner;
GRANT SELECT ON TABLE component_schemas_v TO db_global__ops_support_role;
GRANT SELECT ON TABLE component_schemas_v TO db_global__developer_role;


SET search_path = tools, pg_catalog;

--
-- Name: space; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE space FROM PUBLIC;
REVOKE ALL ON TABLE space FROM db_global_owner;
GRANT ALL ON TABLE space TO db_global_owner;
GRANT SELECT ON TABLE space TO db_global__developer_role;


SET search_path = code_internal, pg_catalog;

--
-- Name: object_type; Type: ACL; Schema: code_internal; Owner: db_global_owner
--

REVOKE ALL ON TABLE object_type FROM PUBLIC;
REVOKE ALL ON TABLE object_type FROM db_global_owner;
GRANT ALL ON TABLE object_type TO db_global_owner;
GRANT SELECT ON TABLE object_type TO db_global__developer_role;


--
-- Name: objects; Type: ACL; Schema: code_internal; Owner: db_global_owner
--

REVOKE ALL ON TABLE objects FROM PUBLIC;
REVOKE ALL ON TABLE objects FROM db_global_owner;
GRANT ALL ON TABLE objects TO db_global_owner;
GRANT INSERT,UPDATE ON TABLE objects TO cabar_base_role;
GRANT SELECT ON TABLE objects TO db_global__developer_role;


--
-- Name: objects_v; Type: ACL; Schema: code_internal; Owner: db_global_owner
--

REVOKE ALL ON TABLE objects_v FROM PUBLIC;
REVOKE ALL ON TABLE objects_v FROM db_global_owner;
GRANT ALL ON TABLE objects_v TO db_global_owner;
GRANT SELECT ON TABLE objects_v TO db_global__developer_role;


--
-- Name: templates_v; Type: ACL; Schema: code_internal; Owner: db_global_owner
--

REVOKE ALL ON TABLE templates_v FROM PUBLIC;
REVOKE ALL ON TABLE templates_v FROM db_global_owner;
GRANT ALL ON TABLE templates_v TO db_global_owner;
GRANT SELECT ON TABLE templates_v TO db_global__developer_role;


SET search_path = inheritance, pg_catalog;

--
-- Name: child_objects_exclude_tables; Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON TABLE child_objects_exclude_tables FROM PUBLIC;
REVOKE ALL ON TABLE child_objects_exclude_tables FROM db_global_owner;
GRANT ALL ON TABLE child_objects_exclude_tables TO db_global_owner;
GRANT SELECT ON TABLE child_objects_exclude_tables TO db_global__developer_role;


SET search_path = tools, pg_catalog;

--
-- Name: tables; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE tables FROM PUBLIC;
REVOKE ALL ON TABLE tables FROM db_global_owner;
GRANT ALL ON TABLE tables TO db_global_owner;
GRANT SELECT ON TABLE tables TO db_global__read_role;
GRANT SELECT ON TABLE tables TO db_global__developer_role;


--
-- Name: tables_v; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE tables_v FROM PUBLIC;
REVOKE ALL ON TABLE tables_v FROM db_global_owner;
GRANT ALL ON TABLE tables_v TO db_global_owner;
GRANT SELECT ON TABLE tables_v TO db_global__read_role;
GRANT SELECT ON TABLE tables_v TO db_global__developer_role;


SET search_path = inheritance, pg_catalog;

--
-- Name: child_objects_v; Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON TABLE child_objects_v FROM PUBLIC;
REVOKE ALL ON TABLE child_objects_v FROM db_global_owner;
GRANT ALL ON TABLE child_objects_v TO db_global_owner;
GRANT SELECT ON TABLE child_objects_v TO db_global__developer_role;


--
-- Name: child_objects_exclude_tables_v; Type: ACL; Schema: inheritance; Owner: db_global_owner
--

REVOKE ALL ON TABLE child_objects_exclude_tables_v FROM PUBLIC;
REVOKE ALL ON TABLE child_objects_exclude_tables_v FROM db_global_owner;
GRANT ALL ON TABLE child_objects_exclude_tables_v TO db_global_owner;
GRANT SELECT ON TABLE child_objects_exclude_tables_v TO db_global__developer_role;


SET search_path = object_monitor, pg_catalog;

--
-- Name: monitors; Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON TABLE monitors FROM PUBLIC;
REVOKE ALL ON TABLE monitors FROM db_global_owner;
GRANT ALL ON TABLE monitors TO db_global_owner;
GRANT SELECT ON TABLE monitors TO db_global__read_role;


--
-- Name: monitors_consumers; Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON TABLE monitors_consumers FROM PUBLIC;
REVOKE ALL ON TABLE monitors_consumers FROM db_global_owner;
GRANT ALL ON TABLE monitors_consumers TO db_global_owner;
GRANT SELECT ON TABLE monitors_consumers TO db_global__read_role;


--
-- Name: monitors_tables; Type: ACL; Schema: object_monitor; Owner: db_global_owner
--

REVOKE ALL ON TABLE monitors_tables FROM PUBLIC;
REVOKE ALL ON TABLE monitors_tables FROM db_global_owner;
GRANT ALL ON TABLE monitors_tables TO db_global_owner;
GRANT SELECT ON TABLE monitors_tables TO db_global__read_role;


SET search_path = pgq, pg_catalog;

--
-- Name: consumer; Type: ACL; Schema: pgq; Owner: skytools
--

REVOKE ALL ON TABLE consumer FROM PUBLIC;
REVOKE ALL ON TABLE consumer FROM skytools;
GRANT ALL ON TABLE consumer TO skytools;
GRANT SELECT ON TABLE consumer TO PUBLIC;


--
-- Name: event_template; Type: ACL; Schema: pgq; Owner: skytools
--

REVOKE ALL ON TABLE event_template FROM PUBLIC;
REVOKE ALL ON TABLE event_template FROM skytools;
GRANT ALL ON TABLE event_template TO skytools;
GRANT SELECT ON TABLE event_template TO PUBLIC;


--
-- Name: queue; Type: ACL; Schema: pgq; Owner: skytools
--

REVOKE ALL ON TABLE queue FROM PUBLIC;
REVOKE ALL ON TABLE queue FROM skytools;
GRANT ALL ON TABLE queue TO skytools;
GRANT SELECT ON TABLE queue TO PUBLIC;


--
-- Name: retry_queue; Type: ACL; Schema: pgq; Owner: skytools
--

REVOKE ALL ON TABLE retry_queue FROM PUBLIC;
REVOKE ALL ON TABLE retry_queue FROM skytools;
GRANT ALL ON TABLE retry_queue TO skytools;
GRANT SELECT ON TABLE retry_queue TO PUBLIC;


--
-- Name: subscription; Type: ACL; Schema: pgq; Owner: skytools
--

REVOKE ALL ON TABLE subscription FROM PUBLIC;
REVOKE ALL ON TABLE subscription FROM skytools;
GRANT ALL ON TABLE subscription TO skytools;
GRANT SELECT ON TABLE subscription TO PUBLIC;


--
-- Name: tick; Type: ACL; Schema: pgq; Owner: skytools
--

REVOKE ALL ON TABLE tick FROM PUBLIC;
REVOKE ALL ON TABLE tick FROM skytools;
GRANT ALL ON TABLE tick TO skytools;
GRANT SELECT ON TABLE tick TO PUBLIC;


SET search_path = tools, pg_catalog;

--
-- Name: activity; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE activity FROM PUBLIC;
REVOKE ALL ON TABLE activity FROM db_global_owner;
GRANT ALL ON TABLE activity TO db_global_owner;
GRANT SELECT ON TABLE activity TO PUBLIC;
GRANT SELECT ON TABLE activity TO db_global__developer_role;


--
-- Name: df; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE df FROM PUBLIC;
REVOKE ALL ON TABLE df FROM db_global_owner;
GRANT ALL ON TABLE df TO db_global_owner;
GRANT SELECT ON TABLE df TO PUBLIC;
GRANT SELECT ON TABLE df TO db_global__ops_support_role;
GRANT SELECT ON TABLE df TO db_global__developer_role;


--
-- Name: application_columns; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE application_columns FROM PUBLIC;
REVOKE ALL ON TABLE application_columns FROM db_global_owner;
GRANT ALL ON TABLE application_columns TO db_global_owner;
GRANT SELECT ON TABLE application_columns TO db_global__developer_role;
GRANT SELECT ON TABLE application_columns TO db_global__read_role;


--
-- Name: application_indexes; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE application_indexes FROM PUBLIC;
REVOKE ALL ON TABLE application_indexes FROM db_global_owner;
GRANT ALL ON TABLE application_indexes TO db_global_owner;
GRANT SELECT ON TABLE application_indexes TO db_global__read_role;
GRANT SELECT ON TABLE application_indexes TO db_global__developer_role;


--
-- Name: column_info; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE column_info FROM PUBLIC;
REVOKE ALL ON TABLE column_info FROM db_global_owner;
GRANT ALL ON TABLE column_info TO db_global_owner;
GRANT SELECT ON TABLE column_info TO PUBLIC;
GRANT SELECT ON TABLE column_info TO db_global__developer_role;


--
-- Name: df_index; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE df_index FROM PUBLIC;
REVOKE ALL ON TABLE df_index FROM db_global_owner;
GRANT ALL ON TABLE df_index TO db_global_owner;
GRANT SELECT ON TABLE df_index TO PUBLIC;
GRANT SELECT ON TABLE df_index TO db_global__developer_role;
GRANT SELECT ON TABLE df_index TO db_global__ops_support_role;


--
-- Name: dump_time; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE dump_time FROM PUBLIC;
REVOKE ALL ON TABLE dump_time FROM db_global_owner;
GRANT ALL ON TABLE dump_time TO db_global_owner;
GRANT SELECT ON TABLE dump_time TO PUBLIC;
GRANT SELECT ON TABLE dump_time TO db_global__developer_role;


--
-- Name: indexes; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE indexes FROM PUBLIC;
REVOKE ALL ON TABLE indexes FROM db_global_owner;
GRANT ALL ON TABLE indexes TO db_global_owner;
GRANT SELECT ON TABLE indexes TO PUBLIC;
GRANT SELECT ON TABLE indexes TO db_global__developer_role;


--
-- Name: lock_waits; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE lock_waits FROM PUBLIC;
REVOKE ALL ON TABLE lock_waits FROM db_global_owner;
GRANT ALL ON TABLE lock_waits TO db_global_owner;
GRANT SELECT ON TABLE lock_waits TO db_global__ops_support_role;
GRANT SELECT ON TABLE lock_waits TO db_global__developer_role;


--
-- Name: object_types; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE object_types FROM PUBLIC;
REVOKE ALL ON TABLE object_types FROM db_global_owner;
GRANT ALL ON TABLE object_types TO db_global_owner;
GRANT INSERT,UPDATE ON TABLE object_types TO cabar_base_role;
GRANT SELECT ON TABLE object_types TO db_global__read_role;
GRANT SELECT ON TABLE object_types TO db_global__developer_role;


--
-- Name: ownership; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE ownership FROM PUBLIC;
REVOKE ALL ON TABLE ownership FROM db_global_owner;
GRANT ALL ON TABLE ownership TO db_global_owner;
GRANT SELECT ON TABLE ownership TO db_global__developer_role;


--
-- Name: ownership_bad; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE ownership_bad FROM PUBLIC;
REVOKE ALL ON TABLE ownership_bad FROM db_global_owner;
GRANT ALL ON TABLE ownership_bad TO db_global_owner;
GRANT SELECT ON TABLE ownership_bad TO db_global__developer_role;


--
-- Name: role_members; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE role_members FROM PUBLIC;
REVOKE ALL ON TABLE role_members FROM db_global_owner;
GRANT ALL ON TABLE role_members TO db_global_owner;
GRANT SELECT ON TABLE role_members TO db_global__developer_role;


--
-- Name: schema_patches; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE schema_patches FROM PUBLIC;
REVOKE ALL ON TABLE schema_patches FROM db_global_owner;
GRANT ALL ON TABLE schema_patches TO db_global_owner;
GRANT SELECT ON TABLE schema_patches TO db_global__read_role;
GRANT SELECT ON TABLE schema_patches TO db_global__developer_role;


--
-- Name: schema_patches_v; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE schema_patches_v FROM PUBLIC;
REVOKE ALL ON TABLE schema_patches_v FROM db_global_owner;
GRANT ALL ON TABLE schema_patches_v TO db_global_owner;
GRANT SELECT ON TABLE schema_patches_v TO db_global__developer_role;
GRANT SELECT ON TABLE schema_patches_v TO db_global__read_role;


--
-- Name: seed_objects; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE seed_objects FROM PUBLIC;
REVOKE ALL ON TABLE seed_objects FROM db_global_owner;
GRANT ALL ON TABLE seed_objects TO db_global_owner;
GRANT INSERT,UPDATE ON TABLE seed_objects TO cabar_base_role;
GRANT SELECT ON TABLE seed_objects TO db_global__read_role;
GRANT SELECT ON TABLE seed_objects TO db_global__developer_role;


--
-- Name: seed_objects_v; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE seed_objects_v FROM PUBLIC;
REVOKE ALL ON TABLE seed_objects_v FROM db_global_owner;
GRANT ALL ON TABLE seed_objects_v TO db_global_owner;
GRANT SELECT ON TABLE seed_objects_v TO db_global__read_role;
GRANT SELECT ON TABLE seed_objects_v TO db_global__developer_role;


--
-- Name: seed_objects__dump; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE seed_objects__dump FROM PUBLIC;
REVOKE ALL ON TABLE seed_objects__dump FROM db_global_owner;
GRANT ALL ON TABLE seed_objects__dump TO db_global_owner;
GRANT SELECT ON TABLE seed_objects__dump TO db_global__developer_role;
GRANT SELECT ON TABLE seed_objects__dump TO db_global__read_role;


--
-- Name: stat_activity; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE stat_activity FROM PUBLIC;
REVOKE ALL ON TABLE stat_activity FROM db_global_owner;
GRANT ALL ON TABLE stat_activity TO db_global_owner;
GRANT SELECT ON TABLE stat_activity TO PUBLIC;
GRANT SELECT ON TABLE stat_activity TO db_global__developer_role;


--
-- Name: stats_small; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE stats_small FROM PUBLIC;
REVOKE ALL ON TABLE stats_small FROM db_global_owner;
GRANT ALL ON TABLE stats_small TO db_global_owner;
GRANT SELECT ON TABLE stats_small TO PUBLIC;
GRANT SELECT ON TABLE stats_small TO db_global__developer_role;


--
-- Name: sub_asserts; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE sub_asserts FROM PUBLIC;
REVOKE ALL ON TABLE sub_asserts FROM db_global_owner;
GRANT ALL ON TABLE sub_asserts TO db_global_owner;
GRANT SELECT ON TABLE sub_asserts TO PUBLIC;
GRANT SELECT ON TABLE sub_asserts TO db_global__developer_role;


--
-- Name: table_sequences; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE table_sequences FROM PUBLIC;
REVOKE ALL ON TABLE table_sequences FROM db_global_owner;
GRANT ALL ON TABLE table_sequences TO db_global_owner;
GRANT SELECT ON TABLE table_sequences TO db_global__read_role;
GRANT SELECT ON TABLE table_sequences TO db_global__developer_role;


--
-- Name: tables_id_seq; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON SEQUENCE tables_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE tables_id_seq FROM db_global_owner;
GRANT ALL ON SEQUENCE tables_id_seq TO db_global_owner;
GRANT USAGE ON SEQUENCE tables_id_seq TO db_global__app_owner_role;


--
-- Name: tests; Type: ACL; Schema: tools; Owner: db_global_owner
--

REVOKE ALL ON TABLE tests FROM PUBLIC;
REVOKE ALL ON TABLE tests FROM db_global_owner;
GRANT ALL ON TABLE tests TO db_global_owner;
GRANT SELECT ON TABLE tests TO db_global__developer_role;


--
-- PostgreSQL database dump complete
--

COMMIT;
